package oracle.fsgbu.obscfcm.services.report;

import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;
import org.springframework.web.servlet.handler.MappedInterceptor;
import oracle.fsgbu.plato.logger.PlatoLogger;

import com.fasterxml.jackson.databind.ObjectMapper;

import brave.sampler.Sampler;
import oracle.fsgbu.plato.core.filter.PlatoCorsFilter;
import oracle.fsgbu.plato.core.interceptor.PlatoInterceptor;
import oracle.fsgbu.sms.rbac.interceptor.RbacInterceptor;

@Configuration
@EnableTransactionManagement
@EnableAspectJAutoProxy(proxyTargetClass = true)
public class ApplicationConfig extends WebMvcConfigurerAdapter {

  private static final PlatoLogger LOG = new PlatoLogger(ApplicationConfig.class);
  
  @Bean
  public FilterRegistrationBean corsFilterRegistration () {
         FilterRegistrationBean registration = new FilterRegistrationBean();
         registration.setFilter(new PlatoCorsFilter());
         registration.addUrlPatterns("*");
         registration.setOrder(0);
         return registration;
  }


  @Bean
  public MappedInterceptor gemInterceptor(PlatoInterceptor platoInterceptor) {
    LOG.info("Added gemInterceptor for fetching the application headers");
    return new MappedInterceptor(new String[] {"/**"}, platoInterceptor);
  }
  
  @Bean
  public MappedInterceptor gcsInterceptor() {
	LOG.info("Added gcsInterceptor for fetching the application headers");
	return new MappedInterceptor(new String[] { "/**" },
			new oracle.fsgbu.gus.persistence.provider.GcsInterceptor());
  }
  
  @Bean
	public MappedInterceptor rbacInterceptor() {
		LOG.info("Added rbacInterceptor for fetching the application headers");
		return new MappedInterceptor(new String[] { "/**" }, new RbacInterceptor());
	} 
  
  @Bean
  @LoadBalanced
  public RestTemplate restTemplate() {
      RestTemplate restTemplate = new RestTemplate();
      MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
      converter.setObjectMapper(new ObjectMapper());
      restTemplate.getMessageConverters().add(converter);
      return restTemplate;
  }
  
  @Bean
  public Sampler defaultSampler() {
		return Sampler.ALWAYS_SAMPLE;
  }
}
