package oracle.fsgbu.obscfcm.services.report.client;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import io.micrometer.core.instrument.util.StringUtils;
import oracle.fsgbu.accountMaintenance.api.model.AccentryWebModel;
import oracle.fsgbu.accountMaintenance.api.model.AccentrycodeWebModel;
import oracle.fsgbu.accountMaintenance.api.model.AccountingTransactionCollection;
import oracle.fsgbu.accountMaintenance.api.model.ExternalWebModel;
import oracle.fsgbu.accountMaintenance.api.model.IntaccmapWebModel;
import oracle.fsgbu.accountMaintenance.api.model.RoleWebModel;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class AccountMaintenanceServiceClientImpl implements AccountMaintenanceServiceClient{
	
	private static final PlatoLogger LOGGER = new PlatoLogger(AccountMaintenanceServiceClientImpl.class);
	
	
	@Value("${obscfcm.account-maintenance.service.appId}")
	private String appId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	public AccountMaintenanceServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}

	@Override
	public RoleWebModel getAccountRoleTWData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<RoleWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ROLE_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, RoleWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountRoleTWData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public RoleWebModel getAccountRoleTMData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<RoleWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ROLE_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, RoleWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountRoleTMData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public AccentryWebModel getAccountEntryTWData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<AccentryWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ENTRY_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AccentryWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountEntryTWData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public AccentryWebModel getAccountEntryTMData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<AccentryWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ENTRY_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AccentryWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountEntryTMData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public AccentrycodeWebModel getAccountEntryCodeTWData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<AccentrycodeWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ENTRY_CODE_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AccentrycodeWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountEntryCodeTWData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public AccentrycodeWebModel getAccountEntryCodeTMData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<AccentrycodeWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_ACCOUNT_ENTRY_CODE_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AccentrycodeWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getAccountEntryCodeTMData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public IntaccmapWebModel getInternalAccountMapTWData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<IntaccmapWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_INTERNAL_ACCOUNT_MAP_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, IntaccmapWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getInternalAccountMapTWData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public IntaccmapWebModel getInternalAccountMapTMData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<IntaccmapWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_INTERNAL_ACCOUNT_MAP_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, IntaccmapWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getInternalAccountMapTMData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public ExternalWebModel getExternalAccountMapTWData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<ExternalWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_EXTERNAL_ACCOUNT_MAP_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, ExternalWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getExternalAccountMapTWData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}

	@Override
	public ExternalWebModel getExternalAccountMapTMData(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<ExternalWebModel> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("resourceId", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_EXTERNAL_ACCOUNT_MAP_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, ExternalWebModel.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getExternalAccountMapTMData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}
	
	@Override
	public AccountingTransactionCollection getAccountingDataByFinRefNo(String userId, String branchCode, String finRefNo, String authorizationStatus) {
		
		LOGGER.info("Getting accounting data from Account Service");
		try {
			
			ResponseEntity<AccountingTransactionCollection> response = null;
			Map<String, Object> params = new HashMap<>();
			params.put("refNo", finRefNo);
			params.put("includecloseandunauth", Boolean.TRUE);
			params.put("limit", 0);
			params.put("offset", 0);
		
			String formUrl = null ;
			if(StringUtils.isNotBlank(authorizationStatus) && authorizationStatus.equals(ReportConstants.AUTHORIZED)) {
				formUrl = protocol + ReportConstants.GET_ACCOUNTING_DATA_TM_URL ;
			}else if(StringUtils.isBlank(authorizationStatus) || authorizationStatus.equals(ReportConstants.UNAUTHORIZED)){
				formUrl = protocol + ReportConstants.GET_ACCOUNTING_DATA_TW_URL ;
			}
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AccountingTransactionCollection.class, params);
			
			if (response.getBody() != null) {
				return response.getBody();
			}
		} catch (Exception e) {
			
			LOGGER.error("Error in getAccountingData:: {}",e);
		}
		
		return null;
	}
}
