package oracle.fsgbu.obscfcm.services.report.client;

import oracle.fsgbu.cmc.services.branch.dto.CmcTmSystemDatesCollection;
import oracle.fsgbu.cmc.services.branch.service.wrapper.CmcTmSystemDatesWrapper;
import oracle.fsgbu.cmc.services.cmc.dto.CoreBranchCollection;
import oracle.fsgbu.cmc.services.cmc.service.wrapper.CmcTmCoreBranchWrapper;
import oracle.fsgbu.cmc.services.cmc.dto.CoreBranchDTO;
import oracle.fsgbu.gus.persistence.provider.GlobalHolder;
import oracle.fsgbu.plato.logger.PlatoLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class CmcBranchServiceClientImpl implements CmcBranchServiceClient {

	private final PlatoLogger LOGGER = new PlatoLogger(CmcBranchServiceClientImpl.class);

	
	@Autowired
	private CmcTmCoreBranchWrapper cmcTmCoreBranchWrapper;

	@Autowired
	private CmcTmSystemDatesWrapper cmcTmSystemDatesWrapper;

	public CmcBranchServiceClientImpl(CmcTmCoreBranchWrapper cmcTmCoreBranchWrapper) {
		this.cmcTmCoreBranchWrapper = cmcTmCoreBranchWrapper;
	}

	@Autowired
	public CmcBranchServiceClientImpl(CmcTmSystemDatesWrapper cmcTmSystemDatesWrapper) {
		this.cmcTmSystemDatesWrapper = cmcTmSystemDatesWrapper;
	}

	@Override
	public String getCurrentBusinessDate(String userId, String branchCode) {
		LOGGER.info("Inside getCurrentBusinessDate");
		try {
			ResponseEntity<CmcTmSystemDatesCollection> response =
					cmcTmSystemDatesWrapper.getAll(null, null, null,
							branchCode, null, null, branchCode);
			if (response != null && null != response.getBody() && null != response.getBody().getData()
					&& response.getBody().getData().size() > 0) {
				String responseBody = response.getBody().getData().get(0).getToday();
				LOGGER.info("response body: Branch Date is {} ", responseBody);
				LOGGER.info("End getCurrentBusinessDate()");
				return responseBody;
			}
		} catch (Exception ex) {
			LOGGER.error("An error has occurred in getCurrentBusinessDate: {}", ex);
		}
		LOGGER.info("Returning null from getCurrentBusinessDate...");
		return null;
	}
	
	@Override
	public CoreBranchCollection getBranchDetails() {
		LOGGER.info("In getBranchDetails()");
		try {
			ResponseEntity<CoreBranchCollection> responseEntity = cmcTmCoreBranchWrapper.getAll(null, null, null, null,
					null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, GlobalHolder.getRequestContext().getCurrBranch());

			if (responseEntity != null && responseEntity.getBody() != null) {
				LOGGER.info("response body: {}", responseEntity.getBody());
				LOGGER.info("End getBranchDetails()");
				return responseEntity.getBody();
			}

		} catch (Exception e) {
			LOGGER.error("Error in getBranchDetails(): {}", e);
		}
		LOGGER.info("End getBranchDetails() returning null");
		return null;
	}

	@Override
	public String getBranchCurrency(String branchCode) {
		LOGGER.info("Inside getBranchCurrency");
		CoreBranchCollection branchCollection = this.getBranchDetails();
		String localBranchCurrency = null;
		if (branchCollection != null && branchCollection.getData() != null && !branchCollection.getData().isEmpty()) {
			Optional<CoreBranchDTO> isBranchPresent = branchCollection.getData().parallelStream()
					.filter(branchDetails -> branchDetails.getBranchCode() != null
							&& branchDetails.getBranchCode().equalsIgnoreCase(branchCode))
					.findFirst();
			if (isBranchPresent.isPresent()) {
				CoreBranchDTO branchDetails = isBranchPresent.get();
				localBranchCurrency = String.valueOf(branchDetails.getBranchLcy());
			}
		}
		LOGGER.info("Branch code :{}  and its currency :{}",branchCode,localBranchCurrency);
		return localBranchCurrency;
	}
}
