package oracle.fsgbu.obscfcm.services.report.client;

import java.math.BigDecimal;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.model.CurrencyRateCalculatorDtoWrapper;
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto;
import oracle.fsgbu.obscfcm.services.report.util.MapperUtil;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class CmcCurrencyServiceClientImpl implements CmcCurrencyServiceClient{
	
	private final PlatoLogger logger = new PlatoLogger(CmcCurrencyServiceClientImpl.class);
	
	@Value("${cmncore.cmc-branch.service.appId}")
	private String appId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	public CmcCurrencyServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}
	
	public HttpEntity<Object> convertToHttpEntity() {
		HttpHeaders headers = ReportingUtil.buildHeaders(appId);
		return new HttpEntity<>(headers);
	}
	
	@Override
	public CurrencyUtilityDto getConvertedCcyValue(String ccy1, String ccy2, BigDecimal ccy1Amt, String inrateDate,
			String userId, String branchCode) {
	
		ResponseEntity<CurrencyRateCalculatorDtoWrapper> currencyUtility = null;
		
		try {
			String formUrl = protocol + ReportConstants.CURRENCY_SERVICES ;
			UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(formUrl).path("/"+branchCode).path("/"+ccy1).path("/"+ccy2)
					.path("/STANDARD").path("/M").path("/"+ccy1Amt.toString()).path("/Y").path("/"+inrateDate);
			HttpEntity<Object> entity = convertToHttpEntity();
			currencyUtility = restTemplate.exchange(builder.toUriString(), HttpMethod.GET, entity, CurrencyRateCalculatorDtoWrapper.class);
		} catch (Exception e) {
			logger.error("Error in getConvertedCcyValue: {}",e.getMessage());
		}
		
		if (currencyUtility != null) {
			CurrencyUtilityDto currencyUtilityDto = MapperUtil.INSTANCE.clone(currencyUtility.getBody().getData());
			return currencyUtilityDto;
		}
		
		return null;
		
	}

}
