package oracle.fsgbu.obscfcm.services.report.client;

import oracle.fsgbu.eventService.api.model.CreditNoteModel;
import oracle.fsgbu.eventService.api.model.InvoiceDto;
import oracle.fsgbu.eventService.api.model.PaymentDto;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import oracle.fsgbu.plato.logger.PlatoLogger;

import oracle.fsgbu.eventService.api.model.InvoiceDto;
import oracle.fsgbu.eventService.api.model.PaymentDto;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.eventService.api.model.FileUploadDto;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import java.util.ArrayList;
import java.util.List;

@Service
public class EventServiceClientImpl implements EventServiceClient{
	
	private static final PlatoLogger LOGGER = new PlatoLogger(EventServiceClientImpl.class);
	
	@Value("${obscfcm.report.service.appId}")
	private String appId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	public EventServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}
	
	@Override
	public Boolean saveInvoiceData(InvoiceDto invoiceDto, String userId, String branchCode) {
		
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);
		
		try {
			String formUrl = protocol + ReportConstants.SAVE_INVOICES_SERVICE ;
			HttpEntity<InvoiceDto> entity = new HttpEntity<>(invoiceDto, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in saveInvoiceData: {}",e.getMessage());
		}
		return response.getBody();
	}
	
	@Override
	public Boolean savePaymentData(PaymentDto paymentDto, String userId, String branchCode) {
		
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);
		
		try {
			String formUrl = protocol + ReportConstants.SAVE_PAYMENT_SERVICE ;
			HttpEntity<PaymentDto> entity = new HttpEntity<>(paymentDto, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in savePaymentData : {}",e.getMessage());
		}
		return response.getBody();
	}

	@Override
	public Boolean saveUploadedFileData(FileUploadDto fileUploadDto,  String userId, String branchCode) {

		ResponseEntity<Boolean> response = ResponseEntity.ok(false);

		try {
			String formUrl = protocol + ReportConstants.SAVE_UPLOADED_FILE_SERVICE ;
			HttpEntity<FileUploadDto> entity = new HttpEntity<>(fileUploadDto, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in saveUploadedFileData : {}",e.getMessage());
		}
		return response.getBody();
	}

	@Override
	public Boolean saveCreditNote(CreditNoteModel creditNoteModel, String userId, String branchCode) {
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);

		try {
			String formUrl = protocol + ReportConstants.SAVE_CREDIT_NOTE ;
			HttpEntity<CreditNoteModel> entity = new HttpEntity<>(creditNoteModel, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in saveCreditNote : {}",e.getMessage());
		}
		return response.getBody();
	}
}
