package oracle.fsgbu.obscfcm.services.report.client;

import java.util.HashMap;
import java.util.Map;

import oracle.fsgbu.eventService.api.model.PurchaseOrderCollection;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import oracle.fsgbu.eventService.api.model.AutoDebitTxnCollection;
import oracle.fsgbu.eventService.api.model.InvoiceCollectionDto;
import oracle.fsgbu.eventService.api.model.PaymentCollectionDto;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class InstrumentReceivablesServiceClientImpl implements InstrumentReceivablesServiceClient {
	
	private final PlatoLogger logger = new PlatoLogger(InstrumentReceivablesServiceClientImpl.class);

	@Value("${obscfcm.instrument.service.appId}")
	private String appId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;
	
	@Autowired
	public InstrumentReceivablesServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}

	@Override
	public InvoiceCollectionDto getInvoiceByTxnRefNo(String transactionRefNo, String userId, String branchCode) {
		ResponseEntity<InvoiceCollectionDto> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("transactionRefNo", transactionRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_INVOICES_TXN_REF_NO;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, InvoiceCollectionDto.class, params);
		} catch (Exception e) {
			logger.error("Error in getInvoiceByTxnRefNo() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}
	
	@Override
	public AutoDebitTxnCollection getAutoDebitTxns(String userId, String branchCode, String executionId) {
		
		logger.info("Getting auto debit data from Finance Service");
		try {
			ResponseEntity<AutoDebitTxnCollection> response = null;
			
			Map<String, String> params = new HashMap<>();
			params.put(ReportConstants.EXECUTION_ID, executionId);
		
			String formUrl = protocol + ReportConstants.GET_AUTO_DEBIT_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, AutoDebitTxnCollection.class, params);
			
			if (response.getBody() != null) {
				return response.getBody();
			}
		} catch (Exception e) {
			
			logger.error("Error in getAutoDebitTxns() :{}",e);
		}
		
		return null;
	}
	
	@Override
	public InvoiceCollectionDto getInvoiceMaintByTxnRefNo(String transactionRefNo, String userId, String branchCode) {
		logger.info("Getting invoice maint data");
		try {
			ResponseEntity<InvoiceCollectionDto> response = null;
			
			Map<String, String> params = new HashMap<>();
			params.put("transactionRefNo", transactionRefNo);
		
			String formUrl = protocol + ReportConstants.GET_INVOICES_MAINT_TXN_REF_NO;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, InvoiceCollectionDto.class, params);
			
			if (response.getBody() != null) {
				return response.getBody();
			}
		} catch (Exception e) {
			
			logger.error("Error in getInvoiceMaintByTxnRefNo() :{}",e);
		}
		return null;
	}
	
	@Override
	public PaymentCollectionDto getPaymentByPayRefNo(String paymentRefNo, String userId, String branchCode) {
		logger.info("Getting payment maint data");
		try {
			ResponseEntity<PaymentCollectionDto> response = null;
			
			Map<String, String> params = new HashMap<>();
			params.put("paymentRefNo", paymentRefNo);
		
			String formUrl = protocol + ReportConstants.GET_PAYMENT_TXN_REF_NO;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, PaymentCollectionDto.class, params);
			
			if (response.getBody() != null) {
				return response.getBody();
			}
		} catch (Exception e) {
			
			logger.error("Error in getPaymentByPayRefNo() :{}",e);
		}
		return null;
	}
	
	@Override
	public PaymentCollectionDto getPaymentTwByPayRefNo(String paymentRefNo, String userId, String branchCode) {
		ResponseEntity<PaymentCollectionDto> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("paymentRefNo", paymentRefNo);
		params.put("version", "latest");
		try {
			String formUrl = protocol + ReportConstants.GET_PAYMENT_TXN_TW_REF_NO;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, PaymentCollectionDto.class, params);
		} catch (Exception e) {
			logger.error("Error in getPayTWByTxnRefNo() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}
	@Override
	public PurchaseOrderCollection getPoByTransactionRefNo(String transactionRefNo, Boolean isAuth, String userId, String branchCode) {
		logger.info("Getting invoice maint data for transactionRefNo :{},isAuth: {} ,userId : {} ,branchCode : {} ",transactionRefNo,isAuth,userId,branchCode);
		try {
			ResponseEntity<PurchaseOrderCollection> response = null;

			Map<String, String> params = new HashMap<>();
			params.put("transactionRefNo", transactionRefNo);
			params.put("isAuth", isAuth.toString());

			String formUrl = protocol + ReportConstants.GET_PO_BY_INT_NUM;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, PurchaseOrderCollection.class, params);

			if (response.getBody() != null) {
				return response.getBody();
			}
		} catch (Exception e) {

			logger.error("Error in getPoByTransactionRefNo() :{}",e.getMessage());
		}
		return null;
	}
}
