package oracle.fsgbu.obscfcm.services.report.client;

import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.client.RestTemplate;

import com.netflix.hystrix.exception.HystrixBadRequestException;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.obscfcm.services.scfcm.dto.RelationshipmasterWebModel;
import oracle.fsgbu.plato.logger.PlatoLogger;
 

@Service
public class ObscfcmCoreServiceClientImpl implements ObscfcmCoreServiceClient {
	
	private final PlatoLogger LOGGER = new PlatoLogger(ObscfcmCoreServiceClientImpl.class);

	@Value("${obscfcm.core.service.appId}")
	private String appId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;
	
	@Autowired
	public ObscfcmCoreServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}


	 @Override
	    public RelationshipmasterWebModel getByResourceId(String resourceId, String branchCode, String userId) {
	        ResponseEntity<RelationshipmasterWebModel> response = null;
	        LOGGER.info("Making get by resourceId call for resourceId {} , branchcode {}, userId {}", resourceId,branchCode,userId);
	        try {
	        	String formUrl = protocol + ReportConstants.OBSCFCM_CORE_RESOURCE_ID+resourceId+ReportConstants.CORE_RESOURCE_ID_APIPARAMS ;
	        	 LOGGER.info("formUrl {}",formUrl);
	            HttpHeaders headers = ReportingUtil.buildHeadersForEventService(appId, userId, branchCode); 
	            HttpEntity<Object> entity = new HttpEntity<>(headers);
	            response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, RelationshipmasterWebModel.class);
	        } catch (final HttpStatusCodeException ex) {
	            LOGGER.error("Error in getByResourceId :{}",ex);
	            if (ex.getStatusCode() == HttpStatus.NOT_FOUND) {
	                throw new HystrixBadRequestException(null, ex);
	            } else {
	                throw ex;
	            }
	        }
	        if (response != null) {
	           return response.getBody();
	        }
	        LOGGER.info("Exiting getByResourceId with response null");
	        return null;
	    }
	
}
