package oracle.fsgbu.obscfcm.services.report.client;

import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reconService.api.model.ExpDrCrPmtReconDTO;
import oracle.fsgbu.reconService.api.model.FinPmtReconDTO;
import oracle.fsgbu.reconService.api.model.InvCnReconDTO;
import oracle.fsgbu.reconService.api.model.InvPmtReconDTO;
import oracle.fsgbu.reconService.api.model.PmtAllocDTO;
import oracle.fsgbu.reconService.api.model.ReconDataCollection;
import oracle.fsgbu.reconService.api.model.ReconTxnDto;

@Service
public class ReconEventServiceClientImpl implements ReconEventServiceClient{
	
	private static final PlatoLogger LOGGER = new PlatoLogger(ReconEventServiceClientImpl.class);
	
	@Value("${obscfcm.report.service.appId}")
	private String appId;
	
	@Value("${obscfcm.manualrecon.service.appId}")
	private String reconAppId;
	
	@Value("${eureka.protocol:http}")
	private String protocol;
	
	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	public ReconEventServiceClientImpl(RestTemplate restTemplate) {
	    this.restTemplate = restTemplate;
	}
	
	@Override
	public Boolean saveExpDrCrData(ExpDrCrPmtReconDTO expDrCrPmtReconDTO, String userId, String branchCode) {
		
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);
		
		try {
			String formUrl = protocol + ReportConstants.SAVE_EXP_DRCR_RECON_SERVICE ;
			HttpEntity<ExpDrCrPmtReconDTO> entity = new HttpEntity<>(expDrCrPmtReconDTO, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in Exp DrCr Data {}", e);
		}
		return response.getBody();
	}
	@Override
	public Boolean saveInvPmtData(InvPmtReconDTO invPmtReconDTO, String userId, String branchCode) {
		
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);
		
		try {
			String formUrl = protocol + ReportConstants.SAVE_INV_PMT_RECON_SERVICE ;
			HttpEntity<InvPmtReconDTO> entity = new HttpEntity<>(invPmtReconDTO, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in Inv payment Data {}", e);
		}
		return response.getBody();
	}
	
	@Override
	public Boolean saveFinPmtData(FinPmtReconDTO finPmtReconDTO, String userId, String branchCode) {
		
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);
		
		try {
			String formUrl = protocol + ReportConstants.SAVE_FIN_PMT_RECON_SERVICE ;
			HttpEntity<FinPmtReconDTO> entity = new HttpEntity<>(finPmtReconDTO, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in Fin Payment Data {}", e);
		}
		return response.getBody();
	}

	@Override
	public Boolean savePmtAllocData(PmtAllocDTO pmtAllocDTO, String userId, String branchCode) {

		ResponseEntity<Boolean> response = ResponseEntity.ok(false);

		try {
			String formUrl = protocol + ReportConstants.SAVE_PMT_ALLOC_SERVICE;
			HttpEntity<PmtAllocDTO> entity = new HttpEntity<>(pmtAllocDTO, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in Payment Allocation Data {}", e);
		}
		return response.getBody();
	}

	@Override
	public Boolean saveInvCnData(InvCnReconDTO invCnReconDTO, String userId, String branchCode) {
		ResponseEntity<Boolean> response = ResponseEntity.ok(false);

		try {
			String formUrl = protocol + ReportConstants.SAVE_INV_CN_RECON_SERVICE;
			HttpEntity<InvCnReconDTO> entity = new HttpEntity<>(invCnReconDTO, ReportingUtil.buildHeadersForEventService(appId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.POST, entity, Boolean.class);
		} catch (Exception e) {
			LOGGER.error("Error in Invoice Credit Note Data {}", e);
		}
		return response.getBody();
	}

	@Override
	public ReconDataCollection getReconTwData(String reconTxnRefNo, String userId, String branchCode) {
		ResponseEntity<ReconDataCollection> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("reconTxnRefNo", reconTxnRefNo);
		try {
			String formUrl = protocol + ReportConstants.GET_RECON_TW_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(reconAppId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, ReconDataCollection.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getReconTwData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}
	
	@Override
	public ReconTxnDto getReconTmData(String reconTxnRefNo, String userId, String branchCode) {
		ResponseEntity<ReconTxnDto> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("reconTxnRefNo", reconTxnRefNo);
		try {
			String formUrl = protocol + ReportConstants.GET_RECON_TM_DATA_URL;
			HttpEntity<Object> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(reconAppId, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, ReconTxnDto.class, params);
		} catch (Exception e) {
			LOGGER.error("Error in getReconTmData() : {}",e.getMessage());
		}
		if (response != null) {
			return response.getBody();
		}
		return null;
	}
}
