package oracle.fsgbu.obscfcm.services.report.config;

import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Conditional;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.plato.common.util.TenantUtil;
import oracle.fsgbu.plato.core.condition.CloudDeployEnvironmentCondition;
import oracle.fsgbu.plato.core.persistence.provider.PlatoRegistry;
import oracle.fsgbu.plato.logger.PlatoLogger;

@EnableTransactionManagement
@Conditional({ CloudDeployEnvironmentCondition.class })
@Configuration
public class MultiTenantDatabaseConfig implements DatabaseConfiguration {

	@Autowired
	PlatoRegistry platoRegistry;

	private Map<String, String> ds2JNDIMap = new HashMap();

	private static final PlatoLogger LOGGER = new PlatoLogger(MultiTenantDatabaseConfig.class);

	public MultiTenantDatabaseConfig() {
		LOGGER.info("MultiTenantDatabaseConfig created...");

		ds2JNDIMap.put(ReportConstants.OBSCF_REPORT_DS, ReportConstants.OBSCF_REPORT_JNDI);
		ds2JNDIMap.put(ReportConstants.OBSCFCM_REPORT_DS, ReportConstants.OBSCFCM_REPORT_JNDI);
		ds2JNDIMap.put(ReportConstants.CMN_CORE_DS, ReportConstants.CMN_CORE_JNDI);
		ds2JNDIMap.put(ReportConstants.OBCM_REPORT_DS, ReportConstants.OBCM_REPORT_JNDI);
	}

	public DataSource springBatchDataSource(final String datasourceName) {
		DataSource ds = null;
		try {
			String jndiName = this.ds2JNDIMap.get(datasourceName);
			LOGGER.info("jndiName for dynamic datasource:  {}", jndiName);
			ds = platoRegistry.dynamicDataSource(
					ReportConstants.JNDI_NAMING_PREFIX + TenantUtil.getCurrentTenantCacheKey() + "/" + jndiName);
		} catch (Exception ex) {
			LOGGER.error("An error has occurred while creating dynamic datasource in MultiTenantDatabaseConfig class: "
					+ ex.getMessage());
		}
		return ds;
	}

	@Override
	public DataSource getDataSource(final String datasourceName) {
		return springBatchDataSource(datasourceName);
	}
}
