package oracle.fsgbu.obscfcm.services.report.config;

import javax.sql.DataSource;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Conditional;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.jndi.JndiObjectFactoryBean;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.plato.core.condition.DefaultDeployEnvironmentCondition;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Configuration
@Conditional({DefaultDeployEnvironmentCondition.class })
public class ObscfcmReportDatabaseConfig implements DatabaseConfiguration {

	private static final PlatoLogger LOGGER = new PlatoLogger(ObscfcmReportDatabaseConfig.class);

	private DataSource dataSource;

	public ObscfcmReportDatabaseConfig() {
		LOGGER.info("ObscfcmReportDatabaseConfig created...");
	}

	@Primary
	@Bean(name = "obscfcmReportDS")
	public DataSource obscfcmReportDataSource() throws Exception {
		JndiObjectFactoryBean bean = new JndiObjectFactoryBean();
		// TOMCAT Support Changes start
		String jndiName = ReportConstants.OBSCFCM_REPORT_JNDI;
		if (ReportConstants.IS_PLATOBATCH_JNDI_RESOURCES_PREFIX) {
			jndiName = ReportConstants.JNDI_NAMING_PREFIX + ReportConstants.OBSCFCM_REPORT_JNDI;
		} else {
			jndiName = ReportConstants.OBSCFCM_REPORT_JNDI;
		}
		bean.setJndiName(jndiName);
		//// TOMCAT Support Changes end
		bean.setProxyInterface(DataSource.class);
		bean.setLookupOnStartup(false);
		bean.afterPropertiesSet();
		dataSource = (DataSource) bean.getObject();
		return dataSource;
	}
	
	@Bean(name = "obscfReportDS")
	public DataSource obscfReportDataSource() throws Exception {
		JndiObjectFactoryBean bean = new JndiObjectFactoryBean();
		// TOMCAT Support Changes start
		String jndiName = ReportConstants.OBSCF_REPORT_JNDI;
		if (ReportConstants.IS_PLATOBATCH_JNDI_RESOURCES_PREFIX) {
			jndiName = ReportConstants.JNDI_NAMING_PREFIX + ReportConstants.OBSCF_REPORT_JNDI;
		} else {
			jndiName = ReportConstants.OBSCF_REPORT_JNDI;
		}
		bean.setJndiName(jndiName);
		//// TOMCAT Support Changes end
		bean.setProxyInterface(DataSource.class);
		bean.setLookupOnStartup(false);
		bean.afterPropertiesSet();
		dataSource = (DataSource) bean.getObject();
		return dataSource;
	}

	@Bean(name = "commonCoreDS")
	public DataSource cmnCoreDataSource() throws Exception {
		JndiObjectFactoryBean bean = new JndiObjectFactoryBean();
		// TOMCAT Support Changes start
		String jndiName = ReportConstants.CMN_CORE_JNDI;
		if (ReportConstants.IS_PLATOBATCH_JNDI_RESOURCES_PREFIX) {
			jndiName = ReportConstants.JNDI_NAMING_PREFIX + ReportConstants.CMN_CORE_JNDI;
		} else {
			jndiName = ReportConstants.CMN_CORE_JNDI;
		}
		bean.setJndiName(jndiName);
		//// TOMCAT Support Changes end
		bean.setProxyInterface(DataSource.class);
		bean.setLookupOnStartup(false);
		bean.afterPropertiesSet();
		dataSource = (DataSource) bean.getObject();
		return dataSource;
	}
	
	@Bean(name = "obcmReportDS")
	public DataSource obcmReportDataSource() throws Exception {
		JndiObjectFactoryBean bean = new JndiObjectFactoryBean();
		// TOMCAT Support Changes start
		String jndiName = ReportConstants.OBCM_REPORT_JNDI;
		if (ReportConstants.IS_PLATOBATCH_JNDI_RESOURCES_PREFIX) {
			jndiName = ReportConstants.JNDI_NAMING_PREFIX + ReportConstants.OBCM_REPORT_JNDI;
		} else {
			jndiName = ReportConstants.OBCM_REPORT_JNDI;
		}
		bean.setJndiName(jndiName);
		//// TOMCAT Support Changes end
		bean.setProxyInterface(DataSource.class);
		bean.setLookupOnStartup(false);
		bean.afterPropertiesSet();
		dataSource = (DataSource) bean.getObject();
		return dataSource;
	}

	@Override
	public DataSource getDataSource(final String datasourceName) throws Exception {
		switch (datasourceName) {
		case ReportConstants.OBSCF_REPORT_DS:
			return obscfReportDataSource();
		case ReportConstants.OBSCFCM_REPORT_DS:
			return obscfcmReportDataSource();
		case ReportConstants.CMN_CORE_DS:
			return cmnCoreDataSource();
		case ReportConstants.OBCM_REPORT_DS:
			return obcmReportDataSource();
		default:
			return null;
		}
	}
}
