package oracle.fsgbu.obscfcm.services.report.service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import oracle.fsgbu.accountMaintenance.api.model.AccentryWebModel;
import oracle.fsgbu.accountMaintenance.api.model.AccentrycodeWebModel;
import oracle.fsgbu.accountMaintenance.api.model.ExternalWebModel;
import oracle.fsgbu.accountMaintenance.api.model.IntaccmapWebModel;
import oracle.fsgbu.accountMaintenance.api.model.RoleWebModel;
import oracle.fsgbu.accountMaintenance.api.model.ScfTmAccEntryCodeMasterDTO;
import oracle.fsgbu.accountMaintenance.api.model.ScfTmAccEntryMasterDTO;
import oracle.fsgbu.accountMaintenance.api.model.ScfTmAccountRoleMasterDTO;
import oracle.fsgbu.accountMaintenance.api.model.ScfTmExtAccMapMasterDTO;
import oracle.fsgbu.accountMaintenance.api.model.ScfTmIntAccMapMasterDTO;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfTmAccountRoleMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfTmAccountRoleMasterWork;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmAccEntryCodeMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmAccEntryCodeMasterWork;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmAccEntryMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmAccEntryMasterWork;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmExtAccMapMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmExtAccMapMasterWork;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmIntAccMapMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTmIntAccMapMasterWork;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfTmAccountRoleMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfTmAccountRoleMasterWorkRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmAccEntryCodeMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmAccEntryCodeMasterWorkRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmAccEntryMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmAccEntryMasterWorkRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmExtAccMapMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmExtAccMapMasterWorkRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmIntAccMapMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfcmTmIntAccMapMasterWorkRepository;
import oracle.fsgbu.obscfcm.services.report.util.AccountingModelAssembler;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class AccountMaintenanceListenerServiceImpl implements AccountMaintenanceListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(AccountMaintenanceListenerServiceImpl.class);
	private static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd";
	
	@Autowired
	private ScfcmTmAccEntryMasterRepository scfcmTmAccEntryMasterRepository;
	@Autowired
	private ScfcmTmAccEntryMasterWorkRepository scfcmTmAccEntryMasterWorkRepository;
	
	@Autowired
	private ScfTmAccountRoleMasterRepository scfTmAccountRoleMasterRepository;
	
	@Autowired
	private ScfTmAccountRoleMasterWorkRepository scfTmAccountRoleMasterWorkRepository;
	
	@Autowired
	private ScfcmTmAccEntryCodeMasterRepository scfcmTmAccEntryCodeMasterRepository;
	@Autowired
	private ScfcmTmAccEntryCodeMasterWorkRepository scfcmTmAccEntryCodeMasterWorkRepository;
	
	@Autowired
	private ScfcmTmExtAccMapMasterRepository scfcmTmExtAccMapMasterRepository;
	
	@Autowired
	private ScfcmTmExtAccMapMasterWorkRepository scfcmTmExtAccMapMasterWorkRepository;
	
	@Autowired
	private ScfcmTmIntAccMapMasterRepository scfcmTmIntAccMapMasterRepository;
	@Autowired
	private ScfcmTmIntAccMapMasterWorkRepository scfcmTmIntAccMapMasterWorkRepository;
	
	@Autowired
	public AccountMaintenanceListenerServiceImpl() {
	}
	
	private Date dateConverter(String inputDate) throws ParseException {

		DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
		if(inputDate == null) {
			return null;
		}
		return dateFormat.parse(inputDate);
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountRoleTWData(RoleWebModel accRoleWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accRoleWebModel.getResourceId(), accRoleWebModel.getModNo()};
			ScfTmAccountRoleMasterWork scfTmAccountRoleMasterWork = null;
			ScfTmAccountRoleMasterDTO scfTmAccountRoleMasterDTO = accRoleWebModel.getScfTmAccountRoleMaster();
			List<ScfTmAccountRoleMasterWork> scfTmAccountRoleMasterWorkList = scfTmAccountRoleMasterWorkRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfTmAccountRoleMasterWork.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfTmAccountRoleMasterWorkList)) {
				scfTmAccountRoleMasterWork = scfTmAccountRoleMasterWorkList.get(0);
			} else {
				scfTmAccountRoleMasterWork = new ScfTmAccountRoleMasterWork();
			}
			BeanUtils.copyProperties(accRoleWebModel, scfTmAccountRoleMasterWork);
			scfTmAccountRoleMasterWork.setId(accRoleWebModel.getResourceId());
			scfTmAccountRoleMasterWork.setAccRoleCode(scfTmAccountRoleMasterDTO.getAccRoleCode());
			scfTmAccountRoleMasterWork.setAccRoleDescription(scfTmAccountRoleMasterDTO.getAccRoleDescription());
			scfTmAccountRoleMasterWork.setCheckerRemarks(scfTmAccountRoleMasterDTO.getCheckerRemarks());
			scfTmAccountRoleMasterWork.setMakerRemarks(scfTmAccountRoleMasterDTO.getMakerRemarks());
			scfTmAccountRoleMasterWork.setAutoAuth(accRoleWebModel.getAuthStatus());
			scfTmAccountRoleMasterWorkRepository.save(scfTmAccountRoleMasterWork);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountRoleTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountRoleTMData(RoleWebModel accRoleWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accRoleWebModel.getResourceId(), accRoleWebModel.getModNo()};
			ScfTmAccountRoleMaster scfTmAccountRoleMaster = null;
			ScfTmAccountRoleMasterDTO scfTmAccountRoleMasterDTO = accRoleWebModel.getScfTmAccountRoleMaster();
			List<ScfTmAccountRoleMaster> scfTmAccountRoleMasterWorkList = scfTmAccountRoleMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfTmAccountRoleMaster.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfTmAccountRoleMasterWorkList)) {
				scfTmAccountRoleMaster = scfTmAccountRoleMasterWorkList.get(0);
			} else {
				scfTmAccountRoleMaster = new ScfTmAccountRoleMaster();
			}
			BeanUtils.copyProperties(accRoleWebModel, scfTmAccountRoleMaster);
			scfTmAccountRoleMaster.setId(accRoleWebModel.getResourceId());
			scfTmAccountRoleMaster.setAccRoleCode(scfTmAccountRoleMasterDTO.getAccRoleCode());
			scfTmAccountRoleMaster.setAccRoleDescription(scfTmAccountRoleMasterDTO.getAccRoleDescription());
			scfTmAccountRoleMaster.setAutoAuth(accRoleWebModel.getAuthStatus());
			scfTmAccountRoleMasterRepository.save(scfTmAccountRoleMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountRoleTMData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountEntryTMData(AccentryWebModel accentryWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryWebModel.getResourceId(), accentryWebModel.getModNo()};
			ScfcmTmAccEntryMaster scfcmTmAccEntryMaster = null;
			ScfTmAccEntryMasterDTO scfTmAccEntryMasterDTO = accentryWebModel.getScfcmTmAccEntryMaster();
			List<ScfcmTmAccEntryMaster> scfTmAccountRoleMasterList = scfcmTmAccEntryMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmAccEntryMaster.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfTmAccountRoleMasterList)) {
				scfcmTmAccEntryMaster = scfTmAccountRoleMasterList.get(0);
			} else {
				scfcmTmAccEntryMaster = new ScfcmTmAccEntryMaster();
			}
			AccountingModelAssembler.convertAccEntryTMModelToEntity(scfcmTmAccEntryMaster, scfTmAccEntryMasterDTO);
			
			scfcmTmAccEntryMaster.setId(accentryWebModel.getResourceId());
			scfcmTmAccEntryMasterRepository.save(scfcmTmAccEntryMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryTMData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountEntryTWData(AccentryWebModel accentryWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryWebModel.getResourceId(), accentryWebModel.getModNo()};
			ScfcmTmAccEntryMasterWork scfcmTmAccEntryMasterWork = null;
			ScfTmAccEntryMasterDTO scfTmAccEntryMasterDTO = accentryWebModel.getScfcmTmAccEntryMaster();
			List<ScfcmTmAccEntryMasterWork> scfcmTmAccEntryMasterWorkList = scfcmTmAccEntryMasterWorkRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmAccEntryMasterWork.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmAccEntryMasterWorkList)) {
				scfcmTmAccEntryMasterWork = scfcmTmAccEntryMasterWorkList.get(0);
			} else {
				scfcmTmAccEntryMasterWork = new ScfcmTmAccEntryMasterWork();
			}
			AccountingModelAssembler.convertAccEntryTWModelToEntity(scfcmTmAccEntryMasterWork, scfTmAccEntryMasterDTO);
			scfcmTmAccEntryMasterWork.setId(accentryWebModel.getResourceId());
			scfcmTmAccEntryMasterWorkRepository.save(scfcmTmAccEntryMasterWork);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountEntryCodeTWData(AccentrycodeWebModel accentryCodeWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryCodeWebModel.getResourceId(), accentryCodeWebModel.getModNo()};
			ScfcmTmAccEntryCodeMasterWork scfcmTmAccEntryCodeMasterWork = null;
			ScfTmAccEntryCodeMasterDTO scfTmAccEntryMasterDTO = accentryCodeWebModel.getScfcmTmAccEntryCodeMaster();
			List<ScfcmTmAccEntryCodeMasterWork> scfcmTmAccEntryCodeMasterWorkList = scfcmTmAccEntryCodeMasterWorkRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmAccEntryCodeMasterWork.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmAccEntryCodeMasterWorkList)) {
				scfcmTmAccEntryCodeMasterWork = scfcmTmAccEntryCodeMasterWorkList.get(0);
			} else {
				scfcmTmAccEntryCodeMasterWork = new ScfcmTmAccEntryCodeMasterWork();
			}
			AccountingModelAssembler.convertAccEntryCodeTWModelToEntity(scfcmTmAccEntryCodeMasterWork, scfTmAccEntryMasterDTO);
			scfcmTmAccEntryCodeMasterWork.setId(accentryCodeWebModel.getResourceId());
			scfcmTmAccEntryCodeMasterWorkRepository.save(scfcmTmAccEntryCodeMasterWork);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryCodeTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAccountEntryCodeTMData(AccentrycodeWebModel accentryCodeWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryCodeWebModel.getResourceId(), accentryCodeWebModel.getModNo()};
			ScfcmTmAccEntryCodeMaster scfcmTmAccEntryCodeMaster = null;
			ScfTmAccEntryCodeMasterDTO scfTmAccEntryMasterDTO = accentryCodeWebModel.getScfcmTmAccEntryCodeMaster();
			List<ScfcmTmAccEntryCodeMaster> scfcmTmAccEntryCodeMasterWorkList = scfcmTmAccEntryCodeMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmAccEntryCodeMaster.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmAccEntryCodeMasterWorkList)) {
				scfcmTmAccEntryCodeMaster = scfcmTmAccEntryCodeMasterWorkList.get(0);
			} else {
				scfcmTmAccEntryCodeMaster = new ScfcmTmAccEntryCodeMaster();
			}
			AccountingModelAssembler.convertAccEntryCodeTMModelToEntity(scfcmTmAccEntryCodeMaster, scfTmAccEntryMasterDTO);
			scfcmTmAccEntryCodeMaster.setId(accentryCodeWebModel.getResourceId());
			scfcmTmAccEntryCodeMasterRepository.save(scfcmTmAccEntryCodeMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryCodeTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveInternalAccountMapTWData(IntaccmapWebModel accentryWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryWebModel.getResourceId(), accentryWebModel.getModNo()};
			ScfcmTmIntAccMapMasterWork scfcmTmIntAccMapMasterWork = null;
			ScfTmIntAccMapMasterDTO scfTmIntAccMapMasterDTO = accentryWebModel.getScfcmTmIntAccMapMaster();
			List<ScfcmTmIntAccMapMasterWork> scfcmTmIntAccMapMasterWorkList = scfcmTmIntAccMapMasterWorkRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmIntAccMapMasterWork.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmIntAccMapMasterWorkList)) {
				scfcmTmIntAccMapMasterWork = scfcmTmIntAccMapMasterWorkList.get(0);
			} else {
				scfcmTmIntAccMapMasterWork = new ScfcmTmIntAccMapMasterWork();
			}
			AccountingModelAssembler.convertInternalAccountMapTWModelToEntity(scfcmTmIntAccMapMasterWork, scfTmIntAccMapMasterDTO);
			scfcmTmIntAccMapMasterWork.setId(accentryWebModel.getResourceId());
			scfcmTmIntAccMapMasterWorkRepository.save(scfcmTmIntAccMapMasterWork);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryCodeTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveInternalAccountMapTMData(IntaccmapWebModel accentryWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {accentryWebModel.getResourceId(), accentryWebModel.getModNo()};
			ScfcmTmIntAccMapMaster scfcmTmIntAccMapMaster = null;
			ScfTmIntAccMapMasterDTO scfTmIntAccMapMasterDTO = accentryWebModel.getScfcmTmIntAccMapMaster();
			List<ScfcmTmIntAccMapMaster> scfcmTmIntAccMapMasterList = scfcmTmIntAccMapMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmIntAccMapMaster.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmIntAccMapMasterList)) {
				scfcmTmIntAccMapMaster = scfcmTmIntAccMapMasterList.get(0);
			} else {
				scfcmTmIntAccMapMaster = new ScfcmTmIntAccMapMaster();
			}
			AccountingModelAssembler.convertInternalAccountMapTMModelToEntity(scfcmTmIntAccMapMaster, scfTmIntAccMapMasterDTO);
			scfcmTmIntAccMapMaster.setId(accentryWebModel.getResourceId());
			scfcmTmIntAccMapMasterRepository.save(scfcmTmIntAccMapMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveAccountEntryCodeTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}





	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveExternalAccountMapTWData(ExternalWebModel externalWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {externalWebModel.getResourceId(), externalWebModel.getModNo()};
			ScfcmTmExtAccMapMasterWork scfcmTmExtAccMapMasterWork = null;
			ScfTmExtAccMapMasterDTO scfTmExtAccMapMasterDTO = externalWebModel.getScfcmTmExtAccMapMaster();
			List<ScfcmTmExtAccMapMasterWork> scfcmTmExtAccMapMasterWorkList = scfcmTmExtAccMapMasterWorkRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfTmAccountRoleMasterWork.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmExtAccMapMasterWorkList)) {
				scfcmTmExtAccMapMasterWork = scfcmTmExtAccMapMasterWorkList.get(0);
			} else {
				scfcmTmExtAccMapMasterWork = new ScfcmTmExtAccMapMasterWork();
			}
			BeanUtils.copyProperties(scfTmExtAccMapMasterDTO, scfcmTmExtAccMapMasterWork);
			scfcmTmExtAccMapMasterWork.setId(externalWebModel.getResourceId());
			scfcmTmExtAccMapMasterWork.setCheckerRemarks(scfTmExtAccMapMasterDTO.getCheckerRemarks());
			scfcmTmExtAccMapMasterWork.setMakerRemarks(scfTmExtAccMapMasterDTO.getMakerRemarks());
			scfcmTmExtAccMapMasterWork.setAutoAuth(externalWebModel.getAuthStatus());
			scfcmTmExtAccMapMasterWorkRepository.save(scfcmTmExtAccMapMasterWork);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveExternalAccountMapTWData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveExternalAccountMapTMData(ExternalWebModel externalWebModel, String userId, String branchCode) {
		try {
			String[] queryParams = {"id", "modNo"};
			Object[] paramValues = {externalWebModel.getResourceId(), externalWebModel.getModNo()};
			ScfcmTmExtAccMapMaster scfcmTmExtAccMapMaster = null;
			List<ScfcmTmExtAccMapMaster> scfcmTmExtAccMapMasterList = scfcmTmExtAccMapMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ScfcmTmExtAccMapMaster.findByIdModNo");
			if(CollectionUtils.isNotEmpty(scfcmTmExtAccMapMasterList)) {
				scfcmTmExtAccMapMaster = scfcmTmExtAccMapMasterList.get(0);
			} else {
				scfcmTmExtAccMapMaster = new ScfcmTmExtAccMapMaster();
			}
			BeanUtils.copyProperties(externalWebModel, scfcmTmExtAccMapMaster);
			scfcmTmExtAccMapMaster.setId(externalWebModel.getResourceId());
			scfcmTmExtAccMapMaster.setAutoAuth(externalWebModel.getAuthStatus());
			scfcmTmExtAccMapMasterRepository.save(scfcmTmExtAccMapMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveExternalAccountMapTMData Data:: {}", e.getMessage(), e);
			return false;
		}
	}
}
