package oracle.fsgbu.obscfcm.services.report.service;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.FinanceTxnDTO;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.InstrumentTxnDTO;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.ObChargesTxnDTO;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObFinanceTxnTwDetails;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObInvoiceTxnDetails;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObInvoiceTxnTwDetails;
import oracle.fsgbu.obscfcm.services.report.persistence.ObChargeTxnRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObChargeTxnTwRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObFinanceTwDetailsRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvoiceTxnRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvoiceTxnTwRepository;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.core.domain.PlatoEntityManagerType;
import oracle.fsgbu.plato.core.persistence.provider.PlatoEntityManagerConfig;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class ChargesListenerServiceImpl implements ChargesListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(ChargesListenerServiceImpl.class);
	private static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd";
	public static final String TXN_REF_NO = "txnRefNo";
	private static final String CHARGE_CODE = "chargeCode";
	private static final String EVENT_CODE = "eventCode";
	
	@Autowired
	private ObChargeTxnTwRepository obChargeTxnTwRepository;

	@Autowired
	private ObChargeTxnRepository obChargeTxnRepository;

	@Autowired
	private ObInvoiceTxnTwRepository obInvoiceTxnTwRepository;

	@Autowired
	private ObInvoiceTxnRepository obInvoiceTxnRepository;

	@Autowired
	private ObFinanceTwDetailsRepository obFinanceTwDetailsRepository;

	
	@Autowired
	@PlatoEntityManagerConfig(PlatoEntityManagerType.APPLICATION)
	private EntityManager entityManager;

	@Autowired
	public ChargesListenerServiceImpl() {
	}

	private Date dateConverter(String inputDate) throws ParseException {

		DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
		if (inputDate == null) {
			return null;
		}
		return dateFormat.parse(inputDate);
	}

	@Override
	public Boolean saveOBChargesData(String txnRefNo, String chargeCode, String eventCode,
			ObChargesTxnDTO obChargesTWTxnDTO, ObChargesTxnDTO obChargesTxnDTO, String userId, String branchCode) {

		try {
			String[] queryParamsTw = { TXN_REF_NO, CHARGE_CODE, EVENT_CODE};
			String[] paramValuesTw = { txnRefNo, chargeCode, eventCode };
			List<ObChargesTxnTw> onChargesTxnTwList = obChargeTxnTwRepository.executeNamedQueryByCondition(queryParamsTw,
					paramValuesTw, "ObChargesTxnTw.updatePostingStatus");
			
			if(!onChargesTxnTwList.isEmpty()) {
				
				onChargesTxnTwList.forEach(data -> {
					obChargeTxnTwRepository.delete(data);
				});
			}
			
			ObChargesTxnTw obChargesTxn = ReportingUtil.convertDTOToEntity(obChargesTWTxnDTO);
			obChargesTxn.setInstrumentType(obChargesTWTxnDTO.getInstrumentType());
			obChargeTxnTwRepository.save(obChargesTxn);
			
			ObChargesTxn onChargesTxnMain = null;
			if(obChargesTxn.getInstrumentType() != null && !obChargesTxn.getInstrumentType().equalsIgnoreCase("FINANCE")) {
				String[] queryParams = { TXN_REF_NO, CHARGE_CODE };
				String[] paramValues = { obChargesTxn.getTxnRefNo(), obChargesTxn.getChargeCode() };
				List<ObChargesTxn> onChargesTxnMainList = obChargeTxnRepository.executeNamedQueryByCondition(queryParams,
						paramValues, "ObChargesTxn.getByTxnRefNoChargeCode");
				
				if (onChargesTxnMainList != null && !onChargesTxnMainList.isEmpty()) {
					onChargesTxnMain = onChargesTxnMainList.get(0);
					if(onChargesTxnMain.getChargeAmount()!= null) {
						onChargesTxnMain.setChargeAmount(onChargesTxnMain.getChargeAmount()
							.add(obChargesTxn.getChargeAmount() != null ? obChargesTxn.getChargeAmount()
									: new BigDecimal(0)));
					} else {
						onChargesTxnMain.setChargeAmount(obChargesTxn.getChargeAmount() != null ? obChargesTxn.getChargeAmount()
										: new BigDecimal(0));	
					}
					ReportingUtil.setDates(onChargesTxnMain, obChargesTxn);
					obChargeTxnRepository.save(onChargesTxnMain);
				} else {
					onChargesTxnMain = new ObChargesTxn();
					BeanUtils.copyProperties(obChargesTxn, onChargesTxnMain);
					ReportingUtil.setDates(onChargesTxnMain, obChargesTxn);
					obChargeTxnRepository.save(onChargesTxnMain);
				}
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting saveOBChargesData Data:: {}", e.getMessage(), e);
			return false;
		}
		
	}

	@Override
	public Boolean saveOBFinanceDetailsData(String txnRefNo, String eventCode, FinanceTxnDTO financeTxnDTO,
			String userId, String branchCode) {

		try {
			String[] queryParams = { TXN_REF_NO, EVENT_CODE };
			String[] paramValues = { txnRefNo, eventCode };
			List<ObFinanceTxnTwDetails> obFinTxnTwList = obFinanceTwDetailsRepository.executeNamedQueryByCondition(queryParams,
					paramValues, "ObFinanceTxnTwDetails.getByTxnRefNoEventCode");
			
			if(!obFinTxnTwList.isEmpty()) {
				
				obFinTxnTwList.forEach(data -> {
					obFinanceTwDetailsRepository.delete(data);
				});
			}
			
			String[] ignoreProperties = { "financeStartDate", "financeDueDate", "processingDate" };
			ObFinanceTxnTwDetails financeTxn = new ObFinanceTxnTwDetails();
			BeanUtils.copyProperties(financeTxnDTO, financeTxn, ignoreProperties);
			financeTxn.setProcessingDate(ReportingUtil.setEntityDateFormat(financeTxnDTO.getProcessingDate()));
			financeTxn.setFinanceStartDate(ReportingUtil.setEntityDateFormat(financeTxnDTO.getFinanceStartDate()));
			financeTxn.setFinanceDueDate(ReportingUtil.setEntityDateFormat(financeTxnDTO.getFinanceDueDate()));
			obFinanceTwDetailsRepository.save(financeTxn);
			return true;
		}catch(Exception e) {
			LOGGER.error("Error while commiting saveOBFinanceDetailsData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	@Override
	public Boolean saveOBInvoiceDetailsData(String txnRefNo, String invoiceRefNo, String invoiceStatus,
			String invoiceCcy, InstrumentTxnDTO instrumentTxnDTO, String userId, String branchCode) {
		try {
			ObInvoiceTxnTwDetails invoiceTxn = new ObInvoiceTxnTwDetails();
			String[] ignoreProperties = { "invoiceDate", "invoiceDueDate" };
			BeanUtils.copyProperties(instrumentTxnDTO, invoiceTxn, ignoreProperties);
			invoiceTxn.setProcessingDate(ReportingUtil.setEntityDateFormat(instrumentTxnDTO.getProcessingDate()));
			invoiceTxn.setInvoiceDate(ReportingUtil.setEntityDateFormat(instrumentTxnDTO.getInvoiceDate()));
			invoiceTxn.setInvoiceDueDate(ReportingUtil.setEntityDateFormat(instrumentTxnDTO.getInvoiceDueDate()));
			obInvoiceTxnTwRepository.save(invoiceTxn);

			String[] queryParams = { TXN_REF_NO, "invoiceRefNo", "invoiceStatus", "invoiceCcy" };
			String[] paramValues = { instrumentTxnDTO.getTxnRefNo(), instrumentTxnDTO.getInvoiceRefNo(),
					instrumentTxnDTO.getInvoiceStatus(), instrumentTxnDTO.getInvoiceCcy() };
			List<ObInvoiceTxnDetails> invTxnMainCollection = obInvoiceTxnRepository
					.executeNamedQueryByCondition(queryParams, paramValues,
							"ObInvoiceTxnDetails.getByTxnRefNoInvRefNo");
			if (invTxnMainCollection == null || invTxnMainCollection.isEmpty()) {

				ObInvoiceTxnDetails invMain = new ObInvoiceTxnDetails();
				BeanUtils.copyProperties(invoiceTxn, invMain);
				invMain.setObInvTxnId(null);
				obInvoiceTxnRepository.save(invMain);
			}
			return true;
		}catch(Exception e) {
			LOGGER.error("Error while commiting saveOBInvoiceDetailsData Data:: {}", e.getMessage(), e);
			return false;
		}
	}

//	@Override
//	@Transactional("PLATO_TX_MANAGER")
//	public Boolean saveExtCustomerDelinqStatusData(String extCustomerDelinqStatusId,
//			ScftmExtCustomerDelinqStatusHisDTO scftmExtCustomerDelinqStatusHisDTO, String userId, String branchCode) {
//		try {
//			ScftmExtCustomerDelinqStatusHis scftmExtCustomerDelinqStatusHis= new ScftmExtCustomerDelinqStatusHis();
////			String[] queryParams = { "customerId" };
////			String[] queryValues = { scftmExtCustomerDelinqStatusHisDTO.getCustomerId() };
////			List<ScftmExtCustomerDelinqStatusHis> scftmExtCustomerDelinqStatusHisList =  scftmExtCustomerDelinqStatusHisRepository.executeNamedQueryByCondition(queryParams, queryValues, "ScftmExtCustomerDelinqStatusHis.findByPk");
////			if (CollectionUtils.isNotEmpty(scftmExtCustomerDelinqStatusHisList)) {
////				scftmExtCustomerDelinqStatusHis = scftmExtCustomerDelinqStatusHisList.get(0);
////			} else {
////				scftmExtCustomerDelinqStatusHis = new ScftmExtCustomerDelinqStatusHis();
////			}
//			BeanUtils.copyProperties(scftmExtCustomerDelinqStatusHisDTO, scftmExtCustomerDelinqStatusHis);
//			scftmExtCustomerDelinqStatusHis.setId(Long.toString(SnowflakeGenerator.getInstance().nextId()));
//			scftmExtCustomerDelinqStatusHis.setInputDate(scftmExtCustomerDelinqStatusHisDTO.getInputDate());
//	        scftmExtCustomerDelinqStatusHis.setStatusDate(scftmExtCustomerDelinqStatusHis.getStatusDate());
//			
//			scftmExtCustomerDelinqStatusRepository.deleteByKey(extCustomerDelinqStatusId);
//			entityManager.flush();
//			scftmExtCustomerDelinqStatusHisRepository.save(scftmExtCustomerDelinqStatusHis);
//			return true;
//		}
//		 catch(Exception e) {
//			LOGGER.error("Error while commiting saveAccountRoleTWData Data:: {}", e.getMessage(), e);
//			return false;
//			}
//	}
//
//	@Override
//	public Boolean saveCustomerDelinqStatusData(ScftmCustomerDelinqStatusDTO scftmCustomerDelinqStatusDTO,
//			ScftmCustomerDelinqStatusHisDTO scftmCustomerDelinqStatusHisDTO, String userId, String branchCode) {
//		try {
//			ScftmCustomerDelinqStatusHis scftmCustomerDelinqStatusHis= null;
//			String[] queryParamsStatusHis = { "customerId" };
//			String[] queryValuesStatusHis = { scftmCustomerDelinqStatusHisDTO.getCustomerId() };
//			List<ScftmCustomerDelinqStatusHis> scftmCustomerDelinqStatusHisList =  scftmCustomerDelinqStatusHisRepository.executeNamedQueryByCondition(queryParamsStatusHis, queryValuesStatusHis, "ScftmExtCustomerDelinqStatusHis.findByPk");
//			if (CollectionUtils.isNotEmpty(scftmCustomerDelinqStatusHisList)) {
//				scftmCustomerDelinqStatusHis = scftmCustomerDelinqStatusHisList.get(0);
//			} else {
//				scftmCustomerDelinqStatusHis = new ScftmCustomerDelinqStatusHis();
//			}
//			BeanUtils.copyProperties(scftmCustomerDelinqStatusHisDTO, scftmCustomerDelinqStatusHis);
//			scftmCustomerDelinqStatusHis.setId(Long.toString(SnowflakeGenerator.getInstance().nextId()));
//			scftmCustomerDelinqStatusHis.setInputDate(scftmCustomerDelinqStatusHisDTO.getInputDate());
//			scftmCustomerDelinqStatusHis.setStatusDate(scftmCustomerDelinqStatusHis.getStatusDate());
//			
//			scftmCustomerDelinqStatusRepository.deleteByKey(scftmCustomerDelinqStatusDTO.getCustomerId());
//			entityManager.flush();
//			scftmCustomerDelinqStatusHisRepository.save(scftmCustomerDelinqStatusHis);
//			
//			ScftmCustomerDelinqStatus scftmCustomerDelinqStatus = new ScftmCustomerDelinqStatus();
////			String[] queryParamsStatus = { "customerId" };
////			String[] queryValuesStatus = { scftmCustomerDelinqStatusHisDTO.getCustomerId() };
////			List<ScftmCustomerDelinqStatus> scftmCustomerDelinqStatusList =  scftmCustomerDelinqStatusRepository.executeNamedQueryByCondition(queryParamsStatus, queryValuesStatus, "ScftmExtCustomerDelinqStatus.findByPk");
////			if (CollectionUtils.isNotEmpty(scftmCustomerDelinqStatusList)) {
////				scftmCustomerDelinqStatus = scftmCustomerDelinqStatusList.get(0);
////			} else {
////				scftmCustomerDelinqStatus = new ScftmCustomerDelinqStatus();
////			}
//			BeanUtils.copyProperties(scftmCustomerDelinqStatusDTO, scftmCustomerDelinqStatus);
//			scftmCustomerDelinqStatus.setStatusDate(scftmCustomerDelinqStatus.getStatusDate());
//			scftmCustomerDelinqStatusRepository.save(scftmCustomerDelinqStatus);
//			
//			return true;
//		}
//		 catch(Exception e) {
//			LOGGER.error("Error while commiting saveAccountRoleTWData Data:: {}", e.getMessage(), e);
//			return false;
//			}
//	}

}
