package oracle.fsgbu.obscfcm.services.report.service;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import oracle.fsgbu.eventService.api.model.*;
import oracle.fsgbu.obscfcm.services.report.assembler.ScfPoAssembler;
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TmCommodityList;
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TmPurchaseOrder;
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwCommodityList;
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwCommodityListPk;
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwPurchaseOrder;
import oracle.fsgbu.obscfcm.services.report.persistence.po.TmPurchaseOrderRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.po.TwPurchaseOrderRepository;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;

import oracle.fsgbu.accountMaintenance.api.model.AccountingTransactionModel;
import oracle.fsgbu.ob.adapter.dto.OBIntegrationServiceRequest;
import oracle.fsgbu.ob.adapter.dto.OBIntegrationServiceResponse;
import oracle.fsgbu.ob.adapter.provider.OBIntegrationAdapter;
import oracle.fsgbu.obscfcm.services.report.config.ReportConfigDbUtil;
import oracle.fsgbu.obscfcm.alertsgenerator.AlertMessageProcessor;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.Address;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.AdviceRequest;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.AdviceResponse;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningDetailsDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningWrapperDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DeliveryDetailsDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.EventDetailsDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.MediaPreferences;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.UserDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AdvicesClient;
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertDecisionClient;
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertsPlaceholdersClient;
import oracle.fsgbu.obscfcm.services.report.assembler.ScfInvoiceAssembler;
import oracle.fsgbu.obscfcm.services.report.assembler.ScfInvoiceTwAssembler;
import oracle.fsgbu.obscfcm.services.report.assembler.ScfTmCreditNoteAssembler;
import oracle.fsgbu.obscfcm.services.report.client.AlertList;
import oracle.fsgbu.obscfcm.services.report.client.InstrumentReceivablesServiceClient;
import oracle.fsgbu.obscfcm.services.report.client.ReceiverDetailsClient;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.domain.entity.AccountPost;
import oracle.fsgbu.obscfcm.services.report.domain.entity.AccountPostTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.AutoDebitTxnDetails;
import oracle.fsgbu.obscfcm.services.report.domain.entity.AutoDebitTxnMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfTwInvoice;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmFile;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmInvoice;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmPayment;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTwPayment;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTwPaymentPK;
import oracle.fsgbu.obscfcm.services.report.domain.entity.TmCreditNote;
import oracle.fsgbu.obscfcm.services.report.persistence.AccountsTmRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.AccountsTwRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.AutoDebitTxnMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.FileRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.InvoiceRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.PaymentRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.PaymentTwRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfTwInvoiceRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.TmCreditNoteRepository;
import oracle.fsgbu.obscfcm.services.report.util.MapperUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.obscfcm.services.alert.dto.UpdatedPlaceholdersDto;

@Service
public class EventListenerServiceImpl implements EventListenerService {

    private static final PlatoLogger LOGGER = new PlatoLogger(EventListenerServiceImpl.class);
    private static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd";
    private static final String MAKER_DT_STAMP = "makerDateStamp";
	private static final String CHECKER_DT_STAMP = "checkerDtStamp";
	private static final String PROCESSING_DATE = "processingDate";
	private static final String VALUE_DATE = "valueDate";
	private static final String SEQ_NO = "seqNo";
	private static final String REF_NO = "refNo";
	private static final String ID = "id";
    private InvoiceRepository invoiceRepository;
    private PaymentRepository paymentRepository;
    private PaymentTwRepository paymentTwRepository;
    private FileRepository fileRepository;
    private TmCreditNoteRepository tmCreditNoteRepository;

    @Autowired
    AlertMessageProcessor alertMessageProcessor;

    @Autowired
    AlertList alertList;

    @Autowired
    ReceiverDetailsClient receiverDetails;

    @Autowired
    AlertsPlaceholdersClient alertsPlaceholdersClient;

    @Autowired
    AlertDecisionClient alertDecisionClient;

    @Autowired
    ScfTwInvoiceRepository scfTwInvoiceRepository;

    @Autowired
    AdvicesClient advicesClient;
    private List<DecisioningDetailsDTO> alertDecisionList;

    @Autowired
	private AutoDebitTxnMasterRepository autoDebitTxnMasterRepository;
    
    @Autowired
	private AccountsTwRepository accountsTwRepository;
    
    @Autowired
	private AccountsTmRepository accountsTmRepository;
    
    @Autowired
	private OBIntegrationAdapter obIntegrationAdapter;
    
    @Autowired
	private InstrumentReceivablesServiceClient instrumentReceivablesServiceClient;
    
    private NamedParameterJdbcTemplate jdbcTemplate;
    
    @Autowired
	private ApplicationContext context;


	@Autowired
	TwPurchaseOrderRepository twPurchaseOrderRepository;

	@Autowired
	TmPurchaseOrderRepository tmPurchaseOrderRepository;
    
    public EventListenerServiceImpl(InvoiceRepository invoiceRepository, PaymentRepository paymentRepository, FileRepository fileRepository, TmCreditNoteRepository tmCreditNoteRepository, PaymentTwRepository paymentTwRepository) {
        this.invoiceRepository = invoiceRepository;
        this.paymentRepository = paymentRepository;
        this.fileRepository = fileRepository;
        this.tmCreditNoteRepository = tmCreditNoteRepository;
        this.paymentTwRepository = paymentTwRepository;
    }
    
    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean fetchInvoiceData(String transactionRefNo, String userId, String branchCode) {
    	LOGGER.info("Fetching Invoice data from Instruments");
		try {
			InvoiceCollectionDto invoiceCollectionDto = instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(transactionRefNo, userId, branchCode);
			if (invoiceCollectionDto != null) {
				List<InvoiceModel> invoiceTwList = invoiceCollectionDto.getData();
				if (invoiceTwList != null && !invoiceTwList.isEmpty()) {
					invoiceTwList.forEach(invoiceModel -> {
						saveInvoiceData(invoiceModel, userId, branchCode);
					});
				}			
			}
		} catch (Exception e) {
			LOGGER.error("Error while commiting Invoice Data:: {}", e.getMessage(), e);
            return false;
		}
		return true;
    }

    @Transactional("PLATO_TX_MANAGER")
    public void saveInvoiceData(InvoiceModel invoiceModel, String userId, String branchCode) {

        LOGGER.info("Trying to commit Updated Invoice data..");
        InvoiceDto invoiceDto = new InvoiceDto();

        try {
            String[] ignoreFields = {"invoiceDueDate", "invAcceptanceDate", "filter1", "invoiceDate", "shipmentDate", "purchaseOrderDate", "oldExpiryDate", "processingDate"};
            String[] queryParams = {"invoiceId"};
            Object[] paramValues = {invoiceModel.getInvoiceId()};
            ScfcmInvoice invoice = null;
            List<ScfcmInvoice> invoiceList = invoiceRepository.executeNamedQueryByCondition(queryParams, paramValues,
                    "ScfcmInvoic.findByInvoiceId");
            if (invoiceList != null && !invoiceList.isEmpty()) {
                invoice = invoiceList.get(0);
            } else {
                invoice = new ScfcmInvoice();
            }
            BeanUtils.copyProperties(invoiceModel, invoice, ignoreFields);

            invoice.setInvoiceDueDate(dateConverter(invoiceModel.getInvoiceDueDate()));
            invoice.setInvAcceptanceDate(dateConverter(invoiceModel.getInvAcceptanceDate()));
            invoice.setFilter1(dateConverter(invoiceModel.getFiller1()));
            invoice.setInvoiceDate(dateConverter(invoiceModel.getInvoiceDate()));
            invoice.setShipmentDate(dateConverter(invoiceModel.getShipmentDate()));
            invoice.setPurchaseOrderDate(dateConverter(invoiceModel.getPurchaseOrderDate()));
            invoice.setOldExpiryDate(dateConverter(invoiceModel.getOldExpiryDate()));
            invoice.setProcessingDate(dateConverter(invoiceModel.getProcessingDate()));
            invoice.setPaymentDueDate(dateConverter(invoiceModel.getPaymentDueDate()));
            invoice.setFundingReqDate(dateConverter(invoiceModel.getFundingReqDate()));
            invoice.setInvoiceValueDate(dateConverter(invoiceModel.getInvoiceValueDate()));
            ScfInvoiceAssembler.convertInvoiceDTOToInvoiceEntity(invoice, invoiceModel);

            invoiceRepository.merge(invoice);

            BeanUtils.copyProperties(invoiceModel, invoiceDto);
            sendInvoiceOnlineAlert(invoiceDto,userId,branchCode);

        } catch (Exception e) {
            LOGGER.error("Error while commiting Invoice Data:: {}", e.getMessage(), e);
        }
    }
    
	private void sendInvoiceOnlineAlert(InvoiceDto invoiceDto, String userId, String branchCode) {
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(invoiceDto,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning");
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		alertDtoCollection.setInvoiceDto(invoiceDto);
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.INVOICE_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.INVOICE_PLACEHOLDER_TRANSACTION_TYPE);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO, gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensability");
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensability");
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
							eventDetailsObject, ReportConstants.CATEGORY_INVOICE, userId, branchCode);
				}
			}
			}
		}
	}

	  @Override
	    @Transactional("PLATO_TX_MANAGER")
	    public Boolean fetchPaymentData(String paymentRefNo, String userId, String branchCode) {
	    	LOGGER.info("Fetching Payment data from Instruments :: {} ",paymentRefNo);
			try {
				PaymentCollectionDto paymentCollectionDto = instrumentReceivablesServiceClient.getPaymentByPayRefNo(paymentRefNo, userId, branchCode);
				if (paymentCollectionDto != null) {
					List<PaymentDto> payTmList = paymentCollectionDto.getData();
					if (payTmList != null && !payTmList.isEmpty()) {
						payTmList.forEach(payDto -> {
							savePaymentData(payDto, userId, branchCode);
							savePaymentTWData(payDto, userId, branchCode);
						});
					}			
				}
			} catch (Exception e) {
				LOGGER.error("Error while commiting Invoice Data:: {}", e.getMessage(), e);
	            return false;
			}
			return true;
	    }

	
    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean savePaymentData(PaymentDto paymentDto, String userId, String branchCode) {

        LOGGER.info("Trying to commit Updated Payment data..");

        try {
            String[] queryParams = {"paymentRefNo"};
            Object[] paramValues = {paymentDto.getPaymentRefNo()};
            ScfcmPayment payment = null;
            List<ScfcmPayment> paymentList = paymentRepository.executeNamedQueryByCondition(queryParams, paramValues,
                    "ScfcmPayment.findByPaymentRefNo");
            if (paymentList != null && !paymentList.isEmpty()) {
                payment = paymentList.get(0);
            } else {
                payment = new ScfcmPayment();
            }
          payment = MapperUtil.INSTANCE.copyPropertiesTMtoEntity(paymentDto);
            paymentRepository.merge(payment);
            sendPaymentOnlineAlert(paymentDto, userId, branchCode);
            return true;
        } catch (Exception e) {
            LOGGER.error("Error while commiting Payment Data:: {}", e.getMessage(), e);
            return false;
        }
    }
    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean savePaymentTWData(PaymentDto paymentDto, String userId, String branchCode) {

        LOGGER.info("Trying to commit Updated Payment Tw data..");
      try {
            String[] queryParams = {"paymentRefNo", "seqNo"};
            Object[] paramValues = {paymentDto.getPaymentRefNo(), paymentDto.getSeqNo()};
            LOGGER.info("Updated Payment Tw paymentDto.getSeqNo() :: {}",paymentDto.getSeqNo());
            List<ScfcmTwPayment> paymentList = paymentTwRepository.executeNamedQueryByCondition(queryParams, paramValues,
                    "ScfcmTwPayment.findByPaymentRefNo");
            LOGGER.info("paymentList :: {}",paymentList);
            ScfcmTwPayment payment = null;
            String paymentRefNo ="";
            Long seqNo = null;
            if (paymentList != null && !paymentList.isEmpty()) {
                payment = paymentList.get(0);
                paymentRefNo = payment.getId().getPaymentRefNo();
                seqNo = payment.getId().getSeqNo();
                payment = MapperUtil.INSTANCE.copyPropertiestoEntity(paymentDto);
            } else {
                payment = new ScfcmTwPayment();
                payment = MapperUtil.INSTANCE.copyPropertiesTWtoEntity(paymentDto);
                paymentRefNo = paymentDto.getPaymentRefNo();
                seqNo = paymentDto.getSeqNo();
            }
            
            ScfcmTwPaymentPK scfcmTwPaymentPK = new ScfcmTwPaymentPK();
            scfcmTwPaymentPK.setPaymentRefNo(paymentRefNo);
            scfcmTwPaymentPK.setSeqNo(seqNo);
            payment.setId(scfcmTwPaymentPK);
            paymentTwRepository.merge(payment);
    
            return true;
        } catch (Exception e) {
            LOGGER.error("Error while commiting Payment Data:: {}", e.getMessage(), e);
            return false;
        }
    }
    
	private void sendPaymentOnlineAlert(PaymentDto paymentDto, String userId, String branchCode) {
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(paymentDto,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning");
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		alertDtoCollection.setPaymentDto(paymentDto);
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.PAYMENTS_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.PAYMENTS_PLACEHOLDER_TRANSACTION_TYPE);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO,gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensability");
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensability");
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
							eventDetailsObject, ReportConstants.CATEGORY_PAYMENT, userId, branchCode);
				}
					}
			}
		}
	}

    private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final PaymentDto paymentDto, EventDetailsDTO eventDetailsObject,String userId, String branchCode) {
        eventDetailsObject.setRelationshipCode(paymentDto.getRelationshipCode());
        eventDetailsObject.setProgramId(paymentDto.getProgramCode());
        eventDetailsObject.setEvent(ReportConstants.PAYMENT_CREATE_EVENT);
        eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
        eventDetailsObject.setReferenceNumber(paymentDto.getPaymentRefNo());
        return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
    }

    private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final InvoiceDto invoiceDto, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
        eventDetailsObject.setRelationshipCode(invoiceDto.getRelationCode());
        eventDetailsObject.setProgramId(invoiceDto.getProgramCode());
        eventDetailsObject.setEvent(invoiceDto.getAction());
        eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
        eventDetailsObject.setReferenceNumber(invoiceDto.getInvoiceId());
        return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
    }


	private void generateAlert(Object obj, DecisioningDetailsDTO alertDecision, EventDetailsDTO eventDetailsObject,
			String alertCategory, String userId, String branchCode) {
		LOGGER.info("start generateAlert");
		List<Object> placeholder = alertsPlaceholdersClient.getparams(alertCategory, userId, branchCode);
		if (CollectionUtils.isNotEmpty(placeholder)) {
			List<ParameterDTO> newparameterList = alertList.setParams(placeholder, obj);
			List<ParameterDTO> filteredPlaceHolder = new ArrayList<ParameterDTO>();
			for(Object placeholderdata:placeholder) {
				ParameterDTO  placeholderData = (ParameterDTO) placeholderdata;
				if("Y".equalsIgnoreCase(placeholderData.getSqlIdentifier()) && StringUtils.isNotEmpty(placeholderData.getBaseQuery())) {
					filteredPlaceHolder.add(placeholderData);
				}
			}
			if(CollectionUtils.isNotEmpty(filteredPlaceHolder)) {
				for(ParameterDTO placeholderData:filteredPlaceHolder) {	
					LOGGER.info("generateAlert for");
						String placeholderValue = fetchPlaceHolderValue(placeholderData,newparameterList);
						Boolean containsKey = false;
						for (ParameterDTO paramDto : newparameterList) {
							if (placeholderData.getKey().equalsIgnoreCase(paramDto.getKey())) {
								containsKey = true;
								paramDto.setValue(placeholderValue);
							}
						}
						LOGGER.info("containsKey :: {}",containsKey);
						if(!containsKey) {
							ParameterDTO paramDto = new ParameterDTO();
							paramDto.setKey(placeholderData.getKey());
							paramDto.setValue(placeholderValue);
							newparameterList.add(paramDto);						
						}

				}
				
			}
			if (CollectionUtils.isNotEmpty(newparameterList)) {
				eventDetailsObject.setParameters(newparameterList);
				if (StringUtils.isBlank(alertDecision.getDeliveryDetails().get(0).getDocumentID())) {
					processAlertMessage(eventDetailsObject, alertDecision, userId, branchCode);
				} else {
					alertWithDocument(obj, alertDecision, newparameterList);
				}
			}
		}
	}
	
	private String fetchPlaceHolderValue(ParameterDTO placeholderData, List<ParameterDTO> newparameterList) {
		LOGGER.info("inside fetchPlaceHolderValue");
		switch (placeholderData.getApplicationCode()) {
		case ReportConstants.APPLICATION_CODE_OBSCF:
			return getPlaceHolderData(placeholderData,newparameterList,ReportConstants.OBSCF_REPORT_DS);
		case ReportConstants.APPLICATION_CODE_OBSCFCM:
			return getPlaceHolderData(placeholderData,newparameterList,ReportConstants.OBSCFCM_REPORT_DS);
		case ReportConstants.APPLICATION_CODE_CMNCORE:
			return getPlaceHolderData(placeholderData,newparameterList,ReportConstants.CMN_CORE_DS);
		case ReportConstants.APPLICATION_CODE_OBCM:
			return getPlaceHolderData(placeholderData,newparameterList,ReportConstants.OBCM_REPORT_DS);
		default:
			return null;
		}
	
	}

	private String getPlaceHolderData(ParameterDTO placeholderData, List<ParameterDTO> newparameterList,
			String dataSource) {
		String value = null;

		try {
			jdbcTemplate = new NamedParameterJdbcTemplate(
					ReportConfigDbUtil.getDatabaseConfiguration(context).getDataSource(dataSource));
			String query = placeholderData.getBaseQuery().toString();
			LOGGER.info("query :: {}", query);

			boolean flag = true;
			int endIndex = 0;
			String paramkey = "";
			List<String> paramList = new ArrayList<>();
			while (flag) {
				LOGGER.info("inside loop :: {}", endIndex);
				int startIndex = 0;
				if (endIndex == 0) {
					startIndex = query.indexOf(":");
				} else {
					startIndex = query.indexOf(":", endIndex);
				}
				if (startIndex > 0) {
					endIndex = query.indexOf(" ", startIndex);
				} else {
					endIndex = -1;
				}

				if (endIndex > 0) {
					paramkey = query.substring(startIndex + 1, endIndex);
					paramList.add(paramkey);

					flag = true;
				} else if (startIndex > 0) {
					paramkey = query.substring(startIndex, query.length());
					paramList.add(paramkey);
					flag = false;
				} else {
					flag = false;
				}
			}
			Map<String, String> paramMap = new HashMap<>();
			for (String param : paramList) {
				for (ParameterDTO paramDto : newparameterList) {
					if (param.equalsIgnoreCase(paramDto.getKey())) {
						paramMap.put(param, paramDto.getValue());
					}
				}
			}
			List<String> result = jdbcTemplate.query(query, paramMap, new RowMapper<String>() {

				@Override
				public String mapRow(ResultSet rs, int rowNum) throws SQLException {
					return (rs != null ? rs.getString(1) : "");
				}

			});
			LOGGER.info("fetchObscfReportData results :: {}", result);
			if (result.size() > 1) {
				value = null;
			}

		} catch (Exception e) {
			LOGGER.error("Exception placeHolderData :: {}", e);
		}
		return value;
	}



	private void processAlertMessage(EventDetailsDTO eventDetailsObject, DecisioningDetailsDTO alertDecision,
			String... extraHeaders) {
		String userId = null;
		String branchCode = null;
		if (extraHeaders != null && extraHeaders.length > 0) {
			userId = extraHeaders[0];
			branchCode = extraHeaders[1];
		}
		String relationshipCode = eventDetailsObject.getRelationshipCode();
		LOGGER.info("relationshipCode in processAlertMessage:  {}", relationshipCode);

		List<Map<String, String>> contactDetails = alertList.getAlertContactDetails(userId, branchCode,
				relationshipCode, ReportConstants.APPLICATION_CODE);
		LOGGER.info("Retrieved alert contact details from alert services...");

		if (contactDetails != null && contactDetails.size() > 0) {

			if (ReportConstants.BUYER.equals(alertDecision.getRecipient())) {
				filterRecipientsAndSendAlerts(eventDetailsObject, alertDecision, userId, branchCode, contactDetails);
			} else if (ReportConstants.SUPPLIER.equals(alertDecision.getRecipient())) {
				filterRecipientsAndSendAlerts(eventDetailsObject, alertDecision, userId, branchCode, contactDetails);
			}
		} else {
			LOGGER.info("Alert contact details not found !!! Sending alert to AdditionalEmailId");
			populateAndSendAlertMessage(eventDetailsObject, alertDecision, userId, branchCode,
					alertDecision.getAdditionalDetails(), alertDecision.getAdditionalDetails());
		}
	}

    private void filterRecipientsAndSendAlerts(EventDetailsDTO eventDetailsObject,
                                               DecisioningDetailsDTO alertDecision,
                                               String userId,
                                               String branchCode,
			List<Map<String, String>> contactDetails) {
		Map<String, String> emails2Mobile = new HashMap();

		contactDetails.forEach(map -> {
			emails2Mobile.put((String) map.get("email"), (String) map.get("mobile"));
		});

		if (emails2Mobile.size() > 0) {
			emails2Mobile.forEach((email, mobile) -> populateAndSendAlertMessage(eventDetailsObject, alertDecision,
					userId, branchCode, email, mobile));
		}
	}

	private void populateAndSendAlertMessage(EventDetailsDTO eventDetailsObject, DecisioningDetailsDTO alertDecision,
			String userId, String branchCode, String emailId, String phone) {
		UserDTO userDtoObject = new UserDTO();
		List<UserDTO> userList = new ArrayList<>();
		userDtoObject.setUserId(eventDetailsObject.getReferenceNumber());
		userDtoObject.setEmailId(emailId);
		userDtoObject.setPhone(phone);
		userList.add(userDtoObject);
		eventDetailsObject.setUsers(userList);
		try {
			alertMessageProcessor.processAlertMessage(eventDetailsObject, alertDecision, userId, branchCode);
		} catch (Exception e) {
			LOGGER.error("Error occured while sending alert:  {}", e.getMessage());
		}
	}

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean saveUploadedFileData(FileUploadDto fileUploadDto,String userId,String branchCode) {

        LOGGER.info("Trying to commit Uploaded File data..");

        try {
            String[] ignoreFields = {"uploadDate"};
            ScfcmFile scfFile = new ScfcmFile();

            BeanUtils.copyProperties(fileUploadDto, scfFile, ignoreFields);
            scfFile.setUploadDateAndTime(dateConverter(fileUploadDto.getUploadDateAndTime()));
            fileRepository.save(scfFile);
            sendFileUploadOnlineAlerts(fileUploadDto,userId,branchCode);
            return true;
        } catch (Exception e) {
            LOGGER.error("Error while commiting File Data:: {}", e.getMessage(), e);
            return false;
        }
    }
    
	private void sendFileUploadOnlineAlerts(FileUploadDto fileUploadDto, String userId, String branchCode) {
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(fileUploadDto,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning");
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		alertDtoCollection.setFileUploadDto(fileUploadDto);
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.FILE_UPLOAD_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.FILE_UPLOAD_PLACEHOLDER_TRANSACTION_TYPE);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO,gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensability");
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensability");
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
						generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
								eventDetailsObject, ReportConstants.CATEGORY_FILEUPLOAD, userId, branchCode);
					}
				}
			}
		}
	}

    private Date dateConverter(String inputDate) throws ParseException {

        DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
        if (inputDate == null) {
            return null;
        }
        return dateFormat.parse(inputDate);
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean saveInvoiceTwData(InvoiceModel invoiceModel, String userId, String branchCode) {
        try {
			LOGGER.error("Inside saveInvoiceTwData");
            String[] queryParams = {"id", "seqNo"};
            Object[] paramValues = {invoiceModel.getId(), invoiceModel.getSeqNo()};
            List<ScfTwInvoice> twInvoiceList = scfTwInvoiceRepository.executeNamedQueryByCondition(queryParams, paramValues,
                    "ScfTwInvoice.findByIdSeqNo");
			LOGGER.error("Inside saveInvoiceTwData twInvoiceList :: {}",twInvoiceList);
            if (twInvoiceList == null || twInvoiceList.isEmpty()) {
				LOGGER.error("Start of convertTwInvoiceModelToInvoiceEntity invoiceModel :: {}",invoiceModel);
                ScfTwInvoice scfTwInvoice = ScfInvoiceTwAssembler.convertTwInvoiceModelToInvoiceEntity(invoiceModel);
				LOGGER.error("End of convertTwInvoiceModelToInvoiceEntity scfTwInvoice :: {}",scfTwInvoice);
                scfTwInvoiceRepository.save(scfTwInvoice);
				LOGGER.error("After Save");
                sendInvoiceTwOnlineAlerts(invoiceModel,userId,branchCode);

            }
            return true;
        } catch (Exception e) {
            LOGGER.error("Error while commiting Invoice Tw Data:: {}", e.getMessage(), e);
            return false;
        }
    }

	private void sendInvoiceTwOnlineAlerts(InvoiceModel invoiceModel, String userId, String branchCode) {

		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(invoiceModel,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning");
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		InvoiceDto invoiceDto = new InvoiceDto();
		alertDtoCollection.setInvoiceDto(ScfInvoiceTwAssembler.convertInvoiceTwModeltoDTO(invoiceModel, invoiceDto));
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.INVOICE_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.INVOICE_PLACEHOLDER_TRANSACTION_TYPE);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO, gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensability");
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensability");
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
						generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
								eventDetailsObject, ReportConstants.CATEGORY_INVOICE, userId, branchCode);
					}
				}
			}
		}
		LOGGER.error("End of Alert!");
	}

	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final FileUploadDto fileUploadDto,
			EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		eventDetailsObject.setEvent(fileUploadDto.getEventType());
		eventDetailsObject.setReferenceNumber(fileUploadDto.getFileName());
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}


	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final InvoiceModel invoiceModel,
			EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setRelationshipCode(invoiceModel.getRelationCode());
		eventDetailsObject.setProgramId(invoiceModel.getProgramCode());
		eventDetailsObject.setEvent(invoiceModel.getAction().concat(ReportConstants.INVOICE_INPROGRESS_APPENDER));
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		eventDetailsObject.setReferenceNumber(invoiceModel.getInvoiceId());
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean saveCreditNote(CreditNoteModel creditNoteModel,String userId,String branchCode) {
        LOGGER.info("Trying to commit Updated CreditNote data..");
        try {
            TmCreditNote tmCreditNote = ScfTmCreditNoteAssembler.convertTmCreditNoteEntity(creditNoteModel);
            LOGGER.info("Saving credit note.... {}", tmCreditNote.toString());
            tmCreditNoteRepository.save(tmCreditNote);
            sendCreditNoteOnlineAlerts(creditNoteModel,userId,branchCode);
            return true;
        } catch (Exception ex) {
            LOGGER.error("Error while commiting Credit Note Data:: {}", ex.getMessage(), ex);
            return false;
        }
    }
    
	private void sendCreditNoteOnlineAlerts(CreditNoteModel creditNoteModel, String userId, String branchCode) {
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(creditNoteModel,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning");
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		alertDtoCollection.setCreditNoteModel(creditNoteModel);
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.CREDIT_NOTE_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.CREDIT_NOTE_PLACEHOLDER_TRANSACTION_TYPE);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO,gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensibility");
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensibility");
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
						generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
								eventDetailsObject, ReportConstants.CATEGORY_CREDITNOTE, userId, branchCode);
					}
				}
			}
		}
	}

    private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final CreditNoteModel creditNoteModel, EventDetailsDTO eventDetailsObject,String userId, String branchCode) {
        eventDetailsObject.setRelationshipCode(creditNoteModel.getRelationCode());
        eventDetailsObject.setProgramId(creditNoteModel.getProgramCode());
        eventDetailsObject.setEvent(ReportConstants.CREDIT_NOTE_CREATE_EVENT);
        eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
        eventDetailsObject.setReferenceNumber(creditNoteModel.getCreditNoteRefNo());
        return alertDecisionClient.getDecisioningList(eventDetailsObject,userId,branchCode);
    }


	private void alertWithDocument(Object invoiceObj, DecisioningDetailsDTO obj, List<ParameterDTO> newparameterList) {
		// TODO: Refactor required to remove object parameter.
		UpdatedPlaceholdersDto invoiceDto = (UpdatedPlaceholdersDto) invoiceObj;
		AdviceRequest adviceRequest = new AdviceRequest();
		AdviceResponse adviceResponse = new AdviceResponse();
		Address address = new Address();
		List<Address> addresses = new ArrayList<>();
		List<MediaPreferences> mediaPreferencesList = new ArrayList<>();
		HashMap<String, Object> advice = new HashMap<>();
		HashMap<String, Object> details = new HashMap<>();
		HashMap<String, String> to = new HashMap<>();
		HashMap<String, String> from = new HashMap<>();
		HashMap<String, String> item = new HashMap<>();
		MediaPreferences mediaPreferences = new MediaPreferences();
		List<ParameterDTO> originalList = newparameterList;
		List<ParameterDTO> updatedList = changeKey(newparameterList);
		if (obj.getRecipient().equalsIgnoreCase(ReportConstants.BUYER)) {
			List<ParameterDTO> paramobj = receiverDetails.fetchaddress(invoiceDto.getBuyerId());
			List<ParameterDTO> senderDetails = receiverDetails.fetchSenderaddress(invoiceDto.getBranch());

			for (ParameterDTO param : paramobj) {
				to.put(param.getKey(), param.getValue());
			}

			for (ParameterDTO param : senderDetails) {
				from.put(param.getKey(), param.getValue());
			}

		} else if (obj.getRecipient().equalsIgnoreCase(ReportConstants.SUPPLIER)) {
			List<ParameterDTO> paramobj = receiverDetails.fetchaddress(invoiceDto.getSupplierId());
			List<ParameterDTO> senderDetails = receiverDetails.fetchSenderaddress(invoiceDto.getBranch());
			for (ParameterDTO param : paramobj) {
				to.put(param.getKey(), param.getValue());
			}

			for (ParameterDTO param : senderDetails) {
				from.put(param.getKey(), param.getValue());
			}
		}
		List<DeliveryDetailsDTO> listDeliveryDetails = obj.getDeliveryDetails();
		for (DeliveryDetailsDTO deliveryObj : listDeliveryDetails) {
			String finalMessage = advicesClient.messageReplacement(deliveryObj.getAlertText(), originalList);

			for (ParameterDTO param : updatedList) {
				item.put(param.getKey(), param.getValue());
				details.put(param.getKey(), param.getValue());
			}
			details.put("To", to);
			details.put("From", from);
			details.put("item", item);

			advice.put(deliveryObj.getTemplateName(), details);
			address.setEmailTo(obj.getAdditionalDetails());
			address.setEmailCC(obj.getAdditionalDetails());
			address.setEmailBody(finalMessage);
			address.setEmailSubject(obj.getAlertDescription());
			address.setEmailFrom("SCFCM-GENERIC_IN@ORACLE.COM");
			addresses.add(address);
			mediaPreferences.setMediaType(MediaPreferences.MediaTypeEnum.EMAIL);
			mediaPreferences.setAddresses(addresses);
			mediaPreferencesList.add(mediaPreferences);
			adviceRequest.setAdviceName(deliveryObj.getTemplateName());
			adviceRequest.setCustomerId(invoiceDto.getId());
			adviceRequest.setPreviewAdvice("N");
			adviceRequest.setReferenceId(invoiceDto.getInvoiceId());
			adviceRequest.setReferenceNumber(invoiceDto.getInvoiceRefNo());
			adviceRequest.setAdviceData(advice);
			adviceRequest.setMediaPreferences(mediaPreferencesList);
			adviceResponse = advicesClient.sendAlertWithAttachment(adviceRequest);
			LOGGER.info("Advaice Request Data:: {}", adviceRequest.toString());
		}

	}

    private List<ParameterDTO> changeKey(List<ParameterDTO> parameterList) {
        for (ParameterDTO p : parameterList) {
            if (p.getKey().equals("INVOICE_NUMBER")) {
                p.setKey("InvoiceNumber");
            } else if (p.getKey().equals("AMOUNT")) {
                p.setKey("InvoiceAmount");
            } else if (p.getKey().equals("INVOICE_DUE_DATE")) {
                p.setKey("InvoiceDueDate");
            }
        }
        return parameterList;
    }
    
    @Override
    @Transactional("PLATO_TX_MANAGER")
    public Boolean saveAutoDebitInstrumentData(AutoDebitTxnMasterDto autoDebitTxnMasterDto) {
        
 
        LOGGER.info("Trying to commit Updated Auto Debit Txn Main data..");
        
 
        try {
            String[] ignoreFields = { };
            String[] queryParams = {"batchJobExecId"};
            Object[] paramValues = {autoDebitTxnMasterDto.getBatchJobExecId()};
            AutoDebitTxnMaster autoDebitTxnMaster = null;
            List<AutoDebitTxnMaster> autoDebitTxnMainList = autoDebitTxnMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
                    "AutoDebitTxnMain.findByExecutionId");
            if(autoDebitTxnMainList != null && !autoDebitTxnMainList.isEmpty()) {
            	autoDebitTxnMaster = autoDebitTxnMainList.get(0);
            } else {
            	autoDebitTxnMaster = new AutoDebitTxnMaster();
            }
            BeanUtils.copyProperties(autoDebitTxnMasterDto, autoDebitTxnMaster, ignoreFields);
            autoDebitTxnMaster.setTimestamp(dateConverter(autoDebitTxnMasterDto.getTimestamp()));
            autoDebitTxnMaster.setBusinessDate(dateConverter(autoDebitTxnMasterDto.getBusinessDate()));
            
 
            if(autoDebitTxnMasterDto.getAutoDebitTxnDetails()!=null
            && autoDebitTxnMasterDto.getAutoDebitTxnDetails().size()>0) {
                List<AutoDebitTxnDetails> autoDebitTxnDetailsList = new ArrayList<AutoDebitTxnDetails>();
                for(AutoDebitTxnDetailsDto autoDebitTxnDetailsDto : autoDebitTxnMasterDto.getAutoDebitTxnDetails()) {
                    AutoDebitTxnDetails autoDebitTxnDetails = new AutoDebitTxnDetails();
                    BeanUtils.copyProperties(autoDebitTxnDetailsDto, autoDebitTxnDetails);
                    autoDebitTxnDetails.setAutoDebitAmt(bigDecimalConverter(autoDebitTxnDetailsDto.getAutoDebitAmt()));
                    autoDebitTxnDetails.setAllocatedAutoDebitAmt(bigDecimalConverter(autoDebitTxnDetailsDto.getAllocatedAutoDebitAmt()));
                    autoDebitTxnDetailsList.add(autoDebitTxnDetails);
                }
                autoDebitTxnMaster.setAutoDebitTxnDetails(autoDebitTxnDetailsList);
            }
            
 
            autoDebitTxnMasterRepository.save(autoDebitTxnMaster);
            return true;
        } catch(Exception e) {
            LOGGER.error("Error while commiting Auto Debit Txn Data:: {}", e.getMessage(), e);
            return false;
        }
    }
    
    private BigDecimal bigDecimalConverter(String inputBigDecimal) {

		if(inputBigDecimal == null) {
			return null;
		}
		return new BigDecimal(inputBigDecimal);
	}
    
    @Override
	@Transactional("PLATO_TX_MANAGER")
	public void saveAccountsTwData(AccountingTransactionModel accountingTransactionModel) {
		
		try {
			String[] ignoreFields = { MAKER_DT_STAMP, CHECKER_DT_STAMP, VALUE_DATE, PROCESSING_DATE };
			
			AccountPostTw accountsWorkt;
			
			String[] queryParams = {REF_NO, SEQ_NO, ID };
			Object[] paramValues = {accountingTransactionModel.getRefNo(), accountingTransactionModel.getSeqNo(), accountingTransactionModel.getId()};
			List<AccountPostTw> accountsWorktDeatils = accountsTwRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"AccountPostTw.findByreferenceSeqId");
			
			if(accountsWorktDeatils != null && !accountsWorktDeatils.isEmpty()) {
				accountsWorkt = accountsWorktDeatils.get(0);
			} else {
				accountsWorkt = new AccountPostTw();
			}
			
			BeanUtils.copyProperties(accountingTransactionModel, accountsWorkt, ignoreFields);
			
			accountsWorkt.setMakererDateStamp(dateConverter(accountingTransactionModel.getMakerDateStamp()));
			accountsWorkt.setCheckerDateStamp(dateConverter(accountingTransactionModel.getCheckerDateStamp()));
			accountsWorkt.setValueDate(dateConverter(accountingTransactionModel.getValueDate()));
			accountsWorkt.setProcessingDate(dateConverter(accountingTransactionModel.getProcessingDate()));
			
			accountsTwRepository.save(accountsWorkt);
		} catch(Exception e) {
			LOGGER.error("Error while commiting Accounts Work Data:: {}", e.getMessage(), e);
		}
	}
    
    @Override
	@Transactional("PLATO_TX_MANAGER")
	public void saveAccountsTmData(AccountingTransactionModel accountingTransactionModel) {
		
		try {
			String[] ignoreFields = { MAKER_DT_STAMP, CHECKER_DT_STAMP, VALUE_DATE, PROCESSING_DATE };
			
			AccountPost accountPostTm;
			
			String[] queryParams = {REF_NO, SEQ_NO, ID };
			Object[] paramValues = {accountingTransactionModel.getRefNo(), accountingTransactionModel.getSeqNo(),  accountingTransactionModel.getId()};
			List<AccountPost> accountsTmDeatils = accountsTmRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"AccountPost.findByreferenceSeqId");
			
			if(CollectionUtils.isNotEmpty(accountsTmDeatils)) {
				accountPostTm = accountsTmDeatils.get(0);
			} else {
				accountPostTm = new AccountPost();
			}
			
			BeanUtils.copyProperties(accountingTransactionModel, accountPostTm, ignoreFields);
			
			accountPostTm.setMakererDateStamp(dateConverter(accountingTransactionModel.getMakerDateStamp()));
			accountPostTm.setCheckerDateStamp(dateConverter(accountingTransactionModel.getCheckerDateStamp()));
			accountPostTm.setValueDate(dateConverter(accountingTransactionModel.getValueDate()));
			accountPostTm.setProcessingDate(dateConverter(accountingTransactionModel.getProcessingDate()));
			
			accountsTmRepository.save(accountPostTm);
		} catch(Exception e) {
			LOGGER.error("Error while commiting Accounts Main Data:: {}", e.getMessage(), e);
		}
	}
    
    @Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveAutoDebitData(AutoDebitTxnDto autoDebitTxnDto) {
		
		LOGGER.info("Trying to commit Updated Auto Debit data..");
		
		try {
			String[] ignoreFields = { };
			String[] queryParams = {"batchJobExecId"};
			Object[] paramValues = {autoDebitTxnDto.getBatchJobExecId()};
			AutoDebitTxnMaster autoDebitTxnMaster = null;
			List<AutoDebitTxnMaster> autoDebitTxnMainList = autoDebitTxnMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
                    "AutoDebitTxnMaster.findByExecutionId");
            if(autoDebitTxnMainList != null && !autoDebitTxnMainList.isEmpty()) {
            	autoDebitTxnMaster = autoDebitTxnMainList.get(0);
            } else {
            	autoDebitTxnMaster = new AutoDebitTxnMaster();
            }
            BeanUtils.copyProperties(autoDebitTxnDto, autoDebitTxnMaster, ignoreFields);
            autoDebitTxnMaster.setTimestamp(dateConverter(autoDebitTxnDto.getTimestamp()));
            autoDebitTxnMaster.setBusinessDate(dateConverter(autoDebitTxnDto.getBusinessDate()));
            if(autoDebitTxnDto.getAutoDebitTxnDetails()!=null
            && autoDebitTxnDto.getAutoDebitTxnDetails().size()>0) {
                List<AutoDebitTxnDetails> autoDebitTxnDetailsList = new ArrayList<>();
                for(AutoDebitTxnDetailsDto autoDebitTxnDetailsDto : autoDebitTxnDto.getAutoDebitTxnDetails()) {
                    AutoDebitTxnDetails autoDebitTxnDetails = new AutoDebitTxnDetails();
                    BeanUtils.copyProperties(autoDebitTxnDetailsDto, autoDebitTxnDetails);
                    autoDebitTxnDetails.setAutoDebitAmt(bigDecimalConverter(autoDebitTxnDetailsDto.getAutoDebitAmt()));
                    autoDebitTxnDetails.setAllocatedAutoDebitAmt(bigDecimalConverter(autoDebitTxnDetailsDto.getAllocatedAutoDebitAmt()));
                    autoDebitTxnDetailsList.add(autoDebitTxnDetails);
                }
                autoDebitTxnMaster.setAutoDebitTxnDetails(autoDebitTxnDetailsList);
            }
            autoDebitTxnMasterRepository.save(autoDebitTxnMaster);
            return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Auto Debit Data:: {}", e.getMessage(), e);
			return false;
		}
	}



	private void sendOnlineAlertsForPOActions(PurchaseOrderDTO purchaseOrderDTO, String userId, String branchCode) {
		LOGGER.info("Sending alert for putchase order: {} , status :{} , netpoAmount:{},currency :{},poDate:{},makerDate:{}"
				,purchaseOrderDTO.getPoExternalNumber(),purchaseOrderDTO.getPoStatus(),purchaseOrderDTO.getNetPoAmount(),purchaseOrderDTO.getCurrency(),purchaseOrderDTO.getPoDate(),purchaseOrderDTO.getMakerDate());
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		DecisioningWrapperDTO decisioningDetailsResponseWrapper = getDecisioningDetailsResponseWrapper(purchaseOrderDTO,
				eventDetailsObject, userId, branchCode);
		LOGGER.info("Got details from alert decisioning {} ",decisioningDetailsResponseWrapper);
		AlertDtoCollection alertDtoCollection = new AlertDtoCollection();
		Gson gson = new Gson();
		alertDtoCollection.setPurchaseOrderDTO(purchaseOrderDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisioningDetailsResponseWrapper.getDetailsCollection();
		for (DecisioningDetailsDTO alertDecision : alertDecisionList) {
			OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
			HashMap<String, Object> requestParams = new HashMap();
			req.setRequestType(ReportConstants.PO_PLACEHOLDER_REQUEST_TYPE);
			req.setTransactionType(ReportConstants.PO_PLACEHOLDER_TRANSACTION_TYPE);
			LOGGER.info("alertDtoCollection :{}",alertDtoCollection);
			requestParams.put(ReportConstants.PLACEHOLDER_ALERTS_DTO, gson.toJson(alertDtoCollection));
			requestParams.put(ReportConstants.DECISIONING_DETAILS_DTO, gson.toJson(alertDecision));
			requestParams.put(ReportConstants.BRANCH_CODE,branchCode);
			req.setRequestParams(requestParams);
			LOGGER.info("Sending call to  extensability");
			LOGGER.info("alertDecision {} ",gson.toJson(alertDecision));
			OBIntegrationServiceResponse response = obIntegrationAdapter.handleTransaction(req);
			LOGGER.info("Received response from  extensability {} ",response);
			if (response.getStatus() != null && response.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)
					&& response.getResponseParams() != null) {
				HashMap<String, Object> adapterResponse = response.getResponseParams();
				if (adapterResponse != null && adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO) != null) {
					if (alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
						generateAlert(adapterResponse.get(ReportConstants.UPDATED_PLACEHOLDER_DTO), alertDecision,
								eventDetailsObject, ReportConstants.CATEGORY_PO, userId, branchCode);
					}
				}
			}
		}
		LOGGER.info("Alert Send for  {}",purchaseOrderDTO.getPoExternalNumber());
	}
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final PurchaseOrderDTO purchaseOrderDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		LOGGER.info("purchaseOrderDTO {}",purchaseOrderDTO);
		eventDetailsObject.setRelationshipCode(purchaseOrderDTO.getRelationshipCode());
		eventDetailsObject.setProgramId(purchaseOrderDTO.getProgramId());
		eventDetailsObject.setEvent(purchaseOrderDTO.getPoStatus());
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		eventDetailsObject.setReferenceNumber(purchaseOrderDTO.getPoInternalNumber());
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean savePoData(List<PurchaseOrderDTO> poTwList, List<PurchaseOrderDTO> poTmList, String userId, String branchCode){
		try {
			LOGGER.info(" purchaseorder work size : {} , purchaseorder main size ", poTwList.size(),poTmList.size());
			if (poTwList != null && !poTwList.isEmpty()) {
				for (PurchaseOrderDTO purchaseOrderDTO : poTwList) {
					savePoTwData(purchaseOrderDTO, userId, branchCode);
				}
			}
			if (poTmList != null && !poTmList.isEmpty()) {
				for (PurchaseOrderDTO purchaseOrderDTO : poTmList) {
					savePoTMData(purchaseOrderDTO, userId, branchCode);
				}
			}

		}catch(Exception exception){
			LOGGER.error("Error while commiting Purchase order  Data:: {}", exception.getMessage());
			return false;
		}
		return true;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean savePoTMData(PurchaseOrderDTO purchaseOrderDTO, String userId, String branchCode) {
		try {
			LOGGER.info("savePoTMData {} , {}",purchaseOrderDTO.getPoInternalNumber(),purchaseOrderDTO.getSeqNo());
			String[] queryParams = {"poInternalNumbers"};
			Object[] paramValues = {Arrays.asList(purchaseOrderDTO.getPoInternalNumber())};
			List<TmPurchaseOrder> tmPurchaseOrderList = tmPurchaseOrderRepository.executeNamedQueryByCondition(queryParams, paramValues,
					"TmPurchaseOrder.findByInternalNumbers");
			TmPurchaseOrder tmPurchaseOrder = null;
			Long version = null;
			if (tmPurchaseOrderList == null || tmPurchaseOrderList.isEmpty()) {
				LOGGER.info("TM Purchase order  is not found in projection DB ");
				tmPurchaseOrder = new TmPurchaseOrder();
			}else{
				LOGGER.info("TM Purchase order  found in projection DB ");
				tmPurchaseOrder = tmPurchaseOrderList.get(0);
				version = tmPurchaseOrder.getVersion();
			}
			tmPurchaseOrder = MapperUtil.INSTANCE.toTmPurchaseOrder(purchaseOrderDTO);
			tmPurchaseOrder.setVersion(version);
			tmPurchaseOrder.setTmPoShipParameters(MapperUtil.INSTANCE.toTmPOShipParameters(purchaseOrderDTO));
			if(!CollectionUtils.isEmpty(purchaseOrderDTO.getCommodityList())) {
				List<TmCommodityList> commodityList = new ArrayList<>();
				purchaseOrderDTO.getCommodityList().stream().forEach(item -> {
					commodityList.add(MapperUtil.INSTANCE.toTmCommodityList(item));
				});
				tmPurchaseOrder.setTmCommodityList(commodityList);
			}
			tmPurchaseOrderRepository.merge(tmPurchaseOrder);
			sendOnlineAlertsForPOActions(purchaseOrderDTO,  userId,  branchCode);
			return true;
		} catch (Exception e) {
			LOGGER.error("Error while commiting Purchase order TM Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean savePoTwData(PurchaseOrderDTO purchaseOrderDTO, String userId, String branchCode) {
		try {
			LOGGER.info("savePoTwData {} , {}",purchaseOrderDTO.getPoInternalNumber(),purchaseOrderDTO.getSeqNo());
			String[] queryParams = {"poInternalNumber", "seqNo"};
			Object[] paramValues = {purchaseOrderDTO.getPoInternalNumber(), purchaseOrderDTO.getSeqNo()};
			List<TwPurchaseOrder> twITwPurchaseOrderList = twPurchaseOrderRepository.executeNamedQueryByCondition(queryParams, paramValues,
					"TwPurchaseOrder.findByInternalRefAndSeq");
			TwPurchaseOrder twPurchaseOrder = null;
			Long version = null;
			if (twITwPurchaseOrderList != null && !twITwPurchaseOrderList.isEmpty()) {
				twPurchaseOrder = twITwPurchaseOrderList.get(0);
				version = twPurchaseOrder.getVersion();
            } else {
            	twPurchaseOrder = new TwPurchaseOrder();
            }
			twPurchaseOrder = MapperUtil.INSTANCE.toTwPurchaseOrder(purchaseOrderDTO);
			twPurchaseOrder.setVersion(version);
			twPurchaseOrder.setTwPoShipParameters(MapperUtil.INSTANCE.toTwPOShipParameters(purchaseOrderDTO));
			if(!CollectionUtils.isEmpty(purchaseOrderDTO.getCommodityList())) {
				List<TwCommodityList> commList = new ArrayList<>();
				purchaseOrderDTO.getCommodityList().stream().forEach(item -> {
					TwCommodityList commodity = MapperUtil.INSTANCE.toTwCommodityList(item);
					TwCommodityListPk pk = new TwCommodityListPk();
					pk.setSeqNo(purchaseOrderDTO.getSeqNo()!=null?purchaseOrderDTO.getSeqNo():1L);
	                pk.setId(item.getId());
	                commodity.setId(pk);
	                commList.add(commodity);
				});
				twPurchaseOrder.setTwCommodityList(commList);
			}
			twPurchaseOrderRepository.merge(twPurchaseOrder);
			return true;
		} catch (Exception e) {
			LOGGER.error("Error while commiting Purchase order Tw Data:: {}", e.getMessage(), e);
			return false;
		}
	}


}

