package oracle.fsgbu.obscfcm.services.report.service;

import java.math.BigDecimal;
import java.util.*;

import avro.oracle.fsgbu.obscf.invoicing.dto.*;
import oracle.cashflow.scfcm.adapter.dto.CashflowResponse;
import oracle.fsgbu.eventService.api.model.*;
import oracle.fsgbu.gus.persistence.provider.GlobalHolder;
import oracle.fsgbu.gus.persistence.provider.RequestContext;
import oracle.fsgbu.ob.adapter.dto.OBIntegrationServiceRequest;
import oracle.fsgbu.ob.adapter.dto.OBIntegrationServiceResponse;
import oracle.fsgbu.ob.adapter.provider.OBIntegrationAdapter;
import oracle.fsgbu.obscfcm.services.report.client.*;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.stream.annotation.EnableBinding;
import org.springframework.cloud.stream.annotation.StreamListener;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Service;

import avro.oracle.fsgbu.obscf.manualrecon.dto.ReconTmCashflowDto;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateExpDrCrReconDTO;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateExpDrCrReconDTOCollection;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateFinPmtReconDTO;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateFinPmtReconDTOCollection;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateInvCnReconDTO;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateInvCnReconDTOCollection;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateInvPmtReconDTO;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateInvPmtReconDTOCollection;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdatePmtAllocDTO;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdatePmtAllocDTOCollection;
import avro.oracle.fsgbu.obscf.manualrecon.dto.UpdateReconTwDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateAccountEntryCodeDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateAccountEntryDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateAccountEntryTWDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateAccountRoleDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateAccountRoleTWDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateExternalAccountMapDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateExternalAccountMapTWDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateInternalAccountMapDto;
import avro.oracle.fsgbu.obscfcm.accounting.dto.UpdateInternalAccountMapTWDto;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBChargesCollection;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBChargesDto;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBFinanceCollection;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBFinanceDto;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBInvoiceDetailsCollection;
import avro.oracle.fsgbu.obscfcm.charges.dto.UpdateOBInvoiceDetailsDto;
import avro.oracle.fsgbu.obscfcm.services.dto.CreateCustomerDelinqDto;
import avro.oracle.fsgbu.obscfcm.services.dto.CreateCustomerDelinqStatusDto;
import avro.oracle.fsgbu.obscfcm.services.dto.CreateExtCustomerDelinqStatusHisDto;
import avro.oracle.fsgbu.obscfcm.services.dto.CreateExternalDelinqDto;
import oracle.fsgbu.accountMaintenance.api.model.AccentryWebModel;
import oracle.fsgbu.accountMaintenance.api.model.AccentrycodeWebModel;
import oracle.fsgbu.accountMaintenance.api.model.AccountingTransactionCollection;
import oracle.fsgbu.accountMaintenance.api.model.AccountingTransactionModel;
import oracle.fsgbu.accountMaintenance.api.model.ExternalWebModel;
import oracle.fsgbu.accountMaintenance.api.model.IntaccmapWebModel;
import oracle.fsgbu.accountMaintenance.api.model.RoleWebModel;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.FinanceTxnDTO;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.InstrumentTxnDTO;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.ObChargesTxnDTO;
import oracle.fsgbu.obscfcm.services.report.assembler.ScfInvoiceAssembler;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.kafka.PlatoMessageChannels;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ScftmCustomerDelinqStatusDTO;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ScftmExtCustomerDelinqStatusHisDTO;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reconService.api.model.ExpDrCrPmtReconDTO;
import oracle.fsgbu.reconService.api.model.FinPmtReconDTO;
import oracle.fsgbu.reconService.api.model.InvCnReconDTO;
import oracle.fsgbu.reconService.api.model.InvPmtReconDTO;
import oracle.fsgbu.reconService.api.model.PmtAllocDTO;
import oracle.fsgbu.reconService.api.model.ReconDataCollection;
import oracle.fsgbu.reconService.api.model.ReconTxnDto;
import oracle.fsgbu.reconService.api.model.ReconInvCnDto;
import oracle.fsgbu.reconService.api.model.ReconInvPmtDto;

@Service
@EnableBinding(PlatoMessageChannels.class)
public class PlatoCloudStreamListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(PlatoCloudStreamListenerService.class);

	@Autowired
	private EventServiceClient eventServiceClient;
	@Autowired
	private ReconEventServiceClient reconEventServiceClient;

	@Autowired
	private InstrumentReceivablesServiceClient instrumentReceivablesServiceClient;

	@Autowired
	private EventListenerService eventListenerService;
	
	@Autowired
	ReconEventListenerService reconEventListenerService;

	@Autowired
	private AccountMaintenanceServiceClient accountMaintenanceServiceClient;
	
	@Autowired
	AccountMaintenanceListenerService accountMaintenanceListenerService;
	
	@Autowired
	private AccountMaintenanceServiceClient accountServiceClient;
	
	@Autowired
	private CustomerListenerService customerListenerService;
	
	@Autowired
	private ChargesListenerService chargesListenerService;

	@Autowired
	private OBIntegrationAdapter obIntegrationAdapter;
	
	@Autowired
	public PlatoCloudStreamListenerService(EventServiceClient eventServiceClient,ReconEventServiceClient reconEventServiceClient ) {

		this.eventServiceClient = eventServiceClient;
		this.reconEventServiceClient = reconEventServiceClient;
	}

	@StreamListener(ReportConstants.EXP_DRCR_PMT_RECON_UPDATED)
	public void getExpDrCrReconData(Message<UpdateExpDrCrReconDTOCollection> message) {

		LOGGER.info("Message Received on channel expected debit credit Updated Event");
		UpdateExpDrCrReconDTOCollection updateExpDrCrReconDTOCollection = message.getPayload();

		String userId = updateExpDrCrReconDTOCollection.getUserId();
		String branchCode = updateExpDrCrReconDTOCollection.getServiceBranchCode();

		List<UpdateExpDrCrReconDTO> updateExpDrCrReconDTOList =  updateExpDrCrReconDTOCollection.getUpdateExpDrCrReconDTOList();
        for(UpdateExpDrCrReconDTO updateExpDrCrReconDTO : updateExpDrCrReconDTOList) {
		if(updateExpDrCrReconDTO != null) {
			ExpDrCrPmtReconDTO expDrCrPmtReconDto = new ExpDrCrPmtReconDTO();
			expDrCrPmtReconDto.setAmtMatched(numberConverter(updateExpDrCrReconDTO.getAmtMatched()));
			expDrCrPmtReconDto.setAuthDtStamp(stringConverter(updateExpDrCrReconDTO.getAuthDtStamp()));
			expDrCrPmtReconDto.setAuthId(stringConverter(updateExpDrCrReconDTO.getAuthId()));
			expDrCrPmtReconDto.setBeneCode(stringConverter(updateExpDrCrReconDTO.getBeneCode()));
			expDrCrPmtReconDto.setBeneName(stringConverter(updateExpDrCrReconDTO.getBeneName()));
			expDrCrPmtReconDto.setBeneVirtualAccNo(stringConverter(updateExpDrCrReconDTO.getBeneVirtualAccNo()));
			expDrCrPmtReconDto.setBranchCode(stringConverter(updateExpDrCrReconDTO.getBranchCode()));
			expDrCrPmtReconDto.setCorporateId(stringConverter(updateExpDrCrReconDTO.getCorporateId()));
			expDrCrPmtReconDto.setCounterpartyId(stringConverter(updateExpDrCrReconDTO.getCounterpartyId()));
			expDrCrPmtReconDto.setExchRate(numberConverter(updateExpDrCrReconDTO.getExchRate()));
			expDrCrPmtReconDto.setExpDrcrAmt(numberConverter(updateExpDrCrReconDTO.getExpDrcrAmt()));
			expDrCrPmtReconDto.setExpDrcrBankAccNumber(stringConverter(updateExpDrCrReconDTO.getExpDrcrBankAccNumber()));
			expDrCrPmtReconDto.setExpDrcrCcy(stringConverter(updateExpDrCrReconDTO.getExpDrcrCcy()));
			expDrCrPmtReconDto.setExpDrcrCode(stringConverter(updateExpDrCrReconDTO.getExpDrcrCode()));
			expDrCrPmtReconDto.setExpDrcrCodeDesc(stringConverter(updateExpDrCrReconDTO.getExpDrcrCodeDesc()));
			expDrCrPmtReconDto.setExpDrcrDate(stringConverter(updateExpDrCrReconDTO.getExpDrcrDate()));
			expDrCrPmtReconDto.setExpDrcrIndicator(stringConverter(updateExpDrCrReconDTO.getExpDrcrIndicator()));
			expDrCrPmtReconDto.setExpDrcrNarration(stringConverter(updateExpDrCrReconDTO.getExpDrcrNarration()));
			expDrCrPmtReconDto.setExpDrCrReconId(numberConverter(updateExpDrCrReconDTO.getExpDrCrReconId()));

			expDrCrPmtReconDto.setExpDrcrRefNo(stringConverter(updateExpDrCrReconDTO.getExpDrcrRefNo()));
			expDrCrPmtReconDto.setExpDrcrVirtualAccNo(stringConverter(updateExpDrCrReconDTO.getExpDrcrVirtualAccNo()));
			expDrCrPmtReconDto.setExternalAccNo(stringConverter(updateExpDrCrReconDTO.getExternalAccNo()));
			expDrCrPmtReconDto.setExternalDrcrRefNo(stringConverter(updateExpDrCrReconDTO.getExternalDrcrRefNo()));
			expDrCrPmtReconDto.setExternalPmtRefNo(stringConverter(updateExpDrCrReconDTO.getExternalPmtRefNo()));
			expDrCrPmtReconDto.setLhsEntity(stringConverter(updateExpDrCrReconDTO.getLhsEntity()));
			expDrCrPmtReconDto.setMakerDtStamp(stringConverter(updateExpDrCrReconDTO.getMakerDtStamp()));
			expDrCrPmtReconDto.setMakerId(stringConverter(updateExpDrCrReconDTO.getMakerId()));
			expDrCrPmtReconDto.setMatchSeqNo(stringConverter(updateExpDrCrReconDTO.getMatchSeqNo()));
			expDrCrPmtReconDto.setPaymentAmount(numberConverter(updateExpDrCrReconDTO.getPaymentAmount()));
			expDrCrPmtReconDto.setPaymentCcy(stringConverter(updateExpDrCrReconDTO.getPaymentCcy()));
			expDrCrPmtReconDto.setPaymentDate(stringConverter(updateExpDrCrReconDTO.getPaymentDate()));
			expDrCrPmtReconDto.setPaymentMode(stringConverter(updateExpDrCrReconDTO.getPaymentMode()));
			expDrCrPmtReconDto.setPaymentPartyId(stringConverter(updateExpDrCrReconDTO.getPaymentPartyId()));
			expDrCrPmtReconDto.setPaymentPartyName(stringConverter(updateExpDrCrReconDTO.getPaymentPartyName()));
			expDrCrPmtReconDto.setPaymentRefNo(stringConverter(updateExpDrCrReconDTO.getPaymentRefNo()));
			expDrCrPmtReconDto.setPmtStatus(stringConverter(updateExpDrCrReconDTO.getPmtStatus()));
			expDrCrPmtReconDto.setReconAction(stringConverter(updateExpDrCrReconDTO.getReconAction()));
			expDrCrPmtReconDto.setReconCategory(stringConverter(updateExpDrCrReconDTO.getReconCategory()));
			expDrCrPmtReconDto.setReconileBy(stringConverter(updateExpDrCrReconDTO.getReconcileBy()));
			expDrCrPmtReconDto.setReconciledPmtAmt(numberConverter(updateExpDrCrReconDTO.getReconciledPmtAmt()));
			expDrCrPmtReconDto.setReconDate(stringConverter(updateExpDrCrReconDTO.getReconDate()));
			expDrCrPmtReconDto.setReconId(numberConverter(updateExpDrCrReconDTO.getReconId()));
			expDrCrPmtReconDto.setReconRule(stringConverter(updateExpDrCrReconDTO.getReconRule()));
			expDrCrPmtReconDto.setReconRuleType(stringConverter(updateExpDrCrReconDTO.getReconRuleType()));
			expDrCrPmtReconDto.setReconStatus(stringConverter(updateExpDrCrReconDTO.getReconStatus()));
			expDrCrPmtReconDto.setReconTxnRefNo(stringConverter(updateExpDrCrReconDTO.getReconTxnRefNo()));
			expDrCrPmtReconDto.setRevisedDrcrDate(stringConverter(updateExpDrCrReconDTO.getRevisedDrcrDate()));
			expDrCrPmtReconDto.setRhsEntity(stringConverter(updateExpDrCrReconDTO.getRhsEntity()));
			expDrCrPmtReconDto.setStage(stringConverter(updateExpDrCrReconDTO.getStage()));
			expDrCrPmtReconDto.setTransactionStatus(stringConverter(updateExpDrCrReconDTO.getTransactionStatus()));
			expDrCrPmtReconDto.setExtSysRefNo(stringConverter(updateExpDrCrReconDTO.getExtSysRefNo()));
			expDrCrPmtReconDto.setDeReconFlag(stringConverter(updateExpDrCrReconDTO.getDeReconFlag()));
			expDrCrPmtReconDto.setUdfTx1(stringConverter(updateExpDrCrReconDTO.getUdfTx1()));
			expDrCrPmtReconDto.setUdfTx2(stringConverter(updateExpDrCrReconDTO.getUdfTx2()));
			expDrCrPmtReconDto.setUdfTx3(stringConverter(updateExpDrCrReconDTO.getUdfTx3()));
			expDrCrPmtReconDto.setUdfTx4(stringConverter(updateExpDrCrReconDTO.getUdfTx4()));
			expDrCrPmtReconDto.setUdfTx5(stringConverter(updateExpDrCrReconDTO.getUdfTx5()));
			expDrCrPmtReconDto.setUdfTx6(stringConverter(updateExpDrCrReconDTO.getUdfTx6()));
			expDrCrPmtReconDto.setUdfTx7(stringConverter(updateExpDrCrReconDTO.getUdfTx7()));
			expDrCrPmtReconDto.setUdfTx8(stringConverter(updateExpDrCrReconDTO.getUdfTx8()));
			expDrCrPmtReconDto.setUdfTx9(stringConverter(updateExpDrCrReconDTO.getUdfTx9()));
			expDrCrPmtReconDto.setUdfTx10(stringConverter(updateExpDrCrReconDTO.getUdfTx10()));
			expDrCrPmtReconDto.setCashflowType(stringConverter(updateExpDrCrReconDTO.getCashflowType()));
			expDrCrPmtReconDto.setActualDate(String.valueOf(stringConverter(updateExpDrCrReconDTO.getActualDate())));
			expDrCrPmtReconDto.setChannel(stringConverter(updateExpDrCrReconDTO.getChannel()));
			expDrCrPmtReconDto.setApplicationName(stringConverter(updateExpDrCrReconDTO.getApplicationName()));
			expDrCrPmtReconDto.setPayCounterpartyId(stringConverter(updateExpDrCrReconDTO.getPayCounterpartyId()));
			expDrCrPmtReconDto.setRemitterAccNo(stringConverter(updateExpDrCrReconDTO.getRemitterAccNo()));
			expDrCrPmtReconDto.setDcIndicator(stringConverter(updateExpDrCrReconDTO.getDcIndicator()));
			expDrCrPmtReconDto.setAccOwner(stringConverter(updateExpDrCrReconDTO.getAccOwner()));
			expDrCrPmtReconDto.setIsPmtAccVA(stringConverter(updateExpDrCrReconDTO.getIsPmtAccVA()));
			expDrCrPmtReconDto.setPayCounterpartyName(stringConverter(updateExpDrCrReconDTO.getPayCounterpartyName()));
			expDrCrPmtReconDto.setAllocationRequired(stringConverter(updateExpDrCrReconDTO.getAllocationRequired()));
			expDrCrPmtReconDto.setLinkedDeReconTxnRef(stringConverter(updateExpDrCrReconDTO.getLinkedDeReconTxnRef()));
			expDrCrPmtReconDto.setCustomerRefNo(stringConverter(updateExpDrCrReconDTO.getCustomerRefNo()));
			expDrCrPmtReconDto.setRemarks(stringConverter(updateExpDrCrReconDTO.getRemarks()));
			expDrCrPmtReconDto.setReconType(stringConverter(updateExpDrCrReconDTO.getReconType()));
			expDrCrPmtReconDto.setPmtProgramCode(stringConverter(updateExpDrCrReconDTO.getPmtProgramCode()));
			expDrCrPmtReconDto.setPmtRelationshipCode(stringConverter(updateExpDrCrReconDTO.getPmtRelationshipCode()));
			reconEventServiceClient.saveExpDrCrData(expDrCrPmtReconDto, userId, branchCode);
		}
       }
	}
	@StreamListener(ReportConstants.INV_PMT_RECON_UPDATED)
	public void getInvPmtReconData(Message<UpdateInvPmtReconDTOCollection> message) {

		LOGGER.info("Message Received on channel Invoice Payment Recon Updated Event");
		UpdateInvPmtReconDTOCollection updateInvPmtReconDTOCollection = message.getPayload();

		String userId = updateInvPmtReconDTOCollection.getUserId();
		String branchCode = updateInvPmtReconDTOCollection.getServiceBranchCode();

        List<UpdateInvPmtReconDTO> updateInvPmtReconDTOList = updateInvPmtReconDTOCollection.getUpdateInvPmtReconDTOList();
		for(UpdateInvPmtReconDTO updateInvPmtReconDTO:updateInvPmtReconDTOList) {
        if(updateInvPmtReconDTO != null) {
			InvPmtReconDTO invPmtReconDTO = new InvPmtReconDTO();
			invPmtReconDTO.setReconId(numberConverter(updateInvPmtReconDTO.getReconId()));
			invPmtReconDTO.setReconTxnRefNo(stringConverter(updateInvPmtReconDTO.getReconTxnRefNo()));
			invPmtReconDTO.setReconCategory(stringConverter(updateInvPmtReconDTO.getReconCategory()));
			invPmtReconDTO.setReconDate(stringConverter(updateInvPmtReconDTO.getReconDate()));
			invPmtReconDTO.setBranchCode(stringConverter(updateInvPmtReconDTO.getBranchCode()));
			invPmtReconDTO.setReconType(stringConverter(updateInvPmtReconDTO.getReconType()));
			invPmtReconDTO.setReconRuleType(stringConverter(updateInvPmtReconDTO.getReconRuleType()));
			invPmtReconDTO.setReconRule(stringConverter(updateInvPmtReconDTO.getReconRule()));
			invPmtReconDTO.setLhsEntity(stringConverter(updateInvPmtReconDTO.getLhsEntity()));
			invPmtReconDTO.setRhsEntity(stringConverter(updateInvPmtReconDTO.getRhsEntity()));
			invPmtReconDTO.setMakerId(stringConverter(updateInvPmtReconDTO.getMakerId()));
			invPmtReconDTO.setMakerDtStamp(stringConverter(updateInvPmtReconDTO.getMakerDtStamp()));
			invPmtReconDTO.setAuthId(stringConverter(updateInvPmtReconDTO.getAuthId()));
			invPmtReconDTO.setAuthDtStamp(stringConverter(updateInvPmtReconDTO.getAuthDtStamp()));
			invPmtReconDTO.setTransactionStatus(stringConverter(updateInvPmtReconDTO.getTransactionStatus()));
			invPmtReconDTO.setReconAction(stringConverter(updateInvPmtReconDTO.getReconAction()));
			invPmtReconDTO.setStage(stringConverter(updateInvPmtReconDTO.getStage()));
			invPmtReconDTO.setReconcileBy(stringConverter(updateInvPmtReconDTO.getReconcileBy()));
			invPmtReconDTO.setMatchSeqNo(stringConverter(updateInvPmtReconDTO.getMatchSeqNo()));
			invPmtReconDTO.setInvBuyerId(stringConverter(updateInvPmtReconDTO.getInvBuyerId()));
			invPmtReconDTO.setInvSupplierId(stringConverter(updateInvPmtReconDTO.getInvSupplierId()));
			invPmtReconDTO.setInvBuyerName(stringConverter(updateInvPmtReconDTO.getInvBuyerName()));
			invPmtReconDTO.setInvSupplierName(stringConverter(updateInvPmtReconDTO.getInvSupplierName()));
			invPmtReconDTO.setInvRelationCode(stringConverter(updateInvPmtReconDTO.getInvRelationCode()));
			invPmtReconDTO.setExtInvRefNo(stringConverter(updateInvPmtReconDTO.getExtInvRefNo()));
			invPmtReconDTO.setIntInvRefNo(stringConverter(updateInvPmtReconDTO.getIntInvRefNo()));
			invPmtReconDTO.setInvDate(stringConverter(updateInvPmtReconDTO.getInvDate()));
			invPmtReconDTO.setInvDueDate(stringConverter(updateInvPmtReconDTO.getInvDueDate()));
			invPmtReconDTO.setNetInvAmt(numberConverter(updateInvPmtReconDTO.getNetInvAmt()));
			invPmtReconDTO.setInvCcy(stringConverter(updateInvPmtReconDTO.getInvCcy()));
			invPmtReconDTO.setInvVirtualAccNo(stringConverter(updateInvPmtReconDTO.getInvVirtualAccNo()));
			invPmtReconDTO.setInvOsAmt(numberConverter(updateInvPmtReconDTO.getInvOsAmt()));
			invPmtReconDTO.setInvFinanced(stringConverter(updateInvPmtReconDTO.getInvFinanced()));
			invPmtReconDTO.setAmtMatched(numberConverter(updateInvPmtReconDTO.getAmtMatched()));
			invPmtReconDTO.setPaymentRefNo(stringConverter(updateInvPmtReconDTO.getPaymentRefNo()));
			invPmtReconDTO.setPaymentPartyId(stringConverter(updateInvPmtReconDTO.getPaymentPartyId()));
			invPmtReconDTO.setPaymentPartyName(stringConverter(updateInvPmtReconDTO.getPaymentPartyName()));
			invPmtReconDTO.setPaymentDate(stringConverter(updateInvPmtReconDTO.getPaymentDate()));
			invPmtReconDTO.setPaymentMode(stringConverter(updateInvPmtReconDTO.getPaymentMode()));
			invPmtReconDTO.setBeneCode(stringConverter(updateInvPmtReconDTO.getBeneCode()));
			invPmtReconDTO.setBeneName(stringConverter(updateInvPmtReconDTO.getBeneName()));
			invPmtReconDTO.setPaymentAmount(numberConverter(updateInvPmtReconDTO.getPaymentAmount()));
			invPmtReconDTO.setPaymentCcy(stringConverter(updateInvPmtReconDTO.getPaymentCcy()));
			invPmtReconDTO.setExchRate(numberConverter(updateInvPmtReconDTO.getExchRate()));
			invPmtReconDTO.setReconciledPmtAmt(numberConverter(updateInvPmtReconDTO.getReconciledPmtAmt()));
			invPmtReconDTO.setBeneVirtualAccNo(stringConverter(updateInvPmtReconDTO.getBeneVirtualAccNo()));
			invPmtReconDTO.setPmtStatus(stringConverter(updateInvPmtReconDTO.getPmtStatus()));
			invPmtReconDTO.setReconStatus(stringConverter(updateInvPmtReconDTO.getReconStatus()));
			invPmtReconDTO.setInvPmtReconId(numberConverter(updateInvPmtReconDTO.getInvPmtReconId()));
			invPmtReconDTO.setAccountNumber(stringConverter(updateInvPmtReconDTO.getAccountNumber()));
			invPmtReconDTO.setBeneficiaryId(stringConverter(updateInvPmtReconDTO.getBeneficiaryId()));
			invPmtReconDTO.setPayCounterpartyId(stringConverter(updateInvPmtReconDTO.getPayCounterpartyId()));
			invPmtReconDTO.setPayCounterpartyName(stringConverter(updateInvPmtReconDTO.getPayCounterpartyName()));
			invPmtReconDTO.setRemitterAccNo(stringConverter(updateInvPmtReconDTO.getRemitterAccNo()));
			invPmtReconDTO.setDcIndicator(stringConverter(updateInvPmtReconDTO.getDcIndicator()));
			invPmtReconDTO.setRemarks(stringConverter(updateInvPmtReconDTO.getRemarks()));
			invPmtReconDTO.setPmtProgramCode(stringConverter(updateInvPmtReconDTO.getPmtProgramCode()));
			invPmtReconDTO.setPmtRelationshipCode(stringConverter(updateInvPmtReconDTO.getPmtRelationshipCode()));
			invPmtReconDTO.setInvProgramCode(stringConverter(updateInvPmtReconDTO.getInvProgramCode()));
			invPmtReconDTO.setAccOwner(stringConverter(updateInvPmtReconDTO.getAccOwner()));
			invPmtReconDTO.setIsInvAccVa(stringConverter(updateInvPmtReconDTO.getIsInvAccVa()));
			invPmtReconDTO.setIsPmtAccVA(stringConverter(updateInvPmtReconDTO.getIsPmtAccVA()));
			invPmtReconDTO.setPartyPaymentCode(stringConverter(updateInvPmtReconDTO.getPartyPaymentCode()));
			invPmtReconDTO.setCounterpartyCode(stringConverter(updateInvPmtReconDTO.getCounterpartyCode()));
			invPmtReconDTO.setAllocationRequired(stringConverter(updateInvPmtReconDTO.getAllocationRequired()));
			invPmtReconDTO.setLinkedDeReconTxnRef(stringConverter(updateInvPmtReconDTO.getLinkedDeReconTxnRef()));
			invPmtReconDTO.setDeReconFlag(stringConverter(updateInvPmtReconDTO.getDeReconFlag()));
			invPmtReconDTO.setExtPmtRefNo(stringConverter(updateInvPmtReconDTO.getExtPmtRefNo()));
			invPmtReconDTO.setExtSysRefNo(stringConverter(updateInvPmtReconDTO.getExtSysRefNo()));
			invPmtReconDTO.setIsIndirectPayment(stringConverter(updateInvPmtReconDTO.getIsIndirectPayment()));
			invPmtReconDTO.setMessageRefId(stringConverter(updateInvPmtReconDTO.getMessageRefId()));
			reconEventServiceClient.saveInvPmtData(invPmtReconDTO, userId, branchCode);
         }
		}
	}
	@StreamListener(ReportConstants.FIN_PMT_RECON_UPDATED)
	public void getFinPmtReconData(Message<UpdateFinPmtReconDTOCollection> message) {

		LOGGER.info("Message Received on channel Finance Payment Recon Updated Event");
		UpdateFinPmtReconDTOCollection updateFinPmtReconDTOCollection = message.getPayload();

		String userId = updateFinPmtReconDTOCollection.getUserId();
		String branchCode = updateFinPmtReconDTOCollection.getServiceBranchCode();

        List<UpdateFinPmtReconDTO> updateFinPmtReconDTOList =  updateFinPmtReconDTOCollection.getUpdateFinPmtReconDTOList();
	    for(UpdateFinPmtReconDTO updateFinPmtReconDTO:updateFinPmtReconDTOList) {
        if(updateFinPmtReconDTO != null) {
			FinPmtReconDTO finPmtReconDTO = new FinPmtReconDTO();
			finPmtReconDTO.setReconId(numberConverter(updateFinPmtReconDTO.getReconId()));
			finPmtReconDTO.setReconTxnRefNo(stringConverter(updateFinPmtReconDTO.getReconTxnRefNo()));
			finPmtReconDTO.setReconCategory(stringConverter(updateFinPmtReconDTO.getReconCategory()));
			finPmtReconDTO.setReconDate(stringConverter(updateFinPmtReconDTO.getReconDate()));
			finPmtReconDTO.setBranchCode(stringConverter(updateFinPmtReconDTO.getBranchCode()));
			finPmtReconDTO.setReconType(stringConverter(updateFinPmtReconDTO.getReconType()));
			finPmtReconDTO.setReconRuleType(stringConverter(updateFinPmtReconDTO.getReconRuleType()));
			finPmtReconDTO.setReconRule(stringConverter(updateFinPmtReconDTO.getReconRule()));
			finPmtReconDTO.setLhsEntity(stringConverter(updateFinPmtReconDTO.getLhsEntity()));
			finPmtReconDTO.setRhsEntity(stringConverter(updateFinPmtReconDTO.getRhsEntity()));
			finPmtReconDTO.setMakerId(stringConverter(updateFinPmtReconDTO.getMakerId()));
			finPmtReconDTO.setMakerDtStamp(stringConverter(updateFinPmtReconDTO.getMakerDtStamp()));
			finPmtReconDTO.setAuthId(stringConverter(updateFinPmtReconDTO.getAuthId()));
			finPmtReconDTO.setAuthDtStamp(stringConverter(updateFinPmtReconDTO.getAuthDtStamp()));
			finPmtReconDTO.setTransactionStatus(stringConverter(updateFinPmtReconDTO.getTransactionStatus()));
			finPmtReconDTO.setReconAction(stringConverter(updateFinPmtReconDTO.getReconAction()));
			finPmtReconDTO.setStage(stringConverter(updateFinPmtReconDTO.getStage()));
			finPmtReconDTO.setMatchSeqNo(stringConverter(updateFinPmtReconDTO.getMatchSeqNo()));
			finPmtReconDTO.setFinBuyerId(stringConverter(updateFinPmtReconDTO.getFinBuyerId()));
			finPmtReconDTO.setFinSupplierId(stringConverter(updateFinPmtReconDTO.getFinSupplierId()));
			finPmtReconDTO.setFinBuyerName(stringConverter(updateFinPmtReconDTO.getFinBuyerName()));
			finPmtReconDTO.setFinSupplierName(stringConverter(updateFinPmtReconDTO.getFinSupplierName()));
			finPmtReconDTO.setFinProgramCode(stringConverter(updateFinPmtReconDTO.getFinProgramCode()));
			finPmtReconDTO.setFinRefNo(stringConverter(updateFinPmtReconDTO.getFinRefNo()));
			finPmtReconDTO.setFinSeqNo(updateFinPmtReconDTO.getFinSeqNo());
			finPmtReconDTO.setFinStartDt(stringConverter(updateFinPmtReconDTO.getFinStartDt()));
			finPmtReconDTO.setFinEndDt(stringConverter(updateFinPmtReconDTO.getFinEndDt()));
			finPmtReconDTO.setFinPastDueDt(stringConverter(updateFinPmtReconDTO.getFinPastDueDt()));
			finPmtReconDTO.setFinCcy(stringConverter(updateFinPmtReconDTO.getFinCcy()));
			finPmtReconDTO.setFinAmt(numberConverter(updateFinPmtReconDTO.getFinAmt()));
			finPmtReconDTO.setFinOsAmt(numberConverter(updateFinPmtReconDTO.getFinOsAmt()));
			finPmtReconDTO.setIntOsAmt(numberConverter(updateFinPmtReconDTO.getIntOsAmt()));
			finPmtReconDTO.setPenIntOsAmt(numberConverter(updateFinPmtReconDTO.getPenIntOsAmt()));
			finPmtReconDTO.setOverdueIntOsAmt(numberConverter(updateFinPmtReconDTO.getOverdueIntOsAmt()));
			finPmtReconDTO.setTotIntOsAmt(numberConverter(updateFinPmtReconDTO.getTotIntOsAmt()));
			finPmtReconDTO.setAmtMatched(numberConverter(updateFinPmtReconDTO.getAmtMatched()));
			finPmtReconDTO.setPaymentRefNo(stringConverter(updateFinPmtReconDTO.getPaymentRefNo()));
			finPmtReconDTO.setPaymentPartyId(stringConverter(updateFinPmtReconDTO.getPaymentPartyId()));
			finPmtReconDTO.setPaymentPartyName(stringConverter(updateFinPmtReconDTO.getPaymentPartyName()));
			finPmtReconDTO.setPaymentDate(stringConverter(updateFinPmtReconDTO.getPaymentDate()));
			finPmtReconDTO.setPaymentMode(stringConverter(updateFinPmtReconDTO.getPaymentMode()));
			finPmtReconDTO.setBeneCode(stringConverter(updateFinPmtReconDTO.getBeneCode()));
			finPmtReconDTO.setBeneName(stringConverter(updateFinPmtReconDTO.getBeneName()));
			finPmtReconDTO.setPaymentAmount(numberConverter(updateFinPmtReconDTO.getPaymentAmount()));
			finPmtReconDTO.setPaymentCcy(stringConverter(updateFinPmtReconDTO.getPaymentCcy()));
			finPmtReconDTO.setExchRate(numberConverter(updateFinPmtReconDTO.getExchRate()));
			finPmtReconDTO.setReconciledPmtAmt(numberConverter(updateFinPmtReconDTO.getReconciledPmtAmt()));
			finPmtReconDTO.setBeneVirtualAccNo(stringConverter(updateFinPmtReconDTO.getBeneVirtualAccNo()));
			finPmtReconDTO.setPmtProgramCode(stringConverter(updateFinPmtReconDTO.getPmtProgramCode()));
			finPmtReconDTO.setPmtStatus(stringConverter(updateFinPmtReconDTO.getPmtStatus()));
			finPmtReconDTO.setReconStatus(stringConverter(updateFinPmtReconDTO.getReconStatus()));
			finPmtReconDTO.setFinPmtReconId(numberConverter(updateFinPmtReconDTO.getFinPmtReconId()));

			reconEventServiceClient.saveFinPmtData(finPmtReconDTO, userId, branchCode);
          }
		}
	}

	@StreamListener(ReportConstants.PMT_ALLOC_UPDATED)
	public void getPaymentAllocationData(Message<UpdatePmtAllocDTOCollection> message) {

		LOGGER.info("Message Received on channel Payment Allocation Updated Event");
		UpdatePmtAllocDTOCollection updatePmtAllocDTOCollection = message.getPayload();

		String userId = updatePmtAllocDTOCollection.getUserId();
		String branchCode = updatePmtAllocDTOCollection.getServiceBranchCode();

		List<UpdatePmtAllocDTO> updatePmtAllocDTOList = updatePmtAllocDTOCollection.getUpdatePmtAllocDTOList();
		for (UpdatePmtAllocDTO updatePmtAllocDTO : updatePmtAllocDTOList) {
			if (updatePmtAllocDTO != null) {
				PmtAllocDTO pmtAllocDTO = new PmtAllocDTO();
				pmtAllocDTO.setReconId(numberConverter(updatePmtAllocDTO.getReconId()));
				pmtAllocDTO.setReconTxnRefNo(stringConverter(updatePmtAllocDTO.getReconTxnRefNo()));
				pmtAllocDTO.setReconCategory(stringConverter(updatePmtAllocDTO.getReconCategory()));
				pmtAllocDTO.setReconDate(stringConverter(updatePmtAllocDTO.getReconDate()));
				pmtAllocDTO.setBranchCode(stringConverter(updatePmtAllocDTO.getBranchCode()));
				pmtAllocDTO.setReconType(stringConverter(updatePmtAllocDTO.getReconType()));
				pmtAllocDTO.setReconRuleType(stringConverter(updatePmtAllocDTO.getReconRuleType()));
				pmtAllocDTO.setReconRule(stringConverter(updatePmtAllocDTO.getReconRule()));
				pmtAllocDTO.setLhsEntity(stringConverter(updatePmtAllocDTO.getLhsEntity()));
				pmtAllocDTO.setRhsEntity(stringConverter(updatePmtAllocDTO.getRhsEntity()));
				pmtAllocDTO.setMakerId(stringConverter(updatePmtAllocDTO.getMakerId()));
				pmtAllocDTO.setMakerDtStamp(stringConverter(updatePmtAllocDTO.getMakerDtStamp()));
				pmtAllocDTO.setAuthId(stringConverter(updatePmtAllocDTO.getAuthId()));
				pmtAllocDTO.setAuthDtStamp(stringConverter(updatePmtAllocDTO.getAuthDtStamp()));
				pmtAllocDTO.setTransactionStatus(stringConverter(updatePmtAllocDTO.getTransactionStatus()));
				pmtAllocDTO.setReconAction(stringConverter(updatePmtAllocDTO.getReconAction()));
				pmtAllocDTO.setStage(stringConverter(updatePmtAllocDTO.getStage()));
				pmtAllocDTO.setMatchSeqNo(stringConverter(updatePmtAllocDTO.getMatchSeqNo()));
				pmtAllocDTO.setAccOwner(stringConverter(updatePmtAllocDTO.getAccOwner()));
				pmtAllocDTO.setAllocatedAmt(numberConverter(updatePmtAllocDTO.getAllocatedAmt()));
				pmtAllocDTO.setAllocationPct(numberConverter(updatePmtAllocDTO.getAllocationPct()));
				pmtAllocDTO.setAllocAttributes(stringConverter(updatePmtAllocDTO.getAllocAttributes()));
				pmtAllocDTO.setAllocBasis(stringConverter(updatePmtAllocDTO.getAllocBasis()));
				pmtAllocDTO.setAllocEntity(stringConverter(updatePmtAllocDTO.getAllocEntity()));
				pmtAllocDTO.setAllocStatus(stringConverter(updatePmtAllocDTO.getAllocStatus()));
				pmtAllocDTO.setAllocVirtualAccNo(stringConverter(updatePmtAllocDTO.getAllocVirtualAccNo()));
				pmtAllocDTO.setBeneCode(stringConverter(updatePmtAllocDTO.getBeneCode()));
				pmtAllocDTO.setBeneficiaryCode(stringConverter(updatePmtAllocDTO.getBeneficiaryCode()));
				pmtAllocDTO.setBeneficiaryId(stringConverter(updatePmtAllocDTO.getBeneficiaryId()));
				pmtAllocDTO.setBeneficiaryName(stringConverter(updatePmtAllocDTO.getBeneficiaryName()));
				pmtAllocDTO.setBeneVirtualAccNo(stringConverter(updatePmtAllocDTO.getBeneVirtualAccNo()));
				pmtAllocDTO.setCorporateId(stringConverter(updatePmtAllocDTO.getCorporateId()));
				pmtAllocDTO.setCorporateName(stringConverter(updatePmtAllocDTO.getCorporateName()));
				pmtAllocDTO.setCounterpartyCode(stringConverter(updatePmtAllocDTO.getCounterpartyCode()));
				pmtAllocDTO.setCounterpartyId(stringConverter(updatePmtAllocDTO.getCounterpartyId()));
				pmtAllocDTO.setCounterpartyName(stringConverter(updatePmtAllocDTO.getCounterpartyName()));
				pmtAllocDTO.setCreditAccNo(stringConverter(updatePmtAllocDTO.getCreditAccNo()));
				pmtAllocDTO.setDcIndicator(stringConverter(updatePmtAllocDTO.getDcIndicator()));
				pmtAllocDTO.setExtPmtRefNo(stringConverter(updatePmtAllocDTO.getExtPmtRefNo()));
				pmtAllocDTO.setIsPmtAccVA(stringConverter(updatePmtAllocDTO.getIsPmtAccVA()));
				pmtAllocDTO.setObdxRefNo(stringConverter(updatePmtAllocDTO.getObdxRefNo()));
				pmtAllocDTO.setObvamRefNo(stringConverter(updatePmtAllocDTO.getObvamRefNo()));
				pmtAllocDTO.setPartyPaymentCode(stringConverter(updatePmtAllocDTO.getPartyPaymentCode()));
				pmtAllocDTO.setPaymentAmount(numberConverter(updatePmtAllocDTO.getPaymentAmount()));
				pmtAllocDTO.setPaymentCcy(stringConverter(updatePmtAllocDTO.getPaymentCcy()));
				pmtAllocDTO.setPaymentDate(stringConverter(updatePmtAllocDTO.getPaymentDate()));
				pmtAllocDTO.setPaymentMode(stringConverter(updatePmtAllocDTO.getPaymentMode()));
				pmtAllocDTO.setPaymentPartyId(stringConverter(updatePmtAllocDTO.getPaymentPartyId()));
				pmtAllocDTO.setPaymentPartyName(stringConverter(updatePmtAllocDTO.getPaymentPartyName()));
				pmtAllocDTO.setPaymentRefNo(stringConverter(updatePmtAllocDTO.getPaymentRefNo()));
				pmtAllocDTO.setPmtAllocatedAmt(numberConverter(updatePmtAllocDTO.getPmtAllocatedAmt()));
				pmtAllocDTO.setPmtAllocationStatus(stringConverter(updatePmtAllocDTO.getPmtAllocationStatus()));
				pmtAllocDTO.setReconAction(stringConverter(updatePmtAllocDTO.getReconAction()));
				pmtAllocDTO.setReconciledPmtAmt(numberConverter(updatePmtAllocDTO.getReconciledPmtAmt()));
				pmtAllocDTO.setRemitterAccNo(stringConverter(updatePmtAllocDTO.getRemitterAccNo()));
				pmtAllocDTO.setReconAllocPmtId(numberConverter(updatePmtAllocDTO.getReconAllocPmtId()));
				pmtAllocDTO.setProgramCode(stringConverter(updatePmtAllocDTO.getProgramCode()));
				pmtAllocDTO.setRelationshipCode(stringConverter(updatePmtAllocDTO.getRelationshipCode()));
				
				reconEventServiceClient.savePmtAllocData(pmtAllocDTO, userId, branchCode);
			}
		}
	}

	@StreamListener(ReportConstants.PAYMENT_UPDATED)
	public void getPaymentData(Message<UpdatePaymentDto> message) {

		LOGGER.info("Message Received on channel Payment Update Event");
		UpdatePaymentDto updatePaymentDto = message.getPayload();
		String paymentRefNo = updatePaymentDto.getPaymentRefNo();
		String userId = updatePaymentDto.getUserId();
		String branchCode = updatePaymentDto.getServiceBranchCode();
		LOGGER.info("Payment Update paymentRefNo :: {}",paymentRefNo);
		eventListenerService.fetchPaymentData(paymentRefNo, userId, branchCode);
	}
	
	@StreamListener(ReportConstants.PAYMENT_TW_UPDATED)
	public void getPaymentTwData(Message<UpdatePaymentDto> message) {

		LOGGER.info("Message Received on channel Payment Update Tw Event");
		UpdatePaymentDto updatePaymentDto = message.getPayload();

		String userId = updatePaymentDto.getUserId();
		String branchCode = updatePaymentDto.getServiceBranchCode();
		String paymentRefNo = updatePaymentDto.getPaymentRefNo();
		LOGGER.info("Payment Update Tw paymentRefNo :: {}",paymentRefNo);
		PaymentCollectionDto paymentCollectionDto  = instrumentReceivablesServiceClient.getPaymentTwByPayRefNo(paymentRefNo, userId, branchCode);
		LOGGER.info("Payment Update Tw paymentCollectionDto :: {}",paymentCollectionDto);
		if (paymentCollectionDto != null) {
			List<PaymentDto> payTwList = paymentCollectionDto.getData();
			if (payTwList != null && !payTwList.isEmpty()) {
				 payTwList.forEach(payDto -> {
				 eventListenerService.savePaymentTWData(payDto, userId, branchCode);
				});
			}
		}
	 
	}
	
	@StreamListener(ReportConstants.INV_CN_RECON_UPDATED)
	public void getInvCnData(Message<UpdateInvCnReconDTOCollection> message) {
		LOGGER.info("Message Received on channel Invoice Credit Note Update Event");
		UpdateInvCnReconDTOCollection updateInvCnReconDTOCollection = message.getPayload();

		String userId = updateInvCnReconDTOCollection.getUserId();
		String branchCode = updateInvCnReconDTOCollection.getServiceBranchCode();

		List<UpdateInvCnReconDTO> updateInvCnReconDTOList =  updateInvCnReconDTOCollection.getUpdateInvCnReconDTOList();
		for(UpdateInvCnReconDTO updateInvCnReconDTO : updateInvCnReconDTOList) {
			if(updateInvCnReconDTO != null) {
				InvCnReconDTO invCnReconDTO = new InvCnReconDTO();
				LOGGER.info("getInvCnData updateInvCnReconDTO {}",updateInvCnReconDTO);
				invCnReconDTO.setReconId(numberConverter(updateInvCnReconDTO.getReconId()));
				invCnReconDTO.setInvCnReconId(numberConverter(updateInvCnReconDTO.getInvCnReconId()));
				invCnReconDTO.setCnBaseAmount(numberConverter(updateInvCnReconDTO.getCnBaseAmount()));
				invCnReconDTO.setNetCnAmount(numberConverter(updateInvCnReconDTO.getNetCnAmount()));
				invCnReconDTO.setInvOsAmt(numberConverter(updateInvCnReconDTO.getInvOsAmt()));
				invCnReconDTO.setNetInvAmt(numberConverter(updateInvCnReconDTO.getNetInvAmt()));
				invCnReconDTO.setVarianceAmount(numberConverter(updateInvCnReconDTO.getVarianceAmount()));
				invCnReconDTO.setAmtMatched(numberConverter(updateInvCnReconDTO.getAmtMatched()));
				invCnReconDTO.setExchRate(numberConverter(updateInvCnReconDTO.getExchRate()));
				invCnReconDTO.setReconciledCnAmt(numberConverter(updateInvCnReconDTO.getReconciledCnAmt()));

				invCnReconDTO.setInsuranceCoName(stringConverter(updateInvCnReconDTO.getInsuranceCoName()));
				invCnReconDTO.setInvVirtualAccNo(stringConverter(updateInvCnReconDTO.getInvVirtualAccNo()));
				invCnReconDTO.setImportFactorId(stringConverter(updateInvCnReconDTO.getImportFactorId()));
				invCnReconDTO.setImportFactorName(stringConverter(updateInvCnReconDTO.getImportFactorName()));
				invCnReconDTO.setInvProgramCode(stringConverter(updateInvCnReconDTO.getInvProgramCode()));
				invCnReconDTO.setLinkedDeReconTxnRef(stringConverter(updateInvCnReconDTO.getLinkedDeReconTxnRef()));
				invCnReconDTO.setProcBranchDate(stringConverter(updateInvCnReconDTO.getProcBranchDate()));
				invCnReconDTO.setCnBuyerDivCode(stringConverter(updateInvCnReconDTO.getCnBuyerDivCode()));
				invCnReconDTO.setTransactionStatus(stringConverter(updateInvCnReconDTO.getTransactionStatus()));
				invCnReconDTO.setCnSupplierCode(stringConverter(updateInvCnReconDTO.getCnSupplierCode()));
				invCnReconDTO.setCnSupplierDivCode(stringConverter(updateInvCnReconDTO.getCnSupplierDivCode()));
				invCnReconDTO.setCnSupplierName(stringConverter(updateInvCnReconDTO.getCnSupplierName()));
				invCnReconDTO.setInvRelationCode(stringConverter(updateInvCnReconDTO.getInvRelationCode()));
				invCnReconDTO.setInvSupplierName(stringConverter(updateInvCnReconDTO.getInvSupplierName()));
				invCnReconDTO.setProcBranchCode(stringConverter(updateInvCnReconDTO.getProcBranchCode()));
				invCnReconDTO.setFiller1(stringConverter(updateInvCnReconDTO.getFiller1()));
				invCnReconDTO.setCnSupplierId(stringConverter(updateInvCnReconDTO.getCnSupplierId()));
				invCnReconDTO.setFiller2(stringConverter(updateInvCnReconDTO.getFiller2()));
				invCnReconDTO.setCnBuyerId(stringConverter(updateInvCnReconDTO.getCnBuyerId()));
				invCnReconDTO.setCnProgramCode(stringConverter(updateInvCnReconDTO.getCnProgramCode()));
				invCnReconDTO.setFiller3(stringConverter(updateInvCnReconDTO.getFiller3()));
				invCnReconDTO.setMakerDtStamp(stringConverter(updateInvCnReconDTO.getMakerDtStamp()));
				invCnReconDTO.setLhsEntity(stringConverter(updateInvCnReconDTO.getLhsEntity()));
				invCnReconDTO.setRemarks(stringConverter(updateInvCnReconDTO.getRemarks()));
				invCnReconDTO.setCnBuyerCode(stringConverter(updateInvCnReconDTO.getCnBuyerCode()));
				invCnReconDTO.setFiller4(stringConverter(updateInvCnReconDTO.getFiller4()));
				invCnReconDTO.setFiller5(stringConverter(updateInvCnReconDTO.getFiller5()));
				invCnReconDTO.setAuthDtStamp(stringConverter(updateInvCnReconDTO.getAuthDtStamp()));
				invCnReconDTO.setCnBuyerName(stringConverter(updateInvCnReconDTO.getCnBuyerName()));
				invCnReconDTO.setMakerId(stringConverter(updateInvCnReconDTO.getMakerId()));
				invCnReconDTO.setCnInternalId(stringConverter(updateInvCnReconDTO.getCnInternalId()));
				invCnReconDTO.setFiller6(stringConverter(updateInvCnReconDTO.getFiller6()));
				invCnReconDTO.setCnReconStatus(stringConverter(updateInvCnReconDTO.getCnReconStatus()));
				invCnReconDTO.setRhsEntity(stringConverter(updateInvCnReconDTO.getRhsEntity()));
				invCnReconDTO.setStage(stringConverter(updateInvCnReconDTO.getStage()));
				invCnReconDTO.setCnExpiryDate(stringConverter(updateInvCnReconDTO.getCnExpiryDate()));
				invCnReconDTO.setCnNumber(stringConverter(updateInvCnReconDTO.getCnNumber()));
				invCnReconDTO.setReconcileBy(stringConverter(updateInvCnReconDTO.getReconcileBy()));
				invCnReconDTO.setCnCurrecny(stringConverter(updateInvCnReconDTO.getCnCurrecny()));
				invCnReconDTO.setCnDate(stringConverter(updateInvCnReconDTO.getCnDate()));
				invCnReconDTO.setCorporateId(stringConverter(updateInvCnReconDTO.getCorporateId()));
				invCnReconDTO.setMatchSeqNo(stringConverter(updateInvCnReconDTO.getMatchSeqNo()));
				invCnReconDTO.setReconAction(stringConverter(updateInvCnReconDTO.getReconAction()));
				invCnReconDTO.setAuthId(stringConverter(updateInvCnReconDTO.getAuthId()));
				invCnReconDTO.setInvDate(stringConverter(updateInvCnReconDTO.getInvDate()));
				invCnReconDTO.setIntInvRefNo(stringConverter(updateInvCnReconDTO.getIntInvRefNo()));
				invCnReconDTO.setInsuranceCoId(stringConverter(updateInvCnReconDTO.getInsuranceCoId()));
				invCnReconDTO.setInvSupplierId(stringConverter(updateInvCnReconDTO.getInvSupplierId()));
				invCnReconDTO.setExtSysRefNo(stringConverter(updateInvCnReconDTO.getExtSysRefNo()));
				invCnReconDTO.setFiller7(stringConverter(updateInvCnReconDTO.getFiller7()));
				invCnReconDTO.setInvDueDate(stringConverter(updateInvCnReconDTO.getInvDueDate()));
				invCnReconDTO.setExtInvRefNo(stringConverter(updateInvCnReconDTO.getExtInvRefNo()));
				invCnReconDTO.setFiller8(stringConverter(updateInvCnReconDTO.getFiller8()));
				invCnReconDTO.setInvCcy(stringConverter(updateInvCnReconDTO.getInvCcy()));
				invCnReconDTO.setInvBuyerId(stringConverter(updateInvCnReconDTO.getInvBuyerId()));
				invCnReconDTO.setAccOwner(stringConverter(updateInvCnReconDTO.getAccOwner()));
				invCnReconDTO.setFiller9(stringConverter(updateInvCnReconDTO.getFiller9()));
				invCnReconDTO.setIsInvAccVa(stringConverter(updateInvCnReconDTO.getIsInvAccVa()));
				invCnReconDTO.setDeReconFlag(stringConverter(updateInvCnReconDTO.getDeReconFlag()));
				invCnReconDTO.setInvBuyerName(stringConverter(updateInvCnReconDTO.getInvBuyerName()));
				invCnReconDTO.setInvFinanced(stringConverter(updateInvCnReconDTO.getInvFinanced()));
				invCnReconDTO.setAccountNumber(stringConverter(updateInvCnReconDTO.getAccountNumber()));
				invCnReconDTO.setFiller10(stringConverter(updateInvCnReconDTO.getFiller10()));
				invCnReconDTO.setIsCnAccVa(stringConverter(updateInvCnReconDTO.getIsCnAccVa()));
				invCnReconDTO.setReconStatus(stringConverter(updateInvCnReconDTO.getReconStatus()));
				invCnReconDTO.setRemitterAccNo(stringConverter(updateInvCnReconDTO.getRemitterAccNo()));
				invCnReconDTO.setReconCategory(stringConverter(updateInvCnReconDTO.getReconCategory()));
				invCnReconDTO.setReconDate(stringConverter(updateInvCnReconDTO.getReconDate()));
				invCnReconDTO.setBranchCode(stringConverter(updateInvCnReconDTO.getBranchCode()));
				invCnReconDTO.setReconTxnRefNo(stringConverter(updateInvCnReconDTO.getReconTxnRefNo()));
				invCnReconDTO.setReconType(stringConverter(updateInvCnReconDTO.getReconType()));
				invCnReconDTO.setReconRuleType(stringConverter(updateInvCnReconDTO.getReconRuleType()));
				invCnReconDTO.setReconRule(stringConverter(updateInvCnReconDTO.getReconRule()));
				invCnReconDTO.setCnProgramCode(stringConverter(updateInvCnReconDTO.getCnProgramCode()));
				invCnReconDTO.setCnRelationCode(stringConverter(updateInvCnReconDTO.getCnRelationCode()));
				invCnReconDTO.setIsIndirectPayment(stringConverter(updateInvCnReconDTO.getIsIndirectPayment()));
				invCnReconDTO.setMessageRefId(stringConverter(updateInvCnReconDTO.getMessageRefId()));
				LOGGER.info("invCnReconDTO {}",invCnReconDTO);
				reconEventServiceClient.saveInvCnData(invCnReconDTO, userId, branchCode);
			}
		}

	}

	@StreamListener(ReportConstants.INVOICE_TW_UPDATED_EVENT)
	public void getInvoiceTwData(Message<UpdateInvoiceTwDto> message) {
		LOGGER.info("Message Received on channel InvoiceTwUpdate Event.");
		UpdateInvoiceTwDto updateInvoiceTwDto = message.getPayload();
		String transactionRefNo = updateInvoiceTwDto.getTransactionReferenceNo();
		String instrumentType = updateInvoiceTwDto.getInstrumentType();
		LOGGER.info("instrumentType {} .",instrumentType);
		LOGGER.info("branchCode {} .",updateInvoiceTwDto.getBranchCode());
		LOGGER.info("transactionReferenceNo {} .",updateInvoiceTwDto.getTransactionReferenceNo());
		LOGGER.info("currentStageCode {} .",updateInvoiceTwDto.getCurrentStageCode());
		LOGGER.info("currentStageName {} .",updateInvoiceTwDto.getCurrentStageName());
		LOGGER.info("userId {} .",updateInvoiceTwDto.getUserId());
		if( instrumentType==null ||ReportConstants.RECEIVABLE_TYPE_INVOICE.equals(instrumentType)){
			InvoiceCollectionDto invoiceCollectionDto = instrumentReceivablesServiceClient.getInvoiceByTxnRefNo(transactionRefNo, updateInvoiceTwDto.getUserId(), updateInvoiceTwDto.getBranchCode());
			if (invoiceCollectionDto != null) {
				List<InvoiceModel> invoiceTwList = invoiceCollectionDto.getData();
				if (invoiceTwList != null && !invoiceTwList.isEmpty()) {
					invoiceTwList.forEach(invoiceModel -> {
						eventListenerService.saveInvoiceTwData(invoiceModel, updateInvoiceTwDto.getUserId(), updateInvoiceTwDto.getBranchCode());
					});
				}
			}
		} else if(ReportConstants.RECEIVABLE_TYPE_PO.equals(instrumentType)){
			LOGGER.info("PO Update ....");
			PurchaseOrderCollection purchaseOrderCollection = instrumentReceivablesServiceClient.getPoByTransactionRefNo(transactionRefNo, false, updateInvoiceTwDto.getUserId(), updateInvoiceTwDto.getBranchCode());
			if (purchaseOrderCollection != null) {
				List<PurchaseOrderDTO> purchaseOrderDTOList = purchaseOrderCollection.getData();
				if (purchaseOrderDTOList != null && !purchaseOrderDTOList.isEmpty()) {
					purchaseOrderDTOList.forEach(poDto -> {
						eventListenerService.savePoTwData(poDto,updateInvoiceTwDto.getUserId(), updateInvoiceTwDto.getBranchCode());
					});
				}
			}

		}
		else{
			LOGGER.info("Invalid Receivable Type..");
		}
	}
	@StreamListener(ReportConstants.FILE_UPDATED)
	public void getFileData(Message<UpdateFileUploadDto> message) {

		LOGGER.info("Message Received on channel File Update Event");
		UpdateFileUploadDto updateFileUploadDto = message.getPayload();


		if(updateFileUploadDto != null) {
			String userId = updateFileUploadDto.getUserId();
			String branchCode = updateFileUploadDto.getServiceBranchCode();
			FileUploadDto fileUploadDto = new FileUploadDto();


			fileUploadDto.setFileName(stringConverter(updateFileUploadDto.getFileName()));
			fileUploadDto.setUploadDateAndTime(stringConverter(updateFileUploadDto.getUploadDateAndTime()));
			fileUploadDto.setCount(numberConverter(updateFileUploadDto.getCount()));
			fileUploadDto.setTotalFileUploadValue(stringConverter(updateFileUploadDto.getTotalFileUploadValue()));
			fileUploadDto.setEventType(stringConverter(updateFileUploadDto.getEventType()));
			eventServiceClient.saveUploadedFileData(fileUploadDto, userId, branchCode);
		}
	}

	private BigDecimal numberConverter(String inputNumber) {

		if("null".equalsIgnoreCase(inputNumber)) {
			return null;
		}
		return new BigDecimal(inputNumber);
	}

	private String stringConverter(String inputString) {

		if("null".equalsIgnoreCase(inputString)) {
			return null;
		}
		return inputString;
	}

	@StreamListener(ReportConstants.CREDIT_NOTES_UPDATED)
	public void getCreditNoteData(Message<UpdateCreditNotesDto> message) {

		LOGGER.info("Message Received on channel CreditNote Update Event");
		UpdateCreditNotesDto updateCreditNotesDto = message.getPayload();
		LOGGER.info("Received updateCreditNotesDto: {}", updateCreditNotesDto.toString());

		String userId = updateCreditNotesDto.getUserId();
		String branchCode = updateCreditNotesDto.getServiceBranchCode();

		if (updateCreditNotesDto != null) {
			CreditNoteModel creditNoteDTO = new CreditNoteModel();

			creditNoteDTO.setCreditNoteRefNo(stringConverter(updateCreditNotesDto.getCreditNoteRefNo()));
			creditNoteDTO.setFiller1(stringConverter(updateCreditNotesDto.getFiller1()));
			creditNoteDTO.setFiller2(stringConverter(updateCreditNotesDto.getFiller2()));
			creditNoteDTO.setFiller3(stringConverter(updateCreditNotesDto.getFiller3()));
			creditNoteDTO.setFiller4(stringConverter(updateCreditNotesDto.getFiller4()));
			creditNoteDTO.setFiller5(stringConverter(updateCreditNotesDto.getFiller5()));
			creditNoteDTO.setFiller6(stringConverter(updateCreditNotesDto.getFiller6()));
			creditNoteDTO.setFiller7(stringConverter(updateCreditNotesDto.getFiller7()));
			creditNoteDTO.setFiller8(stringConverter(updateCreditNotesDto.getFiller8()));
			creditNoteDTO.setFiller9(stringConverter(updateCreditNotesDto.getFiller9()));
			creditNoteDTO.setProcBranchCode(stringConverter(updateCreditNotesDto.getProcBranchCode()));
			creditNoteDTO.setProcBranchDate(stringConverter(updateCreditNotesDto.getProcBranchDate()));
			creditNoteDTO.setProgramCode(stringConverter(updateCreditNotesDto.getProgramCode()));
			creditNoteDTO.setReasonCode(stringConverter(updateCreditNotesDto.getReasonCode()));
			creditNoteDTO.setRemarks(stringConverter(updateCreditNotesDto.getRemarks()));
			creditNoteDTO.setSeqNo(Long.parseLong(updateCreditNotesDto.getSeqNo()));
			creditNoteDTO.setTransactionRefNo(stringConverter(updateCreditNotesDto.getTransactionRefNo()));
			creditNoteDTO.setUnmatchedAmount(numberConverter(updateCreditNotesDto.getUnmatchedAmount()));

			creditNoteDTO.setCreditNoteId(stringConverter(updateCreditNotesDto.getCreditNoteId()));
			creditNoteDTO.setCnNumber(stringConverter(updateCreditNotesDto.getCnNumber()));
			creditNoteDTO.setCnDate(stringConverter(updateCreditNotesDto.getCnDate()));
			creditNoteDTO.setCnExpiryDate(stringConverter(updateCreditNotesDto.getCnExpiryDate()));
			creditNoteDTO.setSupplierId(stringConverter(updateCreditNotesDto.getSupplierId()));
			creditNoteDTO.setBuyerId(stringConverter(updateCreditNotesDto.getBuyerId()));
			creditNoteDTO.setSupplierName(stringConverter(updateCreditNotesDto.getSupplierName()));
			creditNoteDTO.setSupplierDivCode(stringConverter(updateCreditNotesDto.getSupplierDivCode()));
			creditNoteDTO.setSupplierDivName(stringConverter(updateCreditNotesDto.getSupplierDivName()));
			creditNoteDTO.setBuyerName(stringConverter(updateCreditNotesDto.getBuyerName()));
			creditNoteDTO.setBuyerDivCode(stringConverter(updateCreditNotesDto.getBuyerDivCode()));
			creditNoteDTO.setBuyerDivName(stringConverter(updateCreditNotesDto.getBuyerDivName()));
			creditNoteDTO.setRelationCode(stringConverter(updateCreditNotesDto.getRelationCode()));
			creditNoteDTO.setProgramName(stringConverter(updateCreditNotesDto.getProgramName()));
			creditNoteDTO.setAdjustmentReason(stringConverter(updateCreditNotesDto.getAdjustmentReason()));
			creditNoteDTO.setCurrency(stringConverter(updateCreditNotesDto.getCurrency()));
			creditNoteDTO.setTotalAmount(numberConverter(updateCreditNotesDto.getTotalAmount()));
			creditNoteDTO.setTaxAmount(numberConverter(updateCreditNotesDto.getTaxAmount()));
			creditNoteDTO.setDiscountPercentage(numberConverter(updateCreditNotesDto.getDiscountPercentage()));
			creditNoteDTO.setDiscountAmount(numberConverter(updateCreditNotesDto.getDiscountAmount()));
			creditNoteDTO.setTaxPercentage(numberConverter(updateCreditNotesDto.getTaxPercentage()));
			creditNoteDTO.setFileName(stringConverter(updateCreditNotesDto.getFileName()));
			creditNoteDTO.setFiller(stringConverter(updateCreditNotesDto.getFiller()));
			creditNoteDTO.setCnStatus(stringConverter(updateCreditNotesDto.getCnStatus()));
			creditNoteDTO.setCnReconcilationStatus(stringConverter(updateCreditNotesDto.getCnReconcilationStatus()));
			creditNoteDTO.setBuyerCode(stringConverter(updateCreditNotesDto.getBuyerCode()));
			creditNoteDTO.setSupplierCode(stringConverter(updateCreditNotesDto.getSupplierCode()));
			creditNoteDTO.setSourceTxnId(stringConverter(updateCreditNotesDto.getSourceTxnId()));
			creditNoteDTO.setMiscCharge1Amount(numberConverter(updateCreditNotesDto.getMiscCharge1Amount()));
			creditNoteDTO.setMiscCharge1Desc(stringConverter(updateCreditNotesDto.getMiscCharge1Desc()));
			creditNoteDTO.setMiscCharge2Amount(numberConverter(updateCreditNotesDto.getMiscCharge2Amount()));
			creditNoteDTO.setMiscCharge2Desc(stringConverter(updateCreditNotesDto.getMiscCharge2Desc()));
			creditNoteDTO.setMatchedAmount(numberConverter(updateCreditNotesDto.getMatchedAmount()));
			creditNoteDTO.setCnReconcilationStatus(stringConverter(updateCreditNotesDto.getCnReconcilationStatus()));
			creditNoteDTO.setAmount(numberConverter(updateCreditNotesDto.getAmount()));
			creditNoteDTO.setNetCnAmount(numberConverter(updateCreditNotesDto.getNetCnAmount()));
			creditNoteDTO.setCnCommodityGridList(populateCommodityGridList(updateCreditNotesDto.getUpdateCnCommodityGridDto()));
			creditNoteDTO.setLinkedInvoices(populateLinkedInvoices(updateCreditNotesDto.getUpdateDrCrNoteLinkInvoiceDto()));
			LOGGER.info("Saving creditNoteDTO:  {}", creditNoteDTO.toString());
			eventServiceClient.saveCreditNote(creditNoteDTO, userId, branchCode);
		}
	}

    private List<CNLinkedInvoiceModel> populateLinkedInvoices(List<UpdateDrCrNoteLinkInvoiceDto> updateDrCrNoteLinkInvoiceDto) {
        LOGGER.info("inside populateLinkedInvoices method...");
        List<CNLinkedInvoiceModel> list = new ArrayList<>();
        if (updateDrCrNoteLinkInvoiceDto != null && updateDrCrNoteLinkInvoiceDto.size() > 0) {
            updateDrCrNoteLinkInvoiceDto.stream().forEach(dto -> {
				CNLinkedInvoiceModel model = new CNLinkedInvoiceModel();
				model.setDcLinkId(dto.getDcLinkId());
				model.setSeqNo(Long.parseLong(dto.getSeqNo()));
				model.setInvoiceNumber(dto.getInvoiceNumber());
				model.setIsDnCn(dto.getIsDnCn());
				model.setStatus(dto.getStatus());
				model.setTransactionRefNo(dto.getTransactionRefNo());
				model.setNoteNumber(dto.getNoteNumber());

				list.add(model);
            });
        }
        return list;
    }

    private List<CnCommodityGridModel> populateCommodityGridList(List<UpdateCnCommodityGridDto> updateCnCommodityGridDto) {
        LOGGER.info("inside populateCommodityGridList method...");
        List<CnCommodityGridModel> list = new ArrayList<>();
        if (updateCnCommodityGridDto != null && updateCnCommodityGridDto.size() > 0) {
            updateCnCommodityGridDto.stream().forEach(dto -> {
                CnCommodityGridModel model = new CnCommodityGridModel();
                model.setId(dto.getId());
                model.setSeqNo(Long.parseLong(dto.getSeqNo()));
                model.setCommodityCode(dto.getCommodityCode());
                model.setCommodityName(dto.getCommodityName());
                model.setCurrency(dto.getCurrency());
                model.setCommodityDesc(dto.getCommodityDesc());
                model.setDescriptionOfGoods(dto.getDescriptionOfGoods());
                model.setUnit(dto.getUnit());
                model.setInvoiceNumber(dto.getInvoiceNumber());
                model.setCreditNoteRefNo(dto.getCreditNoteRefNo());
                model.setTransactionRefNo(dto.getTransactionRefNo());
                model.setDiscountAmount(numberConverter(dto.getDiscountAmount()));
                model.setDiscountPercentage(numberConverter(dto.getDiscountPercentage()));
                model.setTaxAmount(numberConverter(dto.getTaxAmount()));
                model.setTaxPercentage(numberConverter(dto.getTaxPercentage()));
                model.setCostPerUnit(numberConverter(dto.getCostPerUnit()));
                model.setTotalCost(numberConverter(dto.getTotalCost()));
                model.setNetCost(numberConverter(dto.getNetCost()));
                model.setQuantity(numberConverter(dto.getQuantity()));
                model.setUnitWeight(numberConverter(dto.getUnitWeight()));
                model.setTotalWeight(numberConverter(dto.getTotalWeight()));

                list.add(model);
            });
        }
        return list;
    }
    
	
	@StreamListener(ReportConstants.RECON_TW_UPDATED) 
	public void getReconTwData(Message<UpdateReconTwDto> message)  {
		LOGGER.info("Inside getReconTwData method...");
		UpdateReconTwDto updateReconTwDto = message.getPayload();
		try {
			ReconDataCollection reconDataCollection = reconEventServiceClient.getReconTwData(updateReconTwDto.getReconTxnRefNo(), updateReconTwDto.getUserId(), updateReconTwDto.getBranchCode());
			if (reconDataCollection != null) {
				List<ReconTxnDto> reconTxnList = reconDataCollection.getData();
				if (reconTxnList != null && !reconTxnList.isEmpty()) {
					ReconTxnDto reconTxnDto = reconTxnList.get(0);
					reconEventListenerService.saveReconTwData(reconTxnDto, updateReconTwDto.getUserId(), updateReconTwDto.getBranchCode());
				}
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getReconTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ROLE_TW_UPDATED) 
	public void getAccountRoleTwData(Message<UpdateAccountRoleTWDto> message)  {
		LOGGER.info("Inside getAccountRoleTwData method...");
		UpdateAccountRoleTWDto updateAccountRoleTWDto = message.getPayload();
		try {
			RoleWebModel accRoleWebModel = accountMaintenanceServiceClient.getAccountRoleTWData(updateAccountRoleTWDto.getTxnRefNo(), updateAccountRoleTWDto.getUserId(), updateAccountRoleTWDto.getServiceBranchCode());
			if (accRoleWebModel != null) {
				accountMaintenanceListenerService.saveAccountRoleTWData(accRoleWebModel, updateAccountRoleTWDto.getUserId(), updateAccountRoleTWDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountRoleTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ROLE_UPDATED) 
	public void getAccountRoleTmData(Message<UpdateAccountRoleDto> message)  {
		LOGGER.info("Inside getAccountRoleTmData method...");
		UpdateAccountRoleDto updateAccountRoleDto = message.getPayload();
		try {
			RoleWebModel accRoleWebModel = accountMaintenanceServiceClient.getAccountRoleTMData(updateAccountRoleDto.getTxnRefNo(), updateAccountRoleDto.getUserId(), updateAccountRoleDto.getServiceBranchCode());
			if (accRoleWebModel != null) {
				accountMaintenanceListenerService.saveAccountRoleTMData(accRoleWebModel, updateAccountRoleDto.getUserId(), updateAccountRoleDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountRoleTmData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ENTRY_TW_UPDATED) 
	public void getAccountEntryTwData(Message<UpdateAccountEntryTWDto> message)  {
		LOGGER.info("Inside getAccountEntryTwData method...");
		UpdateAccountEntryTWDto updateAccountEntryTWDto = message.getPayload();
		try {
			AccentryWebModel accentryWebModel = accountMaintenanceServiceClient.getAccountEntryTWData(updateAccountEntryTWDto.getTxnRefNo(), updateAccountEntryTWDto.getUserId(), updateAccountEntryTWDto.getServiceBranchCode());
			if (accentryWebModel != null) {
				accountMaintenanceListenerService.saveAccountEntryTWData(accentryWebModel, updateAccountEntryTWDto.getUserId(), updateAccountEntryTWDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountEntryTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ENTRY_UPDATED) 
	public void getAccountEntryTmData(Message<UpdateAccountEntryDto> message)  {
		LOGGER.info("Inside getAccountEntryTmData method...");
		UpdateAccountEntryDto updateAccountEntryDto = message.getPayload();
		try {
			AccentryWebModel accentryWebModel = accountMaintenanceServiceClient.getAccountEntryTMData(updateAccountEntryDto.getTxnRefNo(), updateAccountEntryDto.getUserId(), updateAccountEntryDto.getServiceBranchCode());
			if (accentryWebModel != null) {
				accountMaintenanceListenerService.saveAccountEntryTMData(accentryWebModel, updateAccountEntryDto.getUserId(), updateAccountEntryDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountEntryTmData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ENTRY_CODE_TW_UPDATED) 
	public void getAccountEntryCodeTwData(Message<UpdateAccountEntryTWDto> message)  {
		LOGGER.info("Inside getAccountEntryCodeTwData method...");
		UpdateAccountEntryTWDto updateAccountEntryTWDto = message.getPayload();
		try {
			AccentrycodeWebModel accentryCodeWebModel = accountMaintenanceServiceClient.getAccountEntryCodeTWData(updateAccountEntryTWDto.getTxnRefNo(), updateAccountEntryTWDto.getUserId(), updateAccountEntryTWDto.getServiceBranchCode());
			if (accentryCodeWebModel != null) {
				accountMaintenanceListenerService.saveAccountEntryCodeTWData(accentryCodeWebModel, updateAccountEntryTWDto.getUserId(), updateAccountEntryTWDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountEntryCodeTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.ACCOUNT_ENTRY_CODE_UPDATED) 
	public void getAccountEntryCodeTmData(Message<UpdateAccountEntryCodeDto> message)  {
		LOGGER.info("Inside getAccountEntryCodeTmData method...");
		UpdateAccountEntryCodeDto updateAccountEntryCodeDto = message.getPayload();
		try {
			AccentrycodeWebModel accentryCodeWebModel = accountMaintenanceServiceClient.getAccountEntryCodeTMData(updateAccountEntryCodeDto.getTxnRefNo(), updateAccountEntryCodeDto.getUserId(), updateAccountEntryCodeDto.getServiceBranchCode());
			if (accentryCodeWebModel != null) {
				accountMaintenanceListenerService.saveAccountEntryCodeTMData(accentryCodeWebModel, updateAccountEntryCodeDto.getUserId(), updateAccountEntryCodeDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getAccountEntryCodeTmData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.INTERNAL_ACCOUNT_MAP_TW_UPDATED) 
	public void getInternalAccountMapTwData(Message<UpdateInternalAccountMapTWDto> message)  {
		LOGGER.info("Inside getInternalAccountMapTwData method...");
		UpdateInternalAccountMapTWDto updateInternalAccountMapTWDto = message.getPayload();
		try {
			IntaccmapWebModel intaccmapWebModel = accountMaintenanceServiceClient.getInternalAccountMapTWData(updateInternalAccountMapTWDto.getTxnRefNo(), updateInternalAccountMapTWDto.getUserId(), updateInternalAccountMapTWDto.getServiceBranchCode());
			if (intaccmapWebModel != null) {
				accountMaintenanceListenerService.saveInternalAccountMapTWData(intaccmapWebModel, updateInternalAccountMapTWDto.getUserId(), updateInternalAccountMapTWDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getInternalAccountMapTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.INTERNAL_ACCOUNT_MAP_UPDATED) 
	public void getInternalAccountMapTmData(Message<UpdateInternalAccountMapDto> message)  {
		LOGGER.info("Inside getInternalAccountMapTmData method...");
		UpdateInternalAccountMapDto updateInternalAccountMapDto = message.getPayload();
		try {
			IntaccmapWebModel intaccmapWebModel = accountMaintenanceServiceClient.getInternalAccountMapTMData(updateInternalAccountMapDto.getTxnRefNo(), updateInternalAccountMapDto.getUserId(), updateInternalAccountMapDto.getServiceBranchCode());
			if (intaccmapWebModel != null) {
				accountMaintenanceListenerService.saveInternalAccountMapTMData(intaccmapWebModel, updateInternalAccountMapDto.getUserId(), updateInternalAccountMapDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getInternalAccountMapTmData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.EXTERNAL_ACCOUNT_MAP_TW_UPDATED) 
	public void getExternalAccountMapTwData(Message<UpdateExternalAccountMapTWDto> message)  {
		LOGGER.info("Inside getExternalAccountMapTwData method...");
		UpdateExternalAccountMapTWDto updateExternalAccountMapTWDto = message.getPayload();
		try {
			ExternalWebModel externalWebModel = accountMaintenanceServiceClient.getExternalAccountMapTWData(updateExternalAccountMapTWDto.getTxnRefNo(), updateExternalAccountMapTWDto.getUserId(), updateExternalAccountMapTWDto.getServiceBranchCode());
			if (externalWebModel != null) {
				accountMaintenanceListenerService.saveExternalAccountMapTWData(externalWebModel, updateExternalAccountMapTWDto.getUserId(), updateExternalAccountMapTWDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getExternalAccountMapTwData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.EXTERNAL_ACCOUNT_MAP_UPDATED) 
	public void getExternalAccountMapTmData(Message<UpdateExternalAccountMapDto> message)  {
		LOGGER.info("Inside getExternalAccountMapTmData method...");
		UpdateExternalAccountMapDto updateExternalAccountMapDto = message.getPayload();
		try {
			ExternalWebModel externalWebModel = accountMaintenanceServiceClient.getExternalAccountMapTMData(updateExternalAccountMapDto.getTxnRefNo(), updateExternalAccountMapDto.getUserId(), updateExternalAccountMapDto.getServiceBranchCode());
			if (externalWebModel != null) {
				accountMaintenanceListenerService.saveExternalAccountMapTMData(externalWebModel, updateExternalAccountMapDto.getUserId(), updateExternalAccountMapDto.getServiceBranchCode());
			}
		} catch (Exception e) {
			LOGGER.error("Exception in getExternalAccountMapTmData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.AUTO_DEBIT_INSTRUMENT_UPDATED)
    public void getUpdatedAutoDebitInstrument(Message<UpdateAutoDebitInstrumentDto> message) {
		
        LOGGER.info("Message Received on channel getUpdatedAutoDebitInstrument Event");
        if(message != null) {
 
            Set<String> instrumentRefNoSet = new HashSet<>();
 
            // Get Auto Debit Txn data and persist
            UpdateAutoDebitInstrumentDto updateAutoDebitInstrumentDto = message.getPayload();
            AutoDebitTxnCollection autoDebitTxnCollection = instrumentReceivablesServiceClient.getAutoDebitTxns(
            		updateAutoDebitInstrumentDto.getUserId(), updateAutoDebitInstrumentDto.getBranchCode(),
            		updateAutoDebitInstrumentDto.getExecutionId());
            if(autoDebitTxnCollection != null) {
                List<AutoDebitTxnDto> autoDebitTxnList = autoDebitTxnCollection.getData();
                if (CollectionUtils.isNotEmpty(autoDebitTxnList)) {
                    for (AutoDebitTxnDto autoDebitTxnDto: autoDebitTxnList) {
                        eventListenerService.saveAutoDebitData(autoDebitTxnDto);
                        if(CollectionUtils.isNotEmpty(autoDebitTxnDto.getAutoDebitTxnDetails())) {
                            for (AutoDebitTxnDetailsDto autoDebitTxnDetailsDto: autoDebitTxnDto.getAutoDebitTxnDetails()) {
                                instrumentRefNoSet.add(autoDebitTxnDetailsDto.getTxnRefNo());
                            }
                        }
                    }
                }
            }
 
            // Get Accounting data and persist
            if(instrumentRefNoSet.size() > 0) {
                for(String instrumentRefNo: instrumentRefNoSet) {
                    // Get Tw data
                    AccountingTransactionCollection accountingTransactionCollectionTw = accountServiceClient
                            .getAccountingDataByFinRefNo(updateAutoDebitInstrumentDto.getUserId(),
                            		updateAutoDebitInstrumentDto.getBranchCode(), instrumentRefNo,
                                    ReportConstants.UNAUTHORIZED);
                    if(accountingTransactionCollectionTw != null 
                            && accountingTransactionCollectionTw.getData() != null
                            && !accountingTransactionCollectionTw.getData().isEmpty()) {
                        for(AccountingTransactionModel accountingTransactionModel : accountingTransactionCollectionTw.getData()) {
                            eventListenerService.saveAccountsTwData(accountingTransactionModel);
                        }
                    }
                    // Get Tm data
                    AccountingTransactionCollection accountingTransactionCollectionTm = accountServiceClient
                            .getAccountingDataByFinRefNo(updateAutoDebitInstrumentDto.getUserId(),
                            		updateAutoDebitInstrumentDto.getBranchCode(), instrumentRefNo,
                                    ReportConstants.AUTHORIZED);
                    if(accountingTransactionCollectionTm != null 
                            && accountingTransactionCollectionTm.getData() != null
                            && !accountingTransactionCollectionTm.getData().isEmpty()) {
                        for(AccountingTransactionModel accountingTransactionModel : accountingTransactionCollectionTm.getData()) {
                            eventListenerService.saveAccountsTmData(accountingTransactionModel);
                        }
                    }
                }
            }
 
        }
		
//        LOGGER.info("Message Received on channel getUpdatedAutoDebitInstrument Event");
//        if(message != null) {
//            UpdateAutoDebitTxnMasterDto updateAutoDebitTxnMaster = message.getPayload();
//            AutoDebitTxnMasterDto autoDebitTxnMasterDto = new AutoDebitTxnMasterDto();
//            autoDebitTxnMasterDto.setBatchJobExecId(stringConverter(updateAutoDebitTxnMaster.getBatchJobExecId()));
//            autoDebitTxnMasterDto.setTimestamp(stringConverter(updateAutoDebitTxnMaster.getTimestamp()));
//            autoDebitTxnMasterDto.setBranchCode(stringConverter(updateAutoDebitTxnMaster.getBranchCode()));
//            autoDebitTxnMasterDto.setUserId(stringConverter(updateAutoDebitTxnMaster.getUserId()));
//            autoDebitTxnMasterDto.setBusinessDate(stringConverter(updateAutoDebitTxnMaster.getBusinessDate()));
//            autoDebitTxnMasterDto.setStatus(stringConverter(updateAutoDebitTxnMaster.getStatus()));
//            if(updateAutoDebitTxnMaster.getAutoDebitTxnDetailsDTO()!=null
//            && updateAutoDebitTxnMaster.getAutoDebitTxnDetailsDTO().size()>0) {
//                List<AutoDebitTxnDetailsDto> autoDebitTxnDetailsList = new ArrayList<>();
//                for(UpdateAutoDebitTxnDetailsDto updatedAutoDebitTxnDetails: updateAutoDebitTxnMaster.getAutoDebitTxnDetailsDTO()) {
//                    AutoDebitTxnDetailsDto autoDebitTxnDetailsDto = new AutoDebitTxnDetailsDto();
//                    autoDebitTxnDetailsDto.setAutoDebitGrpType(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitGrpType()));
//                    autoDebitTxnDetailsDto.setTxnRefNo(stringConverter(updatedAutoDebitTxnDetails.getTxnRefNo()));
//                    autoDebitTxnDetailsDto.setAutoDebitPartyRole(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitPartyRole()));
//                    autoDebitTxnDetailsDto.setAutoDebitPartyId(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitPartyId()));
//                    autoDebitTxnDetailsDto.setAutoDebitAccType(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitAccType()));
//                    autoDebitTxnDetailsDto.setAutoDebitAmtType(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitAmtType()));
//                    autoDebitTxnDetailsDto.setAutoDebitCcy(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCcy()));
//                    autoDebitTxnDetailsDto.setAutoDebitAmt(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitAmt()));
//                    autoDebitTxnDetailsDto.setGroupId(stringConverter(updatedAutoDebitTxnDetails.getGroupId()));
//                    autoDebitTxnDetailsDto.setProductCode(stringConverter(updatedAutoDebitTxnDetails.getProductCode()));
//                    autoDebitTxnDetailsDto.setId(stringConverter(updatedAutoDebitTxnDetails.getId()));
//                    autoDebitTxnDetailsDto.setAutoDebitPartyName(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitPartyName()));
//                    autoDebitTxnDetailsDto.setAccountRefNo(stringConverter(updatedAutoDebitTxnDetails.getAccountRefNo()));
//                    autoDebitTxnDetailsDto.setExecutionId(stringConverter(updatedAutoDebitTxnDetails.getExecutionId()));
//                    autoDebitTxnDetailsDto.setAllocatedAutoDebitAmt(stringConverter(updatedAutoDebitTxnDetails.getAllocatedAutoDebitAmt()));
//                    autoDebitTxnDetailsDto.setStatus(stringConverter(updatedAutoDebitTxnDetails.getStatus()));
//                    autoDebitTxnDetailsDto.setDebitAccountNumber(stringConverter(updatedAutoDebitTxnDetails.getDebitAccountNumber()));
//                    autoDebitTxnDetailsDto.setSettlementRequestId(stringConverter(updatedAutoDebitTxnDetails.getSettlementRequestId()));
//                    autoDebitTxnDetailsDto.setAutoDebitCounterPartyRole(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCounterPartyRole()));
//                    autoDebitTxnDetailsDto.setAutoDebitCounterPartyId(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCounterPartyId()));
//                    autoDebitTxnDetailsDto.setAutoDebitCounterPartyName(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCounterPartyName()));
//                    autoDebitTxnDetailsDto.setAutoDebitPartyDivCode(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitPartyDivCode()));
//                    autoDebitTxnDetailsDto.setAutoDebitCounterPartyDivCode(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCounterPartyDivCode()));
//                    autoDebitTxnDetailsDto.setAutoDebitPartyDivName(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitPartyDivName()));
//                    autoDebitTxnDetailsDto.setAutoDebitCounterPartyDivName(stringConverter(updatedAutoDebitTxnDetails.getAutoDebitCounterPartyDivName()));
//                    autoDebitTxnDetailsList.add(autoDebitTxnDetailsDto);
//                }
//                autoDebitTxnMasterDto.setAutoDebitTxnDetails(autoDebitTxnDetailsList);
//            }
//            eventListenerService.saveAutoDebitInstrumentData(autoDebitTxnMasterDto);
//            
// 
//        }
    }
	
	@StreamListener(ReportConstants.CREATE_EXTERNAL_DELINQ_STATUS) 
	public void getExternalDelinqStatusData(Message<CreateExternalDelinqDto> message)  {
		try {
			LOGGER.info("Inside getExternalDelinqStatusData method...");
			CreateExternalDelinqDto createExternalDelinqDto = message.getPayload();
			if (createExternalDelinqDto != null) {
				List<CreateExtCustomerDelinqStatusHisDto> createExtCustomerDelinqStatusHisDtoList = createExternalDelinqDto.getCreateExtCustomerDelinqStatusHisDto();
				CreateExtCustomerDelinqStatusHisDto createExtCustomerDelinqStatusHisDto = createExtCustomerDelinqStatusHisDtoList.get(0);
				ScftmExtCustomerDelinqStatusHisDTO  scftmExtCustomerDelinqStatusHisDTO = new ScftmExtCustomerDelinqStatusHisDTO();
				scftmExtCustomerDelinqStatusHisDTO.setActiveStatus(stringConverter(createExtCustomerDelinqStatusHisDto.getActiveStatus()));
				scftmExtCustomerDelinqStatusHisDTO.setChannel(stringConverter(createExtCustomerDelinqStatusHisDto.getChannel()));
				scftmExtCustomerDelinqStatusHisDTO.setCustomerId(stringConverter(createExtCustomerDelinqStatusHisDto.getCustomerId()));
				scftmExtCustomerDelinqStatusHisDTO.setCustomerName(stringConverter(createExtCustomerDelinqStatusHisDto.getCustomerName()));
				scftmExtCustomerDelinqStatusHisDTO.setDelinquencyStatus(stringConverter(createExtCustomerDelinqStatusHisDto.getDelinquencyStatus()));
				scftmExtCustomerDelinqStatusHisDTO.setEvent(stringConverter(createExtCustomerDelinqStatusHisDto.getEvent()));
				scftmExtCustomerDelinqStatusHisDTO.setId(stringConverter(createExtCustomerDelinqStatusHisDto.getId()));
				scftmExtCustomerDelinqStatusHisDTO.setInputDate(ReportingUtil.dateConverter(createExtCustomerDelinqStatusHisDto.getInputDate()));
				scftmExtCustomerDelinqStatusHisDTO.setPriority(stringConverter(createExtCustomerDelinqStatusHisDto.getPriority()));
				scftmExtCustomerDelinqStatusHisDTO.setStatusDate(ReportingUtil.dateConverter(createExtCustomerDelinqStatusHisDto.getStatusDate()));
				customerListenerService.saveExtCustomerDelinqStatusData(createExternalDelinqDto.getExtCustomerDelinqStatusId(), scftmExtCustomerDelinqStatusHisDTO, createExternalDelinqDto.getUserId(), createExternalDelinqDto.getServiceBranchCode());
			}
			LOGGER.info("Exiting getExternalDelinqStatusData method...");
		} catch (Exception e) {
			LOGGER.error("Exception in getExternalDelinqStatusData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.CREATE_CUSTOMER_DELINQ_STATUS) 
	public void getCustomerDelinqStatusData(Message<CreateCustomerDelinqDto> message)  {
		try {
			LOGGER.info("Inside getCustomerDelinqStatusData method...");
			CreateCustomerDelinqDto createCustomerDelinqDto = message.getPayload();
			if (createCustomerDelinqDto != null) {
				List<CreateCustomerDelinqStatusDto> createCustomerDelinqStatusDtoList = createCustomerDelinqDto.getCreateCustomerDelinqStatusDto();
				CreateCustomerDelinqStatusDto createCustomerDelinqStatusDto = createCustomerDelinqStatusDtoList.get(0);
				ScftmCustomerDelinqStatusDTO  scftmCustomerDelinqStatusDTO = new ScftmCustomerDelinqStatusDTO();
				scftmCustomerDelinqStatusDTO.setActiveStatus(stringConverter(createCustomerDelinqStatusDto.getActiveStatus()));
				scftmCustomerDelinqStatusDTO.setChannel(stringConverter(createCustomerDelinqStatusDto.getChannel()));
				scftmCustomerDelinqStatusDTO.setCustomerId(stringConverter(createCustomerDelinqStatusDto.getCustomerId()));
				scftmCustomerDelinqStatusDTO.setCustomerName(stringConverter(createCustomerDelinqStatusDto.getCustomerName()));
				scftmCustomerDelinqStatusDTO.setDelinquencyStatus(stringConverter(createCustomerDelinqStatusDto.getDelinquencyStatus()));
				scftmCustomerDelinqStatusDTO.setEvent(stringConverter(createCustomerDelinqStatusDto.getEvent()));
				scftmCustomerDelinqStatusDTO.setPriority(stringConverter(createCustomerDelinqStatusDto.getPriority()));
				scftmCustomerDelinqStatusDTO.setStatusDate(ReportingUtil.dateConverter(createCustomerDelinqStatusDto.getStatusDate()));
				customerListenerService.saveCustomerDelinqStatusData(scftmCustomerDelinqStatusDTO, createCustomerDelinqDto.getUserId(), createCustomerDelinqDto.getServiceBranchCode());
			}
			LOGGER.info("Exiting getCustomerDelinqStatusData method...");
		} catch (Exception e) {
			LOGGER.error("Exception in getCustomerDelinqStatusData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.OB_CHARGES_UPDATED) 
	public void getOBChargesData(Message<UpdateOBChargesCollection> message)  {
		try {
			LOGGER.info("Inside getOBChargesData method...");
			UpdateOBChargesCollection updateOBChargesCollection = message.getPayload();
			if (updateOBChargesCollection != null) {
				List<UpdateOBChargesDto> updateOBChargesTWDtoList = updateOBChargesCollection.getUpdateOBChargesTWDto();
				UpdateOBChargesDto updateOBChargesTWDtoListDto = updateOBChargesTWDtoList.get(0);
				ObChargesTxnDTO  obChargesTWTxnDTO = new ObChargesTxnDTO();
				
				obChargesTWTxnDTO.setAllowOverride(stringConverter(updateOBChargesTWDtoListDto.getAllowOverride()));
				obChargesTWTxnDTO.setAllowPrcModify(stringConverter(updateOBChargesTWDtoListDto.getAllowPrcModify()));
				obChargesTWTxnDTO.setAllowWaive(stringConverter(updateOBChargesTWDtoListDto.getAllowWaive()));
				obChargesTWTxnDTO.setApplicationCode(stringConverter(updateOBChargesTWDtoListDto.getApplicationCode()));
				obChargesTWTxnDTO.setAutoWaive(stringConverter(updateOBChargesTWDtoListDto.getAutoWaive()));
				obChargesTWTxnDTO.setBranchCode(stringConverter(updateOBChargesTWDtoListDto.getBranchCode()));
				obChargesTWTxnDTO.setCalcEndDate(stringConverter(updateOBChargesTWDtoListDto.getCalcEndDate()));
				obChargesTWTxnDTO.setCalcErrorCode(stringConverter(updateOBChargesTWDtoListDto.getCalcErrorCode()));
				obChargesTWTxnDTO.setCalcFreq(stringConverter(updateOBChargesTWDtoListDto.getCalcFreq()));
				obChargesTWTxnDTO.setCalcRefPeriod(stringConverter(updateOBChargesTWDtoListDto.getCalcRefPeriod()));
				obChargesTWTxnDTO.setCalcStartDate(stringConverter(updateOBChargesTWDtoListDto.getCalcStartDate()));
				obChargesTWTxnDTO.setCalcType(stringConverter(updateOBChargesTWDtoListDto.getCalcType()));
				obChargesTWTxnDTO.setCalcUnits(numberConverter(updateOBChargesTWDtoListDto.getCalcUnits()));
				obChargesTWTxnDTO.setChargeAmount(numberConverter(updateOBChargesTWDtoListDto.getChargeAmount()));
				obChargesTWTxnDTO.setChargeCategory(stringConverter(updateOBChargesTWDtoListDto.getChargeCategory()));
				obChargesTWTxnDTO.setChargeCode(stringConverter(updateOBChargesTWDtoListDto.getChargeCode()));
				obChargesTWTxnDTO.setChargeCodeDesc(stringConverter(updateOBChargesTWDtoListDto.getChargeCodeDesc()));
				obChargesTWTxnDTO.setChargeCriteria(stringConverter(updateOBChargesTWDtoListDto.getChargeCriteria()));
				obChargesTWTxnDTO.setChargeCriteriaValue(numberConverter(updateOBChargesTWDtoListDto.getChargeCriteriaValue()));
				obChargesTWTxnDTO.setChargeGroup(stringConverter(updateOBChargesTWDtoListDto.getChargeGroup()));
				obChargesTWTxnDTO.setChargeType(stringConverter(updateOBChargesTWDtoListDto.getChargeType()));
				obChargesTWTxnDTO.setChgCalcDate(stringConverter(updateOBChargesTWDtoListDto.getChgCalcDate()));
				
				obChargesTWTxnDTO.setChgCriteriaCcy(stringConverter(updateOBChargesTWDtoListDto.getChgCriteriaCcy()));
				obChargesTWTxnDTO.setChgCriteriaCount(stringConverter(updateOBChargesTWDtoListDto.getChgCriteriaCount()));
				obChargesTWTxnDTO.setChgSharing(stringConverter(updateOBChargesTWDtoListDto.getChgSharing()));
				obChargesTWTxnDTO.setChgSharingPct(numberConverter(updateOBChargesTWDtoListDto.getChgSharingPct()));
				obChargesTWTxnDTO.setCollectionFreq(stringConverter(updateOBChargesTWDtoListDto.getCollectionFreq()));
				obChargesTWTxnDTO.setCollectionRefPeriod(stringConverter(updateOBChargesTWDtoListDto.getCollectionRefPeriod()));
				obChargesTWTxnDTO.setCollectionType(stringConverter(updateOBChargesTWDtoListDto.getCollectionType()));
				obChargesTWTxnDTO.setCollectionUnits(numberConverter(updateOBChargesTWDtoListDto.getCollectionUnits()));
				
				
				obChargesTWTxnDTO.setCreditAccNumber(stringConverter(updateOBChargesTWDtoListDto.getCreditAccNumber()));
				obChargesTWTxnDTO.setDebitAccNumber(stringConverter(updateOBChargesTWDtoListDto.getDebitAccNumber()));
				obChargesTWTxnDTO.setEventCode(stringConverter(updateOBChargesTWDtoListDto.getEventCode()));
				obChargesTWTxnDTO.setExchangeRate(numberConverter(updateOBChargesTWDtoListDto.getExchangeRate()));
				obChargesTWTxnDTO.setFirstChgCalcDate(stringConverter(updateOBChargesTWDtoListDto.getFirstChgCalcDate()));
				obChargesTWTxnDTO.setFirstChgCollDate(stringConverter(updateOBChargesTWDtoListDto.getFirstChgCollDate()));
				obChargesTWTxnDTO.setInstrumentType(stringConverter(updateOBChargesTWDtoListDto.getInstrumentType()));
				obChargesTWTxnDTO.setNextChgCalcDate(stringConverter(updateOBChargesTWDtoListDto.getNextChgCalcDate()));
				obChargesTWTxnDTO.setNextChgCollDate(stringConverter(updateOBChargesTWDtoListDto.getNextChgCollDate()));
				
				obChargesTWTxnDTO.setParentChargeCode(stringConverter(updateOBChargesTWDtoListDto.getParentChargeCode()));
				obChargesTWTxnDTO.setPartyId(stringConverter(updateOBChargesTWDtoListDto.getPartyId()));
				obChargesTWTxnDTO.setPartyName(stringConverter(updateOBChargesTWDtoListDto.getPartyName()));
				obChargesTWTxnDTO.setPartyToCharge(stringConverter(updateOBChargesTWDtoListDto.getPartyToCharge()));
				obChargesTWTxnDTO.setPostingDate(stringConverter(updateOBChargesTWDtoListDto.getPostingDate()));
				obChargesTWTxnDTO.setPostingErrorCode(stringConverter(updateOBChargesTWDtoListDto.getPostingErrorCode()));
				obChargesTWTxnDTO.setPriceMethodDesc(stringConverter(updateOBChargesTWDtoListDto.getPriceMethodDesc()));
				obChargesTWTxnDTO.setPriceTemplateId(stringConverter(updateOBChargesTWDtoListDto.getPriceTemplateId()));
				obChargesTWTxnDTO.setReferenceEndDate(stringConverter(updateOBChargesTWDtoListDto.getReferenceEndDate()));
				obChargesTWTxnDTO.setReferenceEndDateValue(stringConverter(updateOBChargesTWDtoListDto.getReferenceEndDateValue()));
				obChargesTWTxnDTO.setReferenceStartDate(stringConverter(updateOBChargesTWDtoListDto.getReferenceStartDate()));
				obChargesTWTxnDTO.setReferenceStartDateValue(stringConverter(updateOBChargesTWDtoListDto.getReferenceStartDateValue()));
				obChargesTWTxnDTO.setTxnCcy(stringConverter(updateOBChargesTWDtoListDto.getTxnCcy()));
				obChargesTWTxnDTO.setTxnRefNo(stringConverter(updateOBChargesTWDtoListDto.getTxnRefNo()));	
				
				
				
				List<UpdateOBChargesDto> updateOBChargesDtoList = updateOBChargesCollection.getUpdateOBChargesDto();
				UpdateOBChargesDto updateOBChargesDtoListDto = updateOBChargesDtoList.get(0);
				ObChargesTxnDTO  obChargesTxnDTO = new ObChargesTxnDTO();
				
				obChargesTxnDTO.setAllowOverride(stringConverter(updateOBChargesDtoListDto.getAllowOverride()));
				obChargesTxnDTO.setAllowPrcModify(stringConverter(updateOBChargesDtoListDto.getAllowPrcModify()));
				obChargesTxnDTO.setAllowWaive(stringConverter(updateOBChargesDtoListDto.getAllowWaive()));
				obChargesTxnDTO.setApplicationCode(stringConverter(updateOBChargesDtoListDto.getApplicationCode()));
				obChargesTxnDTO.setAutoWaive(stringConverter(updateOBChargesDtoListDto.getAutoWaive()));
				obChargesTxnDTO.setBranchCode(stringConverter(updateOBChargesDtoListDto.getBranchCode()));
				obChargesTxnDTO.setCalcEndDate(stringConverter(updateOBChargesDtoListDto.getCalcEndDate()));
				obChargesTxnDTO.setCalcErrorCode(stringConverter(updateOBChargesDtoListDto.getCalcErrorCode()));
				obChargesTxnDTO.setCalcFreq(stringConverter(updateOBChargesDtoListDto.getCalcFreq()));
				obChargesTxnDTO.setCalcRefPeriod(stringConverter(updateOBChargesDtoListDto.getCalcRefPeriod()));
				obChargesTxnDTO.setCalcStartDate(stringConverter(updateOBChargesDtoListDto.getCalcStartDate()));
				obChargesTxnDTO.setCalcType(stringConverter(updateOBChargesDtoListDto.getCalcType()));
				obChargesTxnDTO.setCalcUnits(numberConverter(updateOBChargesDtoListDto.getCalcUnits()));
				obChargesTxnDTO.setChargeAmount(numberConverter(updateOBChargesDtoListDto.getChargeAmount()));
				obChargesTxnDTO.setChargeCategory(stringConverter(updateOBChargesDtoListDto.getChargeCategory()));
				obChargesTxnDTO.setChargeCode(stringConverter(updateOBChargesDtoListDto.getChargeCode()));
				obChargesTxnDTO.setChargeCodeDesc(stringConverter(updateOBChargesDtoListDto.getChargeCodeDesc()));
				obChargesTxnDTO.setChargeCriteria(stringConverter(updateOBChargesDtoListDto.getChargeCriteria()));
				obChargesTxnDTO.setChargeCriteriaValue(numberConverter(updateOBChargesDtoListDto.getChargeCriteriaValue()));
				obChargesTxnDTO.setChargeGroup(stringConverter(updateOBChargesDtoListDto.getChargeGroup()));
				obChargesTxnDTO.setChargeType(stringConverter(updateOBChargesDtoListDto.getChargeType()));
				obChargesTxnDTO.setChgCalcDate(stringConverter(updateOBChargesDtoListDto.getChgCalcDate()));
				
				obChargesTxnDTO.setChgCriteriaCcy(stringConverter(updateOBChargesDtoListDto.getChgCriteriaCcy()));
				obChargesTxnDTO.setChgCriteriaCount(stringConverter(updateOBChargesDtoListDto.getChgCriteriaCount()));
				obChargesTxnDTO.setChgSharing(stringConverter(updateOBChargesDtoListDto.getChgSharing()));
				obChargesTxnDTO.setChgSharingPct(numberConverter(updateOBChargesDtoListDto.getChgSharingPct()));
				obChargesTxnDTO.setCollectionFreq(stringConverter(updateOBChargesDtoListDto.getCollectionFreq()));
				obChargesTxnDTO.setCollectionRefPeriod(stringConverter(updateOBChargesDtoListDto.getCollectionRefPeriod()));
				obChargesTxnDTO.setCollectionType(stringConverter(updateOBChargesDtoListDto.getCollectionType()));
				obChargesTxnDTO.setCollectionUnits(numberConverter(updateOBChargesDtoListDto.getCollectionUnits()));
				
				
				obChargesTxnDTO.setCreditAccNumber(stringConverter(updateOBChargesDtoListDto.getCreditAccNumber()));
				obChargesTxnDTO.setDebitAccNumber(stringConverter(updateOBChargesDtoListDto.getDebitAccNumber()));
				obChargesTxnDTO.setEventCode(stringConverter(updateOBChargesDtoListDto.getEventCode()));
				obChargesTxnDTO.setExchangeRate(numberConverter(updateOBChargesDtoListDto.getExchangeRate()));
				obChargesTxnDTO.setFirstChgCalcDate(stringConverter(updateOBChargesDtoListDto.getFirstChgCalcDate()));
				obChargesTxnDTO.setFirstChgCollDate(stringConverter(updateOBChargesDtoListDto.getFirstChgCollDate()));
				obChargesTxnDTO.setInstrumentType(stringConverter(updateOBChargesDtoListDto.getInstrumentType()));
				obChargesTxnDTO.setNextChgCalcDate(stringConverter(updateOBChargesDtoListDto.getNextChgCalcDate()));
				obChargesTxnDTO.setNextChgCollDate(stringConverter(updateOBChargesDtoListDto.getNextChgCollDate()));
				
				obChargesTxnDTO.setParentChargeCode(stringConverter(updateOBChargesDtoListDto.getParentChargeCode()));
				obChargesTxnDTO.setPartyId(stringConverter(updateOBChargesDtoListDto.getPartyId()));
				obChargesTxnDTO.setPartyName(stringConverter(updateOBChargesDtoListDto.getPartyName()));
				obChargesTxnDTO.setPartyToCharge(stringConverter(updateOBChargesDtoListDto.getPartyToCharge()));
				obChargesTxnDTO.setPostingDate(stringConverter(updateOBChargesDtoListDto.getPostingDate()));
				obChargesTxnDTO.setPostingErrorCode(stringConverter(updateOBChargesDtoListDto.getPostingErrorCode()));
				obChargesTxnDTO.setPriceMethodDesc(stringConverter(updateOBChargesDtoListDto.getPriceMethodDesc()));
				obChargesTxnDTO.setPriceTemplateId(stringConverter(updateOBChargesDtoListDto.getPriceTemplateId()));
				obChargesTxnDTO.setReferenceEndDate(stringConverter(updateOBChargesDtoListDto.getReferenceEndDate()));
				obChargesTxnDTO.setReferenceEndDateValue(stringConverter(updateOBChargesDtoListDto.getReferenceEndDateValue()));
				obChargesTxnDTO.setReferenceStartDate(stringConverter(updateOBChargesDtoListDto.getReferenceStartDate()));
				obChargesTxnDTO.setReferenceStartDateValue(stringConverter(updateOBChargesDtoListDto.getReferenceStartDateValue()));
				obChargesTxnDTO.setTxnCcy(stringConverter(updateOBChargesDtoListDto.getTxnCcy()));
				obChargesTxnDTO.setTxnRefNo(stringConverter(updateOBChargesDtoListDto.getTxnRefNo()));	
				
				
				chargesListenerService.saveOBChargesData(updateOBChargesCollection.getTxnRefNo(), updateOBChargesCollection.getChargeCode(), updateOBChargesCollection.getEventCode(), obChargesTWTxnDTO, obChargesTxnDTO, updateOBChargesCollection.getUserId(), updateOBChargesCollection.getServiceBranchCode());
			}
			LOGGER.info("Exiting getOBChargesData method...");
		} catch (Exception e) {
			LOGGER.error("Exception in getOBChargesData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.OB_FINANCE_UPDATED) 
	public void getOBFinanceDetailsData(Message<UpdateOBFinanceCollection> message)  {
		try {
			LOGGER.info("Inside getOBFinanceDetailsData method...");
			UpdateOBFinanceCollection updateOBFinanceCollection = message.getPayload();
			if (updateOBFinanceCollection != null) {
				List<UpdateOBFinanceDto> updateOBFinanceDtoList = updateOBFinanceCollection.getUpdateOBFinanceDto();
				UpdateOBFinanceDto updateOBFinanceDtoDto = updateOBFinanceDtoList.get(0);
				FinanceTxnDTO  financeTxnDTO = new FinanceTxnDTO();

				financeTxnDTO.setBuyerId(stringConverter(updateOBFinanceDtoDto.getBuyerId()));
				financeTxnDTO.setBuyerName(stringConverter(updateOBFinanceDtoDto.getBuyerName()));
				financeTxnDTO.setEventCode(stringConverter(updateOBFinanceDtoDto.getEventCode()));
				financeTxnDTO.setFinanceAmt(numberConverter(updateOBFinanceDtoDto.getFinanceAmt()));
				financeTxnDTO.setFinanceCcy(stringConverter(updateOBFinanceDtoDto.getFinanceCcy()));
				financeTxnDTO.setFinanceDueDate(stringConverter(updateOBFinanceDtoDto.getFinanceDueDate()));
				financeTxnDTO.setFinanceStartDate(stringConverter(updateOBFinanceDtoDto.getFinanceStartDate()));
				financeTxnDTO.setOutdtandingAmt(numberConverter(updateOBFinanceDtoDto.getOutdtandingAmt()));
				financeTxnDTO.setProcessingDate(stringConverter(updateOBFinanceDtoDto.getProcessingDate()));
				financeTxnDTO.setProductCode(stringConverter(updateOBFinanceDtoDto.getProductCode()));
				financeTxnDTO.setProductCodeDesc(stringConverter(updateOBFinanceDtoDto.getProductCodeDesc()));
				financeTxnDTO.setProgramCode(stringConverter(updateOBFinanceDtoDto.getProgramCode()));
				financeTxnDTO.setRepaymentAmt(numberConverter(updateOBFinanceDtoDto.getRepaymentAmt()));
				financeTxnDTO.setSupplierId(stringConverter(updateOBFinanceDtoDto.getSupplierId()));
				financeTxnDTO.setSupplierName(stringConverter(updateOBFinanceDtoDto.getSupplierName()));
				financeTxnDTO.setTxnRefNo(stringConverter(updateOBFinanceDtoDto.getTxnRefNo()));
				
				chargesListenerService.saveOBFinanceDetailsData(updateOBFinanceCollection.getTxnRefNo(), updateOBFinanceCollection.getEventCode(), financeTxnDTO, updateOBFinanceCollection.getUserId(), updateOBFinanceCollection.getServiceBranchCode());
			}
			LOGGER.info("Exiting getOBFinanceDetailsData method...");
		} catch (Exception e) {
			LOGGER.error("Exception in getOBFinanceDetailsData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.OB_INVOICE_DETAILS_UPDATED) 
	public void getOBInvoiceDetailsData(Message<UpdateOBInvoiceDetailsCollection> message)  {
		try {
			LOGGER.info("Inside getOBInvoiceDetailsData method...");
			UpdateOBInvoiceDetailsCollection updateOBInvoiceDetailsCollection = message.getPayload();
			if (updateOBInvoiceDetailsCollection != null) {
				List<UpdateOBInvoiceDetailsDto> updateOBInvoiceDetailsDtoList = updateOBInvoiceDetailsCollection.getUpdateOBInvoiceDetailsDto();
				UpdateOBInvoiceDetailsDto updateOBInvoiceDetailsDto = updateOBInvoiceDetailsDtoList.get(0);
				InstrumentTxnDTO  instrumentTxnDTO = new InstrumentTxnDTO();

				instrumentTxnDTO.setAcceptanceAmount(numberConverter(updateOBInvoiceDetailsDto.getAcceptanceAmount()));
				instrumentTxnDTO.setBuyerId(stringConverter(updateOBInvoiceDetailsDto.getBuyerId()));
				instrumentTxnDTO.setBuyerName(stringConverter(updateOBInvoiceDetailsDto.getBuyerName()));
				instrumentTxnDTO.setEvent(stringConverter(updateOBInvoiceDetailsDto.getEvent()));
				instrumentTxnDTO.setExtInvoiceRefNo(stringConverter(updateOBInvoiceDetailsDto.getExtInvoiceRefNo()));
				instrumentTxnDTO.setInvoiceCcy(stringConverter(updateOBInvoiceDetailsDto.getInvoiceCcy()));
				instrumentTxnDTO.setInvoiceDate(stringConverter(updateOBInvoiceDetailsDto.getInvoiceDate()));
				instrumentTxnDTO.setInvoiceDueDate(stringConverter(updateOBInvoiceDetailsDto.getInvoiceDueDate()));
				instrumentTxnDTO.setInvoiceRefNo(stringConverter(updateOBInvoiceDetailsDto.getInvoiceRefNo()));
				instrumentTxnDTO.setInvoiceStatus(stringConverter(updateOBInvoiceDetailsDto.getInvoiceStatus()));
				instrumentTxnDTO.setNetInvoiceAmt(numberConverter(updateOBInvoiceDetailsDto.getNetInvoiceAmt()));
				instrumentTxnDTO.setObInvTxnId(stringConverter(updateOBInvoiceDetailsDto.getObInvTxnId()));
				instrumentTxnDTO.setProcessingDate(stringConverter(updateOBInvoiceDetailsDto.getProcessingDate()));
				instrumentTxnDTO.setProgramCode(stringConverter(updateOBInvoiceDetailsDto.getProgramCode()));
				instrumentTxnDTO.setRelation(stringConverter(updateOBInvoiceDetailsDto.getRelation()));
				instrumentTxnDTO.setSupplierId(stringConverter(updateOBInvoiceDetailsDto.getSupplierId()));
				instrumentTxnDTO.setSupplierName(stringConverter(updateOBInvoiceDetailsDto.getSupplierName()));
				instrumentTxnDTO.setTxnRefNo(stringConverter(updateOBInvoiceDetailsDto.getTxnRefNo()));
				
				chargesListenerService.saveOBInvoiceDetailsData(updateOBInvoiceDetailsCollection.getTxnRefNo(), updateOBInvoiceDetailsCollection.getInvoiceRefNo(), updateOBInvoiceDetailsCollection.getInvoiceStatus(), updateOBInvoiceDetailsCollection.getInvoiceCcy(), instrumentTxnDTO, updateOBInvoiceDetailsCollection.getUserId(), updateOBInvoiceDetailsCollection.getServiceBranchCode());
			}
			LOGGER.info("Exiting getOBInvoiceDetailsData method...");
		} catch (Exception e) {
			LOGGER.error("Exception in getOBInvoiceDetailsData {}", e.getMessage());
		}
	}
	
	@StreamListener(ReportConstants.INVOICE_TM_UPDATED_EVENT)
	public void getInvoiceTmData(Message<UpdateInvoiceTmDto> message) {
		LOGGER.info("Message Received on channel InvoiceTmUpdate Event.");
		UpdateInvoiceTmDto updateInvoiceTmDto = message.getPayload();
		String transactionRefNo = updateInvoiceTmDto.getTransactionReferenceNo();
		String instrumentType = updateInvoiceTmDto.getInstrumentType();
		LOGGER.info("instrumentType {} .",instrumentType);
		LOGGER.info("branchCode {} .",updateInvoiceTmDto.getBranchCode());
		LOGGER.info("transactionReferenceNo {} .",updateInvoiceTmDto.getTransactionReferenceNo());
		LOGGER.info("currentStageCode {} .",updateInvoiceTmDto.getCurrentStageCode());
		LOGGER.info("currentStageName {} .",updateInvoiceTmDto.getCurrentStageName());
		LOGGER.info("userId {} .",updateInvoiceTmDto.getUserId());
		if(instrumentType==null || ReportConstants.RECEIVABLE_TYPE_INVOICE.equals(instrumentType)) {
			eventListenerService.fetchInvoiceData(transactionRefNo, updateInvoiceTmDto.getUserId(), updateInvoiceTmDto.getBranchCode());
		}
		else if(ReportConstants.RECEIVABLE_TYPE_PO.equals(instrumentType)){
			LOGGER.info(" Updating purchase order data and sending alerts..");
			PurchaseOrderCollection purchaseOrderCollectionTW = instrumentReceivablesServiceClient.getPoByTransactionRefNo(transactionRefNo, false, updateInvoiceTmDto.getUserId(),updateInvoiceTmDto.getBranchCode());
			PurchaseOrderCollection purchaseOrderCollectionTM = instrumentReceivablesServiceClient.getPoByTransactionRefNo(transactionRefNo, true, updateInvoiceTmDto.getUserId(),updateInvoiceTmDto.getBranchCode());
			List<PurchaseOrderDTO> poTwList = purchaseOrderCollectionTW.getData();
			List<PurchaseOrderDTO> poTmList = purchaseOrderCollectionTM.getData();
			eventListenerService.savePoData(poTwList,poTmList,updateInvoiceTmDto.getUserId(), updateInvoiceTmDto.getBranchCode());
		}

		
	}

	@StreamListener(ReportConstants.INVOICE_CASHFLOW)
	public void sendInvoiceCashflowData(Message<InvoiceTmCashflowDto> message) {
		LOGGER.info("Message Received on channel invoicingCashflow Event.");
		InvoiceTmCashflowDto invoiceTmCashflowDto = message.getPayload();
		String transactionRefNo = invoiceTmCashflowDto.getTransactionReferenceNo();
		InvoiceCollectionDto invoiceCollectionDto = instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(transactionRefNo, invoiceTmCashflowDto.getUserId(), invoiceTmCashflowDto.getBranchCode());
		if (invoiceCollectionDto != null) {
			List<InvoiceModel> invoiceModelList = invoiceCollectionDto.getData();
			if (!CollectionUtils.isEmpty(invoiceModelList)) {
				LOGGER.info("invoiceModelList to be sent to cashflow engine:  {}",invoiceModelList);
				OBIntegrationServiceRequest req = new OBIntegrationServiceRequest();
				HashMap<String, Object> requestParams = new HashMap();
				req.setTransactionType(ReportConstants.SAVE_CASHFLOW);
				req.setRequestType(ReportConstants.CASHFLOW);
				requestParams.put("requestList", invoiceModelList);
				requestParams.put("from",ReportConstants.CATEGORY_INVOICE);
				requestParams.put(ReportConstants.BRANCH_CODE,invoiceTmCashflowDto.getBranchCode());
				requestParams.put(ReportConstants.USERID,invoiceTmCashflowDto.getUserId());
				req.setRequestParams(requestParams);

				RequestContext reqContext = new RequestContext();
				reqContext.setUserId(invoiceTmCashflowDto.getUserId());
				reqContext.setCurrBranch(invoiceTmCashflowDto.getBranchCode());
				GlobalHolder.setRequestcontext(reqContext);

				OBIntegrationServiceResponse response= obIntegrationAdapter.handleTransaction(req);
				LOGGER.info("OBIntegrationServiceResponse :  {}", response);
			}
		}
	}
		
	@StreamListener(ReportConstants.CREATE_CASHFLOW_FROM_RECON) 
	public void sendReconInvPmtCnToCashflow(Message<ReconTmCashflowDto> message)  {
		try {
			LOGGER.info("Inside sendReconInvPmtCnToCashflow method...");
			OBIntegrationServiceRequest req = null;
			List<ReconInvPmtDto> requestIPRList=new ArrayList();
			List<ReconInvCnDto> requestICRList=new ArrayList();
			CashflowResponse cashflowResponse=new CashflowResponse();
			HashMap<String, Object> requestParams = new HashMap<>();
			String partyId=null;
			String extTxnRefNo=null;
			ReconTmCashflowDto reconTmCashflowDto = message.getPayload();
			ReconTxnDto reconTxnDto = reconEventServiceClient.getReconTmData(reconTmCashflowDto.getReconTxnRefNo(), reconTmCashflowDto.getUserId(), reconTmCashflowDto.getBranchCode());
			String reconAction=reconTxnDto.getReconAction();
			if(reconTxnDto.getObReconInvPmtTxnTwDtoList()!=null && !reconTxnDto.getObReconInvPmtTxnTwDtoList().isEmpty())
			{
				requestIPRList=reconTxnDto.getObReconInvPmtTxnTwDtoList();
			}
			if(reconTxnDto.getObReconInvCnTxnTwDtoList()!=null && !reconTxnDto.getObReconInvCnTxnTwDtoList().isEmpty())
			{
				requestICRList=reconTxnDto.getObReconInvCnTxnTwDtoList();
			}
			LOGGER.info("Recon record size ...{}::{}", requestIPRList.size(), requestICRList.size());
			if(requestIPRList!=null && !requestIPRList.isEmpty()) {
				List<ReconInvPmtDto> tempIPRList=new ArrayList();
				tempIPRList.addAll(requestIPRList);
				for(ReconInvPmtDto reconInvPmtDto : tempIPRList) {
					partyId=reconInvPmtDto.getCorporateId();
					extTxnRefNo=reconInvPmtDto.getExtInvRefNo();
					requestParams = new HashMap<>();
					req = new OBIntegrationServiceRequest();
					req.setTransactionType(ReportConstants.GET_CASHFLOW);
					req.setRequestType(ReportConstants.CASHFLOW);
					requestParams.put("partyId", partyId);
					requestParams.put("externalTransactionRefNo", extTxnRefNo);
					requestParams.put(ReportConstants.BRANCH_CODE,reconTmCashflowDto.getBranchCode());
					requestParams.put(ReportConstants.USERID,reconTmCashflowDto.getUserId());
					req.setRequestParams(requestParams);
					RequestContext reqContext = new RequestContext();
					reqContext.setUserId(reconTmCashflowDto.getUserId());
					reqContext.setCurrBranch(reconTmCashflowDto.getBranchCode());
					GlobalHolder.setRequestcontext(reqContext);

					OBIntegrationServiceResponse resp = obIntegrationAdapter.handleTransaction(req);
					if (resp != null && resp.getStatus()!=null && resp.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)) {
						if(resp.getResponseParams().get("cashflowResponse")!=null) {
							cashflowResponse = (CashflowResponse) resp.getResponseParams().get("cashflowResponse");
						}else {
							requestIPRList.remove(reconInvPmtDto);
						}
					}
				}
				LOGGER.info("Recon IPR record size after removal ...{}",requestIPRList.size());
				if (!CollectionUtils.isEmpty(requestIPRList)) {
					req = new OBIntegrationServiceRequest();
					requestParams = new HashMap<>();
					req.setTransactionType(ReportConstants.SAVE_CASHFLOW);
					req.setRequestType(ReportConstants.CASHFLOW);
					requestParams.put("from", "IPR");
					requestParams.put("lastActionPerformed", reconAction);
					LOGGER.info("requestIPRList record size while sending to Cashflow...{}", requestIPRList.size());
					LOGGER.info("requestIPRList to be sent to cashflow engine: {}", requestIPRList);
					requestParams.put("requestList", requestIPRList);
					requestParams.put(ReportConstants.BRANCH_CODE, reconTmCashflowDto.getBranchCode());
					requestParams.put(ReportConstants.USERID, reconTmCashflowDto.getUserId());
					req.setRequestParams(requestParams);
					RequestContext reqContext = new RequestContext();
					reqContext.setUserId(reconTmCashflowDto.getUserId());
					reqContext.setCurrBranch(reconTmCashflowDto.getBranchCode());
					GlobalHolder.setRequestcontext(reqContext);
					OBIntegrationServiceResponse cmResponseIPR = obIntegrationAdapter.handleTransaction(req);
				}
			}
			if(requestICRList!=null && !requestICRList.isEmpty()) {
				List<ReconInvCnDto> tempICRList=new ArrayList();
				tempICRList.addAll(requestICRList);
				for(ReconInvCnDto reconInvCnDto : tempICRList) {
					partyId=reconInvCnDto.getCorporateId();
					extTxnRefNo=reconInvCnDto.getExtInvRefNo();
					requestParams = new HashMap<>();
					req = new OBIntegrationServiceRequest();
					req.setTransactionType(ReportConstants.GET_CASHFLOW);
					req.setRequestType(ReportConstants.CASHFLOW);
					requestParams.put("partyId", partyId);
					requestParams.put("externalTransactionRefNo", extTxnRefNo);
					requestParams.put(ReportConstants.BRANCH_CODE,reconTmCashflowDto.getBranchCode());
					requestParams.put(ReportConstants.USERID,reconTmCashflowDto.getUserId());
					req.setRequestParams(requestParams);
					RequestContext reqContext = new RequestContext();
					reqContext.setUserId(reconTmCashflowDto.getUserId());
					reqContext.setCurrBranch(reconTmCashflowDto.getBranchCode());
					GlobalHolder.setRequestcontext(reqContext);
					OBIntegrationServiceResponse resp = obIntegrationAdapter.handleTransaction(req);
					if (resp != null && resp.getStatus()!=null && resp.getStatus().equalsIgnoreCase(ReportConstants.SUCCESS)) {
						if(resp.getResponseParams().get("cashflowResponse")!=null) {
							cashflowResponse = (CashflowResponse) resp.getResponseParams().get("cashflowResponse");						
						}else {
							requestICRList.remove(reconInvCnDto);
						}
					}
				}
				LOGGER.info("Recon ICR record size after removal... {}",requestICRList.size());
				if (!CollectionUtils.isEmpty(requestICRList)) {
					req = new OBIntegrationServiceRequest();
					requestParams = new HashMap<>();
					req.setTransactionType(ReportConstants.SAVE_CASHFLOW);
					req.setRequestType(ReportConstants.CASHFLOW);
					LOGGER.info("requestICRList record size while sending to Cashflow... {}", requestICRList.size());
					LOGGER.info("requestICRList to be sent to cashflow engine: {}", requestICRList);
					requestParams.put("requestList", requestICRList);
					requestParams.put("lastActionPerformed", reconAction);
					requestParams.put("from", "ICR");
					requestParams.put(ReportConstants.BRANCH_CODE, reconTmCashflowDto.getBranchCode());
					requestParams.put(ReportConstants.USERID, reconTmCashflowDto.getUserId());
					req.setRequestParams(requestParams);
					RequestContext reqContext = new RequestContext();
					reqContext.setUserId(reconTmCashflowDto.getUserId());
					reqContext.setCurrBranch(reconTmCashflowDto.getBranchCode());
					GlobalHolder.setRequestcontext(reqContext);
					OBIntegrationServiceResponse cmResponseICR = obIntegrationAdapter.handleTransaction(req);
				}
			}
			LOGGER.info("Exiting sendReconInvPmtCnToCashflow method...");
		} catch (Exception e) {
			LOGGER.error("Exception in sendReconInvPmtCnToCashflow" , e);
		}
	}
	
}