package oracle.fsgbu.obscfcm.services.report.service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import oracle.fsgbu.obscfcm.alertsgenerator.AlertMessageProcessor;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningDetailsDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningWrapperDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.EventDetailsDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.UserDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertDecisionClient;
import oracle.fsgbu.obscfcm.services.report.client.AlertList;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObExpDrCrPmtReconTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconAllocPmtTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconExecTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconFinPmtTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvCnTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvPmtTxn;
import oracle.fsgbu.obscfcm.services.report.persistence.ObExpDrCrPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObFinPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvCnReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconAllocPmtTxnRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconExecTxnTwRepository;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reconService.api.model.ExpDrCrPmtReconDTO;
import oracle.fsgbu.reconService.api.model.FinPmtReconDTO;
import oracle.fsgbu.reconService.api.model.InvCnReconDTO;
import oracle.fsgbu.reconService.api.model.InvPmtReconDTO;
import oracle.fsgbu.reconService.api.model.PmtAllocDTO;
import oracle.fsgbu.reconService.api.model.ReconTxnDto;

@Service
public class ReconEventListenerServiceImpl implements ReconEventListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(ReconEventListenerServiceImpl.class);
	private static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd";
	
	@Autowired
	private ObReconExecTxnTwRepository obReconExecTxnTwRepository;
	
	private ObExpDrCrPmtReconRepository obExpDrCrPmtReconRepository;
	private ObInvPmtReconRepository obInvPmtReconRepository;
	private ObFinPmtReconRepository obFinPmtReconRepository;
	private ObReconAllocPmtTxnRepository obReconAllocPmtTxnRepository;
	private ObInvCnReconRepository obInvCnReconRepository;
	
	@Autowired
	public ReconEventListenerServiceImpl(ObExpDrCrPmtReconRepository obExpDrCrPmtReconRepository,
			ObInvPmtReconRepository obInvPmtReconRepository,
			ObFinPmtReconRepository obFinPmtReconRepository,
			ObReconAllocPmtTxnRepository obReconAllocPmtTxnRepository,
			ObInvCnReconRepository obInvCnReconRepository) {
		this.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository;
		this.obInvPmtReconRepository = obInvPmtReconRepository;
		this.obFinPmtReconRepository = obFinPmtReconRepository;
		this.obReconAllocPmtTxnRepository = obReconAllocPmtTxnRepository;
		this.obInvCnReconRepository = obInvCnReconRepository;
	}
	
	@Autowired
	AlertDecisionClient alertDecisionClient;
	
	@Autowired
	AlertList alertList;
	
	@Autowired
	AlertMessageProcessor alertMessageProcessor;
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveExpDrCrReconData(ExpDrCrPmtReconDTO expDrCrPmtReconDTO, String userId, String branchCode) {
		
		LOGGER.info("Trying to commit Updated Expected Debit Credit Recon data..");
		
		try {
			String[] ignoreFields = { "makerDtStamp", "authDtStamp", "paymentDate", "reconDate", "expDrcrDate", "revisedDrcrDate" };
			String[] queryParams = {"expDrCrReconId"};
			Object[] paramValues = {expDrCrPmtReconDTO.getExpDrCrReconId()};
			ObExpDrCrPmtReconTxn expDrCrPmtReconTxn = null;
			List<ObExpDrCrPmtReconTxn> expDrCrPmtReconTxnList = obExpDrCrPmtReconRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObExpDrCrPmtReconTxn.findById");
			if(expDrCrPmtReconTxnList != null && !expDrCrPmtReconTxnList.isEmpty()) {
				expDrCrPmtReconTxn = expDrCrPmtReconTxnList.get(0);
			} else {
				expDrCrPmtReconTxn = new ObExpDrCrPmtReconTxn();
			}
			BeanUtils.copyProperties(expDrCrPmtReconDTO, expDrCrPmtReconTxn, ignoreFields);
			expDrCrPmtReconTxn.setAuthDtStamp(dateConverter(expDrCrPmtReconDTO.getAuthDtStamp()));
			expDrCrPmtReconTxn.setMakerDtStamp(dateConverter(expDrCrPmtReconDTO.getMakerDtStamp()));
			expDrCrPmtReconTxn.setExpDrcrDate(dateConverter(expDrCrPmtReconDTO.getExpDrcrDate()));
			expDrCrPmtReconTxn.setPaymentDate(dateConverter(expDrCrPmtReconDTO.getPaymentDate()));
			expDrCrPmtReconTxn.setReconDate(dateConverter(expDrCrPmtReconDTO.getReconDate()));
			expDrCrPmtReconTxn.setRevisedDrcrDate(dateConverter(expDrCrPmtReconDTO.getExpDrcrDate()));
			obExpDrCrPmtReconRepository.save(expDrCrPmtReconTxn);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(expDrCrPmtReconDTO, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(expDrCrPmtReconDTO, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Debit Credit Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final ExpDrCrPmtReconDTO expDrCrPmtReconDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setRelationshipCode(expDrCrPmtReconDTO.getPmtRelationshipCode());
		eventDetailsObject.setProgramId(expDrCrPmtReconDTO.getPmtProgramCode());
		eventDetailsObject.setReferenceNumber(expDrCrPmtReconDTO.getReconTxnRefNo());
		eventDetailsObject.setEvent(ReportConstants.RECON_TM_EPR_EVENT);
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final ExpDrCrPmtReconDTO expDrCrPmtReconDTO, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,expDrCrPmtReconDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveInvPmtReconData(InvPmtReconDTO invPmtReconDTO, String userId, String branchCode) {
		
		LOGGER.info("Trying to commit Updated Invoice Payment Recon data..");
		try {
			String[] ignoreFields = { "makerDtStamp", "authDtStamp", "invDate", "invDueDate", "paymentDate", "reconDate",  "paymentDueDate"};
			String[] queryParams = {"invPmtReconId"};
			Object[] paramValues = {invPmtReconDTO.getInvPmtReconId()};
			ObReconInvPmtTxn obReconInvPmtTxn = null;
			List<ObReconInvPmtTxn> obInvPmtReconTxnList = obInvPmtReconRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObReconInvPmtTxn.findById");
			if(obInvPmtReconTxnList != null && !obInvPmtReconTxnList.isEmpty()) {
				obReconInvPmtTxn = obInvPmtReconTxnList.get(0);
			} else {
				obReconInvPmtTxn = new ObReconInvPmtTxn();
			}
			BeanUtils.copyProperties(invPmtReconDTO, obReconInvPmtTxn,ignoreFields);
			obReconInvPmtTxn.setAuthDtStamp(dateConverter(invPmtReconDTO.getAuthDtStamp()));
			obReconInvPmtTxn.setMakerDtStamp(dateConverter(invPmtReconDTO.getMakerDtStamp()));
			obReconInvPmtTxn.setInvDate(dateConverter(invPmtReconDTO.getInvDate()));
			obReconInvPmtTxn.setPaymentDate(dateConverter(invPmtReconDTO.getPaymentDate()));
			obReconInvPmtTxn.setReconDate(dateConverter(invPmtReconDTO.getReconDate()));
			obReconInvPmtTxn.setInvDueDate(dateConverter(invPmtReconDTO.getInvDueDate()));
			obReconInvPmtTxn.setPaymentDueDate(dateConverter(invPmtReconDTO.getPaymentDueDate()));
			obInvPmtReconRepository.save(obReconInvPmtTxn);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(invPmtReconDTO, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(invPmtReconDTO, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Invoice Payment Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final InvPmtReconDTO invPmtReconDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setRelationshipCode(invPmtReconDTO.getPmtRelationshipCode());
		eventDetailsObject.setProgramId(invPmtReconDTO.getPmtProgramCode());
		eventDetailsObject.setReferenceNumber(invPmtReconDTO.getReconTxnRefNo());
		eventDetailsObject.setEvent(ReportConstants.RECON_TM_IPR_EVENT);
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final InvPmtReconDTO invPmtReconDTO, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,invPmtReconDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveFinPmtReconData(FinPmtReconDTO finPmtReconDTO, String userId, String branchCode) {
		
		LOGGER.info("Trying to commit Updated Finance Payment Recon data..");
		
		try {
			String[] ignoreFields = { "makerDtStamp", "authDtStamp", "finStartDt", "finEndDt", "finPastDueDt","paymentDate", "reconDate" };
			String[] queryParams = {"finPmtReconId"};
			Object[] paramValues = {finPmtReconDTO.getFinPmtReconId()};
			ObReconFinPmtTxn obReconFinPmtTxn = null;
			List<ObReconFinPmtTxn> obFinPmtReconTxnList = obFinPmtReconRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObReconFinPmtTxn.findById");
			if(obFinPmtReconTxnList != null && !obFinPmtReconTxnList.isEmpty()) {
				obReconFinPmtTxn = obFinPmtReconTxnList.get(0);
			} else {
				obReconFinPmtTxn = new ObReconFinPmtTxn();
			}
			
			BeanUtils.copyProperties(finPmtReconDTO, obReconFinPmtTxn,ignoreFields);
			obReconFinPmtTxn.setAuthDtStamp(dateConverter(finPmtReconDTO.getAuthDtStamp()));
			obReconFinPmtTxn.setMakerDtStamp(dateConverter(finPmtReconDTO.getMakerDtStamp()));
			obReconFinPmtTxn.setFinEndDt(dateConverter(finPmtReconDTO.getFinEndDt()));
			obReconFinPmtTxn.setFinPastDueDt(dateConverter(finPmtReconDTO.getFinPastDueDt()));
			obReconFinPmtTxn.setFinStartDt(dateConverter(finPmtReconDTO.getFinStartDt()));
			obReconFinPmtTxn.setPaymentDate(dateConverter(finPmtReconDTO.getPaymentDate()));
			obReconFinPmtTxn.setReconDate(dateConverter(finPmtReconDTO.getReconDate()));
			obFinPmtReconRepository.save(obReconFinPmtTxn);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(finPmtReconDTO, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(finPmtReconDTO, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Finance Payment Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final FinPmtReconDTO finPmtReconDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setProgramId(finPmtReconDTO.getPmtProgramCode());
		eventDetailsObject.setReferenceNumber(finPmtReconDTO.getReconTxnRefNo());
		eventDetailsObject.setEvent(ReportConstants.RECON_TM_FPR_EVENT);
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final FinPmtReconDTO finPmtReconDTO, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,finPmtReconDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}
	
	private Date dateConverter(String inputDate) throws ParseException {

		DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
		if(inputDate == null) {
			return null;
		}
		return dateFormat.parse(inputDate);
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean savePmtAllocData(PmtAllocDTO pmtAllocDTO, String userId, String branchCode) {
        LOGGER.info("START savePmtAllocData...");
		
		try {
			String[] ignoreFields = { "authDtStamp", "paymentDate", "reconDate"};
			String[] queryParams = {"reconAllocPmtId"};
			Object[] paramValues = {pmtAllocDTO.getReconAllocPmtId()};
			ObReconAllocPmtTxn obReconAllocPmtTxn = null;
			List<ObReconAllocPmtTxn> obReconAllocPmtTxnList = obReconAllocPmtTxnRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObReconAllocPmtTxn.findById");
			if(obReconAllocPmtTxnList != null && !obReconAllocPmtTxnList.isEmpty()) {
				obReconAllocPmtTxn = obReconAllocPmtTxnList.get(0);
			} else {
				obReconAllocPmtTxn = new ObReconAllocPmtTxn();
			}
			BeanUtils.copyProperties(pmtAllocDTO, obReconAllocPmtTxn, ignoreFields);
			obReconAllocPmtTxn.setAuthDtStamp(dateConverter(pmtAllocDTO.getAuthDtStamp()));
			obReconAllocPmtTxn.setPaymentDate(dateConverter(pmtAllocDTO.getPaymentDate()));
			obReconAllocPmtTxn.setReconDate(dateConverter(pmtAllocDTO.getReconDate()));
			obReconAllocPmtTxnRepository.save(obReconAllocPmtTxn);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(pmtAllocDTO, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(pmtAllocDTO, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Payment Allocation Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}

	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final PmtAllocDTO pmtAllocDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setRelationshipCode(pmtAllocDTO.getRelationshipCode());
		eventDetailsObject.setProgramId(pmtAllocDTO.getProgramCode());
		eventDetailsObject.setReferenceNumber(pmtAllocDTO.getReconTxnRefNo());
		eventDetailsObject.setEvent(ReportConstants.RECON_TM_VPA_EVENT);
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final PmtAllocDTO pmtAllocDTO, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,pmtAllocDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveInvCnReconData(InvCnReconDTO invCnReconDTO, String userId, String branchCode) {
		LOGGER.info("Trying to commit Updated Invoice Credit Note Recon data..");
		try {
			String[] ignoreFields = {"reconDate","makerDtStamp", "authDtStamp", "cnDate", "cnExpiryDate","procBranchDate","filler1","filler2","invDate","invDueDate"};
			String[] queryParams = {"invCnReconId"};
			Object[] paramValues = {invCnReconDTO.getInvCnReconId()};
			ObReconInvCnTxn obReconInvCnTxn = null;
			List<ObReconInvCnTxn> obInvCnReconTxnList = obInvCnReconRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObReconInvCnTxn.findById");
			if(obInvCnReconTxnList != null && !obInvCnReconTxnList.isEmpty()) {
				obReconInvCnTxn = obInvCnReconTxnList.get(0);
			} else {
				obReconInvCnTxn = new ObReconInvCnTxn();
			}
			BeanUtils.copyProperties(invCnReconDTO, obReconInvCnTxn,ignoreFields);
			obReconInvCnTxn.setReconDate(dateConverter(invCnReconDTO.getReconDate()));
			obReconInvCnTxn.setMakerDtStamp(dateConverter(invCnReconDTO.getMakerDtStamp()));
			obReconInvCnTxn.setAuthDtStamp(dateConverter(invCnReconDTO.getAuthDtStamp()));
			obReconInvCnTxn.setCnDate(dateConverter(invCnReconDTO.getCnDate()));
			obReconInvCnTxn.setCnExpiryDate(dateConverter(invCnReconDTO.getCnExpiryDate()));
			obReconInvCnTxn.setProcBranchDate(dateConverter(invCnReconDTO.getProcBranchDate()));
			obReconInvCnTxn.setFiller1(dateConverter(invCnReconDTO.getFiller1()));
			obReconInvCnTxn.setFiller2(dateConverter(invCnReconDTO.getFiller2()));
			obReconInvCnTxn.setInvDate(dateConverter(invCnReconDTO.getInvDate()));
			obReconInvCnTxn.setInvDueDate(dateConverter(invCnReconDTO.getInvDueDate()));
			obInvCnReconRepository.save(obReconInvCnTxn);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(invCnReconDTO, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(invCnReconDTO, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Invoice Credit Note Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final InvCnReconDTO invCnReconDTO, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject.setRelationshipCode(invCnReconDTO.getCnRelationCode());
		eventDetailsObject.setProgramId(invCnReconDTO.getCnProgramCode());
		eventDetailsObject.setReferenceNumber(invCnReconDTO.getReconTxnRefNo());
		eventDetailsObject.setEvent(ReportConstants.RECON_TM_ICR_EVENT);
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final InvCnReconDTO invCnReconDTO, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,invCnReconDTO);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveReconTwData(ReconTxnDto reconTxnDto, String userId, String branchCode) {
		try {
			String[] queryParams = {"id"};
			Object[] paramValues = {reconTxnDto.getId().toString()};
			ObReconExecTxnTw obReconExecTxnTw = null;
			List<ObReconExecTxnTw> obReconTxnTwList = obReconExecTxnTwRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObReconExecTxnTw.findById");
			if(obReconTxnTwList != null && !obReconTxnTwList.isEmpty()) {
				obReconExecTxnTw = obReconTxnTwList.get(0);
			} else {
				obReconExecTxnTw = new ObReconExecTxnTw();
			}
			obReconExecTxnTw = ReportingUtil.setValuesFromDto(reconTxnDto, obReconExecTxnTw);
			obReconExecTxnTwRepository.save(obReconExecTxnTw);
			EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
			DecisioningWrapperDTO decisioningDetailsResponseWrapper= getDecisioningDetailsResponseWrapper(reconTxnDto, eventDetailsObject, userId, branchCode);
			if(null != decisioningDetailsResponseWrapper && CollectionUtils.isNotEmpty(decisioningDetailsResponseWrapper.getDetailsCollection())) {
				generateAlert(reconTxnDto, decisioningDetailsResponseWrapper, eventDetailsObject, ReportConstants.RECON_CATEGORY, userId, branchCode);
			}
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Recon Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
	private DecisioningWrapperDTO getDecisioningDetailsResponseWrapper(final ReconTxnDto reconTxnDto, EventDetailsDTO eventDetailsObject, String userId, String branchCode) {
		eventDetailsObject = ReportingUtil.setReconTwEventDetails(reconTxnDto, eventDetailsObject);
		eventDetailsObject.setReferenceNumber(reconTxnDto.getReconTxnRefNo());
		eventDetailsObject.setApplicationCode(ReportConstants.APPLICATION_CODE);
		return alertDecisionClient.getDecisioningList(eventDetailsObject, userId, branchCode);
	}
	
	private void generateAlert(final ReconTxnDto reconTxnDto, DecisioningWrapperDTO decisionDetailswrapper, EventDetailsDTO eventDetailsObject,String alertCategory, String userId, String branchCode) {
		List<Object> placeholder=alertList.getparams(alertCategory);
		List<ParameterDTO> newparameterList = alertList.setParams(placeholder,reconTxnDto);
		List<DecisioningDetailsDTO> alertDecisionList = decisionDetailswrapper.getDetailsCollection();
		
		if(CollectionUtils.isNotEmpty(newparameterList)) {
			for(DecisioningDetailsDTO alertDecision :alertDecisionList) {
				if(alertDecision.getAlertFrequency().equals(ReportConstants.ALERT_FREQUENCY_ONETIME)) {
					processAlertMessage(eventDetailsObject, alertDecision, newparameterList, userId, branchCode);
				}
			}
		}
	}
	
	private void processAlertMessage(EventDetailsDTO eventDetailsObject, DecisioningDetailsDTO alertDecision, List<ParameterDTO> parameterList, String... extraHeaders) {
		LOGGER.info("start processAlertMessage()");
		eventDetailsObject.setParameters(parameterList);
		List<UserDTO> userList = new ArrayList<>();
		String userId = null;
		String branchCode = null;
		if(extraHeaders != null && extraHeaders.length > 0) {
			userId = extraHeaders[0];
			branchCode = extraHeaders[1];
		}
		UserDTO userDtoObject = new UserDTO();
		userDtoObject.setUserId("this value needs to be populated");
		userDtoObject.setEmailId(alertDecision.getAdditionalDetails());
		userDtoObject.setPhone(alertDecision.getAdditionalDetails());
		userList.add(userDtoObject);
		eventDetailsObject.setUsers(userList);
		try{
			alertMessageProcessor.processAlertMessage(eventDetailsObject,alertDecision, userId, branchCode);
		} catch (Exception e) {
			LOGGER.error(e.getMessage());
		}
	}
	
}
