package oracle.fsgbu.obscfcm.services.report.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.stream.annotation.EnableBinding;
import org.springframework.cloud.stream.annotation.StreamListener;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Service;

import avro.oracle.fsgbu.obscfcm.services.relationship.dto.UpdateRelationshipMasterDto;
import oracle.fsgbu.obscfcm.services.report.client.ObscfcmCoreServiceClient;
import oracle.fsgbu.obscfcm.services.report.client.RelationshipEventServiceClient;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.kafka.PlatoMessageChannels;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipMasterDTO;
import oracle.fsgbu.obscfcm.services.scfcm.dto.RelationshipmasterWebModel;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
@EnableBinding(PlatoMessageChannels.class)
public class RelationshipCloudStreamListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(RelationshipCloudStreamListenerService.class);

	@Autowired
	private RelationshipEventServiceClient relationshipEventServiceClient;
	
	@Autowired
	private RelationshipEventListenerService relationshipEventListenerService;
	
	@Autowired
	ObscfcmCoreServiceClient obscfcmCoreServiceClient;

	@StreamListener(ReportConstants.RELATIONSHIP_MASTER_CHNL)
	public void getRelationshipData(Message<UpdateRelationshipMasterDto> message) {

		LOGGER.info("TM Message Received on channel relationshipMaster Event");
		UpdateRelationshipMasterDto updatedRelationship = message.getPayload();
		if (updatedRelationship != null) {
			String userId = updatedRelationship.getUserId();
			String branchCode = updatedRelationship.getServiceBranchCode();
			String id = updatedRelationship.getId();
			LOGGER.info("id :: {} ",id);
			LOGGER.info("branchCode :: {} ",branchCode);
			RelationshipmasterWebModel  relationshipmasterWebModel = obscfcmCoreServiceClient.getByResourceId(id, branchCode, userId);
			ObcmTmRelationshipMasterDTO dto=relationshipmasterWebModel.getObcmTmRelationshipMaster();
			LOGGER.info("ObcmTmRelationshipMasterDTO ::: {} ",dto);
			relationshipEventListenerService.saveRelationshipData(dto);
			relationshipEventListenerService.saveRelationshipTwData(dto);
		}
	}
	@StreamListener(ReportConstants.RELATIONSHIP_TW_MASTER_CHNL)
	public void getRelationshipTwData(Message<UpdateRelationshipMasterDto> message) {

		LOGGER.info("TW Message Received from  channel relationshipTwMaster.....");
		UpdateRelationshipMasterDto updatedRelationship = message.getPayload();
		if (updatedRelationship != null) {
			String userId = updatedRelationship.getUserId();
			String branchCode = updatedRelationship.getServiceBranchCode();
			String id = updatedRelationship.getId();
			LOGGER.info("id :: {} ",id);
			LOGGER.info("userId :: {} ",userId);
			LOGGER.info("branchCode :: {} ",branchCode);
			RelationshipmasterWebModel  relationshipmasterWebModel = obscfcmCoreServiceClient.getByResourceId(id, branchCode, userId);
			LOGGER.info("relationshipmasterWebModel :: {} ",relationshipmasterWebModel);
			ObcmTmRelationshipMasterDTO dto=relationshipmasterWebModel.getObcmTmRelationshipMaster();
			dto.setId(id);
			relationshipEventListenerService.saveRelationshipTwData(dto);
		}
	}
}