package oracle.fsgbu.obscfcm.services.report.service;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTmRelationshipMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTwRelationshipMaster;
import oracle.fsgbu.obscfcm.services.report.persistence.ObcmTmRelationshipMasterRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObcmTwRelationshipMasterRepository;
import oracle.fsgbu.obscfcm.services.report.util.MapperUtil;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipMasterDTO;
import oracle.fsgbu.plato.logger.PlatoLogger;

@Service
public class RelationshipEventListenerServiceImpl implements RelationshipEventListenerService {

	private static final PlatoLogger LOGGER = new PlatoLogger(RelationshipEventListenerServiceImpl.class);
	
	@Autowired
	private ObcmTmRelationshipMasterRepository relationshipRepository;
	
	@Autowired
	private ObcmTwRelationshipMasterRepository obcmTwRelationshipMasterRepository;
	
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveRelationshipData(ObcmTmRelationshipMasterDTO relationshipDto) {
		LOGGER.info("Saving  Relationship Data ObcmTmRelationshipMasterDTO:::{}",relationshipDto);
		try {
			String[] queryParams = {"id"};
			Object[] paramValues = {relationshipDto.getId()};
			ObcmTmRelationshipMaster obcmTmRelationshipMaster = null;
			List<ObcmTmRelationshipMaster> obcmTmRelationshipMasterList = relationshipRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObcmTmRelationshipMaster.findByPk");
			if(obcmTmRelationshipMasterList != null && !obcmTmRelationshipMasterList.isEmpty()) {
				obcmTmRelationshipMaster = obcmTmRelationshipMasterList.get(0);
			} else {
				obcmTmRelationshipMaster = new ObcmTmRelationshipMaster();
			}
			LOGGER.info("obcmTmRelationshipMaster :{}",obcmTmRelationshipMaster);
			obcmTmRelationshipMaster = MapperUtil.INSTANCE.toEntity(relationshipDto);
			ObcmTmRelationshipMaster saveRelationship = ReportingUtil.convertTmRelationshipDtoToEntity(relationshipDto, obcmTmRelationshipMaster);
			relationshipRepository.merge(saveRelationship);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Relationship Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public Boolean saveRelationshipTwData(ObcmTmRelationshipMasterDTO relationshipDto) {
		LOGGER.info("Saving  Relationship TW Data....");
		LOGGER.info("ObcmTmRelationshipMasterDTO:::{}",relationshipDto);
		try {
		
			String[] queryParams = {"id","modNo"};
			Integer modNumber =Integer.parseInt(relationshipDto.getModNo()==null?"0":relationshipDto.getModNo());
			Object[] paramValues = {relationshipDto.getId(),modNumber};
			LOGGER.info("Id:::{}",relationshipDto.getId());
			LOGGER.info("Mod:::{}",modNumber);
			ObcmTwRelationshipMaster obcmTwRelationshipMaster = null;
			List<ObcmTwRelationshipMaster> obcmTwRelationshipMasterList = obcmTwRelationshipMasterRepository.executeNamedQueryByCondition(queryParams, paramValues, 
					"ObcmTwRelationshipMaster.findByPk");
			if(obcmTwRelationshipMasterList != null && !obcmTwRelationshipMasterList.isEmpty()) {
				obcmTwRelationshipMaster = obcmTwRelationshipMasterList.get(0);
			} else {
				obcmTwRelationshipMaster = new ObcmTwRelationshipMaster();
			}
			obcmTwRelationshipMaster = MapperUtil.INSTANCE.toTwEntity(relationshipDto);
			 obcmTwRelationshipMaster = ReportingUtil.convertTwRelationshipDtoToEntity(relationshipDto, obcmTwRelationshipMaster);
			obcmTwRelationshipMasterRepository.merge(obcmTwRelationshipMaster);
			return true;
		} catch(Exception e) {
			LOGGER.error("Error while commiting Relationship Data:: {}", e.getMessage(), e);
			return false;
		}
	}
	
}
