package oracle.fsgbu.obscfcm.services.report.service;

import oracle.fsgbu.gus.common.dto.PagingDto;
import oracle.fsgbu.gus.common.utils.GenericUtility;
import oracle.fsgbu.gus.common.utils.QueryOperands;
import oracle.fsgbu.gus.core.client.dto.CoreBranchCollection;
import oracle.fsgbu.gus.core.client.dto.CoreBranchDTO;
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient;
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClient;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto;
import oracle.fsgbu.obscfcm.services.report.persistence.InvoiceRepository;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.obscfcm.services.report.web.ReportCreationServiceController;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reportsTransaction.api.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
@Service
public class ReportCreationServiceImpl implements ReportCreationService {

    private static final PlatoLogger LOGGER = new PlatoLogger(ReportCreationServiceImpl.class);
    private static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd";
    private static final String DATE_FORMAT = "dd-MMM-yyyy";
    private static final String DB_DATE_FORMAT = "dd-MMM-yy";
    private static final String BET_DATE_AND_CLAUSE = "' AND '";
    private static final String UNION_CLAUSE = "UNION ALL ";
    private static final String INVOICE_STATUS_CLAUSE = "WHERE INVOICE_STATUS NOT IN ('" + ReportConstants.INVOICE_STATUS_CANCELLED + "', '" + ReportConstants.INVOICE_STATUS_REJECTED + "') ";
    private static final String PAYMENT_STATUS_CLAUSE = "AND PAYMENT_STATUS NOT IN ('" + ReportConstants.PAYMENT_STATUS_PAID;
    private static final String BUYER_ID_FROM_CLAUSE = "FROM SCFCM_INVOICE INVOICE INNER JOIN (SELECT DISTINCT CORPORATE_ID, PRODUCT FROM OBCM_TM_RELATIONSHIP) REL ON REL.CORPORATE_ID = BUYER_ID ";
    private static final String SUPPLIER_ID_FROM_CLAUSE = "FROM SCFCM_INVOICE INVOICE INNER JOIN (SELECT DISTINCT CORPORATE_ID, PRODUCT FROM OBCM_TM_RELATIONSHIP) REL ON REL.CORPORATE_ID = SUPPLIER_ID ";
    private static final String BUYER_ID_INV_FROM_CLAUSE = " FROM SCFCM_INVOICE INVOICE, OBCM_TM_RELATIONSHIP rel, ob_invoice_fin_parameters_tm finparam WHERE INVOICE.BUYER_ID  = REL.CORPORATE_ID AND  INVOICE.id = finparam.invoice_master_id and REL.relationship_code = INVOICE.relation_code  and invoice.supplier_id = rel.counter_party_id ";
    private static final String SUPPLIER_ID_INV_FROM_CLAUSE = " FROM SCFCM_INVOICE INVOICE, OBCM_TM_RELATIONSHIP rel, ob_invoice_fin_parameters_tm finparam WHERE INVOICE.SUPPLIER_ID  = REL.CORPORATE_ID AND  INVOICE.id = finparam.invoice_master_id and REL.relationship_code = INVOICE.relation_code and invoice.BUYER_ID = rel.counter_party_id ";
    private static final String INV_STATUS_CLAUSE = " AND INVOICE_STATUS NOT IN ('" + ReportConstants.INVOICE_STATUS_CANCELLED + "', '" + ReportConstants.INVOICE_STATUS_REJECTED + "') ";
    private static final String FROM_SLAB = "fromSlab";
    private static final String TO_SLAB = "toSlab";
    private InvoiceRepository invoiceRepository;
    private CmcCurrencyServiceClient cmcCurrencyServiceClient;
    private CmcBranchServiceClient cmcBranchServiceClient;
   
    @Autowired
	private GenericUtility genericUtility;
	
    @Autowired
    public ReportCreationServiceImpl(InvoiceRepository invoiceRepository,
                                     CmcCurrencyServiceClient cmcCurrencyServiceClient,
                                     CmcBranchServiceClient cmcBranchServiceClient) {

        this.invoiceRepository = invoiceRepository;
        this.cmcCurrencyServiceClient = cmcCurrencyServiceClient;
        this.cmcBranchServiceClient = cmcBranchServiceClient;
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public RaisedInvoicesResponseDto getRaisedInvoices(String userId, String branchCode, String fromDate, String toDate, String currency) {

        LOGGER.info("Getting dashboard data for raised invoices");
        try {
            List<String> monthsList = new ArrayList<>();
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            DateFormat dbDateFormat = new SimpleDateFormat(DATE_FORMAT);
            RaisedInvoicesResponseDto response = new RaisedInvoicesResponseDto();
            String branchCcy = null;
            int intFromMonth = 0;
            int intToMonth = 0;
            int year = 0;
            int nextYear = 0;
            String branchDate = cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode);
            if (currency == null) {
                branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            } else {
                branchCcy = currency;
            }
            if (fromDate == null && toDate == null) {
                toDate = branchDate;
                intToMonth = Integer.parseInt(toDate.substring(5, 7));
                Calendar cal = Calendar.getInstance();
                cal.setTime(dateFormat.parse(toDate));
                cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
                toDate = dbDateFormat.format(cal.getTime());
                cal.add(Calendar.MONTH, -6);
                cal.add(Calendar.DATE, 1);
                fromDate = dateFormat.format(cal.getTime());
                intFromMonth = Integer.parseInt(fromDate.substring(5, 7));
                year = Integer.parseInt(fromDate.substring(0, 4));
                fromDate = dbDateFormat.format(cal.getTime());
            } else if (fromDate != null && toDate != null) {
                intFromMonth = Integer.parseInt(fromDate.substring(5, 7));
                intToMonth = Integer.parseInt(toDate.substring(5, 7));
                year = Integer.parseInt(fromDate.substring(0, 4));
                fromDate = dbDateFormat.format(dateFormat.parse(fromDate));
                toDate = dbDateFormat.format(dateFormat.parse(toDate));
            }

            if (intToMonth < intFromMonth) {
                intToMonth = intToMonth + 12;
                nextYear = year + 1;
            }

            getMonthsList(intFromMonth, intToMonth, year, nextYear, monthsList);

            String raisedInvQuery =
                    "SELECT to_char(inv.invoice_date, 'YYYY-MM') AS month, SUM(finparam.finance_amount) AS invoice_amount, inv.currency, 'Y' AS is_financed " +
                            "FROM  scfcm_invoice inv , ob_invoice_fin_parameters_tm finparam ,OBCM_TM_RELATIONSHIP_MASTER RELM, OBCM_TM_RELATIONSHIP_DETAILS RELD " +
                            "WHERE  inv.id = finparam.INVOICE_MASTER_ID AND inv.relation_code= RELM.relationship_code AND RELM.ID = RELD.RELATIONSHIP_MASTER_ID "+
                            "AND ( RELM.corporate_id = inv.buyer_id  OR RELM.corporate_id = inv.supplier_id ) and ( RELD.COUNTERPARTYID = inv.buyer_id OR RELD.COUNTERPARTYID = inv.supplier_id ) " +
                            "AND inv.INVOICE_DATE BETWEEN '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' AND finparam.finance_status IN ('FINANCED', 'PARTIAL_FINANCED') "+
                            "GROUP BY TO_CHAR(inv.INVOICE_DATE,'YYYY-MM'), inv.CURRENCY " + UNION_CLAUSE +
                            "SELECT to_char(inv.invoice_date, 'YYYY-MM') AS month, SUM(inv.net_invoice_amount) AS invoice_amount, inv.currency, 'N' AS is_financed " +
                            "FROM scfcm_invoice inv, ob_invoice_fin_parameters_tm finparam, OBCM_TM_RELATIONSHIP_MASTER RELM, OBCM_TM_RELATIONSHIP_DETAILS RELD " +
                            "WHERE  inv.id = finparam.INVOICE_MASTER_ID AND inv.relation_code= RELM.relationship_code AND RELM.ID = RELD.RELATIONSHIP_MASTER_ID "+
                            "AND ( RELM.corporate_id = inv.buyer_id  OR RELM.corporate_id = inv.supplier_id ) and ( RELD.COUNTERPARTYID = inv.buyer_id OR RELD.COUNTERPARTYID = inv.supplier_id ) " +
                            "AND  INVOICE_DATE BETWEEN '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' AND (finparam.finance_status = 'NOT_FINANCED' OR finparam.finance_status IS NULL) "+
                            "GROUP BY TO_CHAR(inv.INVOICE_DATE,'YYYY-MM'), inv.CURRENCY " + UNION_CLAUSE +
                            "SELECT to_char(inv.invoice_date, 'YYYY-MM') AS month, (SUM(inv.net_invoice_amount) - SUM(finparam.finance_amount)) AS invoice_amount, inv.currency, 'N' AS is_financed " +
                            "FROM  scfcm_invoice inv , ob_invoice_fin_parameters_tm finparam ,OBCM_TM_RELATIONSHIP_MASTER RELM, OBCM_TM_RELATIONSHIP_DETAILS RELD  " +
                            "WHERE  inv.id = finparam.INVOICE_MASTER_ID AND inv.relation_code= RELM.relationship_code AND RELM.ID = RELD.RELATIONSHIP_MASTER_ID "+
                            "AND ( RELM.corporate_id = inv.buyer_id  OR RELM.corporate_id = inv.supplier_id ) and ( RELD.COUNTERPARTYID = inv.buyer_id OR RELD.COUNTERPARTYID = inv.supplier_id ) " +
                            "AND inv.INVOICE_DATE BETWEEN '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' AND finparam.finance_status = 'PARTIAL_FINANCED' "+
                            "GROUP BY TO_CHAR(inv.INVOICE_DATE,'YYYY-MM'), inv.CURRENCY " +
                            "ORDER BY MONTH";

            LOGGER.info("raisedInvQuery:: {}", raisedInvQuery);

            List<Object[]> resultSet = invoiceRepository.executeNativeQuery(raisedInvQuery);

            response.setBranchCcy(branchCcy);
            response.setRaisedInvoices(populateRaisedInvoiceDetails(resultSet, branchCcy, userId, branchCode, branchDate, monthsList));
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getRaisedInvoices:: {}", e.getMessage(), e);
            return null;
        }
    }

    private void getMonthsList(int intFromMonth, int intToMonth, int year, int nextYear, List<String> monthsList) {

        for (int i = intFromMonth; i <= intToMonth; i++) {

            if (i > 12) {
                String month = (i - 12) < 10 ? String.valueOf("0" + (i - 12)) : String.valueOf((i - 12));
                monthsList.add(String.valueOf(nextYear) + "-" + month);
            } else {
                String month = i < 10 ? String.valueOf("0" + i) : String.valueOf(i);
                monthsList.add(String.valueOf(year) + "-" + month);
            }
        }
    }

    private List<RaisedInvDetails> populateRaisedInvoiceDetails(List<Object[]> raisedInvList, String branchCcy, String userId,
                                                                String branchCode, String branchDate, List<String> monthsList) {

        Map<String, RaisedInvDetails> raisedInvoicesMap = new TreeMap<>();
        Map<String, String> monthsMap = ReportingUtil.getMonthsMap();
        raisedInvList.stream()
                .forEach(resultSet -> {
                    RaisedInvDetails invDetail = null;

                    if (resultSet[0] != null && raisedInvoicesMap.get(resultSet[0]) != null) {
                        invDetail = raisedInvoicesMap.get(resultSet[0]);
                        invCurrencyChk(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                    } else if (raisedInvoicesMap.get(resultSet[0]) == null) {
                        invDetail = new RaisedInvDetails();
                        DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);

                        Calendar cal = Calendar.getInstance();
                        int year = Integer.parseInt(((String) resultSet[0]).substring(0, 4));
                        int month = Integer.parseInt(((String) resultSet[0]).substring(5, 7));
                        monthsList.remove(resultSet[0]);
                        cal.set(year, month - 1, 1);
                        String startDate = dateFormat.format(new Date(
                                new GregorianCalendar(year, month - 1,
                                        cal.getActualMinimum(Calendar.DAY_OF_MONTH)).getTime().getTime()));
                        String endDate = dateFormat.format(new Date(
                                new GregorianCalendar(year, month - 1,
                                        cal.getActualMaximum(Calendar.DAY_OF_MONTH)).getTime().getTime()));

                        invDetail.setMonth(monthsMap.get(((String) resultSet[0]).substring(5, 7)).concat("-" + ((String) resultSet[0]).substring(0, 4)));
                        invDetail.setStartDate(startDate);
                        invDetail.setEndDate(endDate);
                        invCurrencyChk(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                        raisedInvoicesMap.put((String) resultSet[0], invDetail);
                    }
                });

        List<RaisedInvDetails> raisedInvoiceList = new ArrayList<>();
        raisedInvoicesMap.entrySet().stream()
                .forEach(entry -> {
                    RaisedInvDetails raisedInv = entry.getValue();
                    if (raisedInv.getFinancedInvAmount() == null) {
                        raisedInv.setFinancedInvAmount(new BigDecimal(0));
                    }
                    if (raisedInv.getNonFinancedInvAmount() == null) {
                        raisedInv.setNonFinancedInvAmount(new BigDecimal(0));
                    }
                    raisedInvoiceList.add(entry.getValue());
                });

        monthsList.stream()
                .forEach(monthEntry -> {
                    RaisedInvDetails invDetail = new RaisedInvDetails();
                    DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);

                    Calendar cal = Calendar.getInstance();
                    int year = Integer.parseInt(monthEntry.substring(0, 4));
                    int month = Integer.parseInt(monthEntry.substring(5, 7));
                    cal.set(year, month - 1, 1);
                    String startDate = dateFormat.format(new Date(
                            new GregorianCalendar(year, month - 1,
                                    cal.getActualMinimum(Calendar.DAY_OF_MONTH)).getTime().getTime()));
                    String endDate = dateFormat.format(new Date(
                            new GregorianCalendar(year, month - 1,
                                    cal.getActualMaximum(Calendar.DAY_OF_MONTH)).getTime().getTime()));

                    invDetail.setMonth(monthsMap.get(monthEntry.substring(5, 7)).concat("-" + monthEntry.substring(0, 4)));
                    invDetail.setStartDate(startDate);
                    invDetail.setEndDate(endDate);
                    invDetail.setFinancedInvAmount(new BigDecimal(0));
                    invDetail.setNonFinancedInvAmount(new BigDecimal(0));
                    raisedInvoiceList.add(invDetail);
                });
        return raisedInvoiceList;
    }

    private void invCurrencyChk(String localCcy, Object[] resultSet, RaisedInvDetails invDetails,
                                String branchCode, String userId, String businessDate) {

        BigDecimal invAmount = (BigDecimal) resultSet[1];
        String invCcy = (String) resultSet[2];
        String isFinanced = (String) resultSet[3];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(localCcy) && invAmount != null && !invAmount.equals(new BigDecimal(0))) {
            currencyUtilityDto =
                    cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, localCcy, invAmount,
                            businessDate, userId, branchCode);
            setInvoiceAmounts(isFinanced, invDetails, currencyUtilityDto.getAmount());
        } else if (invAmount != null) {
            setInvoiceAmounts(isFinanced, invDetails, invAmount);
        }
    }

    private void setInvoiceAmounts(String isFinanced, RaisedInvDetails invDetails, BigDecimal invAmount) {
        if (isFinanced.equalsIgnoreCase("Y")) {
            invDetails.setFinancedInvAmount(invAmount.add(invDetails.getFinancedInvAmount() != null ?
                    invDetails.getFinancedInvAmount() : new BigDecimal(0)));
        } else {
            invDetails.setNonFinancedInvAmount(invAmount.add(invDetails.getNonFinancedInvAmount() != null ?
                    invDetails.getNonFinancedInvAmount() : new BigDecimal(0)));
        }
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public AgingInvoicesResponseDto getAgingInvoices(String userId, String branchCode, Integer offset, Integer limit) {
        LOGGER.info("Getting Aging Invoices branchCode :: {} ",branchCode);
        try {
            int nRecords = 20;
            int totalCount = 0;
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            AgingInvoicesResponseDto response = new AgingInvoicesResponseDto();
            String branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            Date currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
            LOGGER.info("Getting Aging Invoices currentBusinessDate :: {} ",currentBusinessDate);
            String simpleBranchDate = dateFormat.format(currentBusinessDate);    //yyyy-MM-DD format for currency conversion service

            String agingInvQuery = "SELECT PAYMENT_DUE_DATE, SUM(OUTSTANDING_AMOUNT), CURRENCY FROM SCFCM_INVOICE WHERE "
                    + " INVOICE_STATUS NOT IN ('" + ReportConstants.INVOICE_STATUS_CANCELLED + "', '" + ReportConstants.INVOICE_STATUS_REJECTED + "')"
                    + " AND PAYMENT_STATUS NOT IN ('" + ReportConstants.PAYMENT_STATUS_PAID + "') AND PAYMENT_DUE_DATE IS NOT NULL "
                    + " GROUP BY PAYMENT_DUE_DATE ,CURRENCY";
            LOGGER.info("Getting Aging Invoices agingInvQuery :: {} ",agingInvQuery);
            List<Object[]> resultSetAllInv = invoiceRepository.executeNativeQuery(agingInvQuery);
            List<AgingInvDetails> agingInvoicesAll = populateAgingInvoicesDetails(resultSetAllInv, branchCcy, userId, branchCode, currentBusinessDate, simpleBranchDate, false);
            if (null == offset && null == limit) {
                offset = 0;
                limit = nRecords;
            }
            List<AgingInvDetails> agingInvoices = subAgingInvoiceList(agingInvoicesAll, offset, limit);
            response.setAgingInvoices(agingInvoices);

            if (agingInvoices != null && !agingInvoices.isEmpty()) {
                LOGGER.info("agingInvoices size :: {}",agingInvoices.size());
                if (offset == 0 && agingInvoices.size() < limit) {
                    totalCount = agingInvoices.size();
                    LOGGER.info("totalCount if :: {}",totalCount);
                } else {
                    totalCount = agingInvoicesAll.size();
                    LOGGER.info("totalCount else:: {}",totalCount);
                }
            }

            response.setOverdueAgingInvoices(new ArrayList<>());
            response.setCurrency(branchCcy);

            Integer initialOffset = offset;
            Integer initialLimit = limit;
            String[] params = {};
            Object[] values = {};
            QueryOperands[] operator = {};
            List<String> qParams = new ArrayList<String>();
            List<Object> qValues = new ArrayList<Object>();
            List<QueryOperands> operands = new ArrayList<QueryOperands>();

            int nextoff = 0, nextlim = 0, preoff = 0, prelim = 0, pagesize = nRecords;
            nextoff = offset + pagesize;
            nextlim = pagesize;
            if (offset != 0) {
                preoff = (offset - nRecords) < 0 ? 0 : offset - nRecords;
                prelim = limit;
            }
            if (initialOffset != null) {
                nextoff = initialOffset + initialLimit;
                if (offset != 0) {
                    preoff = (initialOffset - initialLimit) < 0 ? 0 : initialOffset - initialLimit;
                }
            }
            if (initialLimit != null) {
                nextlim = initialLimit;
                prelim = initialLimit;
            }
            String baseUrinext = "/web/v1/agingInvoices?" + "offset=" + nextoff + "&limit=" + nextlim;
            String baseUriprev = "/web/v1/agingInvoices?" + "offset=" + preoff + "&limit=" + prelim;


            if (params != null && operator != null && values != null ) {
                for (int i = 0; i < params.length; i++) {
                    baseUrinext = baseUrinext + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                    baseUriprev = baseUriprev + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                }
            }

            PagingDto pagination = new PagingDto();
            if (agingInvoices != null && agingInvoices.size() > 0)
                pagination.setTotalResults(totalCount);
            else
                pagination.setTotalResults(0);
            pagination.getLinks().setNext(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUrinext).withRel("next"));
            pagination.getLinks().setPrev(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUriprev).withRel("prev"));

            if (offset == 0 && limit == 0) {
                pagination.getLinks().setPrev(null);
                pagination.getLinks().setNext(null);
            } else if (offset == 0) {
                baseUriprev = null;
                pagination.getLinks().setPrev(null);
            }
            if (agingInvoices.isEmpty() || nextoff == totalCount || agingInvoices.size() < limit) {
                baseUrinext = null;
                pagination.getLinks().setNext(null);
            }
            response.setPaging(pagination);
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getAgingInvoices:: {}", e.getMessage(), e);
            return null;
        }
    }

    private String getSlabDates(Integer slab, Date currentBusinessDate) {

        DateFormat dbDateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
        Calendar cal = Calendar.getInstance();
        cal.setTime(currentBusinessDate);
        cal.add(Calendar.DATE, slab);
        return dbDateFormat.format(cal.getTime());
    }

    private void setAgingSlabDates(AgingInvDetails invDetail, Map<String, Integer> slabLimitsMap, Date currentBusinessDate) {
     LOGGER.info("inside setAgingSlabDates");
        if (slabLimitsMap.get(FROM_SLAB) != null && slabLimitsMap.get(TO_SLAB) != null) {
            invDetail.setFromDate(getSlabDates(slabLimitsMap.get(FROM_SLAB), currentBusinessDate));
            invDetail.setToDate(getSlabDates(slabLimitsMap.get(TO_SLAB), currentBusinessDate));
        } else if (slabLimitsMap.get(FROM_SLAB) != null) {
            invDetail.setFromDate(getSlabDates(slabLimitsMap.get(FROM_SLAB), currentBusinessDate));
        }
    }

    private void getSlabLimits(String slab, Map<String, Integer> slabLimitsMap) {
    	LOGGER.info("inside getSlabLimits");
    	slabLimitsMap.clear();
        if (slab.contains("-")) {
            String[] slabs = slab.split("-");
            slabLimitsMap.put(FROM_SLAB, Integer.valueOf(slabs[0]));
            slabLimitsMap.put(TO_SLAB, Integer.valueOf(slabs[1]));
        } else if (slab.contains(">")) {
            slabLimitsMap.put(FROM_SLAB, Integer.valueOf(slab.substring(1)));
        }
    }

    private List<AgingInvDetails> populateAgingInvoicesDetails(List<Object[]> queryResultList, String branchCcy,
                                                               String userId, String branchCode, Date currentBusinessDate, String simpleBranchDate, Boolean isOverdue) {
    	LOGGER.info("inside populateAgingInvoicesDetails branchCcy :: {}",branchCcy);
        Map<String, AgingInvDetails> agingInvoicesMap = new TreeMap<>();
        Map<String, Integer> slabLimitsMap = new HashMap<>();
        queryResultList.stream()
                .forEach(resultSet -> {
                    AgingInvDetails invDetail = null;
                    String slab = ReportingUtil.getSlabs((Date) resultSet[0], currentBusinessDate, isOverdue);
                    LOGGER.info("slab :: {}",slab);
                    if (agingInvoicesMap.get(slab) != null) {
                        invDetail = agingInvoicesMap.get(slab);
                        getAmountInLocalCurrency(branchCcy, resultSet, invDetail, branchCode, userId, simpleBranchDate);
                    } else {
                        getSlabLimits(slab, slabLimitsMap);
                        invDetail = new AgingInvDetails();
                        invDetail.setSlab(slab);
                        if (!slab.equalsIgnoreCase("Overdue")) {
                            setAgingSlabDates(invDetail, slabLimitsMap, currentBusinessDate);
                        }
                        getAmountInLocalCurrency(branchCcy, resultSet, invDetail, branchCode, userId, simpleBranchDate);
                        agingInvoicesMap.put(invDetail.getSlab(), invDetail);
                    }
                });
        List<AgingInvDetails> agingInvoiceList = new ArrayList<>();
        agingInvoicesMap.entrySet().stream()
                .forEach(entry -> {
                    AgingInvDetails agingInv = entry.getValue();
                    if (agingInv.getOutstandingAmount() == null) {
                        agingInv.setOutstandingAmount(new BigDecimal(0));
                    }
                    agingInvoiceList.add(entry.getValue());
                });
        LOGGER.info("leaving populateAgingInvoicesDetails agingInvoiceList :: {}",agingInvoiceList);
        return agingInvoiceList;
    }

    private void getAmountInLocalCurrency(String branchCcy, Object[] resultSet, AgingInvDetails invDetail,
                                          String branchCode, String userId, String simpleBranchDate) {
    	LOGGER.info("inside getAmountInLocalCurrency");
        BigDecimal outstandingAmount = (BigDecimal) resultSet[1];
        String invCcy = (String) resultSet[2];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(branchCcy) && outstandingAmount != null && !outstandingAmount.equals(new BigDecimal(0))) {
            currencyUtilityDto = cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, branchCcy, outstandingAmount,
                    simpleBranchDate, userId, branchCode);
            try {
                if (currencyUtilityDto != null) {
                	LOGGER.info("inside currencyUtilityDto");
                    invDetail.setOutstandingAmount(currencyUtilityDto.getAmount().add(invDetail.getOutstandingAmount() != null ?
                            invDetail.getOutstandingAmount() : new BigDecimal(0)));
                }
            } catch (Exception e) {
                LOGGER.error("Error in cmcCurrencyServiceClient-getConvertedCcyValue: {}", e.getMessage());
            }


        } else if (outstandingAmount != null) {
            invDetail.setOutstandingAmount(outstandingAmount.add(invDetail.getOutstandingAmount() != null ?
                    invDetail.getOutstandingAmount() : new BigDecimal(0)));
        }
    }


    @Override
    @Transactional("PLATO_TX_MANAGER")
    public AgingInvoicesResponseDto getOverdueAgingInvoices(String userId, String branchCode) {
        LOGGER.info("Getting overdue aging Invoices..");
        try {
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);

            AgingInvoicesResponseDto response = new AgingInvoicesResponseDto();
            String branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            Date currentBranchDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
            List<Object[]> resultSet = invoiceRepository.getOverdueInvoices(currentBranchDate);
            response.setAgingInvoices(new ArrayList<>());
            response.setOverdueAgingInvoices(populateOverdueAgingInvoicesDetails(resultSet, branchCcy, userId, branchCode, currentBranchDate));
            response.setCurrency(branchCcy);
            return response;

        } catch (Exception e) {
            LOGGER.error("Exception in getOverdueAgingInvoices: {}", e.getMessage(), e);
            return null;
        }
    }
    private List<AgingInvDetails> populateOverdueAgingInvoicesDetails(List<Object[]> resultSet, String branchCcy,
                                                                      String userId, String branchCode, Date currentBranchDate) {
        Map<String, AgingInvDetails> agingInvoicesMap = new TreeMap<>();
        DateFormat dateFormat = new SimpleDateFormat(ReportConstants.SIMPLE_DATE_FORMAT);
        String simpleBranchDate = dateFormat.format(currentBranchDate);
        resultSet.stream().forEach(rset -> {
            AgingInvDetails invDetail = new AgingInvDetails();
            String slab = (String) rset[0];
            LOGGER.info("Start OverdueAgingInvoices processing for slab:{}", slab);
            if (agingInvoicesMap.get(slab) != null) {
                invDetail = agingInvoicesMap.get(slab);
                getAmountInLocalCurrency(branchCcy, rset, invDetail, branchCode, userId, simpleBranchDate);
            } else {
                invDetail = new AgingInvDetails();
                getAmountInLocalCurrency(branchCcy, rset, invDetail, branchCode, userId, simpleBranchDate);
                invDetail.setSlab(slab);
                invDetail.setFromDate(((Timestamp) rset[3]).toLocalDateTime().toLocalDate().toString());
                invDetail.setToDate(((Timestamp) rset[4]).toLocalDateTime().toLocalDate().toString());
                agingInvoicesMap.put(slab, invDetail);
            }
        });
        List<AgingInvDetails> agingInvoiceList = new ArrayList<>();
        LOGGER.info("agingInvoicesMap values={}",agingInvoicesMap);
        agingInvoicesMap.entrySet().stream().forEach(entry -> {
            AgingInvDetails agingInv = entry.getValue();
            if (agingInv.getOutstandingAmount() == null) {
                agingInv.setOutstandingAmount(new BigDecimal(0));
            }
            agingInvoiceList.add(entry.getValue());
        });
        LOGGER.info("leaving populateAgingInvoicesDetails agingInvoiceList :: {}", agingInvoiceList);
        return agingInvoiceList;
    }
    @Override
    @Transactional("PLATO_TX_MANAGER")
    public PayableReceivableResponseDto getPayableReceivables(String userId, String branchCode) {

        LOGGER.info("Getting dashboard data for payables & receivables");
        try {
            PayableReceivableResponseDto response = new PayableReceivableResponseDto();
            Set<String> customerList = new HashSet<>();
            String branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            String branchDate = cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode);

            String raisedInvQuery =
                    "SELECT A.CUSTOMER_ID, A.CUSTOMER, SUM(A.TOTAL_OUTSTANDING_AMOUNT) AS OUTSTANDING_AMT, A.CURRENCY, A.PAYABLE_RECEIVABLE FROM " +
                            "(SELECT BUYER_ID AS CUSTOMER_ID, BUYER_NAME AS CUSTOMER, SUM(OUTSTANDING_AMOUNT) AS TOTAL_OUTSTANDING_AMOUNT, CURRENCY, 'PAYABLE' AS PAYABLE_RECEIVABLE FROM SCFCM_INVOICE " +
                            "WHERE INVOICE_STATUS NOT IN ('REJECTED','CANCELLED') AND PAYMENT_STATUS NOT IN ('PAID') GROUP BY BUYER_ID, BUYER_NAME, CURRENCY " + UNION_CLAUSE +
                            "SELECT SUPPLIER_ID AS CUSTOMER_ID, SUPPLIER_NAME AS CUSTOMER, SUM(OUTSTANDING_AMOUNT) AS TOTAL_OUTSTANDING_AMOUNT, CURRENCY, 'RECEIVABLE' AS PAYABLE_RECEIVABLE FROM SCFCM_INVOICE " +
                            "WHERE INVOICE_STATUS NOT IN ('REJECTED','CANCELLED') AND PAYMENT_STATUS NOT IN ('PAID') " +
                            "GROUP BY SUPPLIER_ID, SUPPLIER_NAME, CURRENCY) A " +
                            "GROUP BY A.CUSTOMER_ID, A.CUSTOMER, A.CURRENCY, A.PAYABLE_RECEIVABLE " +
                            "ORDER BY OUTSTANDING_AMT DESC ";
            List<Object[]> resultSet = invoiceRepository.executeNativeQuery(raisedInvQuery);
            LOGGER.info("resultSet size :{}",resultSet.size());
            List<Object[]> top10CustResultSet = new ArrayList<>();

            for (Object[] result : resultSet) {
                if (customerList.size() < 10) {
                    customerList.add((String) result[1]);
                    top10CustResultSet.add(result);
                }
            }
            response.setBranchCcy(branchCcy);
            response.setCustomerDetails(populateCustomerDetails(top10CustResultSet, branchCcy, userId, branchCode, branchDate));
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getPayableReceivables:: {}", e.getMessage(), e);
            return null;
        }
    }

    private List<CustomerDetails> populateCustomerDetails(List<Object[]> resultSet, String branchCcy, String userId, String branchCode, String branchDate) {

        Map<String, CustomerDetails> customerDetailsMap = new TreeMap<>();
        resultSet.stream()
                .forEach(result -> {
                    CustomerDetails customerDetails = null;

                    if (result[0] != null && customerDetailsMap.get(result[0]) != null) {
                        customerDetails = customerDetailsMap.get(result[0]);
                        currencyChk(branchCcy, result, customerDetails, branchCode, userId, branchDate);
                    } else if (customerDetailsMap.get(result[0]) == null) {
                        customerDetails = new CustomerDetails();
                        customerDetails.setCustomerName((String) result[1]);
                        customerDetails.setCustomerId((String) result[0]);
                        currencyChk(branchCcy, result, customerDetails, branchCode, userId, branchDate);
                        customerDetailsMap.put((String) result[0], customerDetails);
                    }
                });

        List<CustomerDetails> customerDetailsList = new ArrayList<>();
        customerDetailsMap.entrySet().stream()
                .forEach(entry -> {
                    CustomerDetails customerDetails = entry.getValue();
                    if (customerDetails.getPayableAmount() == null) {
                        customerDetails.setPayableAmount(new BigDecimal(0));
                    }
                    if (customerDetails.getReceivableAmount() == null) {
                        customerDetails.setReceivableAmount(new BigDecimal(0));
                    }
                    customerDetails.setTotalAmount(customerDetails.getReceivableAmount().add(customerDetails.getPayableAmount()));
                    customerDetailsList.add(customerDetails);
                });

        return customerDetailsList;
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public BusinessTrendResponseDto getBusinessTrend(String userId, String branchCode) {
        LOGGER.info("Getting business volume trend");
        try {
            List<String> monthsList = new ArrayList<>();
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            DateFormat dbDateFormat = new SimpleDateFormat(DATE_FORMAT);
            BusinessTrendResponseDto response = new BusinessTrendResponseDto();
            String branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            int intFromMonth = 0;
            int intToMonth = 0;
            int year = 0;
            int nextYear = 0;
            String branchDate = cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode);
            String toDate = branchDate;
            intToMonth = Integer.parseInt(toDate.substring(5, 7));
            Calendar cal = Calendar.getInstance();
            cal.setTime(dateFormat.parse(toDate));
            cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
            toDate = dbDateFormat.format(cal.getTime());
            cal.add(Calendar.MONTH, -6);
            int numberOfDays = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
            cal.add(Calendar.DATE, numberOfDays == 30 ? 1 : 2);
            String fromDate = dateFormat.format(cal.getTime());
            intFromMonth = Integer.parseInt(fromDate.substring(5, 7));
            year = Integer.parseInt(fromDate.substring(0, 4));
            fromDate = dbDateFormat.format(cal.getTime());

            if (intToMonth < intFromMonth) {
                intToMonth = intToMonth + 12;
                nextYear = year + 1;
            }

            for (int i = intFromMonth; i <= intToMonth; i++) {
                if (i > 12) {
                    String month = String.valueOf("0" + (i - 12));
                    monthsList.add(String.valueOf(nextYear) + "-" + month);
                } else {
                    String month = i < 10 ? String.valueOf("0" + i) : String.valueOf(i);
                    monthsList.add(String.valueOf(year) + "-" + month);
                }
            }

            String fetchQuery = "SELECT temp.MONTH, SUM(temp.TOTAL_OUTSTANDING_AMOUNT), temp.CURRENCY, TEMP.IS_RECEIVABLE FROM "
                    + " (SELECT TO_CHAR(INVOICE_DATE,'YYYY-MM') AS MONTH, BUYER_ID AS CUSTOMER_ID, BUYER_NAME AS CUSTOMER, "
                    + " SUM(OUTSTANDING_AMOUNT) AS TOTAL_OUTSTANDING_AMOUNT, CURRENCY, 'N' AS IS_RECEIVABLE FROM SCFCM_INVOICE  "
                    + " INNER JOIN OBCM_TM_RELATIONSHIP_MASTER REL on REL.CORPORATE_ID = BUYER_ID "
                    + " WHERE INVOICE_STATUS NOT IN ('" + ReportConstants.INVOICE_STATUS_REJECTED + "','" + ReportConstants.INVOICE_STATUS_CANCELLED + "') "
                    + " AND INVOICE_DATE BETWEEN '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' AND REL.PRODUCT IN ('B','P')"
                    + " GROUP BY TO_CHAR(INVOICE_DATE,'YYYY-MM'), BUYER_ID, BUYER_NAME, CURRENCY "
                    + " UNION ALL"
                    + " SELECT TO_CHAR(INVOICE_DATE,'YYYY-MM') AS MONTH, SUPPLIER_ID AS CUSTOMER_ID, SUPPLIER_NAME AS CUSTOMER, "
                    + " SUM(OUTSTANDING_AMOUNT) AS TOTAL_OUTSTANDING_AMOUNT, CURRENCY, 'Y' AS IS_RECEIVABLE FROM SCFCM_INVOICE  "
                    + " INNER JOIN OBCM_TM_RELATIONSHIP_MASTER REL on REL.CORPORATE_ID = SUPPLIER_ID "
                    + " WHERE INVOICE_STATUS NOT IN ('" + ReportConstants.INVOICE_STATUS_REJECTED + "','" + ReportConstants.INVOICE_STATUS_CANCELLED + "') "
                    + " AND INVOICE_DATE BETWEEN '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' AND REL.PRODUCT in('B','R')"
                    + " GROUP BY TO_CHAR(INVOICE_DATE,'YYYY-MM'), SUPPLIER_ID, SUPPLIER_NAME, CURRENCY "
                    + " ORDER BY CUSTOMER) temp GROUP BY  temp.MONTH, temp.CURRENCY, TEMP.IS_RECEIVABLE ";
            List<Object[]> resultSet = invoiceRepository.executeNativeQuery(fetchQuery);
            response.setBusinessTrend(populateDataForBuinesstrend(resultSet, branchCcy, userId, branchCode, branchDate, monthsList));
            response.setCurrency(branchCcy);
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getBusinessTrend:: {} {}", e.getMessage(), e);
            return null;
        }
    }

    private List<BusinessTrendDetails> populateDataForBuinesstrend(List<Object[]> queryResultList, String branchCcy,
                                                                   String userId, String branchCode, String branchDate, List<String> monthsList) {

        Map<String, BusinessTrendDetails> businessTrendMap = new TreeMap<>();
        Map<String, String> monthsMap = ReportingUtil.getMonthsMap();
        queryResultList.stream()
                .forEach(resultSet -> {
                    BusinessTrendDetails invDetail;
                    if (resultSet[0] != null && businessTrendMap.get(resultSet[0]) != null) {
                        invDetail = businessTrendMap.get(resultSet[0]);
                        invCurrencyConvert(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                    } else if (businessTrendMap.get(resultSet[0]) == null) {
                        invDetail = new BusinessTrendDetails();
                        monthsList.remove(resultSet[0]);
                        invDetail.setMonth(monthsMap.get(((String) resultSet[0]).substring(5, 7)).concat("-" + ((String) resultSet[0]).substring(0, 4)));
                        invCurrencyConvert(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                        businessTrendMap.put((String) resultSet[0], invDetail);
                    }
                });

        List<BusinessTrendDetails> businessTrendList = new ArrayList<>();
        businessTrendMap.entrySet().stream()
                .forEach(entry -> {
                    BusinessTrendDetails raisedInv = entry.getValue();
                    if (raisedInv.getReceivableAmount() == null) {
                        raisedInv.setReceivableAmount(new BigDecimal(0));
                    }
                    if (raisedInv.getPayableAmount() == null) {
                        raisedInv.setPayableAmount(new BigDecimal(0));
                    }
                    if (raisedInv.getTotalAmount() == null) {
                        raisedInv.setTotalAmount(new BigDecimal(0));
                    }
                    businessTrendList.add(entry.getValue());
                });

        monthsList.stream()
                .forEach(monthEntry -> {
                    BusinessTrendDetails invDetail = new BusinessTrendDetails();
                    invDetail.setMonth(monthsMap.get(monthEntry.substring(5, 7)).concat("-" + monthEntry.substring(0, 4)));
                    invDetail.setReceivableAmount(new BigDecimal(0));
                    invDetail.setPayableAmount(new BigDecimal(0));
                    invDetail.setTotalAmount(new BigDecimal(0));
                    businessTrendList.add(invDetail);
                });
        return businessTrendList;
    }

    private void invCurrencyConvert(String localCcy, Object[] resultSet, BusinessTrendDetails invDetails,
                                    String branchCode, String userId, String businessDate) {
        BigDecimal invAmount = (BigDecimal) resultSet[1];
        String invCcy = (String) resultSet[2];
        String isReceivable = (String) resultSet[3];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(localCcy) && invAmount != null && !invAmount.equals(new BigDecimal(0))) {
            currencyUtilityDto =
                    cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, localCcy, invAmount,
                            businessDate, userId, branchCode);
            if(currencyUtilityDto != null)
            setAmounts(isReceivable, invDetails, currencyUtilityDto.getAmount());
        } else if (invAmount != null) {
            setAmounts(isReceivable, invDetails, invAmount);
        }
    }

    private void setAmounts(String isReceivable, BusinessTrendDetails invDetails, BigDecimal invAmount) {
        if (isReceivable.equalsIgnoreCase("Y")) {
            invDetails.setReceivableAmount(invAmount.add(invDetails.getReceivableAmount() != null ?
                    invDetails.getReceivableAmount() : new BigDecimal(0)));
            invDetails.setTotalAmount(invDetails.getReceivableAmount().add(invDetails.getPayableAmount() != null ?
                    invDetails.getPayableAmount() : new BigDecimal(0)));
        } else {
            invDetails.setPayableAmount(invAmount.add(invDetails.getPayableAmount() != null ?
                    invDetails.getPayableAmount() : new BigDecimal(0)));
            invDetails.setTotalAmount(invDetails.getPayableAmount().add(invDetails.getReceivableAmount() != null ?
                    invDetails.getReceivableAmount() : new BigDecimal(0)));
        }
    }

    private void currencyChk(String localCcy, Object[] resultSet, CustomerDetails invDetails,
                             String branchCode, String userId, String businessDate) {

        BigDecimal outstandingAmt = (BigDecimal) resultSet[2];
        String invCcy = (String) resultSet[3];
        String isPayableReceivable = (String) resultSet[4];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(localCcy) && outstandingAmt != null && !outstandingAmt.equals(new BigDecimal(0))) {
            currencyUtilityDto =
                    cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, localCcy, outstandingAmt,
                            businessDate, userId, branchCode);
            if(currencyUtilityDto != null)
            setOutstandingAmounts(isPayableReceivable, invDetails, currencyUtilityDto.getAmount());
        } else if (outstandingAmt != null) {
            setOutstandingAmounts(isPayableReceivable, invDetails, outstandingAmt);
        }
    }

    private void setOutstandingAmounts(String isPayableReceivable, CustomerDetails customerDetails, BigDecimal outstandingAmt) {
        if (isPayableReceivable.equalsIgnoreCase("PAYABLE")) {
            customerDetails.setPayableAmount(outstandingAmt.add(customerDetails.getPayableAmount() != null ?
                    customerDetails.getPayableAmount() : new BigDecimal(0)));
        } else {
            customerDetails.setReceivableAmount(outstandingAmt.add(customerDetails.getReceivableAmount() != null ?
                    customerDetails.getReceivableAmount() : new BigDecimal(0)));
        }
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public RaisedInvoiceDetailsResponseDto getRaisedInvoiceDetails(String appId, String userId, String branchCode,
                                                                   String fromDate, String toDate, String isFinanced, String currency,Integer offset,Integer limit) {

        LOGGER.info("Getting dashboard details for raised invoices details");
        try {
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            DateFormat dbDateFormat = new SimpleDateFormat(DATE_FORMAT);
            RaisedInvoiceDetailsResponseDto response = new RaisedInvoiceDetailsResponseDto();
            String branchCcy = null;
            String branchDate = cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode);
            int nRecords = 20;
            int totalCount = 0;
            List<String> qParams = new ArrayList<>();
            List<Object> qValues = new ArrayList<>();
            List<QueryOperands> operands = new ArrayList<>();
            if(StringUtils.hasText(currency)) {
                qParams.add("currency");
                qValues.add(currency);
                operands.add(QueryOperands.EQUALTO);
            }
            if(StringUtils.hasText(fromDate)) {
                qParams.add("fromDate");
                qValues.add(fromDate);
                operands.add(QueryOperands.EQUALTO);
            }
            if(StringUtils.hasText(toDate)) {
                qParams.add("toDate");
                qValues.add(toDate);
                operands.add(QueryOperands.EQUALTO);
            }
            if(StringUtils.hasText(isFinanced)) {
                qParams.add("isFinanced");
                qValues.add(isFinanced);
                operands.add(QueryOperands.EQUALTO);
            }
            if (null == offset && null == limit) {
                offset = 0;
                limit = nRecords;
            }

            if (currency == null) {
                branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);
            } else {
                branchCcy = currency;
            }
            toDate = dbDateFormat.format(dateFormat.parse(toDate));
            fromDate = dbDateFormat.format(dateFormat.parse(fromDate));


            String raisedInvDetailsQuery = "" ;
            String invoiceSelectQuery = "SELECT CASE WHEN rel.corporate_id = invoice.buyer_id THEN buyer_id ELSE supplier_id END AS customer_id, CASE WHEN rel.corporate_id = invoice.buyer_id THEN buyer_name ELSE supplier_name END AS customer, ";
            String invoiceSelectQuery1 = ", currency FROM scfcm_invoice invoice, obcm_tm_relationship rel, ob_invoice_fin_parameters_tm finparam ";
            
            String invoiceSelectQuery2 = " AND invoice.invoice_date BETWEEN  '" + fromDate + BET_DATE_AND_CLAUSE + toDate + "' " +
            		" and INVOICE.INVOICE_STATUS NOT IN ('REJECTED','CANCELLED') AND ( rel.corporate_id = invoice.buyer_id OR rel.corporate_id = invoice.supplier_id ) AND invoice.id = finparam.invoice_master_id and rel.relationship_code = invoice.relation_code   AND ( rel.counter_party_id = invoice.buyer_id OR rel.counter_party_id = invoice.supplier_id ) GROUP BY CASE WHEN rel.corporate_id = invoice.buyer_id THEN buyer_id ELSE supplier_id END, CASE WHEN rel.corporate_id = invoice.buyer_id THEN buyer_name ELSE supplier_name END, currency ";
            if (isFinanced.equalsIgnoreCase("Y")) {
            	raisedInvDetailsQuery =
                        "SELECT a.customer_id, a.customer, SUM(a.invoice_amount), a.currency FROM( " +
                        		invoiceSelectQuery +
                        		" SUM(finparam.finance_amount) AS invoice_amount " +
                        		invoiceSelectQuery1 +
                        		" WHERE finparam.finance_status IN ( 'FINANCED', 'PARTIAL_FINANCED') "
                        		+ invoiceSelectQuery2 +
                         "  ) a GROUP BY a.customer_id, a.customer, a.currency";
            	LOGGER.info("raisedInvDetailsQuery for financed:: {}",raisedInvDetailsQuery);
            	
            } else {
                  
                raisedInvDetailsQuery =
                "SELECT a.customer_id, a.customer, SUM(a.invoice_amount), a.currency FROM( " +
                		invoiceSelectQuery +
                		" SUM(invoice.net_invoice_amount) AS invoice_amount " +
                		invoiceSelectQuery1 +
                		"WHERE ( finparam.finance_status = 'NOT_FINANCED'  OR finparam.finance_status IS NULL  ) "
                		+ invoiceSelectQuery2 +
                	    UNION_CLAUSE +
                		invoiceSelectQuery +
                		"( SUM(invoice.net_invoice_amount) - SUM(finparam.finance_amount) ) AS invoice_amount " +
                		invoiceSelectQuery1 +
                		"WHERE finparam.finance_status = 'PARTIAL_FINANCED' "
                		+ invoiceSelectQuery2 +
                 "  ) a GROUP BY a.customer_id, a.customer, a.currency";
                LOGGER.info("raisedInvDetailsQuery for non financed:: {}",raisedInvDetailsQuery);
         
            }
            List<Object[]> resultSet = invoiceRepository.executeNativeQuery(raisedInvDetailsQuery);
            response.setBranchCcy(branchCcy);
            List<CustomerDetails> customerDetailsAll = populateInvoiceDetails(resultSet, branchCcy, userId, branchCode, branchDate);
            List<CustomerDetails> customerDetails = subCustomerDetailsList(customerDetailsAll, offset, limit);
            response.setCustomerDetails(customerDetails);

            if (customerDetails != null && !customerDetails.isEmpty()) {
                LOGGER.info("customerDetails size :: {}",customerDetails.size());
                if (offset == 0 && customerDetails.size() < limit) {
                    totalCount = customerDetails.size();
                    LOGGER.info("totalCount if :: {}",totalCount);
                } else {
                    totalCount = customerDetailsAll.size();
                    LOGGER.info("totalCount else:: {}",totalCount);
                }
            }
            Integer initialOffset = offset;
            Integer initialLimit = limit;
            String[] params = {};
            Object[] values = {};
            QueryOperands[] operator = {};

            if (qParams != null && qParams.size() > 0) {
                params = qParams.toArray(new String[qParams.size()]);
                values = qValues.toArray(new Object[qValues.size()]);
                operator = operands.toArray(new QueryOperands[operands.size()]);
            }

            int nextoff = 0, nextlim = 0, preoff = 0, prelim = 0, pagesize = nRecords;
            nextoff = offset + pagesize;
            nextlim = pagesize;
            if (offset != 0) {
                preoff = (offset - nRecords) < 0 ? 0 : offset - nRecords;
                prelim = limit;
            }
            if (initialOffset != null) {
                nextoff = initialOffset + initialLimit;
                if (offset != 0) {
                    preoff = (initialOffset - initialLimit) < 0 ? 0 : initialOffset - initialLimit;
                }
            }
            if (initialLimit != null) {
                nextlim = initialLimit;
                prelim = initialLimit;
            }
            String baseUrinext = "/web/v1/raisedInvoiceDetails?" + "offset=" + nextoff + "&limit=" + nextlim;
            String baseUriprev = "/web/v1/raisedInvoiceDetails?" + "offset=" + preoff + "&limit=" + prelim;


            if (params != null && operator != null && values != null ) {
                for (int i = 0; i < params.length; i++) {
                    baseUrinext = baseUrinext + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                    baseUriprev = baseUriprev + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                }
            }

            PagingDto pagination = new PagingDto();
            if (customerDetails != null && customerDetails.size() > 0)
                pagination.setTotalResults(totalCount);
            else
                pagination.setTotalResults(0);
            pagination.getLinks().setNext(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUrinext).withRel("next"));
            pagination.getLinks().setPrev(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUriprev).withRel("prev"));

            if (offset == 0 && limit == 0) {
                pagination.getLinks().setPrev(null);
                pagination.getLinks().setNext(null);
            } else if (offset == 0) {
                baseUriprev = null;
                pagination.getLinks().setPrev(null);
            }
            if (customerDetails.isEmpty() || nextoff == totalCount || customerDetails.size() < limit) {
                baseUrinext = null;
                pagination.getLinks().setNext(null);
            }
            response.setPaging(pagination);
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getRaisedInvoiceDetails:: {}", e.getMessage(), e);
            return null;
        }
    }

    private List<CustomerDetails> populateInvoiceDetails(List<Object[]> raisedInvList, String branchCcy, String userId,
                                                         String branchCode, String branchDate) {

        Map<String, CustomerDetails> raisedInvoicesMap = new TreeMap<>();
        raisedInvList.stream()
                .forEach(resultSet -> {
                    CustomerDetails invDetail = null;

                    if (resultSet[0] != null && raisedInvoicesMap.get(resultSet[0]) != null) {
                        invDetail = raisedInvoicesMap.get(resultSet[0]);
                        invDetailsCurrencyChk(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                    } else if (raisedInvoicesMap.get(resultSet[0]) == null) {
                        invDetail = new CustomerDetails();

                        invDetail.setCustomerId((String) resultSet[0]);
                        invDetail.setCustomerName((String) resultSet[1]);
                        invDetailsCurrencyChk(branchCcy, resultSet, invDetail, branchCode, userId, branchDate);
                        raisedInvoicesMap.put((String) resultSet[0], invDetail);
                    }
                });

        List<CustomerDetails> raisedInvoiceList = new ArrayList<>();
        raisedInvoicesMap.entrySet().stream()
                .forEach(entry -> raisedInvoiceList.add(entry.getValue()));
        return raisedInvoiceList;
    }

    private void invDetailsCurrencyChk(String localCcy, Object[] resultSet, CustomerDetails invDetails,
                                       String branchCode, String userId, String businessDate) {

        BigDecimal invAmount = (BigDecimal) resultSet[2];
        String invCcy = (String) resultSet[3];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(localCcy) && invAmount != null && !invAmount.equals(new BigDecimal(0))) {
            currencyUtilityDto =
                    cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, localCcy, invAmount,
                            businessDate, userId, branchCode);
            setInvoiceDetailsAmounts(invDetails, currencyUtilityDto.getAmount());
        } else if (invAmount != null) {
            setInvoiceDetailsAmounts(invDetails, invAmount);
        }
    }

    private void setInvoiceDetailsAmounts(CustomerDetails invDetails, BigDecimal invAmount) {
        invDetails.setInvoiceAmount(invAmount.add(invDetails.getInvoiceAmount() != null ?
                invDetails.getInvoiceAmount() : new BigDecimal(0)));
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public AgingInvoiceDetailsResponseDto getAgingInvoiceDetails(String userId, String branchCode, String fromDate, String toDate, Integer offset, Integer limit) {
        LOGGER.info("Getting Aging Invoices details in service");
        try {
        	int nRecords = 20;
            int totalCount = 0;
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            DateFormat dbDateFormat = new SimpleDateFormat(DB_DATE_FORMAT);
            AgingInvoiceDetailsResponseDto response = new AgingInvoiceDetailsResponseDto();

            String invDueDateCondition = null;
            String branchCcy = cmcBranchServiceClient.getBranchCurrency(branchCode);

            Date currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));

            if (fromDate != null && toDate != null) {
                invDueDateCondition = "AND INVOICE_DUE_DATE BETWEEN '" + dbDateFormat.format(dateFormat.parse(fromDate)) + BET_DATE_AND_CLAUSE + dbDateFormat.format(dateFormat.parse(toDate)) + "' ";
            } else if (fromDate != null) {
                invDueDateCondition = "AND INVOICE_DUE_DATE >= '" + dbDateFormat.format(dateFormat.parse(fromDate)) + "' ";
            } else {
                invDueDateCondition = "AND INVOICE_DUE_DATE < '" + dbDateFormat.format(currentBusinessDate) + "' ";
            }

            String agingInvQuery =
                    "SELECT BUYER_ID AS CUSTOMER_ID, BUYER_NAME AS CUSTOMER_NAME, INVOICE_DUE_DATE, SUM(OUTSTANDING_AMOUNT), CURRENCY "
                            + BUYER_ID_FROM_CLAUSE
                            + INVOICE_STATUS_CLAUSE
                            + PAYMENT_STATUS_CLAUSE + "') AND INVOICE_DUE_DATE IS NOT NULL AND REL.PRODUCT IN ('B','P') " + invDueDateCondition
                            + "GROUP BY BUYER_ID, BUYER_NAME, INVOICE_DUE_DATE ,CURRENCY " + UNION_CLAUSE
                            + "SELECT SUPPLIER_ID AS CUSTOMER_ID, SUPPLIER_NAME AS CUSTOMER_NAME, INVOICE_DUE_DATE, SUM(OUTSTANDING_AMOUNT), CURRENCY "
                            + SUPPLIER_ID_FROM_CLAUSE
                            + INVOICE_STATUS_CLAUSE
                            + PAYMENT_STATUS_CLAUSE + "') AND INVOICE_DUE_DATE IS NOT NULL AND REL.PRODUCT IN ('B','R') " + invDueDateCondition
                            + "GROUP BY SUPPLIER_ID, SUPPLIER_NAME, INVOICE_DUE_DATE ,CURRENCY";

            List<Object[]> resultSetAllInv = invoiceRepository.executeNativeQuery(agingInvQuery);
            List<CorporateDueDetails> resultList = populateCustomerDueDetails(resultSetAllInv, branchCcy, userId, branchCode, currentBusinessDate);
            //response.setAgingInvoices(resultList);
            response.setCurrency(branchCcy);
            if (null == offset && null == limit) {
                offset = 0;
                limit = nRecords;
            }
            List<CorporateDueDetails> agingInvoicesDetailsList = subAgingInvoiceDetailsList(resultList, offset, limit);
            response.setAgingInvoices(agingInvoicesDetailsList);

            if (agingInvoicesDetailsList != null && !agingInvoicesDetailsList.isEmpty()) {
                LOGGER.info("agingInvoicesDetailsList size :: {}",agingInvoicesDetailsList.size());
                if (offset == 0 && agingInvoicesDetailsList.size() < limit) {
                    totalCount = agingInvoicesDetailsList.size();
                    LOGGER.info("totalCount if :: {}",totalCount);
                } else {
                    totalCount = resultList.size();
                    LOGGER.info("totalCount else:: {}",totalCount);
                }
            }
            Integer initialOffset = offset;
            Integer initialLimit = limit;
            String[] params = {};
            Object[] values = {};
            QueryOperands[] operator = {};
            List<String> qParams = new ArrayList<String>();
            List<Object> qValues = new ArrayList<Object>();
            List<QueryOperands> operands = new ArrayList<QueryOperands>();

            int nextoff = 0, nextlim = 0, preoff = 0, prelim = 0, pagesize = nRecords;
            nextoff = offset + pagesize;
            nextlim = pagesize;
            if (offset != 0) {
                preoff = (offset - nRecords) < 0 ? 0 : offset - nRecords;
                prelim = limit;
            }
            if (initialOffset != null) {
                nextoff = initialOffset + initialLimit;
                if (offset != 0) {
                    preoff = (initialOffset - initialLimit) < 0 ? 0 : initialOffset - initialLimit;
                }
            }
            if (initialLimit != null) {
                nextlim = initialLimit;
                prelim = initialLimit;
            }
            String baseUrinext = "/web/v1/agingInvoiceDetails?" + "offset=" + nextoff + "&limit=" + nextlim;
            String baseUriprev = "/web/v1/agingInvoiceDetails?" + "offset=" + preoff + "&limit=" + prelim;


            if (params != null && operator != null && values != null ) {
                for (int i = 0; i < params.length; i++) {
                    baseUrinext = baseUrinext + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                    baseUriprev = baseUriprev + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                }
            }

            PagingDto pagination = new PagingDto();
            if (agingInvoicesDetailsList != null && agingInvoicesDetailsList.size() > 0)
                pagination.setTotalResults(totalCount);
            else
                pagination.setTotalResults(0);
            pagination.getLinks().setNext(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUrinext).withRel("next"));
            pagination.getLinks().setPrev(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUriprev).withRel("prev"));

            if (offset == 0 && limit == 0) {
                pagination.getLinks().setPrev(null);
                pagination.getLinks().setNext(null);
            } else if (offset == 0) {
                baseUriprev = null;
                pagination.getLinks().setPrev(null);
            }
            if (agingInvoicesDetailsList.isEmpty() || nextoff == totalCount || agingInvoicesDetailsList.size() < limit) {
                baseUrinext = null;
                pagination.getLinks().setNext(null);
            }
            response.setPaging(pagination);
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getAgingInvoiceDetails:: {}", e.getMessage(), e);
            return null;
        }
    }

    private List<CorporateDueDetails> populateCustomerDueDetails(List<Object[]> resultSet, String branchCcy, String userId, String branchCode, Date currentBusinessDate) {

        Map<String, CorporateDueDetails> customerDetailsMap = new TreeMap<>();
        DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
        String branchDate = dateFormat.format(currentBusinessDate);    //yyyy-MM-DD format for currency conversion service

        for (Object[] result : resultSet) {

            CorporateDueDetails customerDetails = null;

            if (result[0] != null && customerDetailsMap.get(result[0]) != null) {
                customerDetails = customerDetailsMap.get(result[0]);
                dueCurrencyChk(branchCcy, result, customerDetails, branchCode, userId, branchDate);
            } else if (customerDetailsMap.get(result[0]) == null) {
                customerDetails = new CorporateDueDetails();
                customerDetails.setCorporateName((String) result[1]);
                customerDetails.setCorporateId((String) result[0]);
                dueCurrencyChk(branchCcy, result, customerDetails, branchCode, userId, branchDate);
                customerDetailsMap.put((String) result[0], customerDetails);
            }
        }

        List<CorporateDueDetails> customerDetailsList = new ArrayList<>();
        customerDetailsMap.entrySet().stream()
                .forEach(entry -> {
                    CorporateDueDetails customerDetails = entry.getValue();
                    customerDetailsList.add(customerDetails);
                });

        return customerDetailsList;
    }

    private void dueCurrencyChk(String localCcy, Object[] resultSet, CorporateDueDetails invDetails,
                                String branchCode, String userId, String businessDate) {

        BigDecimal outstandingAmt = (BigDecimal) resultSet[3];
        String invCcy = (String) resultSet[4];
        CurrencyUtilityDto currencyUtilityDto = null;
        if (!invCcy.equalsIgnoreCase(localCcy) && outstandingAmt != null && !outstandingAmt.equals(new BigDecimal(0))) {
            currencyUtilityDto =
                    cmcCurrencyServiceClient.getConvertedCcyValue(invCcy, localCcy, outstandingAmt,
                            businessDate, userId, branchCode);
            if(currencyUtilityDto != null && currencyUtilityDto.getAmount() != null) {
            invDetails.setAmountDue(currencyUtilityDto.getAmount().add(invDetails.getAmountDue() != null ? invDetails.getAmountDue() : new BigDecimal(0)));
            }
        } else if (outstandingAmt != null) {
            invDetails.setAmountDue(outstandingAmt.add(invDetails.getAmountDue() != null ? invDetails.getAmountDue() : new BigDecimal(0)));
        }
    }

    @Override
    @Transactional("PLATO_TX_MANAGER")
    public InvoiceDetailsResponseDto getInvoices(String userId, String branchCode,
                                                 String fromDueDate, String toDueDate,
                                                 String fromDate, String toDate, String corporateId, String isFinanced,Integer offset,Integer limit) {
        LOGGER.info("Getting Aging Invoices details in service");
        try {
            DateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            DateFormat dbDateFormat = new SimpleDateFormat(DB_DATE_FORMAT);
            InvoiceDetailsResponseDto response = new InvoiceDetailsResponseDto();

            String invDueDateCondition = null;
            String invDateCondition = null;
            String finStatusCondition = null;
            Date currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
            String invAmount = null;
            String finStatus = "";
            if (fromDate != null && toDate != null) {
                invDateCondition = "AND INVOICE_DATE BETWEEN '" + dbDateFormat.format(dateFormat.parse(fromDate)) + BET_DATE_AND_CLAUSE + dbDateFormat.format(dateFormat.parse(toDate)) + "' ";
            }

            if (fromDueDate != null && toDueDate != null) {
                invDueDateCondition = "AND INVOICE_DUE_DATE BETWEEN '" + dbDateFormat.format(dateFormat.parse(fromDueDate)) + BET_DATE_AND_CLAUSE + dbDateFormat.format(dateFormat.parse(toDueDate)) + "' ";
            } else if (fromDueDate != null) {
                invDueDateCondition = "AND INVOICE_DUE_DATE >= '" + dbDateFormat.format(dateFormat.parse(fromDueDate)) + "' ";
            } else if (fromDate == null && toDate == null) {
                invDueDateCondition = "AND INVOICE_DUE_DATE < '" + dbDateFormat.format(currentBusinessDate) + "' ";
            }
            if("Y".equalsIgnoreCase(isFinanced))
            {
            	finStatusCondition = " AND finparam.finance_status in ('FINANCED','PARTIAL_FINANCED') ";
            	invAmount=" finparam.finance_amount as net_invoice_amount, ";
            	finStatus= "  nvl(finance_status, 'FINANCED') AS finance_status ";
            }
            else
            {
            	finStatusCondition = " AND (finparam.finance_status in ('NOT_FINANCED','PARTIAL_FINANCED') OR finparam.finance_status IS NULL) ";
            	invAmount=" (invoice.net_invoice_amount -  nvl(finparam.finance_amount,0)) as net_invoice_amount , ";
            	finStatus= "  'NOT_FINANCED' AS finance_status ";
            	
            }
            String selectQuery =
                    "SELECT INVOICE_ID, INVOICE_NUMBER, INVOICE_DUE_DATE, INVOICE.CURRENCY, AMOUNT, BUYER_ID, SUPPLIER_ID, BUYER_NAME, SUPPLIER_NAME, INVOICE.PROGRAM_CODE, TAX_AMOUNT, TOTAL_INVOICE_AMOUNT, PO_NUMBER, "
                            + "INVOICE.INVOICE_REF_NO, INVOICE.PAYMENT_STATUS, REASON_CODE, ACCEPTANCE_AMOUNT, INV_ACCEPTANCE_DATE, INVOICE_STATUS, PAID_AMOUNT, REMARKS, INVOICE_DATE, PRE_ACCEPTED, PAYMENT_TERMS, SHIPMENT_DATE, "
                            + "OUTSTANDING_AMOUNT, PURCHASE_ORDER_DATE, STALE_STATUS, DISCOUNT_ALLOWED, DISCOUNT_PERCENTAGE, DISCOUNT_AMOUNT, TAX_PERCENTAGE, INVOICE.PROGRAM_NAME, HIERARCHY_CODE, SHIPMENT_ADDRESS, SHIPMENT_NO, "
                            + "SHIPMENT_COUNTRY, REASON_FOR_EXPORT, TERMS_OF_SALE, "
                            + invAmount +
                            " INVOICE.FINANCE_AMOUNT, INVOICE.FINANCE_PERCENTAGE, BUYER_DIV_CODE, SUPPLIER_DIV_CODE, BUYER_DIV_NAME, SUPPLIER_DIV_NAME, DIVISION_CODE, IS_DISPUTED, DISCREPANCY_CODE, DISPUTE_RESOLVE_REMARKS, RELATION_CODE, ";
            String invQuery = null;
            if (invDateCondition != null) {
                invQuery =
                        selectQuery
                                + finStatus
                                + BUYER_ID_INV_FROM_CLAUSE
                                + INV_STATUS_CLAUSE
                                + finStatusCondition
                                + invDateCondition
                                + "AND INVOICE.BUYER_ID = '" + corporateId + "'" + UNION_CLAUSE
                                + selectQuery
                                + finStatus
                                + SUPPLIER_ID_INV_FROM_CLAUSE
                                + INV_STATUS_CLAUSE
                                + finStatusCondition
                                + invDateCondition
                                + "AND INVOICE.SUPPLIER_ID = '" + corporateId + "'";
            } else if (invDueDateCondition != null) {
                invQuery =
                        selectQuery
                                + finStatus
                                + BUYER_ID_INV_FROM_CLAUSE
                                + INV_STATUS_CLAUSE
                                + finStatusCondition
                                + invDueDateCondition
                                + "AND INVOICE.BUYER_ID = '" + corporateId + "'" + UNION_CLAUSE
                                + selectQuery
                                + finStatus
                                + SUPPLIER_ID_INV_FROM_CLAUSE
                                + INV_STATUS_CLAUSE
                                + finStatusCondition
                                + invDueDateCondition
                                + "AND INVOICE.SUPPLIER_ID = '" + corporateId + "'";
            }

            
            int nRecords = 20;
            int totalCount = 0;
            String[] params = {};
            Object[] values = {};
            QueryOperands[] operator = {};
            List<String> qParams = new ArrayList<String>();
            List<Object> qValues = new ArrayList<Object>();
            List<QueryOperands> operands = new ArrayList<QueryOperands>();

            Integer initialOffset = offset;
            Integer initialLimit = limit;
            
            if(StringUtils.hasText(fromDueDate)) {
    			qParams.add("fromDueDate");
    			qValues.add(fromDueDate);
    			operands.add(QueryOperands.EQUALTO);
    		}
            
            if(StringUtils.hasText(toDueDate)) {
    			qParams.add("toDueDate");
    			qValues.add(toDueDate);
    			operands.add(QueryOperands.EQUALTO);
    		}
            
            if(StringUtils.hasText(fromDate)) {
    			qParams.add("fromDate");
    			qValues.add(fromDate);
    			operands.add(QueryOperands.EQUALTO);
    		}
            if(StringUtils.hasText(toDate)) {
    			qParams.add("toDate");
    			qValues.add(toDate);
    			operands.add(QueryOperands.EQUALTO);
    		}
            if(StringUtils.hasText(isFinanced)) {
    			qParams.add("isFinanced");
    			qValues.add(isFinanced);
    			operands.add(QueryOperands.EQUALTO);
    		}
           
            if (null == offset && null == limit) {
    			offset = 0;
    			limit = nRecords;
    		}
            
            LOGGER.info("invQuery :: {}",invQuery);
            List<Object[]> resultSetAllInv = invoiceRepository.executeNativeQuery(invQuery);
            List<InvoiceDetailsDto> invoiceList1 = populateInvoiceDetails(resultSetAllInv);
            List<InvoiceDetailsDto> invoiceList = subInvoiceList(invoiceList1, offset, limit);
            response.setInvoices(invoiceList);
            if (invoiceList != null && !invoiceList.isEmpty()) {
            	  LOGGER.info("invoiceList :: {}",invoiceList.size());
                if (offset == 0 && invoiceList.size() < limit) {
                	totalCount = invoiceList.size();
                  LOGGER.info("totalCount if :: {}",totalCount);
                } else {
                	totalCount = invoiceList1.size();
      			LOGGER.info("totalCount else:: {}",totalCount);
                }
              }

            LOGGER.info("totalCount :: {}",totalCount);
            
            if (qParams != null && qParams.size() > 0) {
                params = qParams.toArray(new String[qParams.size()]);
                values = qValues.toArray(new Object[qValues.size()]);
                operator = operands.toArray(new QueryOperands[operands.size()]);
            }
            
            
        	int nextoff = 0, nextlim = 0, preoff = 0, prelim = 0, pagesize = nRecords;
            nextoff = offset + pagesize;
            nextlim = pagesize;
            if (offset != 0) {
                preoff = (offset - nRecords) < 0 ? 0 : offset - nRecords;
                prelim = limit;
            }
            if (initialOffset != null) {
                nextoff = initialOffset + initialLimit;
                if (offset != 0) {
                    preoff = (initialOffset - initialLimit) < 0 ? 0 : initialOffset - initialLimit;
                }
            }
            if (initialLimit != null) {
                nextlim = initialLimit;
                prelim = initialLimit;
            }
            String baseUrinext = "/web/v1/invoiceInquiry?" + "offset=" + nextoff + "&limit=" + nextlim;
            String baseUriprev = "/web/v1/invoiceInquiry?" + "offset=" + preoff + "&limit=" + prelim;

            
            if (params != null && operator != null && values != null ) {
                for (int i = 0; i < params.length; i++) {
                    baseUrinext = baseUrinext + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                    baseUriprev = baseUriprev + "&" + params[i] + genericUtility.toOperator(operator[i]) + values[i];
                }
            }

            PagingDto pagination = new PagingDto();
            if (invoiceList != null && invoiceList.size() > 0)
            	pagination.setTotalResults(totalCount);
    		else
    			pagination.setTotalResults(0);
            pagination.getLinks().setNext(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUrinext).withRel("next"));
            pagination.getLinks().setPrev(
                    WebMvcLinkBuilder.linkTo(ReportCreationServiceController.class).slash(baseUriprev).withRel("prev"));

            if (offset == 0 && limit == 0) {
                pagination.getLinks().setPrev(null);
                pagination.getLinks().setNext(null);
            } else if (offset == 0) {
                baseUriprev = null;
                pagination.getLinks().setPrev(null);
            }
            if (invoiceList.isEmpty() || nextoff == totalCount || invoiceList.size() < limit) {
                baseUrinext = null;
                pagination.getLinks().setNext(null);
            }
            response.setPaging(pagination);
            
            return response;
        } catch (Exception e) {
            LOGGER.error("Exception in getAgingInvoiceDetails:: {}", e.getMessage(), e);
            return null;
        }
    }

    private List<InvoiceDetailsDto> populateInvoiceDetails(List<Object[]> resultSetAllInv) {

        List<InvoiceDetailsDto> invoiceList = new ArrayList<>();
        if (resultSetAllInv != null && !resultSetAllInv.isEmpty()) {

            resultSetAllInv.parallelStream()
                    .forEach(invoice -> {

                        InvoiceDetailsDto invoiceDto = new InvoiceDetailsDto();

                        invoiceDto.setInvoiceId((String) invoice[0]);
                        LOGGER.info("invoice id:: {}"+invoice[0]);
                        invoiceDto.setBuyerDivCode((String) invoice[42]);
                        invoiceDto.setBuyerDivName((String) invoice[44]);
                        invoiceDto.setBuyerId((String) invoice[5]);
                        invoiceDto.setBuyerName((String) invoice[7]);
                        invoiceDto.setSupplierDivCode((String) invoice[43]);
                        invoiceDto.setSupplierDivName((String) invoice[45]);
                        invoiceDto.setSupplierId((String) invoice[6]);
                        invoiceDto.setSupplierName((String) invoice[8]);

                        invoiceDto.setCurrency((String) invoice[3]);
                        invoiceDto.setDiscountAllowed((String) invoice[28]);
                        invoiceDto.setDiscrepancyCode((String) invoice[48]);
                        invoiceDto.setDisputeResolveRemarks((String) invoice[49]);
                        invoiceDto.setDivCode((String) invoice[46]);
                        invoiceDto.setHierarchyCode((String) invoice[33]);
                        invoiceDto.setInvoiceNumber((String) invoice[1]);
                        invoiceDto.setInvoiceRefNo((String) invoice[13]);
                        invoiceDto.setInvoiceStatus((String) invoice[18]);
                        invoiceDto.setIsDisputed((String) invoice[47]);
                        invoiceDto.setPaymentStatus((String) invoice[14]);
                        invoiceDto.setPaymentTerms((String) invoice[23]);
                        invoiceDto.setPoNumber((String) invoice[12]);
                        invoiceDto.setPreAccepted((String) invoice[22]);
                        invoiceDto.setProgramCode((String) invoice[9]);
                        invoiceDto.setProgramName((String) invoice[32]);
                        invoiceDto.setReasonCode((String) invoice[15]);
                        invoiceDto.setReasonForExport((String) invoice[37]);
                        invoiceDto.setRemarks((String) invoice[20]);
                        invoiceDto.setShipmentAddr((String) invoice[34]);
                        invoiceDto.setShipmentCountry((String) invoice[36]);
                        invoiceDto.setShipmentNo((String) invoice[35]);
                        invoiceDto.setStaleStatus((String) invoice[27]);
                        invoiceDto.setTermsForSale((String) invoice[38]);
                        invoiceDto.setRelationCode((String) invoice[50]);

                        populateInvoiceDates(invoiceDto, invoice);

                        invoiceDto.setDiscPct((BigDecimal) invoice[29]);
                        invoiceDto.setDiscAmt((BigDecimal) invoice[30]);
                        invoiceDto.setFinanceableAmount((BigDecimal) invoice[40]);
                        invoiceDto.setFinancePct((BigDecimal) invoice[41]);
                        invoiceDto.setAcceptanceAmt((BigDecimal) invoice[16]);
                        invoiceDto.setAmount((BigDecimal) invoice[4]);
                        invoiceDto.setTotalInvoiceAmount((BigDecimal) invoice[11]);
                        invoiceDto.setTaxAmount((BigDecimal) invoice[10]);
                        invoiceDto.setTaxPct((BigDecimal) invoice[31]);
                        invoiceDto.setNetInvoiceAmount((BigDecimal) invoice[39]);
                        invoiceDto.setOutstandingAmount((BigDecimal) invoice[25]);
                        invoiceDto.setPaidAmt((BigDecimal) invoice[19]);
                        if(invoice[51] != null) {
                        invoiceDto.setFinanceStatus((String)invoice[51]);
                        }
                        invoiceList.add(invoiceDto);
                    });
        }
        return invoiceList;
    }

	private List<AgingInvDetails> subAgingInvoiceList(List<AgingInvDetails> invoiceList, Integer offset,Integer limit) {
		LOGGER.info("subInvoiceList offset :: {} , limit :: {}",offset,limit);
		if (invoiceList != null && !invoiceList.isEmpty()) {
			LOGGER.info("subInvoiceList invoiceList :: {}",invoiceList.size());
            if (offset != null && limit != null) {
                if (invoiceList.size() >= (offset + limit)) {
                	invoiceList = invoiceList.subList(offset, (offset + limit));
                } else if (invoiceList.size() < (offset + limit) && invoiceList.size() > offset) {
                	invoiceList = invoiceList.subList(offset, invoiceList.size());
                }
            }
        }
		return invoiceList;
	}
	
	private List<CorporateDueDetails> subAgingInvoiceDetailsList(List<CorporateDueDetails> agingInvoicesDetailsList, Integer offset,Integer limit){
		LOGGER.info("subAgingInvoiceDetailsList offset :: {} , limit :: {}",offset,limit);
		if (agingInvoicesDetailsList != null && !agingInvoicesDetailsList.isEmpty()) {
			LOGGER.info("subAgingInvoiceDetailsList invoiceList :: {}",agingInvoicesDetailsList.size());
            if (offset != null && limit != null) {
                if (agingInvoicesDetailsList.size() >= (offset + limit)) {
                	agingInvoicesDetailsList = agingInvoicesDetailsList.subList(offset, (offset + limit));
                } else if (agingInvoicesDetailsList.size() < (offset + limit) && agingInvoicesDetailsList.size() > offset) {
                	agingInvoicesDetailsList = agingInvoicesDetailsList.subList(offset, agingInvoicesDetailsList.size());
                }
            }
        }
		return agingInvoicesDetailsList;
	}

    private List<CustomerDetails> subCustomerDetailsList(List<CustomerDetails> customerDetailsList, Integer offset,Integer limit) {
        LOGGER.info("subInvoiceList offset :: {} , limit :: {}",offset,limit);
        if (customerDetailsList != null && !customerDetailsList.isEmpty()) {
            LOGGER.info("subInvoiceList invoiceList :: {}",customerDetailsList.size());
            if (offset != null && limit != null) {
                if (customerDetailsList.size() >= (offset + limit)) {
                    customerDetailsList = customerDetailsList.subList(offset, (offset + limit));
                } else if (customerDetailsList.size() < (offset + limit) && customerDetailsList.size() > offset) {
                    customerDetailsList = customerDetailsList.subList(offset, customerDetailsList.size());
                }
            }
        }
        return customerDetailsList;
    }

    private List<InvoiceDetailsDto> subInvoiceList(List<InvoiceDetailsDto> invoiceList, Integer offset,Integer limit) {
        LOGGER.info("subInvoiceList offset :: {} , limit :: {}",offset,limit);
        if (invoiceList != null && !invoiceList.isEmpty()) {
            LOGGER.info("subInvoiceList invoiceList :: {}",invoiceList.size());
            if (offset != null && limit != null) {
                if (invoiceList.size() >= (offset + limit)) {
                    invoiceList = invoiceList.subList(offset, (offset + limit));
                } else if (invoiceList.size() < (offset + limit) && invoiceList.size() > offset) {
                    invoiceList = invoiceList.subList(offset, invoiceList.size());
                }
            }
        }
        return invoiceList;
    }

    private void populateInvoiceDates(InvoiceDetailsDto invoiceDto, Object[] invoice) {

        if (invoice[17] != null) {
            invoiceDto.setInvoiceAcceptanceDate(((Timestamp) invoice[17]).toLocalDateTime().toLocalDate().toString());
        }
        if (invoice[21] != null) {
            invoiceDto.setInvoiceDate(((Timestamp) invoice[21]).toLocalDateTime().toLocalDate().toString());
        }
        if (invoice[2] != null) {
            invoiceDto.setInvoiceDueDate(((Timestamp) invoice[2]).toLocalDateTime().toLocalDate().toString());
        }
        if (invoice[24] != null) {
            invoiceDto.setShipmentDate(((Timestamp) invoice[24]).toLocalDateTime().toLocalDate().toString());
        }
        if (invoice[26] != null) {
            invoiceDto.setPoDate(((Timestamp) invoice[26]).toLocalDateTime().toLocalDate().toString());
        }
    }
}
