package oracle.fsgbu.obscfcm.services.report.service;

import java.math.BigDecimal;

import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfTmInvoicePo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import oracle.fsgbu.obscfcm.services.report.persistence.ScfTmInvoicePoRepository;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import oracle.fsgbu.reportsService.api.model.InvoicesLinkedToPOCollection;

import oracle.fsgbu.reportsService.api.model.InvoicesLinkedToPO;

import oracle.fsgbu.plato.logger.PlatoLogger;


@Service
public class ReportTransactionServiceImpl implements ReportTransactionService {

    private static final PlatoLogger LOGGER = new PlatoLogger(ReportTransactionServiceImpl.class);
    
	
	private ScfTmInvoicePoRepository scfTmInvoicePoRepository;


    public ReportTransactionServiceImpl(ScfTmInvoicePoRepository scfTmInvoicePoRepository) {
        this.scfTmInvoicePoRepository = scfTmInvoicePoRepository;
        
    }
    
    
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public InvoicesLinkedToPOCollection getInvoicesLinkedToPO(String appId, String userId, String branchCode, List<String> poInternalNoList){
		LOGGER.info("Inside getInvoicesLinkedToPO for PO Numbers : ");

		InvoicesLinkedToPOCollection invoicesLinkedToPOCollection=new InvoicesLinkedToPOCollection();
		if (null!=poInternalNoList && !poInternalNoList.isEmpty()) {
			LOGGER.info(poInternalNoList.toString());
			DateFormat dateFormat = new SimpleDateFormat(ReportConstants.SIMPLE_DATE_FORMAT);

			String[] financeRefNoQueryParam = { "poInternalNoList" };
			Object[] financeRefNoParamValues = { poInternalNoList };
			List<InvoicesLinkedToPO> invoicesLinkedToPOList=new ArrayList<>();
			List<ScfTmInvoicePo>  scfTmInvoicePoList=scfTmInvoicePoRepository.executeNamedQueryByCondition(financeRefNoQueryParam, financeRefNoParamValues,
					"ScfTmInvoicePo.findLinkedInvoices");
			if (scfTmInvoicePoList != null && !scfTmInvoicePoList.isEmpty()) {
				LOGGER.info("Records Fetched");
				scfTmInvoicePoList.stream()
				.forEach(resultSet -> {
					InvoicesLinkedToPO invoicesLinkedToPO  =new InvoicesLinkedToPO();
					BeanUtils.copyProperties( resultSet,invoicesLinkedToPO);
					invoicesLinkedToPO.setSeqNo(BigDecimal.valueOf(resultSet.getSeqNo()));
					invoicesLinkedToPO.setPoDate(dateFormat.format(resultSet.getPoDate()));
					invoicesLinkedToPO.setNetPoAmount(resultSet.getNetPoAmount());
					invoicesLinkedToPO.setInvoiceAllocAmount(resultSet.getInvoiceAllocAmount());
					invoicesLinkedToPO.setInvoiceAllocPercentage(resultSet.getInvoiceAllocPercentage());
					invoicesLinkedToPO.setPoAvailableAmount(resultSet.getPoAvailableAmount());
					invoicesLinkedToPO.setPoAvailableAmountPostLink(resultSet.getPoAvailableAmountPostLink());
					invoicesLinkedToPOList.add(invoicesLinkedToPO);
				});
				invoicesLinkedToPOCollection.setData(invoicesLinkedToPOList);
			}
		}
		LOGGER.info("Exiting getInvoicesLinkedToPO");

		return invoicesLinkedToPOCollection;
	}
}

