package oracle.fsgbu.obscfcm.services.report.util;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.BeanUtils;
import org.springframework.http.HttpHeaders;

import avro.oracle.fsgbu.obscfcm.services.relationship.dto.updateRelationshipDetailsDataDtoList;
import avro.oracle.fsgbu.obscfcm.services.relationship.dto.updateRelationshipDetailsDtoList;
import oracle.fsgbu.gus.persistence.provider.GlobalHolder;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.EventDetailsDTO;
import oracle.fsgbu.obscfcm.services.ob.services.charges.web.api.model.ObChargesTxnDTO;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxn;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconAllocPmtTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconExecTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconExpDrCrPmtTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconFinPmtTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvCnTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvPmtTxnTw;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTmRelationshipDetails;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTmRelationshipDetailsExternalCode;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTmRelationshipMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTwRelationshipDetails;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTwRelationshipDetailsExternalCode;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTwRelationshipMaster;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipDetailsDTO;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipDetailsExternalCodeDTO;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipMasterDTO;
import oracle.fsgbu.plato.core.persistence.provider.PlatoHolder;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reconService.api.model.AllocPayReconDto;
import oracle.fsgbu.reconService.api.model.ExpDrCrPayReconDto;
import oracle.fsgbu.reconService.api.model.FinPayReconDto;
import oracle.fsgbu.reconService.api.model.ReconInvCnDto;
import oracle.fsgbu.reconService.api.model.ReconInvPmtDto;
import oracle.fsgbu.reconService.api.model.ReconTxnDto;

public class ReportingUtil {

	private static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd";
	private static final String DATE_FORMAT = "dd-MMM-yyyy";
	private static final String NULL = "null";
    private static ThreadLocal<DateFormat> dateFormat = new ThreadLocal<DateFormat>() {
        private final DateFormat dateFormat = new SimpleDateFormat(DEFAULT_DATE_FORMAT);

        @Override
        protected DateFormat initialValue() {
            return dateFormat;
        }
    };
	private static final PlatoLogger LOGGER = new PlatoLogger(ReportingUtil.class);

	private ReportingUtil() {
		
	}
	
	public static String setDateFormat(Date date) {
	    if (date == null) {
	      return null;
	    }
	    return dateFormat.get().format(date);
	}
	
	public static String converDateFormat(String convertDate) throws ParseException {
		
		SimpleDateFormat sdf= new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		Date date=sdf.parse(convertDate);
		sdf=new SimpleDateFormat(DATE_FORMAT);
		convertDate = sdf.format(date);
		return convertDate;
	}
	
	public static Date convertStrToDate(String dateStr) {
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		if (dateStr == null) {
			return null;
		}
		try {
			return dateFormat.parse(dateStr);
		} catch (ParseException e) {
			throw new RuntimeException("Unable to parse the date");
		}
	}
	
	public static Date convertStringToDate(String dateStr) {
		if (dateStr == null) {
			return null;
		}
		try {
			return dateFormat.get().parse(dateStr);
		} catch (ParseException e) {
			throw new RuntimeException("Unable to parse the date");
		}
	}
	
	public static HttpHeaders buildHeaders(String appId) {
		HttpHeaders httpHeaders = new HttpHeaders();
		if (GlobalHolder.getRequestContext() != null) {
			httpHeaders.add(ReportConstants.APPID, appId);
			httpHeaders.add(ReportConstants.ACCEPT, ReportConstants.CONTENT_TYPE_JSON);
			httpHeaders.add(ReportConstants.CONTENT_TYPE, ReportConstants.CONTENT_TYPE_JSON);
			//TODO check : user id and branch code not picked from request context
			httpHeaders.add(ReportConstants.USERID, GlobalHolder.getRequestContext().getUserId());
			httpHeaders.add(ReportConstants.BRANCH_CODE, GlobalHolder.getRequestContext().getCurrBranch());
			httpHeaders.add(ReportConstants.AUTH_TOKEN, ReportConstants.AUTH_TOKEN_VAL);
		}
		return httpHeaders;
	}
	
	public static HttpHeaders buildHeadersForEventService(String appId, String userId, String branchCode) {
		HttpHeaders httpHeaders = new HttpHeaders();
		httpHeaders.add(ReportConstants.APPID, appId);
		httpHeaders.add(ReportConstants.ACCEPT, ReportConstants.CONTENT_TYPE_JSON);
		httpHeaders.add(ReportConstants.CONTENT_TYPE, ReportConstants.CONTENT_TYPE_JSON);
		httpHeaders.add(ReportConstants.USERID, userId);
		httpHeaders.add(ReportConstants.BRANCH_CODE, branchCode);
		httpHeaders.add(ReportConstants.AUTH_TOKEN, ReportConstants.AUTH_TOKEN_VAL);		
		httpHeaders.add(ReportConstants.ENTITY_ID,  PlatoHolder.getCurrentEntityId());		
		return httpHeaders;
	}
	
	public static Map<String, String> getMonthsMap(){
		
		Map<String, String> monthsMap = new HashMap<>();
		monthsMap.put("01", "JAN");
		monthsMap.put("02", "FEB");
		monthsMap.put("03", "MAR");
		monthsMap.put("04", "APR");
		monthsMap.put("05", "MAY");
		monthsMap.put("06", "JUN");
		monthsMap.put("07", "JUL");
		monthsMap.put("08", "AUG");
		monthsMap.put("09", "SEP");
		monthsMap.put("10", "OCT");
		monthsMap.put("11", "NOV");
		monthsMap.put("12", "DEC");
		return monthsMap;
	}

public static String getSlabs(Date dueDate, Date currentBusinessDate, Boolean isOverdue) {
		
		long timeInMillies;
		if(Boolean.TRUE.equals(isOverdue)) {
			timeInMillies = currentBusinessDate.getTime()-dueDate.getTime();
		} else {
			timeInMillies = dueDate.getTime()-currentBusinessDate.getTime();
		}
		
		Long dayDiff = TimeUnit.DAYS.convert(timeInMillies, TimeUnit.MILLISECONDS);
		
		if(dayDiff<0) {
			return "Overdue";
		}else if(dayDiff>=0 && dayDiff<=30) {
			return "0-30";
		}else if(dayDiff>30 && dayDiff<=60) {
			return "30-60";
		}else if(dayDiff>60 && dayDiff<=90) {
			return "60-90";
		}else if(dayDiff>90 && dayDiff<=120) {
			return "90-120";
		}else {
			return ">120";
		}
	}
	
	public static Integer numberConverter(String inputNumber) {

		if(inputNumber.equalsIgnoreCase("null")) {
			return null;
		}
		return new Integer(inputNumber);
    }
	
	

	
	private static List<ObcmTmRelationshipDetailsExternalCode> convertObcmRelationshipExternalCodeListToEntityList(List<ObcmTmRelationshipDetailsExternalCodeDTO> detailsDataList) {
		List<ObcmTmRelationshipDetailsExternalCode> voList = new ArrayList<>();
		detailsDataList.forEach(entity -> {
			ObcmTmRelationshipDetailsExternalCode vo =new ObcmTmRelationshipDetailsExternalCode();
			BeanUtils.copyProperties(entity, vo);
			vo.setId(entity.getId());
			vo.setRelationshipDetailId(entity.getRelationshipDetailId());
			voList.add(vo);
		});
		return voList;
	}
	
	private static List<ObcmTwRelationshipDetailsExternalCode> convertObcmRelationshipExternalCodeListToEntityListTw(List<ObcmTmRelationshipDetailsExternalCodeDTO> detailsDataList) {
		List<ObcmTwRelationshipDetailsExternalCode> voList = new ArrayList<>();
		detailsDataList.forEach(entity -> {
			ObcmTwRelationshipDetailsExternalCode vo =new ObcmTwRelationshipDetailsExternalCode();
			vo=MapperUtil.INSTANCE.toTwEntity(entity);
			vo.setRelationshipDetailId(entity.getRelationshipDetailId());
			voList.add(vo);
		});
		return voList;
	}
	private static List<ObcmTmRelationshipDetails> convertObcmRelationshipDetailsDTOListToEntityList(List<ObcmTmRelationshipDetailsDTO> dtoList) {
		List<ObcmTmRelationshipDetails> voList = new ArrayList<>();
		dtoList.forEach(entity -> {
			ObcmTmRelationshipDetails vo = new ObcmTmRelationshipDetails();
			BeanUtils.copyProperties(entity, vo);
			
			if(entity.getExternalCodes() != null && !entity.getExternalCodes().isEmpty()) {
		      vo.setObcmTmRelationshipDetailsExternalCodeList(convertObcmRelationshipExternalCodeListToEntityList(entity.getExternalCodes()));
			}
			vo.setExpiresOn(convertStringToDate(entity.getExpiresOn()));
			vo.setNoOfDays(entity.getNoOfDays());
			vo.setMaxDaysOverdue(entity.getMaxDaysOverdue());
			voList.add(vo);
		});
		return voList;
	}
	
	private static List<ObcmTwRelationshipDetails> convertObcmRelationshipDetailsDTOListToEntityListTw(List<ObcmTmRelationshipDetailsDTO> dtoList) {
		List<ObcmTwRelationshipDetails> voList = new ArrayList<>();
		dtoList.forEach(entity -> {
			ObcmTwRelationshipDetails vo = new ObcmTwRelationshipDetails();
			//BeanUtils.copyProperties(entity, vo);
			LOGGER.info("entity {}",entity);
			LOGGER.info("Before {}",vo);
			vo=MapperUtil.INSTANCE.toTwEntity(entity);
			LOGGER.info("After {}",vo);
			if(entity.getExternalCodes() != null && !entity.getExternalCodes().isEmpty()) {
		      vo.setObcmTwRelationshipDetailsExternalCodeList(convertObcmRelationshipExternalCodeListToEntityListTw(entity.getExternalCodes()));
			}
			vo.setExpiresOn(convertStringToDate(entity.getExpiresOn()));
			vo.setNoOfDays(entity.getNoOfDays());
			vo.setMaxDaysOverdue(entity.getMaxDaysOverdue());
			voList.add(vo);
		});
		return voList;
	}

	public static ObcmTmRelationshipMaster convertTmRelationshipDtoToEntity(ObcmTmRelationshipMasterDTO dtoData, ObcmTmRelationshipMaster vo) {
	    if( dtoData.getObcmTmRelationshipDetailsDTO() != null && !dtoData.getObcmTmRelationshipDetailsDTO().isEmpty()) {
	        vo.setObcmTmRelationshipDetailsList(convertObcmRelationshipDetailsDTOListToEntityList(dtoData.getObcmTmRelationshipDetailsDTO()));    
	    }
	    vo.setEffFromDate(convertStringToDate((dtoData.getEffFromDate())));
	    vo.setExpiresOn(convertStringToDate((dtoData.getExpiresOn())));
	    vo.setNoOfDays(dtoData.getNoOfDays());
		vo.setMaxDaysOverdue(dtoData.getMaxDaysOverdue());
	    return vo;
	  }
	
	public static ObcmTwRelationshipMaster convertTwRelationshipDtoToEntity(ObcmTmRelationshipMasterDTO dtoData, ObcmTwRelationshipMaster vo) {
	    if( dtoData.getObcmTmRelationshipDetailsDTO() != null && !dtoData.getObcmTmRelationshipDetailsDTO().isEmpty()) {
	        vo.setObcmTwRelationshipDetailsList(convertObcmRelationshipDetailsDTOListToEntityListTw(dtoData.getObcmTmRelationshipDetailsDTO()));    
	    }
	    vo.setEffFromDate(convertStringToDate((dtoData.getEffFromDate())));
	    vo.setExpiresOn(convertStringToDate((dtoData.getExpiresOn())));
	    vo.setNoOfDays(dtoData.getNoOfDays());
		vo.setMaxDaysOverdue(dtoData.getMaxDaysOverdue());
	    return vo;
	  }
	
	public static List<String>  getMonthAndDateOfGivenRange(String date1, String date2) {
    	List<String> dates = new ArrayList<>();
    	DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd", Locale.ROOT);
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MMM-yyyy", Locale.ROOT);
        YearMonth endMonth = YearMonth.parse(date2, dateFormatter);
        for (YearMonth month = YearMonth.parse(date1, dateFormatter);
                ! month.isAfter(endMonth);
                month = month.plusMonths(1)) {
        	dates.add(month.format(monthFormatter).toUpperCase());
        }
    	
    	return dates;
    }
	
	public static Date setEntityDateFormat(String dateStr) {
		if (dateStr == null) {
			return null;
		}
		try {
			return dateFormat.get().parse(dateStr);
		} catch (ParseException e) {
			return null;
		}
	}

	public static ObReconExecTxnTw setValuesFromDto(ReconTxnDto obReconExecTxnDto, ObReconExecTxnTw obReconExecTxnTw) {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		BeanUtils.copyProperties(obReconExecTxnDto, obReconExecTxnTw);
		if (obReconExecTxnDto.getId() != null) 
		  obReconExecTxnTw.setId(obReconExecTxnDto.getId().toString());
		try {
			if (obReconExecTxnDto.getReconDate() != null)
				obReconExecTxnTw.setReconDate(formatter.parse(obReconExecTxnDto.getReconDate()));
			if (obReconExecTxnDto.getMakerDtStamp() != null)
				obReconExecTxnTw.setMakerDtStamp(formatter.parse(obReconExecTxnDto.getMakerDtStamp()));
			if (obReconExecTxnDto.getAuthDtStamp() != null)
				obReconExecTxnTw.setAuthDtStamp(formatter.parse(obReconExecTxnDto.getAuthDtStamp()));
		} catch (ParseException e) {
			LOGGER.error(e.getMessage());
		}
		if (obReconExecTxnDto.getObReconInvPmtTxnTwDtoList() != null
				&& !obReconExecTxnDto.getObReconInvPmtTxnTwDtoList().isEmpty()) {
			List<ObReconInvPmtTxnTw> childList = new ArrayList<>();
			obReconExecTxnDto.getObReconInvPmtTxnTwDtoList().stream().forEach(childDto -> {
				ObReconInvPmtTxnTw childEntity;
				try {
					childEntity = setChildObject(childDto, obReconExecTxnTw);
					childList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			obReconExecTxnTw.setObReconInvPmtTxnTw(childList);
		}
		if (obReconExecTxnDto.getObReconInvCnTxnTwDtoList()!= null
				&& !obReconExecTxnDto.getObReconInvCnTxnTwDtoList().isEmpty()) {
			List<ObReconInvCnTxnTw> childList = new ArrayList<>();
			obReconExecTxnDto.getObReconInvCnTxnTwDtoList().stream().forEach(childDto -> {
				ObReconInvCnTxnTw childEntity;
				try {
					childEntity = setObReconInvCnTxnTw(childDto, obReconExecTxnTw);
					childList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			obReconExecTxnTw.setObReconInvCnTxnTw(childList);
		}
		if (obReconExecTxnDto.getExpDrCrPayReconDtoList() != null
				&& !obReconExecTxnDto.getExpDrCrPayReconDtoList().isEmpty()) {
			List<ObReconExpDrCrPmtTxnTw> childList = new ArrayList<>();
			obReconExecTxnDto.getExpDrCrPayReconDtoList().stream().forEach(childDto -> {
				ObReconExpDrCrPmtTxnTw childEntity;
				try {
					childEntity = setObReconExpDrCrPmtTxnTw(childDto, obReconExecTxnTw);
					childList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			obReconExecTxnTw.setObReconExpDrCrPmtTxnTw(childList);
		}
		if (obReconExecTxnDto.getAllocPayReconDtoList() != null
				&& !obReconExecTxnDto.getAllocPayReconDtoList().isEmpty()) {
			List<ObReconAllocPmtTxnTw> childList = new ArrayList<>();
			obReconExecTxnDto.getAllocPayReconDtoList().stream().forEach(childDto -> {
				ObReconAllocPmtTxnTw childEntity;
				try {
					childEntity = setObReconAllocPmtTxnTw(childDto, obReconExecTxnTw);
					childList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			obReconExecTxnTw.setObReconAllocPmtTxnTw(childList);
		}
		if (obReconExecTxnDto.getFinPayReconDtoList() != null
				&& !obReconExecTxnDto.getFinPayReconDtoList().isEmpty()) {
			List<ObReconFinPmtTxnTw> childList = new ArrayList<>();
			obReconExecTxnDto.getFinPayReconDtoList().stream().forEach(childDto -> {
				ObReconFinPmtTxnTw childEntity;
				try {
					childEntity = setObReconFinPmtTxnTw(childDto, obReconExecTxnTw);
					childList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			obReconExecTxnTw.setObReconFinPmtTxnTw(childList);
		}
		return obReconExecTxnTw;
	}
	
	public static ObReconInvPmtTxnTw setChildObject(ReconInvPmtDto dto, ObReconExecTxnTw obReconExecTxnTw) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconInvPmtTxnTw entity = new ObReconInvPmtTxnTw();
		BeanUtils.copyProperties(dto, entity);
		entity.setId(dto.getId().toString());
		if (dto.getInvDate() != null)
			entity.setInvDate(formatter.parse(dto.getInvDate()));
		if (dto.getInvDueDate() != null)
			entity.setInvDueDate(formatter.parse(dto.getInvDueDate()));
		if (dto.getPaymentDate() != null)
			entity.setPaymentDate(formatter.parse(dto.getPaymentDate()));
		if (dto.getPaymentDueDate() != null)
			entity.setPaymentDueDate(formatter.parse(dto.getPaymentDueDate()));
		if (dto.getPaymentBusinessDate() != null)
			entity.setPaymentBusinessDate(formatter.parse(dto.getPaymentBusinessDate()));
		if (dto.getReconDate() != null)
			entity.setReconDate(formatter.parse(dto.getReconDate()));
		entity.setObReconExecTxnTw(obReconExecTxnTw);
		entity.setAccountNumber(dto.getAccountNo());
		if ("Y".equalsIgnoreCase(dto.getAllocationRequired())
				&& dto.getAllocPayReconDtoList() != null && !dto.getAllocPayReconDtoList().isEmpty()) {
			List<ObReconAllocPmtTxnTw> allocationTwList = new ArrayList<>();
			dto.getAllocPayReconDtoList().stream().forEach(allocation -> {
				ObReconAllocPmtTxnTw childEntity;
				try {
					childEntity = setManualAllocationPmtTxnTw(allocation);
					childEntity.setReconTxnRefNo(obReconExecTxnTw.getReconTxnRefNo());
					childEntity.setReconAction(obReconExecTxnTw.getReconAction());
					allocationTwList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			entity.setAllocPayReconTwList(allocationTwList);
		}
		return entity;
	}
	
	public static ObReconInvCnTxnTw setObReconInvCnTxnTw(ReconInvCnDto dto, ObReconExecTxnTw obReconExecTxnTw) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconInvCnTxnTw entity = new ObReconInvCnTxnTw();
		BeanUtils.copyProperties(dto, entity);
		entity.setId(dto.getId().toString());
		if (dto.getSeqNo() != null)
			entity.setMatchSeqNo(dto.getSeqNo().toString());
		if (dto.getCnDate() != null)
			entity.setCnDate(formatter.parse(dto.getCnDate()));
		if (dto.getCnExpiryDate() != null)
			entity.setCnExpiryDate(formatter.parse(dto.getCnExpiryDate()));
		if (dto.getInvDate() != null)
			entity.setInvDate(formatter.parse(dto.getInvDate()));
		if (dto.getInvDueDate() != null)
			entity.setInvDueDate(formatter.parse(dto.getInvDueDate()));
		if (dto.getReconDate() != null)
			entity.setReconDate(formatter.parse(dto.getReconDate()));
		if (dto.getProcBranchDate() != null)
			entity.setProcBranchDate(formatter.parse(dto.getProcBranchDate()));
		if (dto.getFiller1() != null)
			entity.setFiller1(formatter.parse(dto.getFiller1()));
		if (dto.getFiller2() != null)
			entity.setFiller2(formatter.parse(dto.getFiller2()));
		if (dto.getPaymentDueDate() != null)
			entity.setPaymentDueDate(formatter.parse(dto.getPaymentDueDate()));
		entity.setObReconExecTxnTw(obReconExecTxnTw);
		return entity;
	}
	
	public static ObReconAllocPmtTxnTw setManualAllocationPmtTxnTw(AllocPayReconDto dto) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconAllocPmtTxnTw entity = new ObReconAllocPmtTxnTw();
		BeanUtils.copyProperties(dto, entity);
		if (dto.getPaymentDate() != null)
			entity.setPaymentDate(formatter.parse(dto.getPaymentDate()));
		return entity;
	}
	
	public static ObReconExpDrCrPmtTxnTw setObReconExpDrCrPmtTxnTw(ExpDrCrPayReconDto dto,
			ObReconExecTxnTw obReconExecTxnTw) throws ParseException {
		
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconExpDrCrPmtTxnTw entity = new ObReconExpDrCrPmtTxnTw();
		BeanUtils.copyProperties(dto, entity);
		entity.setId(dto.getId().toString());
		if (dto.getExpDrcrDate() != null)
			entity.setExpDrcrDate(parseDate(dto.getExpDrcrDate()));
		if (dto.getRevisedDrcrDate() != null)
			entity.setRevisedDrcrDate(parseDate(dto.getRevisedDrcrDate()));
		if (dto.getPaymentDate() != null)
			entity.setPaymentDate(formatter.parse(dto.getPaymentDate()));
		if (dto.getActualDate() != null)
			entity.setActualDate(formatter.parse(dto.getActualDate()));
		if (dto.getReconDate() != null)
			entity.setReconDate(formatter.parse(dto.getReconDate()));
		if (dto.getPaymentBusinessDate() != null)
			entity.setPaymentBusinessDate(formatter.parse(dto.getPaymentBusinessDate()));
		entity.setUdfTx1(dto.getUdfTxt1());
		entity.setUdfTx2(dto.getUdfTxt2());
		entity.setUdfTx3(dto.getUdfTxt3());
		entity.setUdfTx4(dto.getUdfTxt4());
		entity.setUdfTx5(dto.getUdfTxt5());
		entity.setUdfTx6(dto.getUdfTxt6());
		entity.setUdfTx7(dto.getUdfTxt7());
		entity.setUdfTx8(dto.getUdfTxt8());
		entity.setUdfTx9(dto.getUdfTxt9());
		entity.setUdfTx10(dto.getUdfTxt10());
		entity.setObReconExecTxnTw(obReconExecTxnTw);
		entity.setBeneVirtualAccNo(dto.getBeneVirtualAccNo());
		if("Y".equalsIgnoreCase(dto.getAllocationRequired()) && dto.getAllocPayReconDtoList()!=null && !dto.getAllocPayReconDtoList().isEmpty()) {
			List<ObReconAllocPmtTxnTw> allocationTwList = new ArrayList<>();
			dto.getAllocPayReconDtoList().stream().forEach(allocation ->{
				ObReconAllocPmtTxnTw childEntity;
				try {
					childEntity = setManualAllocationPmtTxnTw(allocation);
					childEntity.setReconTxnRefNo(obReconExecTxnTw.getReconTxnRefNo());
					childEntity.setReconAction(obReconExecTxnTw.getReconAction());
					allocationTwList.add(childEntity);
				} catch (ParseException e) {
					LOGGER.error(e.getMessage());
				}
			});
			entity.setAllocPayReconTwList(allocationTwList);
		}
		return entity;
	}
	
	public static ObReconAllocPmtTxnTw setObReconAllocPmtTxnTw(AllocPayReconDto dto,
			ObReconExecTxnTw obReconExecTxnTw) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconAllocPmtTxnTw entity = new ObReconAllocPmtTxnTw();
		BeanUtils.copyProperties(dto, entity);
		entity.setId(dto.getId().toString());
		if (dto.getPaymentBusinessDate() != null)
			entity.setPaymentBusinessDate(formatter.parse(dto.getPaymentBusinessDate()));
		if (dto.getPaymentDate() != null)
			entity.setPaymentDate(formatter.parse(dto.getPaymentDate()));
		if (dto.getReconDate() != null)
			entity.setReconDate(formatter.parse(dto.getReconDate()));
		entity.setReconAction(obReconExecTxnTw.getReconAction());
		entity.setObReconExecTxnTw(obReconExecTxnTw);
		return entity;
	}
	
	public static ObReconFinPmtTxnTw setObReconFinPmtTxnTw(FinPayReconDto dto,
			ObReconExecTxnTw obReconExecTxnTw) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		ObReconFinPmtTxnTw entity = new ObReconFinPmtTxnTw();
		BeanUtils.copyProperties(dto, entity);
		entity.setId(dto.getId().toString());
		if (dto.getMatchSeqNo() != null)
			entity.setMatchSeqNo(dto.getMatchSeqNo().toString());
		if (dto.getFinStartDt() != null)
			entity.setFinStartDt(formatter.parse(dto.getFinStartDt()));
		if (dto.getFinEndDt() != null)
			entity.setFinEndDt(formatter.parse(dto.getFinEndDt()));
		if (dto.getFinPastDueDt() != null)
			entity.setFinPastDueDt(formatter.parse(dto.getFinPastDueDt()));
		if (dto.getReconDate() != null)
			entity.setReconDate(formatter.parse(dto.getReconDate()));
		if (dto.getPaymentDate() != null)
			entity.setPaymentDate(formatter.parse(dto.getPaymentDate()));
		entity.setObReconExecTxnTw(obReconExecTxnTw);
		return entity;
	}
	
	public static Date parseDate(String str) throws ParseException {
		DateFormat formatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
		SimpleDateFormat formatter1 = new SimpleDateFormat("dd/MM/yyyy");	
		Date date;
		try {
			date = formatter1.parse(str);
			date = formatter.parse(formatter.format(date));
		} catch (ParseException e) {
			date = formatter.parse(str);
		}
		return date;
	}
	
	public static EventDetailsDTO setReconTwEventDetails(ReconTxnDto reconTxnDto, EventDetailsDTO eventDetailsObject) {
		if(reconTxnDto.getReconCategory()!=null && reconTxnDto.getReconCategory().equals("IPR") && reconTxnDto.getObReconInvPmtTxnTwDtoList().size()>0) {
			eventDetailsObject.setRelationshipCode(reconTxnDto.getObReconInvPmtTxnTwDtoList().get(0).getPmtRelationshipCode());
			eventDetailsObject.setProgramId(reconTxnDto.getObReconInvPmtTxnTwDtoList().get(0).getPmtProgramCode());
			eventDetailsObject.setEvent(ReportConstants.RECON_TW_IPR_EVENT);
		} else if(reconTxnDto.getReconCategory()!=null && reconTxnDto.getReconCategory().equals("ICR") && reconTxnDto.getObReconInvCnTxnTwDtoList().size()>0) {
			eventDetailsObject.setRelationshipCode(reconTxnDto.getObReconInvCnTxnTwDtoList().get(0).getCnRelationCode());
			eventDetailsObject.setProgramId(reconTxnDto.getObReconInvCnTxnTwDtoList().get(0).getCnProgramCode());
			eventDetailsObject.setEvent(ReportConstants.RECON_TW_ICR_EVENT);
		} else if(reconTxnDto.getReconCategory()!=null && reconTxnDto.getReconCategory().equals("EPR") && reconTxnDto.getExpDrCrPayReconDtoList().size()>0) {
			eventDetailsObject.setRelationshipCode(reconTxnDto.getExpDrCrPayReconDtoList().get(0).getPmtRelationshipCode());
			eventDetailsObject.setProgramId(reconTxnDto.getExpDrCrPayReconDtoList().get(0).getPmtProgramCode());
			eventDetailsObject.setEvent(ReportConstants.RECON_TW_EPR_EVENT);
		} else if(reconTxnDto.getReconCategory()!=null && reconTxnDto.getReconCategory().equals("VPA") && reconTxnDto.getAllocPayReconDtoList().size()>0) {
			eventDetailsObject.setRelationshipCode(reconTxnDto.getAllocPayReconDtoList().get(0).getRelationshipCode());
			eventDetailsObject.setProgramId(reconTxnDto.getAllocPayReconDtoList().get(0).getProgramCode());
			eventDetailsObject.setEvent(ReportConstants.RECON_TW_VPA_EVENT);
		} else if(reconTxnDto.getReconCategory()!=null && reconTxnDto.getReconCategory().equals("FPR") && reconTxnDto.getFinPayReconDtoList().size()>0) {
			eventDetailsObject.setProgramId(reconTxnDto.getFinPayReconDtoList().get(0).getPmtProgramCode());
			eventDetailsObject.setEvent(ReportConstants.RECON_TW_FPR_EVENT);
		}
		return eventDetailsObject;
	}
	
	public static Date dateConverter(String inputDate) throws ParseException {

        DateFormat dateFormat = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
        if (inputDate == null) {
            return null;
        }
        return dateFormat.parse(inputDate);
    }
	
	public static ObChargesTxnTw convertDTOToEntity(ObChargesTxnDTO dto) {
		ObChargesTxnTw entity = null;
	    if (dto != null) {
	      entity = new ObChargesTxnTw();
	      BeanUtils.copyProperties(dto, entity);
	    }
	    return entity;
	 }
	
	public static void setDates(ObChargesTxn onChargesTxnMain, ObChargesTxnTw obChargesTxn) {

		if (obChargesTxn.getChgCalcDate() != null) {
			onChargesTxnMain.setChgCalcDate(obChargesTxn.getChgCalcDate());
		}
		if (obChargesTxn.getFirstChgCalcDate() != null) {
			onChargesTxnMain.setFirstChgCalcDate(obChargesTxn.getFirstChgCalcDate());
		}
		if (obChargesTxn.getFirstChgCollDate() != null) {
			onChargesTxnMain.setFirstChgCollDate(obChargesTxn.getFirstChgCollDate());
		}
		if (obChargesTxn.getNextChgCalcDate() != null) {
			onChargesTxnMain.setNextChgCalcDate(obChargesTxn.getNextChgCalcDate());
		}
		if (obChargesTxn.getNextChgCollDate() != null) {
			onChargesTxnMain.setNextChgCollDate(obChargesTxn.getNextChgCollDate());
		}
		if (obChargesTxn.getCalcEndDate() != null) {
			onChargesTxnMain.setCalcEndDate(obChargesTxn.getCalcEndDate());
		}
		if (obChargesTxn.getCalcStartDate() != null) {
			onChargesTxnMain.setCalcStartDate(obChargesTxn.getCalcStartDate());
		}
	}
	public static String formatIntegerString(Integer number) {
		String modNo="-1";
		if(number!=null) {
				modNo= number.toString();
		}
	return modNo;
	}

	public static boolean isNull(Object object) {
		if (object == null || object.toString().isEmpty() || "".equalsIgnoreCase(object.toString())) {
			return true;
		}
		return false;
	}
	
}
