package oracle.fsgbu.obscfcm.services.report.web;

import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.ChronoUnit;

import java.util.List;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.ApiParam;
import oracle.fsgbu.obscfcm.services.report.exception.InvalidDateRangeException;
import oracle.fsgbu.obscfcm.services.report.exception.InvalidTrendLineDateException;
import oracle.fsgbu.obscfcm.services.report.service.ReconDashboardService;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.reconDashboard.api.model.ReconCategoryResponseDto;
import oracle.fsgbu.reconDashboard.api.model.ReconTopCorporateResponseDto;
import oracle.fsgbu.reconDashboard.api.model.ReconTrendGraphResponseDto;
import oracle.fsgbu.reconDashboard.web.api.ReconDashboardServiceApi;

@RestController
public class ReconDashboardServiceController implements ReconDashboardServiceApi {

	@Autowired
	private ReconDashboardService reconDashboardService;
	
	@Autowired
	public ReconDashboardServiceController(ReconDashboardService reconDashboardService) {
		this.reconDashboardService = reconDashboardService;
	}
	
	@Override
	public ResponseEntity<ReconCategoryResponseDto> getInvoiceReconCategoryList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getInvoiceReconCategoryList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconTrendGraphResponseDto> getInvoiceReconTrendGraphList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getInvoiceReconTrendGraphList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconCategoryResponseDto> getCashFlowReconCategoryList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getCashFlowReconCategoryList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconTrendGraphResponseDto> getCashFlowReconTrendGraphList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getCashFlowReconTrendGraphList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconTopCorporateResponseDto> getInvoiceReconTopCorporateList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Recon Type") @Valid @RequestParam(value = "reconType", required = true) String reconType,
			@ApiParam(value = "Trendline Flag") @Valid @RequestParam(value = "trendlineFlag", required = false) String trendlineFlag,
			@ApiParam(value = "Trendline Date") @Valid @RequestParam(value = "trendlineDate", required = false) String trendlineDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		boolean isTrendLineDateValid = validateTrendLineDate(fromDate, toDate, trendlineFlag, trendlineDate);
		if(!isTrendLineDateValid) {
			throw new InvalidTrendLineDateException();
		}
		response = reconDashboardService.getInvoiceReconTopCorporateList(userId, branchCode, fromDate, toDate, reconType, trendlineFlag, trendlineDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconCategoryResponseDto> getAllocationReconCategoryList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getAllocationReconCategoryList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconTrendGraphResponseDto> getAllocationReconTrendGraphList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getAllocationReconTrendGraphList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconCategoryResponseDto> getFinanceReconCategoryList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getFinanceReconCategoryList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconTrendGraphResponseDto> getFinanceReconTrendGraphList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getFinanceReconTrendGraphList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconTopCorporateResponseDto> getAllocationReconTopCorporateList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Recon Type") @Valid @RequestParam(value = "reconType", required = true) String reconType,
			@ApiParam(value = "Trendline Flag") @Valid @RequestParam(value = "trendlineFlag", required = false) String trendlineFlag,
			@ApiParam(value = "Trendline Date") @Valid @RequestParam(value = "trendlineDate", required = false) String trendlineDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		boolean isTrendLineDateValid = validateTrendLineDate(fromDate, toDate, trendlineFlag, trendlineDate);
		if(!isTrendLineDateValid) {
			throw new InvalidTrendLineDateException();
		}
		response = reconDashboardService.getAllocationReconTopCorporateList(userId, branchCode, fromDate, toDate, reconType, trendlineFlag, trendlineDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconTopCorporateResponseDto> getCashflowReconTopCorporateList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Recon Type") @Valid @RequestParam(value = "reconType", required = true) String reconType,
			@ApiParam(value = "Trendline Flag") @Valid @RequestParam(value = "trendlineFlag", required = false) String trendlineFlag,
			@ApiParam(value = "Trendline Date") @Valid @RequestParam(value = "trendlineDate", required = false) String trendlineDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		boolean isTrendLineDateValid = validateTrendLineDate(fromDate, toDate, trendlineFlag, trendlineDate);
		if(!isTrendLineDateValid) {
			throw new InvalidTrendLineDateException();
		}
		response = reconDashboardService.getCashflowReconTopCorporateList(userId, branchCode, fromDate, toDate, reconType, trendlineFlag, trendlineDate, corporateId);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<ReconTopCorporateResponseDto> getFinanceReconTopCorporateList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Recon Type") @Valid @RequestParam(value = "reconType", required = true) String reconType,
			@ApiParam(value = "Trendline Flag") @Valid @RequestParam(value = "trendlineFlag", required = false) String trendlineFlag,
			@ApiParam(value = "Trendline Date") @Valid @RequestParam(value = "trendlineDate", required = false) String trendlineDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		boolean isTrendLineDateValid = validateTrendLineDate(fromDate, toDate, trendlineFlag, trendlineDate);
		if(!isTrendLineDateValid) {
			throw new InvalidTrendLineDateException();
		}
		response = reconDashboardService.getFinanceReconTopCorporateList(userId, branchCode, fromDate, toDate, reconType, trendlineFlag, trendlineDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconTrendGraphResponseDto> getReconciliationTrendGraphList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate);
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		response = reconDashboardService.getReconciliationTrendGraphList(userId, branchCode, fromDate, toDate, corporateId);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<ReconTopCorporateResponseDto> getReconciliationTopCorporateList(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "From Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "reconType Type") @Valid @RequestParam(value = "reconType", required = true) String reconType,
			@ApiParam(value = "Trendline Flag") @Valid @RequestParam(value = "trendlineFlag", required = false) String trendlineFlag,
			@ApiParam(value = "Trendline Date") @Valid @RequestParam(value = "trendlineDate", required = false) String trendlineDate,
			@ApiParam(value = "Corporate ID") @Valid @RequestParam(value = "corporateId", required = false) String corporateId){
		
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		boolean isDateValid = validateDateRange(fromDate, toDate );
		if(!isDateValid) {
			throw new InvalidDateRangeException();
		}
		boolean isTrendLineDateValid = validateTrendLineDate(fromDate, toDate, trendlineFlag, trendlineDate);
		if(!isTrendLineDateValid) {
			throw new InvalidTrendLineDateException();
		}
		response = reconDashboardService.getReconciliationTopCorporateList(userId, branchCode, fromDate, toDate, reconType, trendlineFlag, trendlineDate, corporateId);
		return ResponseEntity.ok(response);
	}

	private boolean validateDateRange(String fromDate, String toDate) {
		boolean isvalidDateRange = true;
		long monthsBetween = ChronoUnit.MONTHS.between(YearMonth.from(LocalDate.parse(fromDate)),
				YearMonth.from(LocalDate.parse(toDate)));
		if (monthsBetween > 6) {
			isvalidDateRange = false;
		}
		return isvalidDateRange;
	}
	private boolean validateTrendLineDate(String fromDate, String toDate, String trendlineFlag, String trendlineDate) {
		boolean isvalidDateRange = true;
		if("Y".equalsIgnoreCase(trendlineFlag)) {
			List<String> monthsWithinRange = ReportingUtil.getMonthAndDateOfGivenRange(fromDate, toDate);
			if(!monthsWithinRange.contains(trendlineDate)) {
				isvalidDateRange = false;
			}
		}
		return isvalidDateRange;
	}
	
}