package oracle.fsgbu.obscfcm.services.report.web;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.ApiParam;
import oracle.fsgbu.obscfcm.services.report.service.ReportCreationService;
import oracle.fsgbu.reportsTransaction.api.model.AgingInvoiceDetailsResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.AgingInvoicesResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.BusinessTrendResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.InvoiceDetailsResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.PayableReceivableResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.RaisedInvoiceDetailsResponseDto;
import oracle.fsgbu.reportsTransaction.api.model.RaisedInvoicesResponseDto;
import oracle.fsgbu.reportsTransaction.web.api.ReportCreationServiceApi;

@RestController
public class ReportCreationServiceController implements ReportCreationServiceApi {

	@Autowired
	private ReportCreationService reportCreationService;
	
	@Autowired
	public ReportCreationServiceController(ReportCreationService reportCreationService) {
		this.reportCreationService = reportCreationService;
	}
	
	@Override
	public ResponseEntity<RaisedInvoicesResponseDto> getRaisedInvoices(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "Start Date of period") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "End Date of period") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "Currency in which data needs to be sent") @Valid @RequestParam(value = "currency", required = false) String currency) {
		
		RaisedInvoicesResponseDto response = reportCreationService.getRaisedInvoices(userId, branchCode, fromDate, toDate, currency);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<AgingInvoicesResponseDto> getAgingInvoices(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "offset is starting no of records to be fetched") @Valid @RequestParam(value = "offset", required = false) Integer offset,
			@ApiParam(value = "limit is maximum no of records to be fetched") @Valid @RequestParam(value = "limit", required = false) Integer limit){
		AgingInvoicesResponseDto response = reportCreationService.getAgingInvoices(userId, branchCode, offset,limit);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<AgingInvoicesResponseDto> getOverdueAgingInvoices(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode){
		
		AgingInvoicesResponseDto response = reportCreationService.getOverdueAgingInvoices(userId, branchCode);
		return ResponseEntity.ok(response);
	}	
	
	@Override
	public ResponseEntity<BusinessTrendResponseDto> getBusinessTrend(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode){
		
		BusinessTrendResponseDto response = reportCreationService.getBusinessTrend(userId, branchCode);
		return ResponseEntity.ok(response);
	}
	
	@Override
	public ResponseEntity<PayableReceivableResponseDto> getPayableReceivables(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode) {
		
		PayableReceivableResponseDto response = reportCreationService.getPayableReceivables(userId, branchCode);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<RaisedInvoiceDetailsResponseDto> getRaisedInvoiceDetails(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@ApiParam(value = "Start Date of period") @Valid @RequestParam(value = "fromDate", required = true) String fromDate,
			@ApiParam(value = "End Date of period") @Valid @RequestParam(value = "toDate", required = true) String toDate,
			@NotNull @ApiParam(value = "", required = true) @Valid @RequestParam(value = "isFinanced", required = true) String isFinanced,
			@ApiParam(value = "Currency in which data needs to be sent") @Valid @RequestParam(value = "currency", required = false) String currency,
			@ApiParam(value = "offset is starting no of records to be fetched") @Valid @RequestParam(value = "offset", required = false) Integer offset,
			@ApiParam(value = "limit is maximum no of records to be fetched") @Valid @RequestParam(value = "limit", required = false) Integer limit) {
		
		RaisedInvoiceDetailsResponseDto response = reportCreationService.getRaisedInvoiceDetails(appId, userId, branchCode, fromDate, toDate, isFinanced, currency,offset,limit);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<AgingInvoiceDetailsResponseDto> getAgingInvoiceDetails(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@NotNull @ApiParam(value = "From Invoice Due Date", required = true) @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@NotNull @ApiParam(value = "To Invoice Due Date", required = true) @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "offset is starting no of records to be fetched") @Valid @RequestParam(value = "offset", required = false) Integer offset,
			@ApiParam(value = "limit is maximum no of records to be fetched") @Valid @RequestParam(value = "limit", required = false) Integer limit) {
		
		AgingInvoiceDetailsResponseDto response = reportCreationService.getAgingInvoiceDetails(userId, branchCode, fromDate, toDate, offset,limit);
		return ResponseEntity.ok(response);
	}

	@Override
	public ResponseEntity<InvoiceDetailsResponseDto> getInvoices(
			@ApiParam(value = "Application id" ,required=true) @RequestHeader(value="appId", required=true) String appId,
			@ApiParam(value = "user id" ,required=true) @RequestHeader(value="userId", required=true) String userId,
			@ApiParam(value = "Branch Code" ,required=true) @RequestHeader(value="branchCode", required=true) String branchCode,
			@NotNull @ApiParam(value = "Corporate Id", required = true) @Valid @RequestParam(value = "corporateId", required = true) String corporateId,
			@ApiParam(value = "From Invoice Due Date") @Valid @RequestParam(value = "fromDueDate", required = false) String fromDueDate,
			@ApiParam(value = "To Invoice Due Date") @Valid @RequestParam(value = "toDueDate", required = false) String toDueDate,
			@ApiParam(value = "From Invoice Date") @Valid @RequestParam(value = "fromDate", required = false) String fromDate,
			@ApiParam(value = "To Invoice Date") @Valid @RequestParam(value = "toDate", required = false) String toDate,
			@ApiParam(value = "isFinanced") @Valid @RequestParam(value = "isFinanced", required = false) String isFinanced,
			@ApiParam(value = "offset is starting no of records to be fetched") @Valid @RequestParam(value = "offset", required = false) Integer offset,
			@ApiParam(value = "limit is maximum no of records to be fetched") @Valid @RequestParam(value = "limit", required = false) Integer limit
			) {
		
		InvoiceDetailsResponseDto response = reportCreationService.getInvoices(userId, branchCode, fromDueDate, toDueDate, fromDate, toDate, corporateId, isFinanced, offset, limit);
		
		return ResponseEntity.ok(response);
	}
}