package oracle.fsgbu.customer.test.assembler

import oracle.fsgbu.eventService.api.model.InvoiceCommodityGridModel
import oracle.fsgbu.eventService.api.model.InvoiceModel
import oracle.fsgbu.eventService.api.model.InvoiceOfferDTO
import oracle.fsgbu.obscfcm.services.report.assembler.ScfInvoiceTwAssembler
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfTwInvoice
import spock.lang.Shared
import spock.lang.Specification;
import spock.lang.Unroll;

class ScfInvoiceTwAssemblerSpec extends Specification{
	
  @Shared mockDate = new String("2019-08-12");
  @Shared mockAmount = new BigDecimal("123456");
  @Shared mockString = new String("Test Data");
  @Shared discountPercentage = new BigDecimal("88");
	
	
	
  @Unroll("test convertEntityModelToOfferParameters method")
  def"test convertEntityModelToOfferParameters method"(){
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  InvoiceOfferDTO invoiceOfferDTO = new InvoiceOfferDTO()
	  invoiceOfferDTO.setEarlyPaymentDate(mockDate)
	  invoiceOfferDTO.setDiscountAmount(mockAmount)
	  invoiceOfferDTO.setDiscountPercentage(discountPercentage)
	  invoiceOfferDTO.setNotionalEarlyPayAmt(mockAmount)
	  invoiceOfferDTO.setChgPriceTemplateId("Test123")
	  invoiceOfferDTO.setCurrency("USD")
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setInvoiceOffer(invoiceOfferDTO)
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
	  assembler.convertEntityModelToOfferParameters(vo, entity)
  }
	
  @Unroll("test convertEntityModelToOfferParameters null")
  def"test convertEntityModelToOfferParameters null"(){
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
	  assembler.convertEntityModelToOfferParameters(vo, entity)
  }
	
  @Unroll("test convertEntityModelToPayParameters method")
  def"test convertEntityModelToPayParameters method"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setReconTxnStatus("Test");
	  vo.setReconPmtAmountCcy("USD");
	  vo.setReconPmtAmount(mockAmount)
	  vo.setPaymentParty("12300")
	  vo.setPaymentMode("Card")
	  vo.setPaidAmount(mockAmount)
		
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
	  assembler.convertEntityModelToPayParameters(vo, entity)
	
  }
	
  @Unroll("test convertEntityModelToShipParameters method")
  def"test convertEntityModelToShipParameters method"(){
		
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setShipmentRefNo("999999");
	  vo.setShipmentNo("555555");
	  vo.setShipmentDate(mockDate);
	  vo.setShipmentCountry("France");
	  vo.setShipmentAddress("Test 123");
		
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
	  assembler.convertEntityModelToShipParameters(vo, entity)
  }
	
  @Unroll("test convertEntityModelToFinParameters method")
  def"test convertEntityModelToFinParameters method"(){
		
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId(mockString);
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo(mockString)
	  vo.setTransactionRefNo(mockString)
	  vo.setProgramName(mockString);
	  vo.setProgramCode(mockString);
	  vo.setOldFinanceAmount(vo.getOldFinanceAmount());
	  vo.setOldAssignmentAmount(vo.getOldAssignmentAmount());
	  vo.setIsIndirectPayment(mockString);
	  vo.setIsPUA(mockString);
	  vo.setIsFinanced(mockString);
	  vo.setIsAssigned(mockString);
	  vo.setIsApproved(mockString);
	  vo.setInsuranceCoName(mockString);
	  vo.setInsuranceCoId(mockString);
	  vo.setImportFactorName(mockString);
	  vo.setImportFactorId(mockString);
	  vo.setFundingReqAmt(mockAmount);
	  vo.setAssignmentParty(mockString);
		
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
	  assembler.convertEntityModelToFinParameters(vo, entity)
  }
	
  @Unroll("test convertMainInvoiceTwCommGridVOToEntity method")
  def"test convertMainInvoiceTwCommGridVOToEntity method"(){
		
	  given:
	  InvoiceCommodityGridModel vo = new InvoiceCommodityGridModel()
	  vo.setId(mockString);
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo(mockString)
		
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
		
	  when:
		
	  def response  =  assembler.convertMainInvoiceTwCommGridVOToEntity(vo, mockString)
		
	  then:
	  response!=null
  }
	
  @Unroll("test convertMainInvoiceTwCommGridVOToEntity null")
  def"test convertMainInvoiceTwCommGridVOToEntity null"(){
		
	  given:
	  InvoiceCommodityGridModel vo = null
		
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
		
	  when:
		
	  def response  =  assembler.convertMainInvoiceTwCommGridVOToEntity(vo, mockString)
		
	  then:
	  response==null
  }
	
  @Unroll("test convertTwInvoiceModelToInvoiceEntity method")
  def"test convertTwInvoiceModelToInvoiceEntity method"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  InvoiceOfferDTO invoiceOfferDTO = new InvoiceOfferDTO()
	  invoiceOfferDTO.setEarlyPaymentDate(mockDate)
	  invoiceOfferDTO.setDiscountAmount(mockAmount)
	  invoiceOfferDTO.setDiscountPercentage(discountPercentage)
	  invoiceOfferDTO.setNotionalEarlyPayAmt(mockAmount)
	  invoiceOfferDTO.setChgPriceTemplateId("Test123")
	  invoiceOfferDTO.setCurrency("USD")
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setInvoiceOffer(invoiceOfferDTO)
	  vo.setReconTxnStatus("Test");
	  vo.setReconPmtAmountCcy("USD");
	  vo.setReconPmtAmount(mockAmount)
	  vo.setPaymentParty("12300")
	  vo.setPaymentMode("Card")
	  vo.setPaidAmount(mockAmount)
	  vo.setShipmentRefNo("999999");
	  vo.setShipmentNo("555555");
	  vo.setShipmentDate(mockDate);
	  vo.setShipmentCountry("France");
	  vo.setShipmentAddress("Test 123");
	  vo.setProgramName(mockString);
	  vo.setProgramCode(mockString);
	  vo.setOldFinanceAmount(vo.getOldFinanceAmount());
	  vo.setOldAssignmentAmount(vo.getOldAssignmentAmount());
	  vo.setIsIndirectPayment(mockString);
	  vo.setIsPUA(mockString);
	  vo.setIsFinanced(mockString);
	  vo.setIsAssigned(mockString);
	  vo.setIsApproved(mockString);
	  vo.setInsuranceCoName(mockString);
	  vo.setInsuranceCoId(mockString);
	  vo.setImportFactorName(mockString);
	  vo.setImportFactorId(mockString);
	  vo.setFundingReqAmt(mockAmount);
	  vo.setAssignmentParty(mockString);
	  InvoiceCommodityGridModel vo1 = new InvoiceCommodityGridModel()
	  vo1.setId(mockString);
	  vo1.setSeqNo(12345)
	  vo1.setInvoiceRefNo(mockString)
		def list = new ArrayList<>()
		list.add(vo1)
		vo.setInvoiceCommodityGridList(list)
	  ScfTwInvoice entity = new ScfTwInvoice()
	  ScfInvoiceTwAssembler assembler = new ScfInvoiceTwAssembler()
		
	  when:
		
	  def response  =  assembler.convertTwInvoiceModelToInvoiceEntity(vo)
		
	  then:
	  response!=null
  }
	
}