package oracle.fsgbu.customer.test.assembler

import org.mockito.Mockito

import lombok.Data
import oracle.fsgbu.eventService.api.model.InvoiceCommodityGridModel
import oracle.fsgbu.eventService.api.model.LinkedInvoiceModel
import oracle.fsgbu.eventService.api.model.TmDrCrNoteLinkInvoiceModel
import oracle.fsgbu.eventService.api.model.InvoiceDto
import oracle.fsgbu.eventService.api.model.InvoiceModel
import oracle.fsgbu.eventService.api.model.InvoiceOfferDTO
import avro.oracle.fsgbu.obscf.invoicing.dto.UpdateInvoiceCommodityGridDto
import avro.oracle.fsgbu.obscf.invoicing.dto.UpdateDrCrNoteLinkInvoiceDto
import oracle.fsgbu.obscfcm.services.report.assembler.ScfInvoiceAssembler
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmInvoice
import oracle.fsgbu.obscfcm.services.report.domain.entity.TmInvoiceOfferParameters
import spock.lang.Shared
import spock.lang.Specification;
import spock.lang.Unroll;

@Data
class ScfcmInvoiceAssemblerSpec extends Specification{
	
  @Shared mockDate = new String("2019-08-12");
  @Shared mockAmount = new BigDecimal("123456");
  @Shared mockString = new String("Test Data");
  @Shared discountPercentage = new BigDecimal("88");
	
	
	
  /*@Unroll("test convertEntityModelToOfferParameters method")
  def"test convertEntityModelToOfferParameters method"(){
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  TmInvoiceOfferParameters offer = new TmInvoiceOfferParameters()
	  vo.setEarlyPaymentDate(mockDate)
	  vo.setDiscountAmount(mockAmount)
	  vo.setDiscountPercentage(discountPercentage)
	  vo.setNotionalEarlyPayAmt(mockAmount)
	  vo.setChgPriceTemplateId("Test123")
	  vo.setCurrency("USD")
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	   
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  assembler.convertEntityModelToOfferParameters(vo, entity)
  }
	
  @Unroll("test convertEntityModelToOfferParameters null")
  def"test convertEntityModelToOfferParameters null"(){
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  assembler.convertEntityModelToOfferParameters(vo, entity)
  }
	*/
  @Unroll("test convertEntityModelToPayParameters method")
  def"test convertEntityModelToPayParameters method"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setReconTxnStatus("Test");
	  vo.setReconPmtAmountCcy("USD");
	  vo.setReconPmtAmount(mockAmount)
	  vo.setPaymentParty("12300")
	  vo.setPaymentMode("Card")
	  vo.setPaidAmount(mockAmount)
	  vo.setPaySeqNo(BigDecimal.ZERO)
		
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  assembler.convertEntityModelToPayParameters(vo, entity)
	
  }
  
  @Unroll("test convertEntityModelToPayParameters method null check")
  def"test convertEntityModelToPayParameters method null check"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertEntityModelToPayParameters(vo, entity)
	
  }
	
  @Unroll("test convertEntityModelToShipParameters method")
  def"test convertEntityModelToShipParameters method"(){
		
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId("123456");
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setShipmentRefNo("999999");
	  vo.setShipmentNo("555555");
	  vo.setShipmentDate(mockDate);
	  vo.setShipmentCountry("France");
	  vo.setShipmentAddress("Test 123");
	  vo.setShipSeqNo(BigDecimal.ZERO)
		
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  assembler.convertEntityModelToShipParameters(vo, entity)
  }
  
  @Unroll("test convertEntityModelToShipParameters method null check")
  def"test convertEntityModelToShipParameters method null check"(){
		
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertEntityModelToShipParameters(vo, entity)
	  
  }
	
  @Unroll("test convertEntityModelToFinParameters method")
  def"test convertEntityModelToFinParameters method"(){
		
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setId(mockString);
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo(mockString)
	  vo.setTransactionRefNo(mockString)
	  vo.setProgramName(mockString);
	  vo.setProgramCode(mockString);
	  vo.setOldFinanceAmount(vo.getOldFinanceAmount());
	  vo.setOldAssignmentAmount(vo.getOldAssignmentAmount());
	  vo.setIsIndirectPayment(mockString);
	  vo.setIsPUA(mockString);
	  vo.setIsFinanced(mockString);
	  vo.setIsAssigned(mockString);
	  vo.setIsApproved(mockString);
	  vo.setInsuranceCoName(mockString);
	  vo.setInsuranceCoId(mockString);
	  vo.setImportFactorName(mockString);
	  vo.setImportFactorId(mockString);
	  vo.setFundingReqAmt(mockAmount);
	  vo.setAssignmentParty(mockString);
	  vo.setFinanceAmount(BigDecimal.ZERO)
	  vo.setFinancePercentage(BigDecimal.ZERO)
	  vo.setFinSeqNo(BigDecimal.ZERO)
	  vo.setAssignmentAmount(BigDecimal.ZERO)
	  vo.setCreditCoverAmount(BigDecimal.ZERO)
		
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  assembler.convertEntityModelToFinParameters(vo, entity)
  }
  
  @Unroll("test convertEntityModelToFinParameters method null check")
  def"test convertEntityModelToFinParameters method null check"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()	
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertEntityModelToFinParameters(vo, entity)
  }
	
  /*@Unroll("test convertMainInvoiceTmCommGridVOToEntity method")
  def"test convertMainInvoiceTmCommGridVOToEntity method"(){
		
	  given:
	  InvoiceCommodityGridModel vo = new InvoiceCommodityGridModel()
	  vo.setId(mockString);
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo(mockString)
		
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
		
	  when:
		
	  def response  =  assembler.convertMainInvoiceTwCommGridVOToEntity(vo, mockString)
		
	  then:
	  response!=null
  }
	
  @Unroll("test convertMainInvoiceTwCommGridVOToEntity null")
  def"test convertMainInvoiceTwCommGridVOToEntity null"(){
		
	  given:
	  InvoiceCommodityGridModel vo = null
		
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
		
	  when:
		
	  def response  =  assembler.convertMainInvoiceTwCommGridVOToEntity(vo, mockString)
		
	  then:
	  response==null
  }*/
	
  @Unroll("test convertInvoiceDTOToInvoiceEntity method")
  def"test convertInvoiceDTOToInvoiceEntity method"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  //vo.setEarlyPaymentDate(mockDate)
	  vo.setDiscountAmount(mockAmount)
	  vo.setDiscountPercentage(discountPercentage)
	  //vo.setNotionalEarlyPayAmt(mockAmount)
	  //vo.setChgPriceTemplateId("Test123")
	  vo.setCurrency("USD")
	  vo.setId("123456");
	  vo.setInvoiceRefNo("129087")
	  vo.setTransactionRefNo("864456")
	  vo.setReconTxnStatus("Test");
	  vo.setReconPmtAmountCcy("USD");
	  vo.setReconPmtAmount(mockAmount)
	  vo.setPaymentParty("12300")
	  vo.setPaymentMode("Card")
	  vo.setPaidAmount(mockAmount)
	  vo.setShipmentRefNo("999999");
	  vo.setShipmentNo("555555");
	  vo.setShipmentDate(mockDate);
	  vo.setShipmentCountry("France");
	  vo.setShipmentAddress("Test 123");
	  vo.setProgramName(mockString);
	  vo.setProgramCode(mockString);
	  vo.setOldFinanceAmount(vo.getOldFinanceAmount());
	  vo.setOldAssignmentAmount(vo.getOldAssignmentAmount());
	  vo.setIsIndirectPayment(mockString);
	  vo.setIsPUA(mockString);
	  vo.setIsFinanced(mockString);
	  vo.setIsAssigned(mockString);
	  vo.setIsApproved(mockString);
	  vo.setInsuranceCoName(mockString);
	  vo.setInsuranceCoId(mockString);
	  vo.setImportFactorName(mockString);
	  vo.setImportFactorId(mockString);
	  vo.setFundingReqAmt(mockAmount);
	  vo.setAssignmentParty(mockString);
	  /*InvoiceCommodityGridModel vo = new InvoiceCommodityGridModel()
	  vo.setId(mockString);
	  vo.setSeqNo(12345)
	  vo.setInvoiceRefNo(mockString)*/
		
	  TmDrCrNoteLinkInvoiceModel TmDrCrNoteLinkInvoiceModel = new TmDrCrNoteLinkInvoiceModel()
	  def list = new ArrayList<>()
	  list.add(TmDrCrNoteLinkInvoiceModel)
	  vo.setTmDrCrNoteLinkInvoiceList(list)
	  
	  InvoiceOfferDTO InvoiceOfferDTO = new InvoiceOfferDTO()
	  def list1 = new ArrayList<>()
	  list1.add(InvoiceOfferDTO)
	  vo.setInvoiceOfferList(list1)
	  
	  ScfcmInvoice entity = new ScfcmInvoice()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
		
	  
		
	  assembler.convertInvoiceDTOToInvoiceEntity(entity, vo);
		
  }
  
  @Unroll("test convertInvoiceDTOToInvoiceEntity method null check 1")
  def"test convertInvoiceDTOToInvoiceEntity method null check 1"(){
		
	  given:
	  InvoiceModel vo = new InvoiceModel()
	  vo.setInvoiceCommodityGridList(new ArrayList<InvoiceCommodityGridModel>())
	  vo.setLinkedInvoiceList(new ArrayList<LinkedInvoiceModel>())
	  vo.setAcceptanceAmount(BigDecimal.ZERO)
	  vo.setAmount(BigDecimal.ZERO)
	  vo.setDiscountAmount(BigDecimal.ZERO)
	  vo.setNetInvoiceAmount(BigDecimal.ZERO)
	  vo.setOutstandingAmount(BigDecimal.ZERO)
	  vo.setTaxAmount(BigDecimal.ZERO)
	  vo.setTotalInvoiceAmount(BigDecimal.ZERO)
	  //vo.setOfferSeqNo(BigDecimal.ZERO)
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  ScfcmInvoice entity = new ScfcmInvoice()
	
	  assembler.convertInvoiceDTOToInvoiceEntity(entity, vo);
		
  }
  
  @Unroll("test convertInvoiceDTOToInvoiceEntity method null check 2")
  def"test convertInvoiceDTOToInvoiceEntity method null check 2"(){
		
	  given:
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	
	  assembler.convertInvoiceDTOToInvoiceEntity(null, null);
		
  }

  @Unroll("test convertInvocieCommGridEntityListToVOList method")
  def"test convertInvocieCommGridEntityListToVOList method"(){
	  
	  given:
	  List<InvoiceCommodityGridModel> commodityList = new ArrayList<InvoiceCommodityGridModel>();
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertInvocieCommGridEntityListToVOList(commodityList);
	  
  }
  
  @Unroll("test convertInvoiceCommGridVOToEntity method")
  def"test convertInvoiceCommGridVOToEntity method"(){
	  
	  given:
	  InvoiceCommodityGridModel model = new InvoiceCommodityGridModel();
	  model.setDiscountAmount(BigDecimal.ZERO)
	  model.setDiscountPercentage(BigDecimal.ZERO)
	  model.setNetCost(BigDecimal.ZERO)
	  model.setQuantity(BigDecimal.ZERO)
	  model.setTaxAmount(BigDecimal.ZERO)
	  model.setTaxPercentage(BigDecimal.ZERO)
	  model.setCostPerUnit(BigDecimal.ZERO)
	  model.setTotalCost(BigDecimal.ZERO)
	  model.setTotalWeight(BigDecimal.ZERO)
	  model.setUnitWeight(BigDecimal.ZERO)
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertInvoiceCommGridVOToEntity(model);
	  
  }
  
  @Unroll("test convertInvoiceCommGridVOToEntity method null model")
  def"test convertInvoiceCommGridVOToEntity method null model"(){
	  
	  given:
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertInvoiceCommGridVOToEntity(null);
	  
  }
  
  @Unroll("test convertInvoiceCommGridVOToEntity method null check")
  def"test convertInvoiceCommGridVOToEntity method null check"(){
	  
	  given:
	  InvoiceCommodityGridModel model = new InvoiceCommodityGridModel();
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertInvoiceCommGridVOToEntity(model);
	  
  }
  
  @Unroll("test convertLinkedInvoiceVOToEntity method")
  def"test convertLinkedInvoiceVOToEntity method"(){
	  
	  given:
	  TmDrCrNoteLinkInvoiceModel model = new TmDrCrNoteLinkInvoiceModel();
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertLinkedInvoiceVOToEntity(model);
	  
  }
  
  @Unroll("test convertLinkedInvoiceVOToEntity method null check")
  def"test convertLinkedInvoiceVOToEntity method null check"(){
	  
	  given:
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertLinkedInvoiceVOToEntity(null);
	  
  }
  
  @Unroll("test convertLinkedInvoiceVOListToEntityList method")
  def"test convertLinkedInvoiceVOListToEntityList method"(){
	  
	  given:
	  List<TmDrCrNoteLinkInvoiceModel> linkedInvoiceList = new ArrayList<TmDrCrNoteLinkInvoiceModel>()
	  linkedInvoiceList.add(new TmDrCrNoteLinkInvoiceModel())
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.convertLinkedInvoiceVOListToEntityList(linkedInvoiceList);
	  
  }
  
  @Unroll("test populateLinkedInvoices method")
  def"test populateLinkedInvoices method"(){
	  
	  given:
	  List<UpdateDrCrNoteLinkInvoiceDto> updateDrCrNoteLinkInvoiceDtoList = new ArrayList<UpdateDrCrNoteLinkInvoiceDto>()
	  UpdateDrCrNoteLinkInvoiceDto updateDrCrNoteLinkInvoiceDto = new UpdateDrCrNoteLinkInvoiceDto()
	  updateDrCrNoteLinkInvoiceDto.seqNo = "1";
	  updateDrCrNoteLinkInvoiceDtoList.add(updateDrCrNoteLinkInvoiceDto)
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.populateLinkedInvoices(updateDrCrNoteLinkInvoiceDtoList);
	  
  }
  
  @Unroll("test populateLinkedInvoices method null check")
  def"test populateLinkedInvoices method null check"(){
	  
	  given:
	  List<UpdateDrCrNoteLinkInvoiceDto> updateDrCrNoteLinkInvoiceDtoList = new ArrayList<UpdateDrCrNoteLinkInvoiceDto>()
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.populateLinkedInvoices(updateDrCrNoteLinkInvoiceDtoList);
	  
  }
  
  @Unroll("test populateCommodityGridList method")
  def"test populateCommodityGridList method"(){
	  
	  given:
	  List<UpdateInvoiceCommodityGridDto> updateCommodityGridDtoList = new ArrayList<UpdateInvoiceCommodityGridDto>()
	  UpdateInvoiceCommodityGridDto updateInvoiceCommodityGridDto = new UpdateInvoiceCommodityGridDto()
	  updateInvoiceCommodityGridDto.seqNo = "1"
	  updateInvoiceCommodityGridDto.discountAmount = "100"
	  updateInvoiceCommodityGridDto.discountPercentage = "10"
	  updateInvoiceCommodityGridDto.taxPercentage = "10"
	  updateInvoiceCommodityGridDto.costPerUnit = "1"
	  updateInvoiceCommodityGridDto.totalCost = "100"
	  updateInvoiceCommodityGridDto.netCost = "90"
	  updateInvoiceCommodityGridDto.quantity = "10"
	  updateInvoiceCommodityGridDto.unitWeight = "1"
	  updateInvoiceCommodityGridDto.totalWeight = "100"
	  updateInvoiceCommodityGridDto.taxAmount = "null"
	  updateCommodityGridDtoList.add(updateInvoiceCommodityGridDto)
	  ScfInvoiceAssembler assembler = new ScfInvoiceAssembler()
	  
	  assembler.populateCommodityGridList(updateCommodityGridDtoList);
	  
  }
  
  
}