package oracle.fsgbu.customer.test.client

import com.fasterxml.jackson.annotation.JsonProperty
import oracle.fsgbu.gus.persistence.provider.GlobalHolder
import oracle.fsgbu.gus.persistence.provider.RequestContext
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO
import oracle.fsgbu.obscfcm.services.report.client.AlertListImpl
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObTmAlertPartyContactDetailsModel
import oracle.fsgbu.reconService.api.model.FinPmtReconDTO
import org.springframework.http.ResponseEntity
import org.springframework.web.client.RestTemplate
import spock.lang.Specification
import spock.lang.Unroll

class AlertListImplSpec extends Specification {
    RestTemplate restTemplate = Mock()
    def alertImpl = new AlertListImpl(restTemplate)

    @Unroll("test setParams method")
    def "test setParams method"() {
        given:
        List<Object> objlst = new ArrayList<>()
        ParameterDTO dto = new ParameterDTO()
        dto.setKey("RECON_CATEGORY")
        dto.setValue("getReconCategory")
        objlst.add(dto)

        FinPmtReconDTO finPmtReconDTO = new FinPmtReconDTO()
        finPmtReconDTO.setReconCategory("TEST_RECON_CAT")
        when:
        def list = alertImpl.setParams(objlst, finPmtReconDTO)
        then:
        list != null
    }

    @Unroll("test getparams method")
    def "test getparams method"() {
        given:
        ResponseEntity<Object> response = Mock();
        restTemplate.exchange(_, _, _, _) >> response
        List<Object> body = new ArrayList<>()
        Map map = new LinkedHashMap()
        map.put("key", "key1")
        map.put("value", "value1")
        body.add(map)
        response.getBody() >> body
        when:
        def list = alertImpl.getparams("ALERT_CATEGORY", "userid", "branchCode")
        then:
        list != null
    }

    @Unroll("test getAlertContactDetails method")
    def "test getAlertContactDetails method"() {
        given:
        ResponseEntity<Object> response = Mock();

        Map body = new LinkedHashMap()
        Map innermap = new LinkedHashMap()
        List<ObTmAlertPartyContactDetailsModel> list = new ArrayList<>()
        list.add(getObTmAlertPartyContactDetailsModel())
        innermap.put("ObTmAlertPartyContactDetailsModel", list)
        List<Object> outerList = new ArrayList<>()
        outerList.add(innermap)
        body.put("data", outerList)
        restTemplate.exchange(_, _, _, _, _) >> response
        response.getBody() >> body

        when:
        def contactDetails = alertImpl.getAlertContactDetails("userId", "branchCode", "partyId", "appCode")
        then:
        contactDetails != null
    }

    def getObTmAlertPartyContactDetailsModel() {
//        ObTmAlertPartyContactDetailsModel model = new ObTmAlertPartyContactDetailsModel()
        Map<String, String> model = new LinkedHashMap()
        model.put("id", "id")
        model.put("code", "code")
        model.put("masterId", "masterId")
        model.put("desc", "desc")
        model.put("email","nikhil.kalaskar@oracle.com")
        model.put("mobile", "123456")
        return model
    }
}
