package oracle.fsgbu.customer.test.client

import oracle.fsgbu.cmc.services.branch.dto.CmcTmSystemDatesCollection
import oracle.fsgbu.cmc.services.branch.dto.CmcTmSystemDatesDto
import oracle.fsgbu.cmc.services.branch.service.wrapper.CmcTmSystemDatesWrapper
import oracle.fsgbu.cmc.services.cmc.service.wrapper.CmcTmCoreBranchWrapper
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClientImpl
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.client.HttpClientErrorException
import spock.lang.Specification
import spock.lang.Unroll

import static org.springframework.http.ResponseEntity.ok

class CmcBranchServiceClientImplSpec extends Specification{

	@Unroll("test getCurrentBusinessDate method")
	def"test getCurrentBusinessDate method"(){
		given:
		def branchCode = "004"
		def cmcTmSystemDatesWrapper = Mock(CmcTmSystemDatesWrapper)
		ResponseEntity responseEntity = ok("\"toDay\": \"" + (new Date()).toTimestamp().toString().substring(0, 10) + "\"");
		def cmcTmSystemDatesCollection = new CmcTmSystemDatesCollection()
		List<CmcTmSystemDatesDto> data = new ArrayList<>()
		CmcTmSystemDatesDto dto = new CmcTmSystemDatesDto()
		dto.setToday("2023-04-19")
		data.add(dto)
		cmcTmSystemDatesCollection.setData(data)
		responseEntity = ok(cmcTmSystemDatesCollection)
		cmcTmSystemDatesWrapper.getAll(null, null, null,
				branchCode, null, null, branchCode) >> responseEntity;
		CmcBranchServiceClient branchClient = new CmcBranchServiceClientImpl(cmcTmSystemDatesWrapper);
		when:
		def response = branchClient.getCurrentBusinessDate("CITI001", branchCode)
		println response
		then:
		response != null;
	}

	@Unroll("test getCurrentBusinessDate exception method")
	def "test getCurrentBusinessDate exception method"() {
		given:
		def cmcTmSystemDatesWrapper = Mock(CmcTmSystemDatesWrapper)
		CmcBranchServiceClientImpl branchClient = new CmcBranchServiceClientImpl(cmcTmSystemDatesWrapper);
		def HttpStatusCodeException = new HttpClientErrorException(HttpStatus.BAD_REQUEST)

		cmcTmSystemDatesWrapper.getAll(null, null, null,
				null, null, null, null) >> { throw HttpStatusCodeException }
		when:
		def response = branchClient.getCurrentBusinessDate("CITI001", "004")
		then:
		response == response
	}

	@Unroll("test getBranchDetails method")
	def"test getBranchDetails method"(){
		given:
		def cmcTmCoreBranchWrapper = Mock(CmcTmCoreBranchWrapper)
		CmcBranchServiceClient branchClient = new CmcBranchServiceClientImpl(cmcTmCoreBranchWrapper);
		when:
		def response = branchClient.getBranchDetails()
		then:
		response ==null;
	}

	@Unroll("test getBranchDetails exception method")
	def"test getBranchDetails exception method"(){
		given:
		def cmcTmCoreBranchWrapper = Mock(CmcTmCoreBranchWrapper)

		CmcBranchServiceClientImpl branchClient = new CmcBranchServiceClientImpl(cmcTmCoreBranchWrapper);
		def HttpStatusCodeException = new HttpClientErrorException(HttpStatus.BAD_REQUEST)
		cmcTmCoreBranchWrapper.getAll(null, null, null, null,
				null, null, null, null, null, null,
				null, null, null, null, null, null,
				null, null, null, null)
				>> {throw  HttpStatusCodeException}
		when:
		def response = branchClient.getBranchDetails()
		then:
		response == response
	}
}