package oracle.fsgbu.customer.test.client

import java.sql.Timestamp
import java.time.LocalDate

import org.springframework.http.ResponseEntity
import org.springframework.web.client.RestTemplate

import oracle.fsgbu.gus.response.service.ResponseHandler
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClient
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClientImpl
import oracle.fsgbu.obscfcm.services.report.model.CurrencyRateCalculatorDtoWrapper
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil
import spock.lang.Shared
import spock.lang.Specification
import spock.lang.Unroll

class CmcCurrencyServiceClientImplSpec extends Specification{
	
	 def restTemplate = Mock(RestTemplate)
	 
	 @Shared CurrencyRateCalculatorDtoWrapper dto = new CurrencyRateCalculatorDtoWrapper()
	 @Shared CurrencyUtilityDto currencyDto = new CurrencyUtilityDto()
	 
	 @Unroll("test getConvertedCcyValue method")
	 def"test getConvertedCcyValue"(){
	  given:
		 def restTemplate = Mock(RestTemplate);
		 
		 currencyDto.setAmount(new BigDecimal(100));
		 currencyDto.setRate(new BigDecimal(10));
		 
		 dto.setData(currencyDto);
		 ResponseEntity<CurrencyRateCalculatorDtoWrapper> responseEntity = ResponseEntity.ok(dto);
		 
		 restTemplate.exchange(_,_,_,_)>>responseEntity;
		 
		 CmcCurrencyServiceClientImpl client = new CmcCurrencyServiceClientImpl(restTemplate)
		 client.protocol = "https"
	  
	  when:
	  	def response = client.getConvertedCcyValue("INR","USD",new BigDecimal(10),null,null,null)
	  then:
	  	response != null;
	 }
	 
	 @Unroll("test getConvertedCcyValueWithNullResponse method")
	 def"test getConvertedCcyValueWithNullResponse"(){
	  given:
		 def restTemplate = Mock(RestTemplate);
		 
		 currencyDto.setAmount(new BigDecimal(100));
		 currencyDto.setRate(new BigDecimal(10));
		 
		 dto.setData(currencyDto);
		 ResponseEntity<CurrencyRateCalculatorDtoWrapper> responseEntity = ResponseEntity.ok(dto);
		 
		 restTemplate.exchange(null,null,null,null)>>responseEntity;
		 
		 CmcCurrencyServiceClientImpl client = new CmcCurrencyServiceClientImpl(restTemplate)
	  
	  when:
		  def response = client.getConvertedCcyValue("INR","USD",new BigDecimal(10),null,null,null)
	  then:
		  response == null;
	 }
	 
	 @Unroll("test getConvertedCcyValueWithException method")
	 def"test getConvertedCcyValueWithException"(){
	  given:
		 def restTemplate = Mock(RestTemplate);
		 
		 currencyDto.setAmount(new BigDecimal(100));
		 currencyDto.setRate(new BigDecimal(10));
		 
		 dto.setData(currencyDto);
		 ResponseEntity<CurrencyRateCalculatorDtoWrapper> responseEntity = ResponseEntity.ok(dto);
		 
		 restTemplate.exchange(null,null,null,null)>>responseEntity;
		 
		 CmcCurrencyServiceClientImpl client = new CmcCurrencyServiceClientImpl(null)
	  
	  when:
		  def response = client.getConvertedCcyValue("INR","USD",new BigDecimal(10),null,null,null)
	  then:
		  response == null;
	 }
}