package oracle.fsgbu.customer.test.client

import oracle.fsgbu.accountMaintenance.api.model.RoleWebModel
import oracle.fsgbu.obscfcm.services.report.client.ReceiverDetailsClientImpl
import org.springframework.http.ResponseEntity
import org.springframework.web.client.RestTemplate
import spock.lang.Specification
import spock.lang.Unroll

class ReceiverDetailsClientImplSpec extends Specification {
    RestTemplate restTemplate = Mock()
    ReceiverDetailsClientImpl instance = new ReceiverDetailsClientImpl(restTemplate)

    private void setupResponse(ResponseEntity<RoleWebModel> response, Object model) {
        restTemplate.exchange(_, _, _, _, _) >> response
        response.getBody() >> model
    }

    @Unroll("test fetchaddress method")
    def "test fetchaddress method"() {
        given:
        ResponseEntity<Object> response = Mock()
        Map<String, List<Object>> res = new LinkedHashMap<>()
        List<Object> list = new ArrayList<>()
        Map<String, String> model = new LinkedHashMap<>()

        model.put("customerName1","name1")
        model.put("addressLine1","addressLine1")
        model.put("addressLine2","addressLine2")
        model.put("addressLine3","addressLine3")
        model.put("addressLine4","addressLine4")
        model.put("country","country")
        model.put("pincode","pincode")

        list.add(model)
        res.put("data", list)
        setupResponse(response, res)
        when:
        def resList = instance.fetchaddress("customerNo")
        then:
        resList != null
    }

    @Unroll("test fetchaddress method negative")
    def "test fetchaddress method negative"() {
        given:
        ResponseEntity<Object> response = Mock()
        Map<String, List<Object>> res = new LinkedHashMap<>()
        List<Object> list = new ArrayList<>()
        Map<String, String> model = new LinkedHashMap<>()

        model.put("customerName1","name1")
        model.put("addressLine1","addressLine1")
        model.put("addressLine2","addressLine2")
        model.put("addressLine3","addressLine3")
        model.put("addressLine4","addressLine4")
        model.put("country","country")
        model.put("pincode","pincode")

        list.add(model)
        res.put("data", list)
        when:
        def resList = instance.fetchaddress("customerNo")
        then:
        resList == null
    }

    @Unroll("test fetchSenderaddress method")
    def "test fetchSenderaddress method"() {
        given:
        ResponseEntity<Object> response = Mock()
        Map<String, List<Object>> res = new LinkedHashMap<>()
        List<Object> list = new ArrayList<>()
        Map<String, String> model = new LinkedHashMap<>()

        model.put("branchName","branchName")
        model.put("branchAddr1","branchAddr1")
        model.put("branchAddr2","branchAddr2")
        model.put("branchAddr3","branchAddr3")

        list.add(model)
        res.put("data", list)
        setupResponse(response, res)
        when:
        def resList = instance.fetchSenderaddress("branchCode")
        then:
        resList != null
    }

    @Unroll("test fetchSenderaddress method negative")
    def "test fetchSenderaddress method negative"() {
        given:
        ResponseEntity<Object> response = Mock()
        Map<String, List<Object>> res = new LinkedHashMap<>()
        List<Object> list = new ArrayList<>()
        Map<String, String> model = new LinkedHashMap<>()

        model.put("branchName","branchName")
        model.put("branchAddr1","branchAddr1")
        model.put("branchAddr2","branchAddr2")
        model.put("branchAddr3","branchAddr3")

        list.add(model)
        res.put("data", list)
        when:
        def resList = instance.fetchSenderaddress("branchCode")
        then:
        resList == null
    }

}
