package oracle.fsgbu.customer.test.service

import oracle.fsgbu.accountMaintenance.api.model.AccountingTransactionModel;
import oracle.fsgbu.eventService.api.model.*
import oracle.fsgbu.ob.adapter.dto.OBIntegrationServiceResponse
import oracle.fsgbu.ob.adapter.provider.OBIntegrationAdapter
import oracle.fsgbu.obscfcm.alertsgenerator.AlertMessageProcessor
import oracle.fsgbu.obscfcm.alertsgenerator.dto.*
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AdvicesClient
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertDecisionClient
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertsPlaceholdersClient
import oracle.fsgbu.obscfcm.services.report.client.AlertList
import oracle.fsgbu.obscfcm.services.report.client.InstrumentReceivablesServiceClient
import oracle.fsgbu.obscfcm.services.report.client.ReceiverDetailsClient
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants
import oracle.fsgbu.obscfcm.services.report.domain.entity.AccountPost
import oracle.fsgbu.obscfcm.services.report.domain.entity.AccountPostTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.AutoDebitTxnDetails
import oracle.fsgbu.obscfcm.services.report.domain.entity.AutoDebitTxnMaster
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmInvoice
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTwPayment
import oracle.fsgbu.obscfcm.services.report.domain.entity.ScfcmTwPaymentPK
import oracle.fsgbu.obscfcm.services.report.domain.entity.TmCreditNote
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TmCommodityList
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TmPurchaseOrder
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwPOShipParameters
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwPurchaseOrder
import oracle.fsgbu.obscfcm.services.report.domain.entity.po.TwPurchaseOrderPk
import oracle.fsgbu.obscfcm.services.report.persistence.*
import oracle.fsgbu.obscfcm.services.report.persistence.po.TmPurchaseOrderRepository
import oracle.fsgbu.obscfcm.services.report.persistence.po.TwPurchaseOrderRepository
import oracle.fsgbu.obscfcm.services.report.service.EventListenerServiceImpl
import spock.lang.Shared
import spock.lang.Specification
import spock.lang.Unroll
import oracle.fsgbu.obscfcm.services.alert.dto.UpdatedPlaceholdersDto;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO;
import oracle.fsgbu.obscfcm.services.report.config.ReportConfigDbUtil;


class EventListenerServiceImplSpec extends Specification {
	


	def mockInvoiceRepository = Mock(InvoiceRepository)
	def mockPaymentRepository = Mock(PaymentRepository)
	def mockFileRepository = Mock(FileRepository)
	def mockCreditNoteRepository = Mock(TmCreditNoteRepository)
	def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)
	def alertDecisionClient = Mock(AlertDecisionClient);
	def alertsPlaceholdersClient= Mock(AlertsPlaceholdersClient)
	def alertList = Mock(AlertList)
	def instrumentReceivablesServiceClient = Mock(InstrumentReceivablesServiceClient)
	def mockPaymentTwRepository = Mock(PaymentTwRepository)

	def invoiceDto = new InvoiceDto();
	def paymentDto = new PaymentDto();
	def fileDto = new FileUploadDto()
	def tmCreditNote = new TmCreditNote();
	def invoiceModel = new InvoiceModel();
	@Shared autoDebitTxnDetails = new AutoDebitTxnDetails( invoiceRefNo:"INV200",autoDebitGrpType: "G",txnRefNo: "TXNREF1",autoDebitPartyRole:"B", autoDebitPartyId:"CUST1",autoDebitAccType:"CUST A/C", autoDebitAmtType:"abc",autoDebitCcy:"USD", autoDebitAmt:1000, executionId:"EXECID1","accountRefNo":"AUTO1000")
	@Shared autoDebitTxnMaster = new AutoDebitTxnMaster("batchJobExecId":"AUTODEB01", "timestamp": new Date(), "branchCode":"004", "userId":"SUNIL", "businessDate":new Date() , "status":null);
	TwPurchaseOrderPk id = new TwPurchaseOrderPk(poInternalNumber: "POI123",seqNo: 1)
	TwPOShipParameters twPoShipParameters = new TwPOShipParameters(reqShipmentDate: new Date(), promisedShipmentDate: new Date(),shipmentAddr: "OFSS",city: "Pune",zipCode: "411021",phoneNumber: "1234567890",taxId: "t1")
	@Shared TwPurchaseOrder twPurchaseOrder = new TwPurchaseOrder(id: id, poExternalNumber: "POE123",buyerId: "000380",poStatus: "RAISED",supplierId: "000381",programId: "PROGRAM1"
				,poDate: new Date(),buyerDivCode: "000380D", supplierDivCode: "000381D",termsOfSale: "terms", countryOfOrigin: "IN",miscChrg1Desc: "msd1",miscChrg2Desc: "msd2", filler1: "f1",filler2: "f2",
				filler3: "f3",filler4: "f4", filler5: "f5", filler6: "f6",remarks: "rmk",twPoShipParameters:twPoShipParameters,reasonForExport: "del")	
	TmCommodityList tmCommodityList = new TmCommodityList(commodityCode: "JCB", commodityName: "JCB", quantity: 2)
	TmPurchaseOrder tmPurchaseOrder = new TmPurchaseOrder(buyerId: "ABCD", buyerDivCode: "BACD", buyerName: "ABCD", buyerDivName: "BUYER1", poDate: null,
			poExternalNumber: "A123423", supplierDivCode: "RED", supplierDivName: "DEV1", supplierName: "BANK", paymentTerms: "paymentTerms", preAccepted: "Y",
			basePoAmount: 1000, discountAmount: 0, discountPercent: 0, taxAmount: 0, taxPercent: 0, netPoAmount: 1000, acceptanceAmount: 1000, acceptanceDate: null,
			fundReqAmount: 1000, filler1: "filler1", filler2: "filler2", filler3: "filler3", filler4: "filler4", filler5: "filler5", filler6: "filler6",
			tmCommodityList: Collections.singletonList(tmCommodityList));
	@Unroll("test saveInvoiceData method")
	def"test saveInvoiceData method exception"(){
		given:
		mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())
		AlertList alertList = Mock()
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList

		def alertMessageProcessor = Mock(AlertMessageProcessor)
		alertMessageProcessor.processAlertMessage(_, _, _) >> null
		service.alertMessageProcessor = alertMessageProcessor

		List<ScfcmInvoice> list1 = new ArrayList<>();
		list1.add(new ScfcmInvoice())

		mockInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> list1
		mockInvoiceRepository.save(_)

		when:
		def response = service.saveInvoiceData(invoiceModel, "DEMO", "004")
		then:
		response == response;
	}




	@Unroll("test fetchInvoiceData method 1")
	def"test fetchInvoiceData method 1 null"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)


		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> null

		eventListenerServiceImpl.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		when:
		def response = eventListenerServiceImpl.fetchInvoiceData("123", "DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test fetchInvoiceData method 2")
	def"test fetchInvoiceData method 2 empty inner list"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new InvoiceCollectionDto()
		def invoiceModelList = new ArrayList<InvoiceModel> ()
		dto.setData(invoiceModelList)

		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> dto

		eventListenerServiceImpl.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		when:
		def response = eventListenerServiceImpl.fetchInvoiceData("123", "DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test fetchInvoiceData method 3")
	def"test fetchInvoiceData method 3 null inner details"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new InvoiceCollectionDto()
		def invoiceModelList = new ArrayList<InvoiceModel> ()
		invoiceModelList.add(null)
		dto.setData(invoiceModelList)

		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> dto

		eventListenerServiceImpl.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		when:
		def response = eventListenerServiceImpl.fetchInvoiceData("123", "DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test fetchInvoiceData method 4")
	def"test fetchInvoiceData method 4 details"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new InvoiceCollectionDto()
		def invoiceModelList = new ArrayList<InvoiceModel> ()
		def invoiceModel = new InvoiceModel()
		invoiceModel.setId("1")
		invoiceModelList.add(invoiceModel)
		dto.setData(invoiceModelList)

		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> dto

		eventListenerServiceImpl.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		when:
		def response = eventListenerServiceImpl.fetchInvoiceData("123", "DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test fetchInvoiceData method 5")
	def"test fetchInvoiceData method 5 exception"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new InvoiceCollectionDto()
		def invoiceModelList = new ArrayList<InvoiceModel> ()
		def invoiceModel = new InvoiceModel()
		invoiceModel.setId("1")
		invoiceModelList.add(invoiceModel)
		dto.setData(invoiceModelList)

		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> new Exception()

		eventListenerServiceImpl.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		when:
		def response = eventListenerServiceImpl.fetchInvoiceData("123", "DEMO", "004")
		then:
		response == response;
	}


	@Unroll("test saveInvoiceData method 1")
	def"test saveInvoiceData method 1"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		receiverDetailsClient.fetchaddress(_) >> new ArrayList<ParameterDTO>()
		receiverDetailsClient.fetchSenderaddress(_) >> new ArrayList<ParameterDTO>()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		AlertDecisioningDTO alertDecisioningDTO = Mock()
		alertDecisioningDTO.getDocumentID() >> "123"
		alertDecisioningDTO.getRecipient() >> ReportConstants.SUPPLIER
		detailsCollection.add(alertDecisioningDTO)
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		ParameterDTO parameterDTO = new ParameterDTO()
		parameterDTO.setKey("INVOICE_NUMBER")
		newparameterList.add(parameterDTO)
		alertList.setParams(_,_) >> newparameterList
		
		eventListenerServiceImpl.alertMessageProcessor = alertMessageProcessor
		eventListenerServiceImpl.alertList = alertList
		eventListenerServiceImpl.alertsPlaceholdersClient = alertsPlaceholdersClient
		eventListenerServiceImpl.alertDecisionClient = alertDecisionClient
		eventListenerServiceImpl.invoiceRepository=mockInvoiceRepository
		eventListenerServiceImpl.advicesClient = advicesClient
		eventListenerServiceImpl.receiverDetails = receiverDetailsClient

		List<ScfcmInvoice> list1 = new ArrayList<>();
		list1.add(new ScfcmInvoice())

		mockInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> list1



		when:
		def response = eventListenerServiceImpl.saveInvoiceData(invoiceModel, "DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test saveInvoiceData method 2")
	def"test saveInvoiceData method 2"(){
		given:

		def mockInvoiceRepository = Mock(InvoiceRepository)
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		receiverDetailsClient.fetchaddress(_) >> new ArrayList<ParameterDTO>()
		receiverDetailsClient.fetchSenderaddress(_) >> new ArrayList<ParameterDTO>()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		AlertDecisioningDTO alertDecisioningDTO = Mock()
		alertDecisioningDTO.getDocumentID() >> "123"
		alertDecisioningDTO.getRecipient() >> ReportConstants.SUPPLIER
		detailsCollection.add(alertDecisioningDTO)
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		ParameterDTO parameterDTO = new ParameterDTO()
		parameterDTO.setKey("AMOUNT")
		newparameterList.add(parameterDTO)
		alertList.setParams(_,_) >> newparameterList
		
		eventListenerServiceImpl.alertMessageProcessor = alertMessageProcessor
		eventListenerServiceImpl.alertList = alertList
		eventListenerServiceImpl.alertsPlaceholdersClient = alertsPlaceholdersClient
		eventListenerServiceImpl.alertDecisionClient = alertDecisionClient
		eventListenerServiceImpl.invoiceRepository=mockInvoiceRepository
		eventListenerServiceImpl.advicesClient = advicesClient
		eventListenerServiceImpl.receiverDetails = receiverDetailsClient
		List<ScfcmInvoice> list1 = new ArrayList<>();

		mockInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> list1



		when:
		def response = eventListenerServiceImpl.saveInvoiceData(invoiceModel, "DEMO", "004")
		then:
		response == response;
	}

	
	def"test saveInvoiceData null list  method "(){
		given:

		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		receiverDetailsClient.fetchaddress(_) >> new ArrayList<ParameterDTO>()
		receiverDetailsClient.fetchSenderaddress(_) >> new ArrayList<ParameterDTO>()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		AlertDecisioningDTO alertDecisioningDTO = Mock()
		detailsCollection.add(alertDecisioningDTO)
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		ParameterDTO parameterDTO = new ParameterDTO()
		parameterDTO.setKey("AMOUNT")
		newparameterList.add(parameterDTO)
		alertList.setParams(_,_) >> newparameterList
		eventListenerServiceImpl.alertMessageProcessor = alertMessageProcessor
		eventListenerServiceImpl.alertList = alertList
		eventListenerServiceImpl.alertsPlaceholdersClient = alertsPlaceholdersClient
		eventListenerServiceImpl.alertDecisionClient = alertDecisionClient
		eventListenerServiceImpl.advicesClient = advicesClient
		eventListenerServiceImpl.receiverDetails = receiverDetailsClient
		def mockInvoiceRepository = Mock(InvoiceRepository)
		eventListenerServiceImpl.invoiceRepository=mockInvoiceRepository
		mockInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null

		when:
		def response = eventListenerServiceImpl.saveInvoiceData(invoiceModel, "DEMO", "004")
		then:
		response == response;
	}
	def"test saveInvoiceData list method "(){
		given:

		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		receiverDetailsClient.fetchaddress(_) >> new ArrayList<ParameterDTO>()
		receiverDetailsClient.fetchSenderaddress(_) >> new ArrayList<ParameterDTO>()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		AlertDecisioningDTO alertDecisioningDTO = Mock()
		alertDecisioningDTO.getDocumentID() >> "123"
		alertDecisioningDTO.getRecipient() >> ReportConstants.BUYER
		detailsCollection.add(alertDecisioningDTO)
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		ParameterDTO parameterDTO = new ParameterDTO()
		parameterDTO.setKey("INVOICE_DUE_DATE")
		newparameterList.add(parameterDTO)
		alertList.setParams(_,_) >> newparameterList
		eventListenerServiceImpl.alertMessageProcessor = alertMessageProcessor
		eventListenerServiceImpl.alertList = alertList
		eventListenerServiceImpl.alertsPlaceholdersClient = alertsPlaceholdersClient
		eventListenerServiceImpl.alertDecisionClient = alertDecisionClient
		eventListenerServiceImpl.advicesClient = advicesClient
		eventListenerServiceImpl.receiverDetails = receiverDetailsClient
		def mockInvoiceRepository = Mock(InvoiceRepository)
		eventListenerServiceImpl.invoiceRepository=mockInvoiceRepository

		List<ScfcmInvoice> list = new ArrayList();
		def scfcmInvoice  = new ScfcmInvoice()
		scfcmInvoice.setInvoiceDueDate(new Date())
		list.add(scfcmInvoice)

		mockInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> list

		when:
		def response = eventListenerServiceImpl.saveInvoiceData(invoiceModel, "DEMO", "004")
		then:
		response == response;
	}
	
	@Unroll("test fetchPaymentData method")
	def "test fetchPaymentData method"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new PaymentCollectionDto()
		def payTwList = new  ArrayList<PaymentDto> ()
		payTwList.add(paymentDto)
		dto.setData(payTwList)
		InstrumentReceivablesServiceClient instrumentReceivablesServiceClient = Mock()
		service.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		instrumentReceivablesServiceClient.getPaymentByPayRefNo(_,_,_) >> dto
		when:
		def response =  service.fetchPaymentData("paymentRefNo", "userId", "branchCode")
		then:
		response == response;
	}
	@Unroll("test fetchPaymentData  method exception")
	def "test fetchPaymentData method exception"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new PaymentCollectionDto()
		def payTwList = new  ArrayList<PaymentDto> ()
		payTwList.add(paymentDto)
		dto.setData(payTwList)
		InstrumentReceivablesServiceClient instrumentReceivablesServiceClient = Mock()
		service.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		instrumentReceivablesServiceClient.getPaymentByPayRefNo(_,_,_) >> new Exception()
		when:
		def response = service.fetchPaymentData("paymentRefNo", "userId", "branchCode")
		then:
		response == response;
	}

	@Unroll("test savePaymentData method")
	def"test savePaymentData method exception"(){
		given:
		def mockPaymentRepository = Mock(PaymentRepository)

		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)

		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList

		List<PaymentDto> list1 = new ArrayList<>();
		list1.add(paymentDto)
		mockPaymentRepository.executeNamedQueryByCondition(_,_,_) >> list1

		when:
		def response = service.savePaymentData(paymentDto,"DEMO", "004")
		then:
		response == false;
	}

	@Unroll("test savePaymentData method")
	def"test savePaymentData method "(){
		given:

		def mockPaymentRepository = Mock(PaymentRepository)
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("")

		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList

		List list1 = new ArrayList();

		mockPaymentRepository.executeNamedQueryByCondition(_,_,_) >> list1

		when:
		def response = service.savePaymentData(paymentDto,"DEMO", "004")
		then:
		response !=null;
	}

	def"test savePaymentData null list  method "(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList
		def mockPaymentRepository = Mock(PaymentRepository)
		service.paymentRepository=mockPaymentRepository
		mockPaymentRepository.executeNamedQueryByCondition(_,_,_) >> null

		when:
		def response = service.savePaymentData(paymentDto,"DEMO", "004")
		then:
		response != null;
	}

	@Unroll("test saveUploadedFileData method")
	def"test saveUploadedFileData method exception"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)

		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList

		service.fileRepository=mockFileRepository
		List<PaymentDto> list1 = new ArrayList<>();
		list1.add(fileDto)
		mockFileRepository.save(_) >>new Exception()

		when:
		def response = service.saveUploadedFileData(fileDto,"DEMO", "004")
		then:
		response == false;
	}

	@Unroll("test saveUploadedFileData method")
	def"test saveUploadedFileData method "(){
		given:

		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository,mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList


		service.fileRepository=mockFileRepository
		service.fileRepository=mockFileRepository

		List list1 = new ArrayList();

		mockFileRepository.save(_)

		when:
		def response = service.saveUploadedFileData(fileDto,"DEMO", "004")
		then:
		response == response;
	}

	@Unroll("test saveInvoiceTwData method")
	def"test saveInvoiceTwData method exception"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		detailsCollection.add(new AlertDecisioningDTO())
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient

		List<InvoiceModel> list = new ArrayList<>();
		invoiceModel.setAction("DEV")
		list.add(invoiceModel)
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> list
		mockScfTwInvoiceRepository.save(_)

		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response == false;
	}

	
	@Unroll("test saveInvoiceTwData1")
	def"test saveInvoiceTwData method1"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		
		List<Object> objlst = new ArrayList<>();
		ParameterDTO param = new ParameterDTO();
		param.setKey("INVOICE_NUMBER");
		param.setValue("INVOICE_NUMBER");
		objlst.add(param);

		
		receiverDetailsClient.fetchaddress(_) >> objlst
		receiverDetailsClient.fetchSenderaddress(_) >> objlst

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("D1")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		def updatedPlaceholdersDto = new UpdatedPlaceholdersDto();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, updatedPlaceholdersDto)
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		

		
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> objlst
		
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> objlst
		
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient
		service.invoiceRepository=mockInvoiceRepository
		service.advicesClient = advicesClient
		service.receiverDetails = receiverDetailsClient

		
		service.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		mockScfTwInvoiceRepository.save(_)

		invoiceModel.setAction("action")
		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response ==response;
	}
	
	
	
	@Unroll("test saveInvoiceTwData2")
	def"test saveInvoiceTwData method2"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		
		List<Object> objlst = new ArrayList<>();
		ParameterDTO param = new ParameterDTO();
		param.setKey("INVOICE_NUMBER");
		param.setValue("INVOICE_NUMBER");
		objlst.add(param);

		
		receiverDetailsClient.fetchaddress(_) >> objlst
		receiverDetailsClient.fetchSenderaddress(_) >> objlst

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		//deliveryDetailsDTO.setDocumentID("D1")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("BUYER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		def updatedPlaceholdersDto = new UpdatedPlaceholdersDto();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, updatedPlaceholdersDto)
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> list2 = new ArrayList<>()
		Map<String, String> model = new LinkedHashMap()
		model.put("id", "id")
		model.put("code", "code")
		model.put("masterId", "masterId")
		model.put("desc", "desc")
		model.put("email","nikhil.kalaskar@oracle.com")
		model.put("mobile", "123456")
		list2.add(model)
		alertList.getAlertContactDetails(_,_,_,_) >> list2
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> objlst
		
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> objlst
		
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient
		service.invoiceRepository=mockInvoiceRepository
		service.advicesClient = advicesClient
		service.receiverDetails = receiverDetailsClient

		
		service.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		mockScfTwInvoiceRepository.save(_)

		invoiceModel.setAction("action")
		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response ==response;
	}
	
	
	@Unroll("test saveInvoiceTwData3")
	def"test saveInvoiceTwData method3"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		
		List<Object> objlst = new ArrayList<>();
		ParameterDTO param = new ParameterDTO();
		param.setKey("INVOICE_NUMBER");
		param.setValue("INVOICE_NUMBER");
		objlst.add(param);

		
		receiverDetailsClient.fetchaddress(_) >> objlst
		receiverDetailsClient.fetchSenderaddress(_) >> objlst

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		//deliveryDetailsDTO.setDocumentID("D1")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		def updatedPlaceholdersDto = new UpdatedPlaceholdersDto();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, updatedPlaceholdersDto)
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> list2 = new ArrayList<>()
		Map<String, String> model = new LinkedHashMap()
		model.put("id", "id")
		model.put("code", "code")
		model.put("masterId", "masterId")
		model.put("desc", "desc")
		model.put("email","nikhil.kalaskar@oracle.com")
		model.put("mobile", "123456")
		list2.add(model)
		alertList.getAlertContactDetails(_,_,_,_) >> list2
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> objlst
		
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> objlst
		
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient
		service.invoiceRepository=mockInvoiceRepository
		service.advicesClient = advicesClient
		service.receiverDetails = receiverDetailsClient

		
		service.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		mockScfTwInvoiceRepository.save(_)

		invoiceModel.setAction("action")
		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response ==response;
	} 
	
	
	
	@Unroll("test saveInvoiceTwData4")
	def"test saveInvoiceTwData method4"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		
		List<Object> objlst = new ArrayList<>();
		ParameterDTO param = new ParameterDTO();
		param.setKey("INVOICE_NUMBER");
		param.setValue("INVOICE_NUMBER");
		objlst.add(param);

		
		receiverDetailsClient.fetchaddress(_) >> objlst
		receiverDetailsClient.fetchSenderaddress(_) >> objlst

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("D1")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("BUYER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		def updatedPlaceholdersDto = new UpdatedPlaceholdersDto();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, updatedPlaceholdersDto)
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> list2 = new ArrayList<>()
		Map<String, String> model = new LinkedHashMap()
		model.put("id", "id")
		model.put("code", "code")
		model.put("masterId", "masterId")
		model.put("desc", "desc")
		model.put("email","nikhil.kalaskar@oracle.com")
		model.put("mobile", "123456")
		list2.add(model)
		alertList.getAlertContactDetails(_,_,_,_) >> list2
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> objlst
		
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> objlst
		
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient
		service.invoiceRepository=mockInvoiceRepository
		service.advicesClient = advicesClient
		service.receiverDetails = receiverDetailsClient

		
		service.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		mockScfTwInvoiceRepository.save(_)

		invoiceModel.setAction("action")
		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response ==response;
	}
	
	@Unroll("test saveInvoiceTwData5")
	def"test saveInvoiceTwData method5"(){
		given:
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		ReceiverDetailsClient receiverDetailsClient = Mock()
		advicesClient.messageReplacement(_,_) >> "MOCK_STRING"
		
		List<Object> objlst = new ArrayList<>();
		ParameterDTO param = new ParameterDTO();
		param.setKey("INVOICE_NUMBER");
		param.setValue("INVOICE_NUMBER");
		objlst.add(param);

		
		receiverDetailsClient.fetchaddress(_) >> objlst
		receiverDetailsClient.fetchSenderaddress(_) >> objlst

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("D1")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		def updatedPlaceholdersDto = new UpdatedPlaceholdersDto();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, updatedPlaceholdersDto)
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> list2 = new ArrayList<>()
		Map<String, String> model = new LinkedHashMap()
		model.put("id", "id")
		model.put("code", "code")
		model.put("masterId", "masterId")
		model.put("desc", "desc")
		model.put("email","nikhil.kalaskar@oracle.com")
		model.put("mobile", "123456")
		list2.add(model)
		alertList.getAlertContactDetails(_,_,_,_) >> list2
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> objlst
		
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> objlst
		
		service.alertMessageProcessor = alertMessageProcessor
		service.alertList = alertList
		service.alertsPlaceholdersClient = alertsPlaceholdersClient
		service.alertDecisionClient = alertDecisionClient
		service.invoiceRepository=mockInvoiceRepository
		service.advicesClient = advicesClient
		service.receiverDetails = receiverDetailsClient

		
		service.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		mockScfTwInvoiceRepository.save(_)

		invoiceModel.setAction("action")
		when:
		def response = service.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response ==response;
	}

	
	def"test saveInvoiceTwData null list  method "(){
		given:

		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
		List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
		detailsCollection.add(new AlertDecisioningDTO())
		decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
		alertDecisionClient.getList(_,_) >> decisioningDetailsResponseWrapper
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList
		eventListenerServiceImpl.alertMessageProcessor = alertMessageProcessor
		eventListenerServiceImpl.alertList = alertList
		eventListenerServiceImpl.alertsPlaceholdersClient = alertsPlaceholdersClient
		eventListenerServiceImpl.alertDecisionClient = alertDecisionClient
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)
		eventListenerServiceImpl.scfTwInvoiceRepository=mockScfTwInvoiceRepository
		mockScfTwInvoiceRepository.executeNamedQueryByCondition(_,_,_) >> null
		eventListenerServiceImpl.scfTwInvoiceRepository = mockScfTwInvoiceRepository
		
		invoiceModel.setAction("DEV")

		when:
		def response = eventListenerServiceImpl.saveInvoiceTwData(invoiceModel,"DEV", "004")
		then:
		response != null;
	} 

	@Unroll("test saveCreditNote method 1")
	def"test saveCreditNote method 1"(){
		given:
		CreditNoteModel creditNoteModel = new CreditNoteModel();
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList


		when:
		def response = service.saveCreditNote(creditNoteModel,"DEMO", "004")
		then:
		response != null;
	}

	
	@Unroll("test saveCreditNote method 2")
	def"test saveCreditNote method 2"(){
		given:
		CreditNoteModel creditNoteModel = new CreditNoteModel();
		def mockScfTwInvoiceRepository = Mock(ScfTwInvoiceRepository)


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		service.alertDecisionClient = alertDecisionClient

		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)

		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter

		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse

		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())

		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList
		when:
		def response = service.saveCreditNote(creditNoteModel,"DEMO", "004")
		then:
		response != null;
	}

	def"test saveCreditNote null list  method "(){
		given:
		CreditNoteModel creditNoteModel = new CreditNoteModel()
		AlertMessageProcessor alertMessageProcessor = Mock()
		AlertList alertList = Mock()
		AlertsPlaceholdersClient alertsPlaceholdersClient = Mock()
		AlertDecisionClient alertDecisionClient = Mock()
		AdvicesClient advicesClient = Mock()
		alertDecisionClient.getList(_,_) >> null
		EventListenerServiceImpl eventListenerServiceImpl = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository, mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)

		when:
		def response = eventListenerServiceImpl.saveCreditNote(creditNoteModel,"DEMO", "004")
		then:
		response != null;
	}

	@Unroll("test generateAlert method")
	def "test generateAlert method" (){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)


		InvoiceDto invoiceDto = new InvoiceDto()
		invoiceDto.setInvoiceRefNo("INVTEST")
		invoiceDto.setSeqNo(1)
		invoiceDto.setRelationCode("ABCD")
		invoiceDto.setBranch("branchCode1")
		invoiceDto.setBuyerCode("Buyer1")

		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("nikhil.kalaskar@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency("ONETIME");
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)

		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		eventDetailsObject.setEvent("CREATE");

		AlertList alertList = Mock()
		service.alertList = alertList
		List<Object> placeHolders = new ArrayList<>()
		placeHolders.add(new ParameterDTO())
		AlertsPlaceholdersClient alertsPlaceholdersClient= Mock()
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeHolders

		ParameterDTO dto = new ParameterDTO()
		dto.setKey("abc")
		dto.setValue("xyz")
		List<ParameterDTO> paramList = new ArrayList<>()
		paramList.add(dto)
		alertList.setParams(_,_) >> paramList

		when:
		service.generateAlert(invoiceDto,
				decisioningDetailsDTO,
				eventDetailsObject,
				ReportConstants.CATEGORY_INVOICE,
				"DEM0","004")
		then:
		noExceptionThrown()
	}

	@Unroll("test generateAlert method1")
	def "test generateAlert method1" (){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)


		InvoiceDto invoiceDto = new InvoiceDto()
		invoiceDto.setInvoiceRefNo("INVTEST")
		invoiceDto.setSeqNo(1)
		invoiceDto.setRelationCode("ABCD")
		invoiceDto.setBranch("branchCode1")
		invoiceDto.setBuyerCode("Buyer1")

		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("nikhil.kalaskar@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency("ONETIME");
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)

		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		eventDetailsObject.setEvent("CREATE");

		AlertList alertList = Mock()
		service.alertList = alertList
		List<Object> placeHolders = new ArrayList<>()
		ParameterDTO dto1 = new ParameterDTO()
		dto1.setKey("abc")
		dto1.setValue("xyz")
		dto1.setSqlIdentifier("Y")
		dto1.setApplicationCode("OBSCFCM")
		dto1.setBaseQuery("select invoice_number from scfcm_invoice")
		placeHolders.add(dto1)
		AlertsPlaceholdersClient alertsPlaceholdersClient= Mock()
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeHolders

		ParameterDTO dto = new ParameterDTO()
		dto.setKey("abc")
		dto.setValue("xyz")
		dto.setSqlIdentifier("Y")
		dto.setApplicationCode("OBSCFCM")
		dto.setBaseQuery("select invoice_number from scfcm_invoice")
		List<ParameterDTO> paramList = new ArrayList<>()
		paramList.add(dto)
		alertList.setParams(_,_) >> paramList

		when:
		service.generateAlert(invoiceDto,
				decisioningDetailsDTO,
				eventDetailsObject,
				ReportConstants.CATEGORY_INVOICE,
				"DEM0","004")
		then:
		noExceptionThrown()
	}

	@Unroll("test populateAndSendAlertMessage method")
	def "test populateAndSendAlertMessage method"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)

		EventDetailsDTO eventDetailsObject = getEventDetailsDTO()
		DecisioningDetailsDTO decisioningDetailsDTO = getDecisioningDetailsDTO()
		AlertMessageProcessor alertMessageProcessor = Mock();
		service.alertMessageProcessor = alertMessageProcessor
		alertMessageProcessor.processAlertMessage(_, _, _) >> null
		when:
		service.populateAndSendAlertMessage(eventDetailsObject, decisioningDetailsDTO, "userId", "branchCode",
				"harsh.k.agarwal@oracle.com", "123456")
		then:
		noExceptionThrown()
	}

	@Unroll("test populateAndSendAlertMessage throws Exception method")
	def "test populateAndSendAlertMessage throws Exception method"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)

		EventDetailsDTO eventDetailsObject = getEventDetailsDTO().setEvent(null)
		DecisioningDetailsDTO decisioningDetailsDTO = getDecisioningDetailsDTO()
		AlertMessageProcessor alertMessageProcessor = Mock();
		service.alertMessageProcessor = alertMessageProcessor
		alertMessageProcessor.processAlertMessage(_, _, _, _) >> null
		when:
		service.populateAndSendAlertMessage(eventDetailsObject, decisioningDetailsDTO, "userId", "branchCode",
				"harsh.k.agarwal@oracle.com", "123456")
		then:
		thrown(Exception.class)
	}

	private DecisioningDetailsDTO getDecisioningDetailsDTO() {
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		decisioningDetailsDTO.setAdditionalDetails("harsh.k.agarwal@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("INVOICE");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency("ONETIME");
		decisioningDetailsDTO
	}

	private EventDetailsDTO getEventDetailsDTO() {
		EventDetailsDTO eventDetailsObject = new EventDetailsDTO();
		eventDetailsObject.setEvent("CREATE");
		eventDetailsObject
	}

	private InvoiceCollectionDto getInvoiceCollectionDto() {
		InvoiceCollectionDto invoiceCollectionDto = new InvoiceCollectionDto()
		invoiceCollectionDto
	}

	@Unroll("test fetchInvoiceData method")
	def "test fetchInvoiceData method"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		InvoiceCollectionDto invoiceCollectionDto = getInvoiceCollectionDto()
		InstrumentReceivablesServiceClient instrumentReceivablesServiceClient = Mock()
		service.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		instrumentReceivablesServiceClient.getInvoiceMaintByTxnRefNo(_,_,_) >> null
		when:
		service.saveInvoiceData(invoiceModel, "userId", "branchCode")
		then:
		noExceptionThrown()
	}


	@Unroll("test saveAccountsTwData method")
	def"test saveAccountsTwData"(){
		given:


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def accountsTwRepository =Mock(AccountsTwRepository)
		AccountingTransactionModel accountingTransactionModel = new AccountingTransactionModel()
		service.accountsTwRepository = accountsTwRepository;

		accountingTransactionModel.setRefNo("FINTEST123")
		accountingTransactionModel.setSeqNo(1)
		List<AccountPostTw> accountsWorktDeatils = new ArrayList<>()
		def obscfcmAccountPostTw = Mock(AccountPostTw);
		accountsWorktDeatils.add(obscfcmAccountPostTw);
		accountsTwRepository.executeNamedQueryByCondition(_,_,_) >> accountsWorktDeatils

		when:
		def response = service.saveAccountsTwData(accountingTransactionModel)
		then:
		response == response;
	}


	@Unroll("test saveAccountsTmData method")
	def"test saveAccountsTmData"(){
		given:


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def accountsTmRepository =Mock(AccountsTmRepository)
		AccountingTransactionModel accountingTransactionModel = new AccountingTransactionModel()
		service.accountsTmRepository = accountsTmRepository;

		accountingTransactionModel.setRefNo("FINTEST123")
		accountingTransactionModel.setSeqNo(1)
		List<AccountPost> accountsWorktDeatils = new ArrayList<>()
		def obscfcmAccountPostTw = Mock(AccountPost);
		accountsWorktDeatils.add(obscfcmAccountPostTw);
		accountsTmRepository.executeNamedQueryByCondition(_,_,_) >> accountsWorktDeatils

		when:
		def response = service.saveAccountsTmData(accountingTransactionModel)
		then:
		response == response;
	}


	@Unroll("test saveAutoDebitData method")
	def"test saveAutoDebitData"(){
		given:


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def autoDebitTxnMasterRepository =Mock(AutoDebitTxnMasterRepository)
		service.autoDebitTxnMasterRepository = autoDebitTxnMasterRepository;

		List<AutoDebitTxnMaster> accountsWorktDeatils = new ArrayList<>()
		def obscfcmAccountPostTw = Mock(AutoDebitTxnMaster);
		accountsWorktDeatils.add(obscfcmAccountPostTw);
		autoDebitTxnMasterRepository.executeNamedQueryByCondition(_,_,_) >> accountsWorktDeatils
		
		def input = new AutoDebitTxnDto()
		def innerinput = new AutoDebitTxnDetailsDto()
		innerinput.setAutoDebitGrpType("Y")
		def list  = new ArrayList<>()
		list.add(innerinput)
		input.setAutoDebitTxnDetails(list)

		when:
		def response = service.saveAutoDebitData(input)
		then:
		response == response;
	}
	
	
	@Unroll("test saveAutoDebitInstrumentData method")
	def"test saveAutoDebitInstrumentData"(){
		given:


		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def autoDebitTxnMasterRepository =Mock(AutoDebitTxnMasterRepository)
		service.autoDebitTxnMasterRepository = autoDebitTxnMasterRepository;
		
		List<AutoDebitTxnDetails> autoDebitTxnDetailsList = new ArrayList();
		autoDebitTxnDetailsList.add(autoDebitTxnDetails);
		autoDebitTxnMaster.setAutoDebitTxnDetails(autoDebitTxnDetailsList);
		List<AutoDebitTxnMaster> masterList = new ArrayList<>();
		masterList.add(autoDebitTxnMaster);
		
		autoDebitTxnMasterRepository.executeNamedQueryByCondition(_,_,_) >> masterList

		AutoDebitTxnDetailsDto dto = new AutoDebitTxnDetailsDto()
		dto.setAccountRefNo("1")
		def list1 = new ArrayList<>()
		list1.add(dto)
		def input = new AutoDebitTxnMasterDto()
		input.setBatchJobExecId("1")
		input.setAutoDebitTxnDetails(list1)
		when:
		def response = service.saveAutoDebitInstrumentData(input)
		then:
		response == response;
	}
	
	@Unroll("test fetchPaymentData method1")
	def "test fetchPaymentData method1"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		def  dto = new PaymentCollectionDto()
		def payTwList = new  ArrayList<PaymentDto> ()
		payTwList.add(paymentDto)
		dto.setData(payTwList)
		InstrumentReceivablesServiceClient instrumentReceivablesServiceClient = Mock()
		service.instrumentReceivablesServiceClient = instrumentReceivablesServiceClient
		instrumentReceivablesServiceClient.getPaymentByPayRefNo(_,_,_) >> dto
		
		ScfcmTwPayment ScfcmTwPayment = new ScfcmTwPayment();
		ScfcmTwPaymentPK pk = new ScfcmTwPaymentPK()
		pk.setPaymentRefNo("1")
		pk.setSeqNo(1L)
		ScfcmTwPayment.setId(pk)
		def list = new ArrayList<>()
		list.add(ScfcmTwPayment)
		mockPaymentTwRepository.executeNamedQueryByCondition(_,_,_) >> list
		when:
		def response =  service.fetchPaymentData("paymentRefNo", "userId", "branchCode")
		then:
		response == response;
	}
	
	@Unroll("test savePoData method1")
	def "test savePoData method1"(){
		given:
		EventListenerServiceImpl service = new EventListenerServiceImpl(mockInvoiceRepository, mockPaymentRepository,
				mockFileRepository, mockCreditNoteRepository, mockPaymentTwRepository)
		
		
		service.twPurchaseOrderRepository = Mock(TwPurchaseOrderRepository) {
			executeNamedQueryByCondition(_, _,"TwPurchaseOrder.findByInternalRefAndSeq") >> Arrays.asList(twPurchaseOrder);
		}
		
		service.tmPurchaseOrderRepository = Mock(TmPurchaseOrderRepository) {
			executeNamedQueryByCondition(_, _,"TmPurchaseOrder.findByInternalNumbers") >> Arrays.asList(tmPurchaseOrder);
		}
		
		List<PurchaseOrderDTO> purchaseOrderDTOList = new ArrayList<PurchaseOrderDTO>();
		List<PoCommodityListDTO> commodityList = Arrays.asList(new PoCommodityListDTO(commodityCode: "JCB", commodityName: "JCB", quantity: 2))
		PurchaseOrderDTO purchaseOrderDTO = new PurchaseOrderDTO(poInternalNumber: "POI123",seqNo: 1, poExternalNumber: "POE123",buyerId: "000380",poStatus: "RAISED",supplierId: "000381",programId: "PROGRAM1"
				,poDate: new Date(),buyerDivCode: "000380D", supplierDivCode: "000381D",termsOfSale: "terms", countryOfOrigin: "IN",miscChrg1Desc: "msd1",miscChrg2Desc: "msd2", filler1: "f1",filler2: "f2",
				filler3: "f3",filler4: "f4", filler5: "f5", filler6: "f6",remarks: "rmk", commodityList: commodityList);
		purchaseOrderDTOList.add(purchaseOrderDTO)
		
		DecisioningWrapperDTO decisioningWrapperDTO= new DecisioningWrapperDTO();
		List<DecisioningDetailsDTO> list = new ArrayList<>();
		DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO();
		List<DeliveryDetailsDTO> deliveryDetailsDTOList = new ArrayList<>();
		DeliveryDetailsDTO deliveryDetailsDTO = new DeliveryDetailsDTO();
		deliveryDetailsDTO.setAlertText("hi");
		deliveryDetailsDTO.setDocumentID("")
		deliveryDetailsDTOList.add(deliveryDetailsDTO);
		decisioningDetailsDTO.setAdditionalDetails("abc@oracle.com");
		decisioningDetailsDTO.setEvent("CREATE");
		decisioningDetailsDTO.setAlertCategory("PURCHASE_ORDER");
		decisioningDetailsDTO.setRecipient("SUPPLIER");
		decisioningDetailsDTO.setAlertFrequency(ReportConstants.ALERT_FREQUENCY_ONETIME);
		decisioningDetailsDTO.setDeliveryDetails(deliveryDetailsDTOList)
		list.add(decisioningDetailsDTO);
		decisioningWrapperDTO.setDetailsCollection(list)
		
		service.alertDecisionClient = alertDecisionClient
		alertDecisionClient.getDecisioningList(_, _,_) >> decisioningWrapperDTO
		
		OBIntegrationAdapter obIntegrationAdapter = Mock()
		service.obIntegrationAdapter = obIntegrationAdapter
		
		OBIntegrationServiceResponse adapterReponse = new OBIntegrationServiceResponse();
		Map<String, Object> reponseParams = new HashMap<>();
		reponseParams.put(ReportConstants.UPDATED_PLACEHOLDER_DTO, new Object())
		adapterReponse.setStatus(ReportConstants.SUCCESS)
		adapterReponse.setResponseParams(reponseParams)
		obIntegrationAdapter.handleTransaction(_) >> adapterReponse
		
		List<Object> placeholderList = new ArrayList<>()
		placeholderList.add(new Object())
		
		service.alertList = alertList
		service.alertsPlaceholdersClient=alertsPlaceholdersClient
		alertsPlaceholdersClient.getparams(_,_,_) >> placeholderList
		List<ParameterDTO> newparameterList = new ArrayList<>()
		newparameterList.add(new ParameterDTO())
		alertList.setParams(_,_) >> newparameterList
		
		when:
		def response =  service.savePoData(purchaseOrderDTOList, purchaseOrderDTOList, "userId", "branchCode")
		
		then:
		response == response;
		
	}
}