package oracle.fsgbu.customer.test.service

import oracle.fsgbu.gus.core.client.dto.CoreBranchCollection
import oracle.fsgbu.gus.core.client.dto.CoreBranchDTO
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClient
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto
import oracle.fsgbu.obscfcm.services.report.persistence.ObExpDrCrPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObFinPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconAllocPmtTxnRepository
import oracle.fsgbu.obscfcm.services.report.service.ReconDashboardServiceImpl
import spock.lang.Specification
import spock.lang.Unroll

class ReconDashbordServiceImplSpec extends Specification{

	def mockObInvPmtReconRepository = Mock(ObInvPmtReconRepository)
	def mockObExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
	def mockObFinPmtReconRepository = Mock(ObFinPmtReconRepository)
	def mockObReconAllocPmtTxnRepository = Mock(ObReconAllocPmtTxnRepository)
	def mockCmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
	def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)

	@Unroll("test getInvoiceReconCategoryList method")
	def"test getInvoiceReconCategoryList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository, 
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getInvoiceReconCategoryList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getInvoiceReconTrendGraphList method")
	def"test getInvoiceReconTrendGraphList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10), "MARCH", "YEAR", "MARCH-2021"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getInvoiceReconTrendGraphList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getCashFlowReconCategoryList method")
	def"test getCashFlowReconCategoryList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getCashFlowReconCategoryList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getCashFlowReconTrendGraphList method")
	def"test getCashFlowReconTrendGraphList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10), "MARCH", "2021", "MARCH-2021"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getCashFlowReconTrendGraphList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getAllocationReconCategoryList method")
	def"test getAllocationReconCategoryList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObReconAllocPmtTxnRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getAllocationReconCategoryList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getAllocationReconTrendGraphList method")
	def"test getAllocationReconTrendGraphList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10), "MARCH", "2021", "MARCH-2021"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObReconAllocPmtTxnRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getAllocationReconTrendGraphList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getFinanceReconCategoryList method")
	def"test getFinanceReconCategoryList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObFinPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getFinanceReconCategoryList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getFinanceReconTrendGraphList method")
	def"test getFinanceReconTrendGraphList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10), "MARCH", "2021", "MARCH-2021"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObFinPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getFinanceReconTrendGraphList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}
	
	@Unroll("test getInvoiceReconCorporateDetailsList method")
	def"test getInvoiceReconCorporateDetailsList method"(){
		given:
		
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		Object [] object  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		
		ReconDashboardServiceImpl limitDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = limitDashboardServiceImpl.getInvoiceReconCorporateDetailsList("DEV","004", "2020-04-04", "2020-10-10", "M", "MATCHED", "201914", "M", null, null )
		then:
		response==response
	}
	
	@Unroll("test getFinanceReconCorporateDetailsList method")
	def"test getFinanceReconCorporateDetailsList method"(){
		given:
		
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		Object [] object  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","FPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObFinPmtReconRepository.executeNativeQuery(_) >> resultList
		
		ReconDashboardServiceImpl limitDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = limitDashboardServiceImpl.getFinanceReconCorporateDetailsList("DEV","004", "2020-04-04", "2020-10-10", "M", "MATCHED", "201914", "M", null, null )
		then:
		response==response
	}
	
	@Unroll("test getCashflowReconCorporateDetailsList method")
	def"test getCashflowReconCorporateDetailsList method"(){
		given:
		
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		Object [] object  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","EPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList
		
		ReconDashboardServiceImpl limitDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = limitDashboardServiceImpl.getCashflowReconCorporateDetailsList("DEV","004", "2020-04-04", "2020-10-10", "M", "MATCHED", "201914", "M", null, null )
		then:
		response==response
	}
	
	@Unroll("test getAllocationReconCorporateDetailsList method")
	def"test getAllocationReconCorporateDetailsList method"(){
		given:
		
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		Object [] object  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","EPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObReconAllocPmtTxnRepository.executeNativeQuery(_) >> resultList
		
		ReconDashboardServiceImpl limitDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = limitDashboardServiceImpl.getAllocationReconCorporateDetailsList("DEV","004", "2020-04-04", "2020-10-10", "M", "MATCHED", "201914", "M", null, null )
		then:
		response==response
	}
	
	@Unroll("test getReconciliationCorporateDetailsList method")
	def"test getReconciliationCorporateDetailsList method"(){
		given:
		
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		Object [] object  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","EPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		
		ReconDashboardServiceImpl limitDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = limitDashboardServiceImpl.getReconciliationCorporateDetailsList("DEV","004", "2020-04-04", "2020-10-10", "M", "MATCHED", "201914", "M", null, null )
		then:
		response==response
	}
	
	
	@Unroll("test getInvoiceReconTopCorporateList method")
	def"test getInvoiceReconTopCorporateList method"(){
		given:
		
		String query1 = " SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID,  NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,      PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME      FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,      PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME      FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,      PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME        FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,      PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME        FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'I'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' ";
		String query2 = " SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.CORPORATE_NAME) AS CORPORATE_COUNT FROM( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL     AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL     AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'    AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'I'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001'     ) A   GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE  ROWNUM <=5";
	
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(query2) >> resultList
		
		mockObInvPmtReconRepository.executeNativeQuery(query1) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getInvoiceReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "U","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getInvoiceReconTopCorporateList method1")
	def"test getInvoiceReconTopCorporateList method1"(){
		given:
		
		String query1 = " SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE =  '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' UNION ALL     SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME       FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='000381' ";
		String query2 = " SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM ( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN(  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL     SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN (  ) AND PAYMENT.PAYMENT_STATUS = ''      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001'  )A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC ) WHERE  ROWNUM <=5";
	
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(query2) >> resultList
		
		mockObInvPmtReconRepository.executeNativeQuery(query1) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getInvoiceReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "I","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getCashflowReconTopCorporateList method")
	def"test getCashflowReconTopCorporateList method"(){
		given:
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getCashflowReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "U","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getCashflowReconTopCorporateList method1")
	def"test getCashflowReconTopCorporateList method1"(){
		given:
			
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getCashflowReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "I","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getFinanceReconTopCorporateList method")
	def"test getFinanceReconTopCorporateList method"(){
		given:
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		mockObFinPmtReconRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getFinanceReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "U","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getFinanceReconTopCorporateList method1")
	def"test getFinanceReconTopCorporateList method1"(){
		given:
			
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		
		mockObFinPmtReconRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getFinanceReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "I","Y","2019-01-13","00001")
		then:
		response==response
	}

	
	@Unroll("test getAllocationReconTopCorporateList method")
	def"test getAllocationReconTopCorporateList method"(){
		given:
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		mockObReconAllocPmtTxnRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getAllocationReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "U","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	@Unroll("test getAllocationReconTopCorporateList method1")
	def"test getAllocationReconTopCorporateList method1"(){
		given:
			
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(_) >> resultList
		
		mockObReconAllocPmtTxnRepository.executeNativeQuery(_) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getAllocationReconTopCorporateList("DEV","004","2019-01-13","2019-04-13", "I","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	
	@Unroll("test getReconciliationTrendGraphList method")
	def"test getReconciliationTrendGraphList method"(){
		given:
		
		Object [] object  = ["AUTO_RECON", new BigDecimal(10), "MARCH", "YEAR", "MARCH-2021"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		mockObExpDrCrPmtReconRepository.executeNativeQuery(_) >> resultList

		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getReconciliationTrendGraphList("DEV","004","2019-01-13","2019-04-13", "201914")
		then:
		response==response
	}

	
	
	@Unroll("test getReconciliationTopCorporateList method")
	def"test getReconciliationTopCorporateList method"(){
		given:
		
		String query1 = "SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME            FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN  (  )     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE ='004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME   FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN (  )     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE ='004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME    FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN (  )     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PMT_PROGRAM_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO    AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME 	     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER                        AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME 		     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')	  AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL                      AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' UNION ALL 	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')    AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL                       AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('2019-01-13', 'yyyy/mm/dd')    AND TO_DATE ('2019-04-13', 'yyyy/mm/dd') AND REL.CORPORATE_ID ='000381' ";
		String query2 = " SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM (  	SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN (  )   JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE ='004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN (  )     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE ='004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN (  )     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PMT_PROGRAM_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO    AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN (  )  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER                        AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')	  AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  			     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL                      AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' UNION ALL 				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')    AND RPAYMENT.RECON_TYPE IN (  )   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL                       AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = '004'  AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('2019-01-13', 'MON-YYYY'), 'MON-YYYY') AND REL.CORPORATE_ID ='00001' 	)A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE  ROWNUM <=5";
	
		
		Object [] object  = ["DANONE","000381",
			new BigDecimal(1000000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		
		Object [] object1  = ["PEGATRON","201914","MATCHED","RECONCILE",
			new BigDecimal(1000000), new BigDecimal(800000),new BigDecimal(200000),"USD","IPR","123456",
			"C","201914",  "PEGATRON", "201914","PEGATRON"]
		List<Object []> resultList1 = new ArrayList()
		resultList1.add(object1)
		mockObInvPmtReconRepository.executeNativeQuery(query2) >> resultList
		
		mockObInvPmtReconRepository.executeNativeQuery(query1) >> resultList1
		
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		
		
		ReconDashboardServiceImpl reconDashboardServiceImpl= new ReconDashboardServiceImpl( mockObInvPmtReconRepository, mockObExpDrCrPmtReconRepository,
			mockObFinPmtReconRepository, mockObReconAllocPmtTxnRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
		def response = reconDashboardServiceImpl.getReconciliationTopCorporateList("DEV","004","2019-01-13","2019-04-13", "U","Y","2019-01-13","00001")
		then:
		response==response
	}
	
	
	
	
}
