package oracle.fsgbu.customer.test.service

import oracle.fsgbu.obscfcm.alertsgenerator.AlertMessageProcessor
import oracle.fsgbu.obscfcm.alertsgenerator.dto.AlertDecisioningDTO
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningDetailsDTO
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningDetailsResponseWrapper
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DecisioningWrapperDTO
import oracle.fsgbu.obscfcm.alertsgenerator.dto.DeliveryDetailsDTO
import oracle.fsgbu.obscfcm.alertsgenerator.dto.EventDetailsDTO
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO
import oracle.fsgbu.obscfcm.alertsgenerator.restclient.AlertDecisionClient
import oracle.fsgbu.obscfcm.services.report.client.AlertList
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObExpDrCrPmtReconTxn
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconFinPmtTxn
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvPmtTxn
import oracle.fsgbu.obscfcm.services.report.persistence.ObExpDrCrPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObFinPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvCnReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvPmtReconRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconAllocPmtTxnRepository
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconExecTxnTwRepository
import oracle.fsgbu.obscfcm.services.report.service.ReconEventListenerServiceImpl
import oracle.fsgbu.reconService.api.model.ExpDrCrPmtReconDTO
import oracle.fsgbu.reconService.api.model.FinPmtReconDTO
import oracle.fsgbu.reconService.api.model.InvCnReconDTO
import oracle.fsgbu.reconService.api.model.InvPmtReconDTO
import oracle.fsgbu.reconService.api.model.PmtAllocDTO
import oracle.fsgbu.reconService.api.model.ReconTxnDto
import spock.lang.Specification
import spock.lang.Unroll


class ReconListenerServiceImplSpec extends Specification {

    def obExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
    def obInvPmtReconRepository = Mock(ObInvPmtReconRepository)
    def obFinPmtReconRepository = Mock(ObFinPmtReconRepository)
    def obReconAllocPmtTxnRepository = Mock(ObReconAllocPmtTxnRepository)
    def obInvCnReconRepository = Mock(ObInvCnReconRepository)
    def invCnReconDTO = new InvCnReconDTO()
    def expDrCrPmtReconDTO = new ExpDrCrPmtReconDTO();
    def invPmtReconDTO = new InvPmtReconDTO();
    def finPmtReconDTO = new FinPmtReconDTO();
    def pmtAllocDTO = new PmtAllocDTO();
    def obExpDrCrPmtReconTxn = new ObExpDrCrPmtReconTxn()
    def obInvPmtReconTxn = new ObReconInvPmtTxn()
    def obFinPmtReconTxn = new ObReconFinPmtTxn()

    @Unroll("test saveInvCnReconData method")
    def "test saveInvCnReconData method empty list "() {
        given:

        def mockObInvCnReconRepository = Mock(ObInvCnReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvCnReconRepository = mockObInvCnReconRepository
        List list = new ArrayList();
        mockObInvCnReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvCnReconData(invCnReconDTO, "DEV", "004")
        then:
        response == true;
    }

    def "test saveInvCnReconData method null list "() {
        given:

        def mockObInvCnReconRepository = Mock(ObInvCnReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvCnReconRepository = mockObInvCnReconRepository
        mockObInvCnReconRepository.executeNamedQueryByCondition(_, _, _) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvCnReconData(invCnReconDTO, "DEV", "004")
        then:
        response == true;
    }

    def "test saveInvCnReconData exception method "() {
        given:
        def mockObInvCnReconRepository = Mock(ObInvCnReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvCnReconRepository = mockObInvCnReconRepository
        List<InvCnReconDTO> list = new ArrayList<>();
        list.add(invCnReconDTO)
        mockObInvCnReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvCnReconData(invCnReconDTO, "DEV", "004")
        then:
        response == false;
    }


    @Unroll("test saveExpDrCrReconData method")
    def "test saveExpDrCrReconData method empty list "() {
        given:

        def obExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository
        List list = new ArrayList();
        obExpDrCrPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveExpDrCrReconData(expDrCrPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveExpDrCrReconData method null list "() {
        given:

        def obExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository

        obExpDrCrPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveExpDrCrReconData(expDrCrPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveExpDrCrReconData exception method "() {
        given:

        def obExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository

        List<ExpDrCrPmtReconDTO> list = new ArrayList<>();
        list.add(expDrCrPmtReconDTO)
        obExpDrCrPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveExpDrCrReconData(expDrCrPmtReconDTO, "DEV", "004")
        then:
        response == false;


    }

    def "test saveExpDrCrReconData method "() {
        given:

        def obExpDrCrPmtReconRepository = Mock(ObExpDrCrPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository

        List<ObExpDrCrPmtReconTxn> list = new ArrayList<>();
        list.add(obExpDrCrPmtReconTxn)
        obExpDrCrPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveExpDrCrReconData(expDrCrPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    @Unroll("test saveInvPmtReconData method")
    def "test saveInvPmtReconData method empty list"() {
        given:

        def obInvPmtReconRepository = Mock(ObInvPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvPmtReconRepository = obInvPmtReconRepository
        List list = new ArrayList();

        obInvPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvPmtReconData(invPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveInvPmtReconData  exception method "() {
        given:

        def obInvPmtReconRepository = Mock(ObInvPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvPmtReconRepository = obInvPmtReconRepository
        List<InvPmtReconDTO> list = new ArrayList<>()
        list.add(invPmtReconDTO)
        obInvPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvPmtReconData(invPmtReconDTO, "DEV", "004")
        then:
        response == false;


    }

    def "test saveInvPmtReconData method "() {
        given:

        def obInvPmtReconRepository = Mock(ObInvPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvPmtReconRepository = obInvPmtReconRepository
        List<ObReconInvPmtTxn> list = new ArrayList<>()
        list.add(obInvPmtReconTxn)
        obInvPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvPmtReconData(invPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveInvPmtReconData null method "() {
        given:

        def obInvPmtReconRepository = Mock(ObInvPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obInvPmtReconRepository = obInvPmtReconRepository
        obInvPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveInvPmtReconData(invPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    @Unroll("test saveFinPmtReconData method")
    def "test saveFinPmtReconData empty list method "() {
        given:
        def obFinPmtReconRepository = Mock(ObFinPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obFinPmtReconRepository = obFinPmtReconRepository
        List<FinPmtReconDTO> list = new ArrayList<>()
        obFinPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveFinPmtReconData(finPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveFinPmtReconData exception method "() {
        given:
        def obFinPmtReconRepository = Mock(ObFinPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obFinPmtReconRepository = obFinPmtReconRepository
        List<FinPmtReconDTO> list = new ArrayList()
        list.add(finPmtReconDTO)

        obFinPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveFinPmtReconData(finPmtReconDTO, "DEV", "004")
        then:
        response == false;


    }

    def "test saveFinPmtReconData  method "() {
        given:
        def obFinPmtReconRepository = Mock(ObFinPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obFinPmtReconRepository = obFinPmtReconRepository
        List<ObReconFinPmtTxn> list = new ArrayList()
        list.add(obFinPmtReconTxn)

        obFinPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveFinPmtReconData(finPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }

    def "test saveFinPmtReconData null method "() {
        given:
        def obFinPmtReconRepository = Mock(ObFinPmtReconRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obFinPmtReconRepository = obFinPmtReconRepository

        obFinPmtReconRepository.executeNamedQueryByCondition(_, _, _) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.saveFinPmtReconData(finPmtReconDTO, "DEV", "004")
        then:
        response == true;


    }


    @Unroll("test savePmtAllocData method")
    def "test savePmtAllocData method empty list "() {

        given:
        def mockObReconAllocPmtTxnRepository = Mock(ObReconAllocPmtTxnRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obReconAllocPmtTxnRepository = mockObReconAllocPmtTxnRepository
        List list = new ArrayList();
        mockObReconAllocPmtTxnRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.savePmtAllocData(pmtAllocDTO, "DEV", "004")
        then:
        response == true;

    }

    def "test savePmtAllocData method null list "() {

        given:
        def mockObReconAllocPmtTxnRepository = Mock(ObReconAllocPmtTxnRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obReconAllocPmtTxnRepository = mockObReconAllocPmtTxnRepository
        mockObReconAllocPmtTxnRepository.executeNamedQueryByCondition(_, _, _) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.savePmtAllocData(pmtAllocDTO, "DEV", "004")
        then:
        response == true;
    }

    def "test savePmtAllocData exception method "() {
        given:
        def mockObReconAllocPmtTxnRepository = Mock(ObReconAllocPmtTxnRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obReconAllocPmtTxnRepository = mockObReconAllocPmtTxnRepository
        List<PmtAllocDTO> list = new ArrayList<>();
        list.add(pmtAllocDTO)
        mockObReconAllocPmtTxnRepository.executeNamedQueryByCondition(_, _, _) >> list
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        when:
        def response = service.savePmtAllocData(pmtAllocDTO, "DEV", "004")
        then:
        response == false;
    }

    @Unroll("test saveReconTwData method")
    def "test saveReconTwData method empty list "() {

        given:
        def mockObReconExecTxnTwRepository = Mock(ObReconExecTxnTwRepository)
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        service.obReconExecTxnTwRepository = mockObReconExecTxnTwRepository
        List list = new ArrayList();
        mockObReconExecTxnTwRepository.executeNamedQueryByCondition(_, _, _) >> list
        mockObReconExecTxnTwRepository.save(_) >> null
        AlertMessageProcessor alertMessageProcessor = Mock()
        AlertList alertList = Mock()
        AlertDecisionClient alertDecisionClient = Mock()
        DecisioningDetailsResponseWrapper decisioningDetailsResponseWrapper = new DecisioningDetailsResponseWrapper()
        List<AlertDecisioningDTO> detailsCollection = new ArrayList<AlertDecisioningDTO>()
        detailsCollection.add(new AlertDecisioningDTO())
        decisioningDetailsResponseWrapper.setDetailsCollection(detailsCollection)
        alertDecisionClient.getList(_, _) >> decisioningDetailsResponseWrapper
        List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>()
        newparameterList.add(new ParameterDTO())
        alertList.setParams(_, _) >> newparameterList
        service.alertMessageProcessor = alertMessageProcessor
        service.alertList = alertList
        service.alertDecisionClient = alertDecisionClient
        ReconTxnDto reconTxnDto = new ReconTxnDto()
        reconTxnDto.setId(new BigDecimal(0))
        when:
        def response = service.saveReconTwData(reconTxnDto, "DEV", "004")
        then:
        response == true;

    }

    @Unroll("test generateAlert method ExpDrCrPmtReconDTO")
    def "test generateAlert method ExpDrCrPmtReconDTO"() {
        given:
        ReconEventListenerServiceImpl service = new ReconEventListenerServiceImpl(obExpDrCrPmtReconRepository, obInvPmtReconRepository, obFinPmtReconRepository, obReconAllocPmtTxnRepository, obInvCnReconRepository)
        ExpDrCrPmtReconDTO expDrCrPmtReconDTO = new ExpDrCrPmtReconDTO()
        FinPmtReconDTO finPmtReconDTO = new FinPmtReconDTO()
        InvCnReconDTO invCnReconDTO = new InvCnReconDTO()
        InvPmtReconDTO invPmtReconDTO = new InvPmtReconDTO()
        PmtAllocDTO pmtAllocDTO = new PmtAllocDTO()
        ReconTxnDto reconTxnDto = new ReconTxnDto()
        setupAlerts(service)

        when:
        service.generateAlert(expDrCrPmtReconDTO, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        service.generateAlert(finPmtReconDTO, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        service.generateAlert(invCnReconDTO, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        service.generateAlert(invPmtReconDTO, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        service.generateAlert(pmtAllocDTO, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        service.generateAlert(reconTxnDto, getDecisioningWrapperDTO(), getEventDetailsDTO(), "category", "userid", "branchcode")
        then:
        noExceptionThrown()
    }

    private void setupAlerts(ReconEventListenerServiceImpl service) {
        AlertList alertList = Mock()
        service.alertList = alertList
        List<Object> placeHolders = new ArrayList<>()
        placeHolders.add(new Object())
        alertList.getparams(_) >> placeHolders

        List<ParameterDTO> newparameterList = new ArrayList<>()
        ParameterDTO pdto = new ParameterDTO()
        pdto.setKey("key1")
        pdto.setValue("value1")
        newparameterList.add(pdto)
        alertList.setParams(_, _) >> newparameterList

        AlertMessageProcessor alertMessageProcessor = Mock()
        service.alertMessageProcessor = alertMessageProcessor
        alertMessageProcessor.processAlertMessage(_, _, _, _) >> null
    }

    DecisioningWrapperDTO getDecisioningWrapperDTO() {
        DecisioningWrapperDTO dto = new DecisioningWrapperDTO()
        List<DecisioningDetailsDTO> detailsCollection = new ArrayList<>()
        DecisioningDetailsDTO decisioningDetailsDTO = new DecisioningDetailsDTO()
        decisioningDetailsDTO.setAdditionalDetails("nikhil.kalaskar@oracle.com");
        decisioningDetailsDTO.setEvent("DISBURSEMENT");

        decisioningDetailsDTO.setLimitPartyType("SUPPLIER");
        decisioningDetailsDTO.setIsRootEntity("N");
        decisioningDetailsDTO.setAlertCategory("LIMITS");
        decisioningDetailsDTO.setRecipient("SUPPLIER");
        decisioningDetailsDTO.setAlertFrequency("ONETIME");
        List<DeliveryDetailsDTO> deliveryDetails = new ArrayList<>()
        DeliveryDetailsDTO delDetailsDTO = new DeliveryDetailsDTO()
        delDetailsDTO.setAlertText("hi");
        deliveryDetails.add(delDetailsDTO)
        decisioningDetailsDTO.setDeliveryDetails(deliveryDetails)
        detailsCollection.add(decisioningDetailsDTO)
        dto.detailsCollection(detailsCollection)
        return dto
    }

    EventDetailsDTO getEventDetailsDTO() {
        EventDetailsDTO dto = new EventDetailsDTO()
        dto.setEvent("DISBURSEMENT");
        return dto
    }
}