package oracle.fsgbu.customer.test.service

import static org.junit.Assert.*

import java.text.DateFormat
import java.text.SimpleDateFormat
import java.sql.Timestamp
import java.time.LocalDate

import oracle.fsgbu.gus.core.client.dto.CoreBranchCollection
import oracle.fsgbu.gus.core.client.dto.CoreBranchDTO
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClient
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto
import oracle.fsgbu.obscfcm.services.report.persistence.InvoiceRepository
import oracle.fsgbu.obscfcm.services.report.service.ReportCreationServiceImpl
import oracle.fsgbu.reportsTransaction.api.model.RaisedInvDetails

import org.junit.Test

import spock.lang.Shared
import spock.lang.Specification
import spock.lang.Unroll

class ReportCreationServiceImplSpec extends Specification{
	
	def mockInvoiceRepository = Mock(InvoiceRepository)
	def mockCmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
	def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
	
	@Unroll("test getAgingInvoices method")
	def"test getAgingInvoices method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:	
		def response = reportCreationService.getAgingInvoices("DEV","004",0,10)
		then:
		response!=null
	}
	
	
	
	@Unroll("test getAgingInvoices method1")
	def"test getAgingInvoices method1"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:
		def response = reportCreationService.getAgingInvoices("DEV","004",null,null)
		then:
		response!=null
	}
	
	@Unroll("test getAgingInvoices method2")
	def"test getAgingInvoices method2"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:
		def response = reportCreationService.getAgingInvoices("DEV","004",2,2)
		then:
		response!=null
	}
	
	@Unroll("test getAgingInvoices method3")
	def"test getAgingInvoices method3"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:
		def response = reportCreationService.getAgingInvoices("DEV","004",0,0)
		then:
		response!=null
	}
	@Unroll("test getAgingInvoices method")
	def"test getAgingInvoices method exception"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:	
		def response = reportCreationService.getAgingInvoices("DEV","004",0,10)
		then:
		response==null
	}
	
	@Unroll("test getOverdueAgingInvoices method")
	def"test getOverdueAgingInvoices method"(){
		
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["0-30",new BigDecimal(25000),"TWD",Timestamp.valueOf(LocalDate.of(2019, 4, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 4, 30).atStartOfDay())]
		Object [] object1  = ["0-30",new BigDecimal(10000),"USD",Timestamp.valueOf(LocalDate.of(2019, 4, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 4, 30).atStartOfDay())]
		Object [] object2  = ["60-90",new BigDecimal(15000),"TWD",Timestamp.valueOf(LocalDate.of(2019, 5, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 6, 30).atStartOfDay())]

		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object1)
		resultList.add(object2)
		invoiceRepository.getOverdueInvoices(_) >> resultList

		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(11000))
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
		when:
		def response = reportCreationService.getOverdueAgingInvoices("DEV","004")
		then:
		response!=null
		
	}
	
	@Unroll("test getOverdueAgingInvoices method different ccy")
	def"test getOverdueAgingInvoices method different ccy"(){
		
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["0-30",new BigDecimal(25000),"TWD",Timestamp.valueOf(LocalDate.of(2019, 4, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 4, 30).atStartOfDay())]
		Object [] object1  = ["0-30",new BigDecimal(10000),"USD",Timestamp.valueOf(LocalDate.of(2019, 4, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 4, 30).atStartOfDay())]
		Object [] object2  = ["60-90",new BigDecimal(15000),"TWD",Timestamp.valueOf(LocalDate.of(2019, 5, 1).atStartOfDay()),Timestamp.valueOf(LocalDate.of(2019, 6, 30).atStartOfDay())]

		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object1)
		resultList.add(object2)
		invoiceRepository.getOverdueInvoices(_) >> resultList
		
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(12)
		currencyUtilityDto.setRate(12)
		mockCmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >>currencyUtilityDto
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		reportCreationService.cmcCurrencyServiceClient = mockCmcCurrencyServiceClient
		genericUtility.toOperator(_) >> ""
		when:
		def response = reportCreationService.getOverdueAgingInvoices("DEV","004")
		then:
		response!=null
		
	}
	
	
	@Unroll("test getOverdueAgingInvoices method")
	def"test getOverdueAgingInvoices method exception"(){
		
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		
		when:
		def response = reportCreationService.getOverdueAgingInvoices("DEV","004")
		then:
		response==null
		
	}
	
	@Unroll("test getBusinessTrend method")
	def"test getBusinessTrend method"(){
		
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
		def response = reportCreationService.getBusinessTrend("DEV","004")
		then:
		response!=null
		
		}
		
		
		@Unroll("test getBusinessTrend method1")
		def"test getBusinessTrend method1"(){
			
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
			CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
			List<CoreBranchDTO> branchDTOs = new ArrayList<>();
			branchDTOs.add(coreBranchDto)
			coreBranchCollection.setData(branchDTOs)
			mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
			Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","Y"]
			List<Object []> resultList = new ArrayList()
			resultList.add(object)
			invoiceRepository.executeNativeQuery(_) >> resultList
			CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
			currencyUtilityDto.setAmount(new BigDecimal(100))
			mockCmcBranchServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			reportCreationService.cmcBranchServiceClient = mockCmcBranchServiceClient
				when:
				def response = reportCreationService.getBusinessTrend("DEV","004")
			then:
			response!=null
			
			}
			
			
			@Unroll("test getBusinessTrend method2")
			def"test getBusinessTrend method2"(){
				
				given:
				def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
				def invoiceRepository = Mock(InvoiceRepository)
				String currentDate = "2019-04-13";
				mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
				def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"USD");
				CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
				List<CoreBranchDTO> branchDTOs = new ArrayList<>();
				branchDTOs.add(coreBranchDto)
				coreBranchCollection.setData(branchDTOs)
				mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
				Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","Y"]
				List<Object []> resultList = new ArrayList()
				resultList.add(object)
				invoiceRepository.executeNativeQuery(_) >> resultList
				
				ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
				when:
				def response = reportCreationService.getBusinessTrend("DEV","004")
				then:
				response!=null
				
				}
		
		
		@Unroll("test getBusinessTrend method")
		def"test getBusinessTrend method exception"(){
			
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
			CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
			List<CoreBranchDTO> branchDTOs = new ArrayList<>();
			branchDTOs.add(coreBranchDto)
			coreBranchCollection.setData(branchDTOs)
			mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
			Object [] object  = ["2018-08",new BigDecimal(25000),"TWD"]
			List<Object []> resultList = new ArrayList()
			resultList.add(object)
			invoiceRepository.executeNativeQuery(_) >> resultList
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			when:
			def response = reportCreationService.getBusinessTrend("DEV","004")
			then:
			response==null
			
			}
		
		@Unroll("test getRaisedInvoices method")
		def"test getRaisedInvoices method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		Object [] object2  = ["2018-08",new BigDecimal(26000),"TWD","Y"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object2)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-01-13","2019-04-13",null)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoicesDiffCurrency method")
		def"test getRaisedInvoicesDiffCurrency method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		cmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		Object [] object2  = ["2018-08",new BigDecimal(26000),"TWD","Y"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object2)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-12-13","2020-10-13",null)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoicesOneYear method")
		def"test getRaisedInvoicesOneYear method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		String branchCcy = "TWD";
		mockCmcBranchServiceClient.getBranchCurrency(_)>> branchCcy
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-01-13","2019-12-13",null)
			then:
			response!=null
		}
		
		@Unroll("test getRaisedInvoicesNoBranch method")
		def"test getRaisedInvoicesNoBranch method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"005",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-01-13","2019-04-13",null)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoicesException method")
		def"test getRaisedInvoicesException method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-01-13","2019-04-13","GBP")
			then:
			response==null
		}
		
		@Unroll("test getRaisedInvoicesFromToDateNull method")
		def"test getRaisedInvoicesFromToDateNull method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		String branchCcy = "TWD";
		mockCmcBranchServiceClient.getBranchCurrency(_)>> branchCcy
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004",null,null,null)
			then:
			response!=null
		}
		
		@Unroll("test getRaisedInvoicesToDateNull method")
		def"test getRaisedInvoicesToDateNull method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004","2019-01-13",null,null)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoicesFromDateNull method")
		def"test getRaisedInvoicesFromDateNull method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["2018-08",new BigDecimal(25000),"TWD","N"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient)
		when:
			def response = reportCreationService.getRaisedInvoices("DEV","004",null,"2019-01-13",null)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoiceDetails method")
		def"test getRaisedInvoiceDetails method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","Y",null,0,10)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoiceDetailsNonFinanced method")
		def"test getRaisedInvoiceDetailsNonFinanced method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"GBP");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		cmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		Object [] object1  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object1)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","N",null,0,10)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoiceDetailsNoBranch method")
		def"test getRaisedInvoiceDetailsNoBranch method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"005",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","Y",null,0,10)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoiceDetailsException method")
		def"test getRaisedInvoiceDetailsException method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426)]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","Y","GBP",0,10)
			then:
			response==null
		}
		
		@Unroll("test getPayableReceivables method")
		def"test getPayableReceivables method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD","PAYABLE"]
		Object [] object1  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD","PAYABLE"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object1)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getPayableReceivables("DEV","004")
			then:
			response!=null
		}
		
		@Unroll("test getPayableReceivables method")
		def"test getPayableReceivables method exception"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"PAYABLE"]
		Object [] object1  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"PAYABLE"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object1)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getPayableReceivables("DEV","004")
			then:
			response==null
		}
		
		@Unroll("test getInvoices method")
		def"test getInvoices method"(){
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			List<Object[]> resultList = new ArrayList<>()
			Object[] invoice1 = ["id", "1", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "12", new BigDecimal(250),
					"13", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "14", "15", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33","51"]
			Object[] invoice2 = ["id", "1", null, "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), null, "12", new BigDecimal(250),
					"13", null, "14", "15", null, new BigDecimal(250), null, "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33","51"]
			resultList.add(invoice1)
			resultList.add(invoice2)
			invoiceRepository.executeNativeQuery(_) >> resultList
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
			
			reportCreationService.genericUtility = genericUtility
			genericUtility.toOperator(_) >> ""
		
			when:
			def response = reportCreationService.getInvoices("DEV","004",fromDueDate,toDueDate,fromDate,toDate,"201930","Y",0,10)
			then:
			response==response
			where:
			fromDueDate	 | toDueDate    | fromDate     | toDate
			"2019-11-06" | "2019-11-06" | null         | null
			"2019-11-06" | null         | null         | null
			null         | null         | "2019-11-06" | "2019-11-06"
			
		}
		
		@Unroll("test getInvoices method1")
		def"test getInvoices method1"(){
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			List<Object[]> resultList = new ArrayList<>()
			Object[] invoice1 = [new BigDecimal(250), "1", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "12", new BigDecimal(250),
					"13", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "14", "15", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33"]
			Object[] invoice2 = [new BigDecimal(250), "1", null, "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), null, "12", new BigDecimal(250),
					"13", null, "14", "15", null, new BigDecimal(250), null, "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33"]
			resultList.add(invoice1)
			resultList.add(invoice2)
			invoiceRepository.executeNativeQuery(_) >> resultList
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		
			def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
			
			reportCreationService.genericUtility = genericUtility
			genericUtility.toOperator(_) >> ""
		
			when:
			def response = reportCreationService.getInvoices("DEV","004",null,null,null,null,"201930","N",null,null)
			then:
			response==response
			where:
			fromDueDate	 | toDueDate    | fromDate     | toDate
			"2019-11-06" | "2019-11-06" | null         | null
			"2019-11-06" | null         | null         | null
			null         | null         | "2019-11-06" | "2019-11-06"
			
		}
		
		
		@Unroll("test getInvoices method2")
		def"test getInvoices method2"(){
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			List<Object[]> resultList = new ArrayList<>()
			Object[] invoice1 = [new BigDecimal(250), "1", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "12", new BigDecimal(250),
					"13", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "14", "15", Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), new BigDecimal(250), Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()), "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33"]
			Object[] invoice2 = [new BigDecimal(250), "1", null, "2", new BigDecimal(250), "3", "4", "5", "6", "7",
					new BigDecimal(250), new BigDecimal(250), "8", "9", "10", "11", new BigDecimal(250), null, "12", new BigDecimal(250),
					"13", null, "14", "15", null, new BigDecimal(250), null, "16", "17", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "18", "19", "20", "21", "22", "23", "24", new BigDecimal(250),
					new BigDecimal(250), new BigDecimal(250), "25", "26", "27", "28", "29", "30", "31", "32","33"]
			resultList.add(invoice1)
			resultList.add(invoice2)
			invoiceRepository.executeNativeQuery(_) >> resultList
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
			
			reportCreationService.genericUtility = genericUtility
			genericUtility.toOperator(_) >> ""
		
			when:
			def response = reportCreationService.getInvoices("DEV","004",null,null,null,null,"201930","N",5,1)
			then:
			response==response
			where:
			fromDueDate	 | toDueDate    | fromDate     | toDate
			"2019-11-06" | "2019-11-06" | null         | null
			"2019-11-06" | null         | null         | null
			null         | null         | "2019-11-06" | "2019-11-06"
			
		}
		
		@Unroll("test getInvoicesEmptyList method")
		def"test getInvoicesEmptyList method"(){
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			Object [] object  = [Timestamp.valueOf(LocalDate.of(2019, 9, 1).atStartOfDay()),new BigDecimal(25000),"TWD"]
			invoiceRepository.executeNativeQuery(_) >> new ArrayList<>()
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			
			when:
			def response = reportCreationService.getInvoices("DEV","004",fromDueDate,toDueDate,fromDate,toDate,"201930","Y",0,10)
			then:
			response==response
			where:
			fromDueDate	 | toDueDate    | fromDate     | toDate
			"2019-11-06" | "2019-11-06" | null         | null
			
		}
		
		@Unroll("test getInvoicesNullList method")
		def"test getInvoicesNullList method"(){
			given:
			def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
			def invoiceRepository = Mock(InvoiceRepository)
			String currentDate = "2019-04-13";
			mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
			invoiceRepository.executeNativeQuery(_) >> null
			
			ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
			
			when:
			def response = reportCreationService.getInvoices("DEV","004",fromDueDate,toDueDate,fromDate,toDate,"201930","Y",0,10)
			then:
			response==response
			where:
			fromDueDate	 | toDueDate    | fromDate     | toDate
			"2019-11-06" | "2019-11-06" | null         | null
			
		}
		
		@Unroll("test getAgingInvoiceDetails method")
		def"test getAgingInvoiceDetails method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		cmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		Object [] object  = ["38203","custoFuture Groupmer","2019-04-03",new BigDecimal(2342426),"TWD"]
		Object [] object2  = ["38203","custoFuture Groupmer","2019-04-03",new BigDecimal(2342426),"GBP"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		resultList.add(object2)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
	
		
		when:
			def response = reportCreationService.getAgingInvoiceDetails("DEV","004",fromDueDate,toDueDate,0,0)
		then:
			response==response
		where:
			fromDueDate	 | toDueDate
			"2019-11-06" | "2019-11-06"
			"2019-11-06" | null
			null         | "2019-11-06"
			null         | null
		}
		
		@Unroll("test getAgingInvoiceDetailsNoBranch method")
		def"test getAgingInvoiceDetailsNoBranch method"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def cmcCurrencyServiceClient = Mock(CmcCurrencyServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		CurrencyUtilityDto currencyUtilityDto = new CurrencyUtilityDto()
		currencyUtilityDto.setAmount(new BigDecimal(100))
		cmcCurrencyServiceClient.getConvertedCcyValue(_,_,_,_,_,_) >> currencyUtilityDto
		Object [] object  = ["38203","custoFuture Groupmer","2019-04-03",new BigDecimal(2342426),"TWD"]
		Object [] object2  = ["38203","custoFuture Groupmer","2019-04-03",new BigDecimal(2342426),"GBP"]
		invoiceRepository.executeNativeQuery(_) >> new ArrayList()
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getAgingInvoiceDetails("DEV","005",fromDueDate,toDueDate,0,0)
		then:
			response==response
		where:
			fromDueDate	 | toDueDate
			"2019-11-06" | "2019-11-06"
		}
		
		@Unroll("test getRaisedInvoiceDetails method offset1")
		def"test getRaisedInvoiceDetails method offset1"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		def genericUtility = Mock(oracle.fsgbu.gus.common.utils.GenericUtility)
		
		reportCreationService.genericUtility = genericUtility
		genericUtility.toOperator(_) >> ""
	
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","Y",null,5,10)
			then:
			response==response
		}
		
		@Unroll("test getRaisedInvoiceDetails method offset2")
		def"test getRaisedInvoiceDetails method offset2"(){
		given:
		def mockCmcBranchServiceClient = Mock(CmcBranchServiceClient)
		def invoiceRepository = Mock(InvoiceRepository)
		String currentDate = "2019-04-13";
		mockCmcBranchServiceClient.getCurrentBusinessDate(_,_)>> currentDate
		def coreBranchDto = new CoreBranchDTO(sourceBranchCode:"004",countryCode:"US",branchName:"New York",branchCode:"004",branchLcy:"TWD");
		CoreBranchCollection coreBranchCollection = new CoreBranchCollection();
		List<CoreBranchDTO> branchDTOs = new ArrayList<>();
		branchDTOs.add(coreBranchDto)
		coreBranchCollection.setData(branchDTOs)
		mockCmcBranchServiceClient.getBranchDetails() >> coreBranchCollection
		Object [] object  = ["38203","custoFuture Groupmer",new BigDecimal(2342426),"TWD"]
		List<Object []> resultList = new ArrayList()
		resultList.add(object)
		invoiceRepository.executeNativeQuery(_) >> resultList
		
		ReportCreationServiceImpl reportCreationService= new ReportCreationServiceImpl( invoiceRepository, mockCmcCurrencyServiceClient, mockCmcBranchServiceClient )
		when:
			def response = reportCreationService.getRaisedInvoiceDetails("OBSCFCMREPORT","DEV","004","2019-01-13","2019-04-13","Y",null,null,null)
			then:
			response==response
		}
}

