package oracle.fsgbu.customer.test.util

import spock.lang.Specification;
import spock.lang.Unroll;
import java.math.BigDecimal
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.ZoneId

import spock.lang.Shared
import oracle.fsgbu.gus.persistence.provider.GlobalHolder
import oracle.fsgbu.gus.persistence.provider.RequestContext
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxn
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObChargesTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconAllocPmtTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconExecTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconExpDrCrPmtTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconFinPmtTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvCnTxnTw
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObReconInvPmtTxnTw
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil
import oracle.fsgbu.reconService.api.model.AllocPayReconDto
import oracle.fsgbu.reconService.api.model.ExpDrCrPayReconDto
import oracle.fsgbu.reconService.api.model.FinPayReconDto
import oracle.fsgbu.reconService.api.model.ReconInvCnDto
import oracle.fsgbu.reconService.api.model.ReconInvPmtDto
import oracle.fsgbu.reconService.api.model.ReconTxnDto
import org.junit.Test
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipMasterDTO;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTmRelationshipMaster;
import oracle.fsgbu.obscfcm.services.report.domain.entity.ObcmTwRelationshipMaster
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipDetailsDTO;
import oracle.fsgbu.obscfcm.services.scfcm.dto.ObcmTmRelationshipDetailsExternalCodeDTO;
import oracle.fsgbu.reconService.api.model.ReconTxnDto;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.EventDetailsDTO;
import oracle.fsgbu.reconService.api.model.AllocPayReconDto;
import oracle.fsgbu.reconService.api.model.FinPayReconDto;


	
class ReportingUtilSpec extends Specification{
	
	@Shared stringDate = new String("2019-08-12");
	@Shared obcmTmRelationshipMasterDTO = new ObcmTmRelationshipMasterDTO("autoAcceptance": "N",
		"corporateId": "201901",
		"corporateName": "PEGATRON",
		"effFromDate": "2019-10-08",
		"relationshipCode": "RC001",
		"relationshipName": "RC001",
		"expiresOn": "2020-02-29",
		"product": "B",
		"noOfDays": null,
		"maxDate": null,
		"txnControllerRefNo": null,
		"obcmTmRelationshipDetailsDTO": null)
	
	@Shared obcmRelationsDetailsDto = new ObcmTmRelationshipDetailsDTO(
		expiresOn: "2020-02-29", noOfDays: 2,externalCodes:null)

	@Shared allocPayReconDtoList = new AllocPayReconDto(reconTxnRefNo : "FR01",allocVirtualAccNo : "FR01", allocEntity : "LHS", allocAttributes : "RI001", allocStatus : "P",allocatedAmt : 300,allocationPct : 5, pmtAllocatedAmt : 200, obvamRefNo : "RI001", corporateId : "004",counterpartyId : "201901",obdxRefNo : "FR01", beneficiaryId : 1L, paymentPartyId : "RI001", paymentDate : "004",paymentMode : "EFT",beneCode : "FR01", paymentAmount : 300, paymentCcy : "USD", reconciledPmtAmt : 200,beneVirtualAccNo : "8973",pmtReconStatus : "FR01", pmtAllocationStatus : "A", reconAction : "RI001", reconDate : "004",extPmtRefNo : "er34",paymentRefNo : "FR01", remitterAccNo :"r234", creditAccNo : "RI001", dcIndicator : "004", accOwner : "RI001", allocBasis : "004");
	
	@Shared finPayReconDtoList = new FinPayReconDto(reconTxnRefNo : "FR01",matchSeqNo : "01", finBuyerId : "201901", finSupplierId : "000419", finBuyerName : "PEGATRON",finSupplierName : "WINBOND",finProgramCode : "FR01", finRefNo : "REF2", finSeqNo : "2", finStartDt : "2022-05-02",finEndDt : "2022-05-10",finPastDueDt : "2022-05-08", finCcy : "USD", finAmt : 1000, finOsAmt : 500,intOsAmt : 200,penIntOsAmt : 300, overdueIntOsAmt : 100, totIntOsAmt : 1000, amtMatched : 600,paymentRefNo : "pay02",paymentPartyId : "FR01", paymentPartyName :"payment", paymentDate : "2022-05-06", paymentMode : "EFT",beneCode : "001",beneName : "abc", paymentAmount : 800, paymentCcy : "RI001", exchRate : 1,reconciledPmtAmt : 300,beneVirtualAccNo : "FR01", pmtStatus : "A", reconStatus : "RI001", reconDate : "004",reconcileBy : "B",allocationStatus : 0, allocationRequired : "N");
	
	@Shared reconTxnDto=new ReconTxnDto( id : 1,reconTxnRefNo : "FR01", reconCategory : "IPR", reconDate : "RI001", branchCode : "004", reconType : "PR001", reconRuleType : "OP001", reconRule : "PRG001", lhsEntity : "PRG NAME", rhsEntity : "BUY001",makerId : "BUYER 01",  makerDtStamp : "SUPP001",authId : "SUPP NAME",authDtStamp : "20-12-2018", transactionStatus : "PRG NAME", reconAction : "BUY001",stage : "BUYER 01",  corporateId : "SUPP001",allocBasis : "SUPP NAME",totalAllocatedAmt : 1000, finPayReconDtoList :null, allocPayReconDtoList : null)
	
	
	@Unroll("test buildHeaders method")
	def"test buildHeaders method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		def globalHolder = Mock(GlobalHolder)
		def reqContext = Mock(RequestContext)
		globalHolder.setRequestcontext(reqContext)
		
		when:
		
		def response  =  utils.buildHeaders("SCFCORE")
		
		then:
		response!=null
	}
	
	@Unroll("test buildHeadersForEventService method")
	def"test buildHeadersForEventService method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		def globalHolder = Mock(GlobalHolder)
		def reqContext = Mock(RequestContext)
		globalHolder.setRequestcontext(null)
		
		when:
		
		def response  =  utils.buildHeadersForEventService("SCFCORE","SWETA","004")
		
		then:
		response!=null
		
	}
	
	@Unroll("test setDateFormatNull method")
	def"test setDateFormatNull method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.setDateFormat(null)
		
		then:
		response==null
		
	}
	
	@Unroll("test setDateFormat method")
	def"test setDateFormat method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.setDateFormat(new Date())
		
		then:
		response==response
		
	}
	
	@Unroll("Test converDateFormat method for SUCCESS")
	def "test converDateFormat method for SUCCESS"(){
		given:
		ReportingUtil utils = new ReportingUtil()
 
		when:
        //def responseForNull = utils.converDateFormat(null);
        def response = utils.converDateFormat(stringDate);
        //def responseException = utils.converDateFormat("2011 11 19");
 
        then:
       // println responseForNull
       // responseForNull == null
        println response
        response != null
       // println responseException
        //thrown 
 

	}
	
	@Unroll("test setDateFormat method")
	def"test getSlabByDueDate method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.getSlabs(new Date(),new Date(), false)
		
		then:
		 println response
        response != null
		
	}
	
	@Unroll("test setDateFormat method")
	def"test getOverdueSlabByDueDate method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.getSlabs(new Date(),new Date(), true)
		
		then:
		 println response
        response != null
		
	}
	
	@Unroll("Test convertStringToDate method for SUCCESS")
	def "test convertStringToDate method for SUCCESS"(){
		given:
		ReportingUtil scfCmCoreMasterUtils = new ReportingUtil();
 
		when:
		def responseForNull = scfCmCoreMasterUtils.convertStringToDate(null);
		def response = scfCmCoreMasterUtils.convertStringToDate(stringDate);
		def responseException = scfCmCoreMasterUtils.convertStringToDate("2011 11 19");
 
		then:
		println responseForNull
		responseForNull == null
		println response
		response != null
		println responseException
		thrown RuntimeException
 
	}
	
	@Unroll("Test 1 setValuesFromDto method")
	def "test 1 setValuesFromDto method"(){
		given:
		ReportingUtil reportingUtil = new ReportingUtil()
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		ObReconExecTxnTw obReconExecTxnTw = new ObReconExecTxnTw()
 
		when:
		def response = reportingUtil.setValuesFromDto(obReconExecTxnDto, obReconExecTxnTw)
 
		then:
		response != null
 
	}
	
	@Unroll("Test 2 setValuesFromDto method")
	def "test 2 setValuesFromDto method"(){
		
		given:
		
		ReportingUtil reportingUtil = new ReportingUtil()
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		ObReconExecTxnTw obReconExecTxnTw = new ObReconExecTxnTw()
 
		when:
		def response = reportingUtil.setValuesFromDto(obReconExecTxnDto, obReconExecTxnTw)
 
		then:
		response != null
 
	}
	
	@Unroll("Test 3 setValuesFromDto method")
	def "test 3 setValuesFromDto method"(){
		
		given:
		def mockDate = "2021-01-01"
		ReportingUtil reportingUtil = new ReportingUtil()
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate(mockDate)
		obReconExecTxnDto.setMakerDtStamp(mockDate)
		obReconExecTxnDto.setAuthDtStamp(mockDate)
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		obReconInvPmtTxnTw.setInvDate(mockDate)
		obReconInvPmtTxnTw.setInvDueDate(mockDate)
		obReconInvPmtTxnTw.setPaymentDate(mockDate)
		obReconInvPmtTxnTw.setPaymentDueDate(mockDate)
		obReconInvPmtTxnTw.setPaymentBusinessDate(mockDate)
		obReconInvPmtTxnTw.setReconDate(mockDate)
		obReconInvPmtTxnTw.setAllocationRequired("Y")
		List<AllocPayReconDto> allocPayReconDtoList1 = new ArrayList<AllocPayReconDto>()
		AllocPayReconDto allocPayReconDto = new AllocPayReconDto()
		allocPayReconDto.setPaymentDate(mockDate)
		allocPayReconDtoList1.add(allocPayReconDto)
		obReconInvPmtTxnTw.setAllocPayReconDtoList(allocPayReconDtoList1)
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		obReconInvCnTxnTw.setSeqNo(new BigDecimal(0))
		obReconInvCnTxnTw.setCnDate(mockDate)
		obReconInvCnTxnTw.setCnExpiryDate(mockDate)
		obReconInvCnTxnTw.setInvDate(mockDate)
		obReconInvCnTxnTw.setInvDueDate(mockDate)
		obReconInvCnTxnTw.setReconDate(mockDate)
		obReconInvCnTxnTw.setProcBranchDate(mockDate)
		obReconInvCnTxnTw.setFiller1(mockDate)
		obReconInvCnTxnTw.setFiller2(mockDate)
		obReconInvCnTxnTw.setPaymentDueDate(mockDate)
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		obReconExpDrCrPmtTxnTw.setExpDrcrDate(mockDate)
		obReconExpDrCrPmtTxnTw.setRevisedDrcrDate(mockDate)
		obReconExpDrCrPmtTxnTw.setPaymentDate(mockDate)
		obReconExpDrCrPmtTxnTw.setActualDate(mockDate)
		obReconExpDrCrPmtTxnTw.setReconDate(mockDate)
		obReconExpDrCrPmtTxnTw.setPaymentBusinessDate(mockDate)
		obReconExpDrCrPmtTxnTw.setAllocationRequired("Y")
		obReconExpDrCrPmtTxnTw.setAllocPayReconDtoList(allocPayReconDtoList1)
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		obReconAllocPmtTxnTw.setPaymentBusinessDate(mockDate)
		obReconAllocPmtTxnTw.setPaymentDate(mockDate)
		obReconAllocPmtTxnTw.setReconDate(mockDate)
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		obReconFinPmtTxnTw.setMatchSeqNo("0")
		obReconFinPmtTxnTw.setFinStartDt(mockDate)
		obReconFinPmtTxnTw.setFinEndDt(mockDate)
		obReconFinPmtTxnTw.setFinPastDueDt(mockDate)
		obReconFinPmtTxnTw.setReconDate(mockDate)
		obReconFinPmtTxnTw.setPaymentDate(mockDate)
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		ObReconExecTxnTw obReconExecTxnTw = new ObReconExecTxnTw()
 
		when:
		def response = reportingUtil.setValuesFromDto(obReconExecTxnDto, obReconExecTxnTw)
 
		then:
		response != null
 
	}
	
	@Unroll("Test parseDate method")
	def "test parseDate method"(){
		given:
		ReportingUtil reportingUtil = new ReportingUtil()
 
		when:
		def response = reportingUtil.parseDate("19/11/2011");
		def responseException = reportingUtil.parseDate("2011 11 19")
 
		then:
		response != null
		thrown Exception
 
	}
	
	@Unroll("test convertStrToDate method")
	def"test convertStrToDate method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.convertStrToDate("abc")
		
		then:
		final RuntimeException exception = thrown()
		
	}
	
	@Unroll("test convertStrToDate method null")
	def"test convertStrToDate method null"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.convertStrToDate(null)
		
		then:
		response == null
		
	}
	
	@Unroll("test convertStrToDate method")
	def"test convertStrToDate method1"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.convertStrToDate("2019-01-01 01:01:01")
		
		then:
		response != null
		
	}
	
	
	@Unroll("test getSlabs")
	def"test getSlabs"(){
		given:
		
		ReportingUtil utils = new ReportingUtil();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -30);
		Date dateBefore30Days = cal.getTime();
		Date now = new Date()
		

		
		when:
		
		def response  =  utils.getSlabs(now,dateBefore30Days,true)
		
		then:
		response == "Overdue"
		
	}
	
	@Unroll("test getSlabs 0-30")
	def"test getSlabs 0-30"(){
		given:
		
		ReportingUtil utils = new ReportingUtil();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -30);
		Date dateBefore30Days = cal.getTime();
		Date now = new Date()
				
		when:
		
		def response  =  utils.getSlabs(dateBefore30Days,now,true)
			
		then:
		response == "0-30"
		
	}
	
	@Unroll("test getSlabs 30-60")
	def"test getSlabs 30-60"(){
		given:
		
		ReportingUtil utils = new ReportingUtil();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -60);
		Date dateBefore30Days = cal.getTime();
		Date now = new Date()
		

		
		when:
		
		def response  =  utils.getSlabs(dateBefore30Days,now,true)
		
		then:
		response == "30-60"
		
	}
	
	@Unroll("test getSlabs 60-90")
	def"test getSlabs 60-90"(){
		given:
		
		ReportingUtil utils = new ReportingUtil();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -90);
		Date dateBefore30Days = cal.getTime();
		Date now = new Date()
		

		
		when:
		
		def response  =  utils.getSlabs(dateBefore30Days,now,true)
		
		then:
		response == "60-90"
		
	}
	
	@Unroll("test getSlabs 90-120")
	def"test getSlabs 90-120"(){
		given:
		
		ReportingUtil utils = new ReportingUtil();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -120);
		Date dateBefore30Days = cal.getTime();
		Date now = new Date()
		

		
		when:
		
		def response  =  utils.getSlabs(dateBefore30Days,now,true)
		
		then:
		response == "90-120"
		
	}
	
	@Unroll("test numberConverter")
	def"test numberConverter"(){
		given:		
		ReportingUtil utils = new ReportingUtil();
		
		when:
		
		def response  =  utils.numberConverter("1")
		
		then:
		response == 1
		
	}
	
	
	@Unroll("test numberConverter null")
	def"test numberConverter null"(){
		given:
		ReportingUtil utils = new ReportingUtil();
		
		when:
		
		def response  =  utils.numberConverter("null")
		
		then:
		response == null
		
	}
	
	@Unroll("test convertTmRelationshipDtoToEntity method")
	def"test convertTmRelationshipDtoToEntity method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		def globalHolder = Mock(GlobalHolder)
		def reqContext = Mock(RequestContext)
		globalHolder.setRequestcontext(reqContext)
		
		def obcmTmRelationshipDetailsDTOList = new ArrayList<>()
		def externalCodes = new ArrayList<>()
		
		ObcmTmRelationshipDetailsExternalCodeDTO externalCodesDTO = new ObcmTmRelationshipDetailsExternalCodeDTO()
        externalCodesDTO.setExternalCode("externalCode")
        externalCodesDTO.setDivisionCode("divisionCode")
        externalCodesDTO.setCorporateDivisionCode("corporateDivisionCode")
		externalCodesDTO.setModNo(1)
        externalCodes.add(externalCodesDTO)
		obcmRelationsDetailsDto.setExternalCodes(externalCodes)
		obcmTmRelationshipDetailsDTOList.add(obcmRelationsDetailsDto)
		obcmTmRelationshipMasterDTO.setObcmTmRelationshipDetailsDTO(obcmTmRelationshipDetailsDTOList)
		
		when:
		
		def response  =  utils.convertTmRelationshipDtoToEntity(obcmTmRelationshipMasterDTO,new ObcmTmRelationshipMaster())
		
		then:
		response!=null
	}
	
	@Unroll("test convertTwRelationshipDtoToEntity method")
	def"test convertTwRelationshipDtoToEntity method"(){
		given:
		
		ReportingUtil utils = new ReportingUtil()
		def globalHolder = Mock(GlobalHolder)
		def reqContext = Mock(RequestContext)
		globalHolder.setRequestcontext(reqContext)
		
		def obcmTmRelationshipDetailsDTOList = new ArrayList<>()
		def externalCodes = new ArrayList<>()
		
		ObcmTmRelationshipDetailsExternalCodeDTO externalCodesDTO = new ObcmTmRelationshipDetailsExternalCodeDTO()
		externalCodesDTO.setExternalCode("externalCode")
		externalCodesDTO.setDivisionCode("divisionCode")
		externalCodesDTO.setCorporateDivisionCode("corporateDivisionCode")
		externalCodesDTO.setModNo(1)
		externalCodes.add(externalCodesDTO)
		obcmRelationsDetailsDto.setExternalCodes(externalCodes)
		obcmTmRelationshipDetailsDTOList.add(obcmRelationsDetailsDto)
		obcmTmRelationshipMasterDTO.setObcmTmRelationshipDetailsDTO(obcmTmRelationshipDetailsDTOList)
		
		when:
		
		def response  =  utils.convertTwRelationshipDtoToEntity(obcmTmRelationshipMasterDTO,new ObcmTwRelationshipMaster())
		
		then:
		response!=null
	}
	
	@Unroll("test getMonthAndDateOfGivenRange method")
	def"test getMonthAndDateOfGivenRange method"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		
		when:
		
		def response  =  utils.getMonthAndDateOfGivenRange("2020-02-29","2020-01-29")
		
		then:
		response!=null
	}
	
	@Unroll("test getMonthAndDateOfGivenRange method1")
	def"test getMonthAndDateOfGivenRange method1"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		
		when:
		
		def response  =  utils.getMonthAndDateOfGivenRange("2020-02-29","2020-03-29")
		
		then:
		response!=null
	}
	
	@Unroll("test setReconTwEventDetails method1")
	def"test setReconTwEventDetails method1"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		obReconExecTxnDto.setReconCategory("IPR")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		when:
		
		def response  =  utils.setReconTwEventDetails(obReconExecTxnDto,new EventDetailsDTO())
		
		then:
		response!=null
	}
	
	@Unroll("test setReconTwEventDetails method2")
	def"test setReconTwEventDetails method2"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		obReconExecTxnDto.setReconCategory("ICR")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		when:
		
		def response  =  utils.setReconTwEventDetails(obReconExecTxnDto,new EventDetailsDTO())
		
		then:
		response!=null
	}
	
	
	@Unroll("test setReconTwEventDetails method3")
	def"test setReconTwEventDetails method3"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		obReconExecTxnDto.setReconCategory("EPR")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		when:
		
		def response  =  utils.setReconTwEventDetails(obReconExecTxnDto,new EventDetailsDTO())
		
		then:
		response!=null
	}
	
	
	@Unroll("test setReconTwEventDetails method4")
	def"test setReconTwEventDetails method4"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		obReconExecTxnDto.setReconCategory("VPA")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		when:
		
		def response  =  utils.setReconTwEventDetails(obReconExecTxnDto,new EventDetailsDTO())
		
		then:
		response!=null
	}
	
	
	@Unroll("test setReconTwEventDetails method5")
	def"test setReconTwEventDetails method5"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		ReconTxnDto obReconExecTxnDto = new ReconTxnDto()
		obReconExecTxnDto.setId(new BigDecimal(0))
		obReconExecTxnDto.setReconDate("2021-01-01")
		obReconExecTxnDto.setMakerDtStamp("2021-01-01")
		obReconExecTxnDto.setAuthDtStamp("2021-01-01")
		obReconExecTxnDto.setReconCategory("FPR")
		
		// Test Invoice Payment data
		ReconInvPmtDto obReconInvPmtTxnTw = new ReconInvPmtDto()
		obReconInvPmtTxnTw.setId(new BigDecimal(0))
		List<ReconInvPmtDto> obReconInvPmtTxnTwDtoList = new ArrayList<ReconInvPmtDto>()
		obReconInvPmtTxnTwDtoList.add(obReconInvPmtTxnTw)
		
		// Test Invoice Credit Note data
		ReconInvCnDto obReconInvCnTxnTw = new ReconInvCnDto()
		obReconInvCnTxnTw.setId(new BigDecimal(0))
		List<ReconInvCnDto> obReconInvCnTxnTwDtoList = new ArrayList<ReconInvCnDto>()
		obReconInvCnTxnTwDtoList.add(obReconInvCnTxnTw)
		
		// Test Expected CF to Payment data
		ExpDrCrPayReconDto obReconExpDrCrPmtTxnTw = new ExpDrCrPayReconDto()
		obReconExpDrCrPmtTxnTw.setId(new BigDecimal(0))
		List<ExpDrCrPayReconDto> expDrCrPayReconDtoList = new ArrayList<ExpDrCrPayReconDto>()
		expDrCrPayReconDtoList.add(obReconExpDrCrPmtTxnTw)
		
		// Test Allocation Payment data
		AllocPayReconDto obReconAllocPmtTxnTw = new AllocPayReconDto()
		obReconAllocPmtTxnTw.setId(new BigDecimal(0))
		List<AllocPayReconDto> allocPayReconDtoList = new ArrayList<AllocPayReconDto>()
		allocPayReconDtoList.add(obReconAllocPmtTxnTw)
		
		// Test Finance Payment data
		FinPayReconDto obReconFinPmtTxnTw = new FinPayReconDto()
		obReconFinPmtTxnTw.setId(new BigDecimal(0))
		List<FinPayReconDto> finPayReconDtoList = new ArrayList<FinPayReconDto>()
		finPayReconDtoList.add(obReconFinPmtTxnTw)
		
		obReconExecTxnDto.setObReconInvPmtTxnTwDtoList(obReconInvPmtTxnTwDtoList)
		obReconExecTxnDto.setObReconInvCnTxnTwDtoList(obReconInvCnTxnTwDtoList)
		obReconExecTxnDto.setExpDrCrPayReconDtoList(expDrCrPayReconDtoList)
		obReconExecTxnDto.setAllocPayReconDtoList(allocPayReconDtoList)
		obReconExecTxnDto.setFinPayReconDtoList(finPayReconDtoList)
		when:
		
		def response  =  utils.setReconTwEventDetails(obReconExecTxnDto,new EventDetailsDTO())
		
		then:
		response!=null
	}
	
	@Unroll("test setDates method1")
	def"test setDates method1"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		ObChargesTxnTw ObChargesTxnTw = new ObChargesTxnTw()
		
		when:
		
		def response  =  utils.setDates(new ObChargesTxn(),ObChargesTxnTw)
		
		then:
		response==null
	}
	
	
	@Unroll("test setDates method2")
	def"test setDates method2"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		ObChargesTxnTw ObChargesTxnTw = new ObChargesTxnTw()
		ObChargesTxnTw.setChgCalcDate(new Date())
		ObChargesTxnTw.setFirstChgCalcDate(new Date())
		ObChargesTxnTw.setNextChgCalcDate(new Date())
		ObChargesTxnTw.setNextChgCollDate(new Date())
		ObChargesTxnTw.setCalcEndDate(new Date())
		ObChargesTxnTw.setCalcStartDate(new Date())
	
		when:
		
		def response  =  utils.setDates(new ObChargesTxn(),ObChargesTxnTw)
		
		then:
		response==null
	}

	@Unroll("test formatIntegerString method2")
	def"test formatIntegerString method2"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.formatIntegerString(1)
		
		then:
		response!=null
	}

	
	@Unroll("test formatIntegerString method1")
	def"test formatIntegerString method1"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.formatIntegerString(null)
		
		then:
		response!=null
	}
	
	
	@Unroll("test isNull method1")
	def"test isNull method1"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.isNull(null)
		
		then:
		response!=null
	}
	
	
	@Unroll("test isNull method2")
	def"test isNull method2"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.isNull("")
		
		then:
		response!=null
	}
	
	@Unroll("test isNull method3")
	def"test isNull method3"(){
		given:
		ReportingUtil utils = new ReportingUtil()
		
		when:
		
		def response  =  utils.isNull("xyz")
		
		then:
		response!=null
	}
	
	
}