package oracle.fsgbu.obscfcm.services.report.client;

import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.util.AlertServiceUtility;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;
import oracle.fsgbu.plato.logger.PlatoLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Method;
import java.util.*;

@Service
public class AlertListImpl implements AlertList {

	@Autowired
	private RestTemplate restTemplate;

	@Value("${eureka.protocol:http}")
	private String protocol;

	@Autowired
	public AlertListImpl(RestTemplate restTemplate) {
		this.restTemplate = restTemplate;
	}
	
	private static final PlatoLogger LOGGER = new PlatoLogger(AlertListImpl.class);
	
	
	@Override
	public List<ParameterDTO> setParams(List<Object> objlst, Object dtoObj) {		
		List<ParameterDTO> newparameterList = new ArrayList<ParameterDTO>();
		HashMap<String,String> hmap= new HashMap<>();
		for(Object obj:objlst) {	
			ParameterDTO  placeholderlist = (ParameterDTO) obj;
			hmap.put(placeholderlist.getKey(), placeholderlist.getValue());
		}
		newparameterList=toValue(dtoObj,hmap);
		return newparameterList;
	}
	
	
	private List<ParameterDTO> toValue(Object dtoObject,HashMap<String,String> hmap) {
		List<ParameterDTO> parameterList = new ArrayList<ParameterDTO>();
		Method[] declaredMethods = dtoObject.getClass().getDeclaredMethods();
        Arrays.asList(declaredMethods).forEach(method -> {
        for(Map.Entry mapElement: hmap.entrySet()) {
        	ParameterDTO paramObject = new ParameterDTO();
            try {
                if (mapElement.getValue().equals(method.getName())) {
                 Object returnValue=method.invoke(dtoObject, null);
              	 String methodValue = (returnValue != null ) ? returnValue.toString() : "";
              	 paramObject.setKey(mapElement.getKey().toString());
              	 paramObject.setValue(methodValue);
              	 parameterList.add(paramObject);
                }     
            } catch (Exception ex) {
                LOGGER.error("Method not found {}", method.getName());
            }	
        }
       });
		return parameterList;
	}
	
		
	
	/**
	 * @param alertCategory
	 * @param extraHeaders
	 * extraHeaders[0] : User Id
	 * extraHeaders[1] : Branch Code
	 * @return placeholderList
	 */
	@Override
	public List<Object> getparams(String alertCategory, String... extraHeaders) {
		LOGGER.info("Inside getPlaceholders");
		ResponseEntity<Object> response = null;
		HttpHeaders headers = new HttpHeaders();
		String userId = null;
		String branchCode = null;
		if(extraHeaders != null && extraHeaders.length > 0) {
			userId = extraHeaders[0];
			branchCode = extraHeaders[1];
		}
		headers = AlertServiceUtility.buildHeaders(ReportConstants.ALERTS_APPID, userId, branchCode);
		try {
			HttpEntity<String> entity = new HttpEntity<>(headers);
			
			String formUrl = protocol + ReportConstants.GET_LIST_PLACEHOLDERS;
			formUrl = formUrl + ReportConstants.ALERT_CATEGORY;
			formUrl = formUrl + alertCategory;
			
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity,Object.class);
		} catch (Exception e) {
			
			LOGGER.error("Error in getMessage()");
		}
		if (response != null) {
			
			List<Object> responseBody = (List<Object>) response.getBody();
			List<Object> placeholderList= new ArrayList<>();
			List<Object> s =responseBody;
			for(Object s1:s) {
				 LinkedHashMap  params = (LinkedHashMap) s1;
				 ParameterDTO paramObject = new ParameterDTO();
				 paramObject.setKey(params.get("key").toString());
				 paramObject.setValue(params.get("value").toString());
				 placeholderList.add(paramObject);
			}
			return placeholderList;
		}
		return null;
	}

	@Override
	public List<Map<String, String>> getAlertContactDetails(String userId, String branchCode, String partyId, String applicationCode) {
		List<Map<String, String>> contactDetails = new ArrayList<Map<String, String>>();
		ResponseEntity<Object> response = null;
		Map<String, String> params = new java.util.HashMap<>();
		params.put("partyId", partyId);
		params.put("applicationCode", applicationCode);
		try {
			String formUrl = protocol + ReportConstants.GET_ALERT_CONTACTS_SERVICE;
			HttpEntity<String> entity = new HttpEntity<>(ReportingUtil.buildHeadersForEventService(ReportConstants.ALERTS_APPID, userId, branchCode));
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, Object.class, params);
			LinkedHashMap body = (response != null) ? (LinkedHashMap) response.getBody() : null;

			List data = (body != null) ? (List) body.get("data"): new ArrayList();

			data.forEach(dataObj -> {
				Map mapContactDetails = (Map)((List)((LinkedHashMap) dataObj).get("ObTmAlertPartyContactDetailsModel")).get(0);
				contactDetails.add(mapContactDetails);
			});

		} catch (Exception e) {
			LOGGER.error("Error in getAlertContactDetails() : {}",e.getMessage());
		}
		if (response != null && contactDetails.size() > 0) {
			return contactDetails;
		}
		return null;
	}
}
