package oracle.fsgbu.obscfcm.services.report.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import oracle.fsgbu.obscfcm.alertsgenerator.dto.ParameterDTO;
import oracle.fsgbu.obscfcm.alertsgenerator.utility.AlertServiceUtility;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.obscfcm.services.report.util.ReportingUtil;

@Service
public class ReceiverDetailsClientImpl implements ReceiverDetailsClient {

	@Value("${eureka.protocol:http}")
	private String protocol;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	public ReceiverDetailsClientImpl(RestTemplate restTemplate) {
		this.restTemplate = restTemplate;
	}

	private static final PlatoLogger LOGGER = new PlatoLogger(ReceiverDetailsClientImpl.class);
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public List<ParameterDTO> fetchaddress(String customerNo) {
		ResponseEntity<Object> response = null;
		HttpHeaders headers = new HttpHeaders();
		//TODO Added for Alert PDF generation
		headers = ReportingUtil.buildHeaders(ReportConstants.CMC_APPID);
		Map<String, String> params = new HashMap<>();
		params.put("customerNo", customerNo);
		try {
			HttpEntity<String> entity = new HttpEntity<>(headers);
			
			String formUrl = protocol + ReportConstants.GET_CUSTOMER_ADDRESS;
		
			
			
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, Object.class, params);
		} catch (Exception e) {
		
			LOGGER.error("Error in getMessage()",e);
		}
		if (response != null) {
			 List<ParameterDTO> addList= new ArrayList<>();
			 Object obj= response.getBody();
			 LinkedHashMap  paramslistnew = (LinkedHashMap) obj;
			 ArrayList<Object> objlst= (ArrayList<Object>) paramslistnew.get("data");
			 for(Object o:objlst) {
				 LinkedHashMap  paramslistnew2 = (LinkedHashMap) o;
				 ParameterDTO paramObject2 = new ParameterDTO();
				 if(paramslistnew2.containsKey("customerName1")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("receiverName");
					 paramObject.setValue(paramslistnew2.get("customerName1").toString());
					 addList.add(paramObject); 
				 }
				 if(paramslistnew2.containsKey("addressLine1")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("AddrLine1");
					 paramObject.setValue(paramslistnew2.get("addressLine1").toString());
					 addList.add(paramObject); 
	 			 } 
				 if(paramslistnew2.containsKey("addressLine2")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("AddrLine2");
					 paramObject.setValue(paramslistnew2.get("addressLine2").toString());
					 addList.add(paramObject); 
	 			 }  
				 if(paramslistnew2.containsKey("addressLine3")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("AddrLine3");
					 paramObject.setValue(paramslistnew2.get("addressLine3").toString());
					 addList.add(paramObject); 
	 			 } 
				 if(paramslistnew2.containsKey("addressLine4")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("city");
					 paramObject.setValue(paramslistnew2.get("addressLine4").toString());
					 addList.add(paramObject); 
	 			 } 
				 if(paramslistnew2.containsKey("country")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("country");
					 paramObject.setValue(paramslistnew2.get("country").toString());
					 addList.add(paramObject); 
	 			 }  
				 if(paramslistnew2.containsKey("pincode")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("pincode");
					 paramObject.setValue(paramslistnew2.get("pincode").toString());
					 addList.add(paramObject); 
	 			 }
			 }

			
			 return addList;
		}

		
		return null;
	}

	@Override
	public List<ParameterDTO> fetchSenderaddress(String branchCode) {
		ResponseEntity<Object> response = null;
		HttpHeaders headers = new HttpHeaders();
		//TODO Added for Alert PDF generation
		headers = ReportingUtil.buildHeaders(ReportConstants.CMC_APPID);
		Map<String, String> params = new HashMap<>();
		params.put("branchCode", branchCode);
		try {
			HttpEntity<String> entity = new HttpEntity<>(headers);
			
			String formUrl = protocol + ReportConstants.GET_SENDER_ADDRESS;
			
			
			response = restTemplate.exchange(formUrl, HttpMethod.GET, entity, Object.class, params);
		} catch (Exception e) {
		
			LOGGER.error("Error in getMessage()",e);
		}
		if (response != null) {
			 List<ParameterDTO> senderAddList= new ArrayList<>();
			 Object obj= response.getBody();
			 LinkedHashMap  paramslistnew = (LinkedHashMap) obj;
			 ArrayList<Object> objlst= (ArrayList<Object>) paramslistnew.get("data");
			 for(Object o:objlst) {
				 LinkedHashMap  paramslistnew2 = (LinkedHashMap) o;
				 ParameterDTO paramObject2 = new ParameterDTO();
				 if(paramslistnew2.containsKey("branchName")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("senderName");
					 paramObject.setValue(paramslistnew2.get("branchName").toString());
					 senderAddList.add(paramObject); 
				 }
				 if(paramslistnew2.containsKey("branchAddr1")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("senderAddrLine1");
					 paramObject.setValue(paramslistnew2.get("branchAddr1").toString());
					 senderAddList.add(paramObject); 
	 			 } 
				 if(paramslistnew2.containsKey("branchAddr2")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("senderAddrLine2");
					 paramObject.setValue(paramslistnew2.get("branchAddr2").toString());
					 senderAddList.add(paramObject); 
	 			 }  
				 if(paramslistnew2.containsKey("branchAddr3")) {
					 ParameterDTO paramObject = new ParameterDTO();
					 paramObject.setKey("senderAddrLine3");
					 paramObject.setValue(paramslistnew2.get("branchAddr3").toString());
					 senderAddList.add(paramObject); 
	 			 } 
			 }

			
			 return senderAddList;
	}
		return null;
	
	
	}
}
