package oracle.fsgbu.obscfcm.services.report.service;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import oracle.fsgbu.gus.core.client.dto.CoreBranchCollection;
import oracle.fsgbu.gus.core.client.dto.CoreBranchDTO;
import oracle.fsgbu.obscfcm.services.report.client.CmcBranchServiceClient;
import oracle.fsgbu.obscfcm.services.report.client.CmcCurrencyServiceClient;
import oracle.fsgbu.obscfcm.services.report.constants.ReportConstants;
import oracle.fsgbu.obscfcm.services.report.model.CurrencyUtilityDto;
import oracle.fsgbu.obscfcm.services.report.persistence.ObExpDrCrPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObFinPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObInvPmtReconRepository;
import oracle.fsgbu.obscfcm.services.report.persistence.ObReconAllocPmtTxnRepository;
import oracle.fsgbu.plato.logger.PlatoLogger;
import oracle.fsgbu.reconDashboard.api.model.CategoryPercentageDto;
import oracle.fsgbu.reconDashboard.api.model.CorporateDetailsDto;
import oracle.fsgbu.reconDashboard.api.model.CorporateMasterDto;
import oracle.fsgbu.reconDashboard.api.model.ReconCategoryResponseDto;
import oracle.fsgbu.reconDashboard.api.model.ReconCorporateDetailsResponseDto;
import oracle.fsgbu.reconDashboard.api.model.ReconTopCorporateResponseDto;
import oracle.fsgbu.reconDashboard.api.model.ReconTrendGraphResponseDto;
import oracle.fsgbu.reconDashboard.api.model.TrendGraphCountDto;

@Service
public class ReconDashboardServiceImpl implements ReconDashboardService {
	private static final PlatoLogger LOGGER = new PlatoLogger(ReconEventListenerServiceImpl.class);
	private ObInvPmtReconRepository obInvPmtReconRepository;
	private ObExpDrCrPmtReconRepository obExpDrCrPmtReconRepository;
	private ObFinPmtReconRepository obFinPmtReconRepository;
	private ObReconAllocPmtTxnRepository obReconAllocPmtTxnRepository;
	private CmcCurrencyServiceClient cmcCurrencyServiceClient;
	private CmcBranchServiceClient cmcBranchServiceClient;
	private final String UNION = "UNION ";
	private final String UNION_ALL = "UNION ALL ";
	private final String ROWNUM_CLAUSE = " ROWNUM <=5";

	@Autowired
	public ReconDashboardServiceImpl(
			ObInvPmtReconRepository obInvPmtReconRepository,
			ObExpDrCrPmtReconRepository obExpDrCrPmtReconRepository,
			ObFinPmtReconRepository obFinPmtReconRepository,
			ObReconAllocPmtTxnRepository obReconAllocPmtTxnRepository,
			CmcCurrencyServiceClient cmcCurrencyServiceClient,
			CmcBranchServiceClient cmcBranchServiceClient) {

		this.obInvPmtReconRepository = obInvPmtReconRepository;
		this.obExpDrCrPmtReconRepository = obExpDrCrPmtReconRepository;
		this.obFinPmtReconRepository = obFinPmtReconRepository;
		this.obReconAllocPmtTxnRepository = obReconAllocPmtTxnRepository;
		this.cmcCurrencyServiceClient = cmcCurrencyServiceClient;
		this.cmcBranchServiceClient = cmcBranchServiceClient;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconCategoryResponseDto getInvoiceReconCategoryList(String userId, String branchCode, String fromDate, String toDate, String corporateId) {
		LOGGER.info("Getting InvoiceReconCategory List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, SUM(A.COUNT_RECON) AS COUNT_RECON FROM( " ) 
		.append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON      FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE WHERE RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE  WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE  WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE      WHERE REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL     AND  REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
		.append("     ) A GROUP BY A.RECON_CATEGORY ");

		LOGGER.info("getInvoiceReconCategoryList Query :{}",selectQuery.toString());
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		response.setCategoryPercentage(populateReconCountList(resultSet, "Invoice Recon"));
		return response;
		
	}
	
	private List<CategoryPercentageDto> populateReconCountList(List<Object[]> categoryResultSet, String group) {
		
		
		List<CategoryPercentageDto> categoryList = new ArrayList<>();
		AtomicInteger id= new AtomicInteger(0);
		categoryResultSet.stream()
		 .forEach(resultSet -> {
			 CategoryPercentageDto recontCountDetails = null;

			if(resultSet[0] != null ) {
				recontCountDetails = new CategoryPercentageDto();
				recontCountDetails.setId(new BigDecimal(id.incrementAndGet()));
				recontCountDetails.setGroup(group);
				recontCountDetails.setSeries((String) resultSet[0]);
				recontCountDetails.setValue((BigDecimal) resultSet[1]);
				categoryList.add(recontCountDetails);
			}
		 });

		
		return categoryList;
	}
	
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTrendGraphResponseDto getInvoiceReconTrendGraphList(String userId, String branchCode, String fromDate, String toDate, String corporateId) {
        LOGGER.info("Getting InvoiceReconTrendGraph List in service");
        
        StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
        String reconFormToDateQuery = formDateQuery(fromDate,toDate);
        StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
        StringBuilder selectQuery = new StringBuilder();
        selectQuery.append(" SELECT A.RECON_CATEGORY, COUNT(A.PAYMENT_REF_NO), TO_CHAR(A.PAYMENT_DATE, 'MON') AS MONTH , TO_CHAR(A.PAYMENT_DATE, 'YYYY') AS YEAR, TO_CHAR(A.PAYMENT_DATE, 'MON-YYYY') AS PAYMENT_DATE  FROM(  " ) 
        .append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE      FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE WHERE RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO, PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE  WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO, PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO, PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO, PAYMENT.PAYMENT_DATE  FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' )  AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY,RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE  WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY,RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY,RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY,RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_INV_PMT_TXN RPAYMENT  JOIN SCFCM_PAYMENT PAYMENT ON  RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ('M','A')     AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' WHERE RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE      FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE      WHERE REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL     AND  REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL   AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.PAYMENT_REF_NO IS NULL     AND PAYMENT.FLAG = 'I'      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
        .append("     ) A GROUP BY A.PAYMENT_DATE, A.RECON_CATEGORY ORDER BY A.PAYMENT_DATE ");

		LOGGER.info("getInvoiceReconTrendGraphList Query :{}",selectQuery.toString());
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
        ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
        response.setCategoryCount(populateReconTrendGraphList(resultSet, fromDate, toDate, "Invoice Recon"));
        return response;
	}
	
private List<TrendGraphCountDto> populateReconTrendGraphList(List<Object[]> trendGraphResultSet, String fromDate, String toDate, String series ) {
		
		
		List<TrendGraphCountDto> trendList = new ArrayList<>();
		AtomicInteger id= new AtomicInteger(0);
		trendGraphResultSet.stream()
		 .forEach(resultSet -> {
			 TrendGraphCountDto recontCountDetails = null;

			if(resultSet[0] != null ) {
				recontCountDetails = new TrendGraphCountDto();
				recontCountDetails.setId(new BigDecimal(id.incrementAndGet()));
				recontCountDetails.setSeries((String) resultSet[0]);
				recontCountDetails.setValue((BigDecimal) resultSet[1]);
				recontCountDetails.setMonth((String) resultSet[2]);
				recontCountDetails.setYear((String) resultSet[3]);
				recontCountDetails.setDate((String) resultSet[4]);
				trendList.add(recontCountDetails);
			}
		 });
		
		List<String> monthsWithinRange = getMonthAndDateOfGivenRange(fromDate, toDate);
		
		
		return getMissingDataPoints(trendList, monthsWithinRange, series, id);
		
	}

private List<TrendGraphCountDto> getMissingDataPoints(List<TrendGraphCountDto> actualList, List<String> monthsWithinRange, String series, AtomicInteger id){
	BigDecimal delfaultValue =new BigDecimal("0");
	
	
	
	if("Allocation".equals(series)) {
		List<TrendGraphCountDto> filteredAllocatedList = monthsWithinRange.stream()
			      .filter(selectedDate -> !actualList.stream()
			        .anyMatch(dto -> 
			        dto.getSeries().equals("Allocated") && 
			        selectedDate.equals(dto.getDate())))
			        .collect(Collectors.mapping(
			        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
			        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Allocated").value(delfaultValue),
			            Collectors.toList()));
	  	
	  	List<TrendGraphCountDto> filteredPartialllyAllocatedList = monthsWithinRange.stream()
	  		      .filter(selectedDate -> !actualList.stream()
	  		        .anyMatch(dto -> 
	  		        dto.getSeries().equals("Partiallly Allocated") && 
	  		      selectedDate.equals(dto.getDate())))
	  		        .collect(Collectors.mapping(
	  		        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
	  		        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Partiallly Allocated").value(delfaultValue),
	  		            Collectors.toList()));
	  	
	  	List<TrendGraphCountDto> filteredUnallocatedList = monthsWithinRange.stream()
			      .filter(selectedDate -> !actualList.stream()
			        .anyMatch(dto -> 
			        dto.getSeries().equals("Unallocated") && 
			        selectedDate.equals(dto.getDate())))
			        .collect(Collectors.mapping(
			        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
			        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Unallocated").value(delfaultValue),
			            Collectors.toList()));
	  	
	  	List<TrendGraphCountDto> filteredFailedList = monthsWithinRange.stream()
	  		      .filter(selectedDate -> !actualList.stream()
	  		        .anyMatch(dto -> 
	  		        dto.getSeries().equals("Failed") && 
	  		      selectedDate.equals(dto.getDate())))
	  		        .collect(Collectors.mapping(
	  		        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
	  		        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Failed").value(delfaultValue),
	  		            Collectors.toList()));
	  	
	  	actualList.addAll(filteredAllocatedList);
	  	actualList.addAll(filteredPartialllyAllocatedList);
	  	actualList.addAll(filteredUnallocatedList);
	  	actualList.addAll(filteredFailedList);
	}else {
		List<TrendGraphCountDto> filteredAutoReconciledList = monthsWithinRange.stream()
			      .filter(selectedDate -> !actualList.stream()
			        .anyMatch(dto -> 
			        dto.getSeries().equals("Auto Reconciled") && 
			        selectedDate.equals(dto.getDate())))
			        .collect(Collectors.mapping(
			        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
			        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Auto Reconciled").value(delfaultValue),
			            Collectors.toList()));
	  	
	  	List<TrendGraphCountDto> filteredManualReconciledList = monthsWithinRange.stream()
	  		      .filter(selectedDate -> !actualList.stream()
	  		        .anyMatch(dto -> 
	  		        dto.getSeries().equals("Manual Reconciled") && 
	  		      selectedDate.equals(dto.getDate())))
	  		        .collect(Collectors.mapping(
	  		        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
	  		        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Manual Reconciled").value(delfaultValue),
	  		            Collectors.toList()));
	  	
	  	
	  	
	  	
	  	if(!series.equals("Trend Graph")) {
	  		List<TrendGraphCountDto> filteredUnreconciledlList = monthsWithinRange.stream()
	    		      .filter(selectedDate -> !actualList.stream()
	    		        .anyMatch(dto -> 
	    		        dto.getSeries().equals("Unreconciled") && 
	    		        selectedDate.equals(dto.getDate())))
	    		        .collect(Collectors.mapping(
	    		        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
	    		        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Unreconciled").value(delfaultValue),
	    		            Collectors.toList()));
	  		actualList.addAll(filteredUnreconciledlList);
	  		
	  		List<TrendGraphCountDto> filteredPartiallyReconciledlList = monthsWithinRange.stream()
	  		      .filter(selectedDate -> !actualList.stream()
	  		        .anyMatch(dto -> 
	  		        dto.getSeries().equals("Partially Reconciled") && 
	  		        selectedDate.equals(dto.getDate())))
	  		        .collect(Collectors.mapping(
	  		        		selectedDate -> new TrendGraphCountDto().id(new BigDecimal(id.incrementAndGet())).date(selectedDate).
	  		        		month(selectedDate.substring(0, 3)).year(selectedDate.substring(4, 8)).series("Partially Reconciled").value(delfaultValue),
	  		            Collectors.toList()));
	  		actualList.addAll(filteredPartiallyReconciledlList);
	  	}
	  	
	  	actualList.addAll(filteredAutoReconciledList);
	  	actualList.addAll(filteredManualReconciledList);
	}
	
	
	
	actualList.sort((o1, o2) -> {
		DateFormat f = new SimpleDateFormat("MMM-yyyy");
		try {
        if (f.parse(o1.getDate()).compareTo(f.parse(o2.getDate())) == 0) {
            return o1.getSeries().compareTo(o2.getSeries());
        } else {
            return f.parse(o1.getDate()).compareTo(f.parse(o2.getDate()));
        } 
		} catch (ParseException e) {
            throw new IllegalArgumentException(e);
        }
    });
	
	return actualList;
}

	private List<CorporateMasterDto> populateCorporateCountList(List<Object[]> corporateDetailsResultSet, String userId,
			String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String series, String reconTypeRequest, String trendlineFlag, String trendlineDate) {

		List<CorporateMasterDto> corporateCountList = new ArrayList<>();
		AtomicInteger id = new AtomicInteger(0);
		corporateDetailsResultSet.stream().forEach(resultSet -> {
			CorporateMasterDto reconCorporateCountDetails = null;

			if (resultSet[0] != null) {
				reconCorporateCountDetails = new CorporateMasterDto();
				reconCorporateCountDetails.setId(new BigDecimal(id.incrementAndGet()));
				reconCorporateCountDetails.setCorporateName((String) resultSet[0]);
				reconCorporateCountDetails.setCorporateID((String) resultSet[1]);
				reconCorporateCountDetails.setCount((BigDecimal) resultSet[2]);
				reconCorporateCountDetails.setSeries(series);
				List<CorporateDetailsDto> corporateDetails = null;

				if (series.equals("Invoice Recon")) {
					corporateDetails = getInvoiceReconCorporateDetailsList(userId, branchCode, fromDate, toDate,
							reconType,paymentStatus, (String) resultSet[1], reconTypeRequest, trendlineFlag, trendlineDate).getCorporateCount();
				}else if(series.equals("Cashflow Recon")) {
					corporateDetails = getCashflowReconCorporateDetailsList(userId, branchCode, fromDate, toDate,
							reconType,paymentStatus, (String) resultSet[1], reconTypeRequest, trendlineFlag, trendlineDate).getCorporateCount();
				}else if(series.equals("Finance Recon")) {
					corporateDetails = getFinanceReconCorporateDetailsList(userId, branchCode, fromDate, toDate,
							reconType,paymentStatus, (String) resultSet[1], reconTypeRequest, trendlineFlag, trendlineDate).getCorporateCount();
				}else if(series.equals("Allocation")) {
					corporateDetails = getAllocationReconCorporateDetailsList(userId, branchCode, fromDate, toDate,
							reconType,paymentStatus, (String) resultSet[1], reconTypeRequest, trendlineFlag, trendlineDate).getCorporateCount();
				}else if(series.equals("All")) {
					corporateDetails = getReconciliationCorporateDetailsList(userId, branchCode, fromDate, toDate,
							reconType,paymentStatus, (String) resultSet[1], reconTypeRequest, trendlineFlag, trendlineDate).getCorporateCount();
				}
				reconCorporateCountDetails.setCorporateDetails(corporateDetails);
				corporateCountList.add(reconCorporateCountDetails);
			}
		});

		return corporateCountList;
	}
	

private List<CorporateDetailsDto> populateCorporateDetails(List<Object[]> corporateDetailsResultSet, String branchCode, String userId) {
	
	String localBranchCCY = cmcBranchServiceClient.getBranchCurrency(branchCode);
	DateFormat dateFormat = new SimpleDateFormat(ReportConstants.SIMPLE_DATE_FORMAT);
	Date currentBusinessDate = null;
	try {
		currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
	} catch (ParseException e) {
		LOGGER.error("Exception in currentBusinessDate:: {}", e.getMessage(), e);
	}
	String simpleBranchDate = dateFormat.format(currentBusinessDate);	//yyyy-MM-DD format for currency conversion service
	
	List<CorporateDetailsDto> corporateDetailsList = new ArrayList<>();
	AtomicInteger id= new AtomicInteger(0);
	corporateDetailsResultSet.parallelStream()
	 .forEach(resultSet -> {
		 CorporateDetailsDto reconCorporateDetails = null;

		if(resultSet[0] != null ) {
			BigDecimal paymentAmount = (BigDecimal) resultSet[4];
			String paymentCCY = (String) resultSet[7];
			BigDecimal paymentAmountInLCY = getAmountInLocalCurrency(localBranchCCY, paymentAmount, paymentCCY, branchCode, userId, simpleBranchDate);
			reconCorporateDetails = new CorporateDetailsDto();
			reconCorporateDetails.setId(new BigDecimal(System.currentTimeMillis()));
			reconCorporateDetails.setCorporateName((String) resultSet[0]);
			reconCorporateDetails.setCorporateID((String) resultSet[1]);
			reconCorporateDetails.setPmtStatus((String) resultSet[2]);
			reconCorporateDetails.setReconAction((String) resultSet[3]);
			reconCorporateDetails.setPaymentAmount(paymentAmount);
			reconCorporateDetails.setAmountReconciled((BigDecimal) resultSet[5]);
			reconCorporateDetails.setAmountNotReconciled((BigDecimal) resultSet[6]);
			reconCorporateDetails.setPaymentCCY(paymentCCY);
			reconCorporateDetails.setReconCategory((String) resultSet[8]);
			reconCorporateDetails.setPaymentNo((String) resultSet[9]);
			reconCorporateDetails.setDcIndicator((String) resultSet[10]);
			reconCorporateDetails.setBuyerId((String) resultSet[11]);
			reconCorporateDetails.setBuyerName((String) resultSet[12]);
			reconCorporateDetails.setSupplierId((String) resultSet[13]);
			reconCorporateDetails.setSupplierName((String) resultSet[14]);
			reconCorporateDetails.setPaymentInLCY(paymentAmountInLCY);
			corporateDetailsList.add(reconCorporateDetails);
		}
	 });

	
	return corporateDetailsList;
}

private List<CorporateDetailsDto> populateAllocationCorporateDetails(List<Object[]> corporateDetailsResultSet, String branchCode, String userId) {
	
	String localBranchCCY = cmcBranchServiceClient.getBranchCurrency(branchCode);
	DateFormat dateFormat = new SimpleDateFormat(ReportConstants.SIMPLE_DATE_FORMAT);
	Date currentBusinessDate = null;
	try {
		currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
	} catch (ParseException e) {
		LOGGER.error("Exception in currentBusinessDate:: {}", e.getMessage(), e);
	}
	String simpleBranchDate = dateFormat.format(currentBusinessDate);	//yyyy-MM-DD format for currency conversion service
	
	List<CorporateDetailsDto> corporateDetailsList = new ArrayList<>();
	AtomicInteger id= new AtomicInteger(0);
	corporateDetailsResultSet.parallelStream()
	 .forEach(resultSet -> {
		 CorporateDetailsDto reconCorporateDetails = null;

		if(resultSet[0] != null ) {
			BigDecimal paymentAmount = (BigDecimal) resultSet[4];
			String paymentCCY = (String) resultSet[7];
			BigDecimal paymentAmountInLCY = getAmountInLocalCurrency(localBranchCCY, paymentAmount, paymentCCY, branchCode, userId, simpleBranchDate);
			reconCorporateDetails = new CorporateDetailsDto();
			reconCorporateDetails.setId(new BigDecimal(System.currentTimeMillis()));
			reconCorporateDetails.setCorporateName((String) resultSet[0]);
			reconCorporateDetails.setCorporateID((String) resultSet[1]);
			reconCorporateDetails.setPmtStatus((String) resultSet[2]);
			reconCorporateDetails.setReconAction((String) resultSet[3]);
			reconCorporateDetails.setPaymentAmount(paymentAmount);
			reconCorporateDetails.setAmountReconciled((BigDecimal) resultSet[5]);
			reconCorporateDetails.setAmountNotReconciled((BigDecimal) resultSet[6]);
			reconCorporateDetails.setPaymentCCY(paymentCCY);
			reconCorporateDetails.setReconCategory((String) resultSet[8]);
			reconCorporateDetails.setPaymentNo((String) resultSet[9]);
			reconCorporateDetails.setDcIndicator((String) resultSet[10]);
			reconCorporateDetails.setAllocationStatus((String) resultSet[11]);
			reconCorporateDetails.setPaymentTowards((String) resultSet[12]);
			reconCorporateDetails.setPaymentStatus((String) resultSet[13]);
			reconCorporateDetails.setPaymentInLCY(paymentAmountInLCY);
			corporateDetailsList.add(reconCorporateDetails);
		}
	 });

	
	return corporateDetailsList;
}

private List<CorporateDetailsDto> populateCashflowCorporateDetails(List<Object[]> corporateDetailsResultSet, String branchCode, String userId) {
	
	String localBranchCCY = cmcBranchServiceClient.getBranchCurrency(branchCode);
	DateFormat dateFormat = new SimpleDateFormat(ReportConstants.SIMPLE_DATE_FORMAT);
	Date currentBusinessDate = null;
	try {
		currentBusinessDate = dateFormat.parse(cmcBranchServiceClient.getCurrentBusinessDate(userId, branchCode));
	} catch (ParseException e) {
		LOGGER.error("Exception in currentBusinessDate:: {}", e.getMessage(), e);
	}
	String simpleBranchDate = dateFormat.format(currentBusinessDate);	//yyyy-MM-DD format for currency conversion service
	
	List<CorporateDetailsDto> corporateDetailsList = new ArrayList<>();
	AtomicInteger id= new AtomicInteger(0);
	corporateDetailsResultSet.parallelStream()
	 .forEach(resultSet -> {
		 CorporateDetailsDto reconCorporateDetails = null;

		if(resultSet[0] != null ) {
			BigDecimal paymentAmount = (BigDecimal) resultSet[4];
			String paymentCCY = (String) resultSet[7];
			BigDecimal paymentAmountInLCY = getAmountInLocalCurrency(localBranchCCY, paymentAmount, paymentCCY, branchCode, userId, simpleBranchDate);
			reconCorporateDetails = new CorporateDetailsDto();
			reconCorporateDetails.setId(new BigDecimal(System.currentTimeMillis()));
			reconCorporateDetails.setCorporateName((String) resultSet[0]);
			reconCorporateDetails.setCorporateID((String) resultSet[1]);
			reconCorporateDetails.setPmtStatus((String) resultSet[2]);
			reconCorporateDetails.setReconAction((String) resultSet[3]);
			reconCorporateDetails.setPaymentAmount(paymentAmount);
			reconCorporateDetails.setAmountReconciled((BigDecimal) resultSet[5]);
			reconCorporateDetails.setAmountNotReconciled((BigDecimal) resultSet[6]);
			reconCorporateDetails.setPaymentCCY(paymentCCY);
			reconCorporateDetails.setReconCategory((String) resultSet[8]);
			reconCorporateDetails.setCashflowType((String) resultSet[9]);
			reconCorporateDetails.setPaymentNo((String) resultSet[10]);
			reconCorporateDetails.setDcIndicator((String) resultSet[11]);
			reconCorporateDetails.setPaymentInLCY(paymentAmountInLCY);
			corporateDetailsList.add(reconCorporateDetails);
		}
	 });

	
	return corporateDetailsList;
}



@Override
@Transactional("PLATO_TX_MANAGER")
public ReconCategoryResponseDto getCashFlowReconCategoryList(String userId, String branchCode, String fromDate, String toDate, String corporateId) {
	LOGGER.info("Getting CashFlowReconCategory List in service");
	try {
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, SUM(A.COUNT_RECON) AS COUNT_RECON FROM(  " ) 
		.append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON        FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'  				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'  				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.ACCOUNT_OWNER       WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'E' " ).append(reconFormToDateQuery).append(corporateIDClause)
		.append("     ) A GROUP BY A.RECON_CATEGORY ");

		LOGGER.info("getCashFlowReconCategoryList Query :{}",selectQuery.toString());

		List<Object[]> resultSet = obExpDrCrPmtReconRepository.executeNativeQuery(selectQuery.toString());
			ReconCategoryResponseDto response = new ReconCategoryResponseDto();
			response.setCategoryPercentage(populateReconCountList(resultSet, "Cashflow Recon"));
			return response;
		} catch (Exception e) {
			LOGGER.error("Exception in getCashFlowReconCategoryList:: {}", e.getMessage(), e);
			return null;
		}
	
}

@Override
@Transactional("PLATO_TX_MANAGER")
public ReconTrendGraphResponseDto getCashFlowReconTrendGraphList(String userId, String branchCode, String fromDate, String toDate, String corporateId) {
	LOGGER.info("Getting CashFlowReconTrendGraph List in service");
	StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
	String reconFormToDateQuery = formDateQuery(fromDate,toDate);
	StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
	StringBuilder selectQuery = new StringBuilder();
	selectQuery.append(" SELECT A.RECON_CATEGORY, COUNT(A.PAYMENT_REF_NO), TO_CHAR(A.PAYMENT_DATE, 'MON') AS MONTH , TO_CHAR(A.PAYMENT_DATE, 'YYYY') AS YEAR, TO_CHAR(A.PAYMENT_DATE, 'MON-YYYY') AS PAYMENT_DATE  FROM(  " ) 
	.append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
	.append(UNION_ALL).append("     SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
	.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED'  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
	.append(UNION_ALL).append("     SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER       AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL  WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
	.append(UNION_ALL).append("     SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'  				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' 				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO     AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED'  				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.ACCOUNT_OWNER       WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause)
	.append(UNION_ALL).append("    SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL        JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause) 
	.append(UNION_ALL).append("    SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)          JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL       AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'       AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause)  
	.append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON       (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL         JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID        AND PAYMENT.ACCOUNT_OWNER IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL      AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'E' " ).append(reconFormToDateQuery).append(corporateIDClause)
	.append("     ) A GROUP BY A.PAYMENT_DATE, A.RECON_CATEGORY ORDER BY A.PAYMENT_DATE ");

	LOGGER.info("getCashFlowReconTrendGraphList Query :{}",selectQuery.toString());

	List<Object[]> resultSet = obExpDrCrPmtReconRepository.executeNativeQuery(selectQuery.toString());
	ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
	response.setCategoryCount(populateReconTrendGraphList(resultSet, fromDate, toDate, "Cashflow Recon"));
	return response;
	
}

@Override
@Transactional("PLATO_TX_MANAGER")
public ReconTopCorporateResponseDto getInvoiceReconTopCorporateList(String userId, String branchCode, String fromDate, String toDate, String reconTypeRequest, String trendlineFlag, String trendlineDate, String corporateId) {
	LOGGER.info("Getting getInvoiceReconTopCorporate List in service");
	
	//reconcategory is mandatory in this API.. need to validate
	// Need to add logic for U
	String reconType = "A".equalsIgnoreCase(reconTypeRequest)? "'A'" : "M".equalsIgnoreCase(reconTypeRequest) ? "'M'" :"P".equalsIgnoreCase(reconTypeRequest) ? "'M','A'" : "";
	String paymentStatus = ("A".equalsIgnoreCase(reconTypeRequest) || "M".equalsIgnoreCase(reconTypeRequest)) ? "MATCHED" : "P".equalsIgnoreCase(reconTypeRequest) ? "PART_MATCHED" : "";
	StringBuilder selectQuery = new StringBuilder();
	StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
	
	StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
	StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
	String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
	StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
	
	if("U".equalsIgnoreCase(reconTypeRequest)) {
		selectQuery.append(" SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.CORPORATE_NAME) AS CORPORATE_COUNT FROM( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO " + 
				"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
				"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
				"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE   " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
				"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
				"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
						"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
						"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
						"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " + 
						"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
						"    WHERE REL.CORPORATE_NAME IS NOT NULL " + 
						"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
						"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
								"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
								"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)     " + 
								"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
								"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL  " + 
								"     AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
								"    WHERE REL.CORPORATE_NAME IS NOT NULL " +  
								"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
								"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
										"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
										"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL    " + 
										"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID   " + 
										"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
										"    WHERE REL.CORPORATE_NAME IS NOT NULL " + 
										"    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'I' ").append(reconFormToDateQuery).append(corporateIDClause).append("    ) A   GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ").append(ROWNUM_CLAUSE);
	}else {
		selectQuery.append(" SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM ( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE   " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID   " + 
				"     AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID   " + 
				"     AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL  " + 
				"     AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ")
		.append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID   " + 
				"     AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(" )A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC ) WHERE ").append(ROWNUM_CLAUSE);
	}
	
	
	List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
	ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
	response.setCorporateCount(populateCorporateCountList(resultSet,userId, branchCode,fromDate,toDate,reconType,paymentStatus,"Invoice Recon", reconTypeRequest, trendlineFlag, trendlineDate));
	
	return response;
	
}

@Transactional("PLATO_TX_MANAGER")
public ReconCorporateDetailsResponseDto getInvoiceReconCorporateDetailsList(String userId, String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String corporateID, String reconTypeRequest, String trendlineFlag, String trendlineDate) {
	LOGGER.info("Getting getInvoiceReconCorporateDetailsList List in service");
	
	//reconcategory is mandatory in this API.. need to validate
	// Need to add logic for U and P
	//String reconType = "A".equalsIgnoreCase(reconCetegory)? "'A'" : "M".equalsIgnoreCase(reconCetegory) ? "'M'" :"P".equalsIgnoreCase(reconCetegory) ? "'M','A'" : "";
	//String paymentStatus = ("A".equalsIgnoreCase(reconCetegory) || "M".equalsIgnoreCase(reconCetegory)) ? "MATCHED" : "P".equalsIgnoreCase(reconCetegory) ? "PART_MATCHED" : "";
	StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateID)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateID).append("' ") : new StringBuilder("");
	StringBuilder selectQuery = new StringBuilder();
	StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
	StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
	String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
	StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
	
	if("U".equalsIgnoreCase(reconTypeRequest)) {
		selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID,  NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
				"    PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME " + 
				"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
				"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
				"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = PAYMENT.RELATIONSHIP_CODE   " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  " +
				"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
				"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
				"    PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME " + 
				"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
				"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
				"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " + 
				"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
				"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
				"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
				"    PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME   " + 
				"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
				"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)     " + 
				"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
				"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL  " + 
				"     AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
				"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'I'  " + 
				"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
				"    PAYMENT.PAYMENT_CURRENCY, 'IPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME   " + 
				"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_RECON_INV_PMT_TXN RPAYMENT ON   " + 
				"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL    " + 
				"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID   " + 
				"     AND PAYMENT.RELATIONSHIP_CODE IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
				"    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'I' ").append(reconFormToDateQuery).append(corporateIDClause);
	} else {
		selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
				"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE   " + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE =  ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
				"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID   " +
				"    AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'"+
						"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
				"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME   " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID   " + 
						"     AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL  " + 
						"     AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
						"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
				"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
				"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
				"    PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID, INV_BUYER_NAME, INV_SUPPLIER_ID, INV_SUPPLIER_NAME  " + 
				"     FROM OB_RECON_INV_PMT_TXN RPAYMENT   " + 
				"     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
				"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
				"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID   " + 
						"     AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
						"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause);
		
	}
	
	List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
	ReconCorporateDetailsResponseDto response = new ReconCorporateDetailsResponseDto();
	response.setCorporateCount(populateCorporateDetails(resultSet, branchCode, userId));
	
	return response;
	
}

	private String formDateQuery(String fromDate, String toDate) {
		if(!StringUtils.isEmpty(fromDate) && !StringUtils.isEmpty(toDate)) {
			return (new StringBuilder(" AND PAYMENT.PAYMENT_DATE BETWEEN TO_DATE ('").append(fromDate).append("', 'yyyy/mm/dd')")
			.append("    AND TO_DATE ('").append(toDate).append("', 'yyyy/mm/dd')")).toString();
		}else {
			return "AND PAYMENT.PAYMENT_DATE >= add_months(sysdate, -6)"; 
		}
	}
	
	private String formDateQueryForTrendline(String date) {
		if(!StringUtils.isEmpty(date)) {
			return (new StringBuilder(" AND TO_CHAR(PAYMENT.PAYMENT_DATE, 'MON-YYYY')  = TO_CHAR(TO_DATE('").append(date).append("', 'MON-YYYY'), 'MON-YYYY')")).toString();
		}else {
			return "";
		}
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconCategoryResponseDto getAllocationReconCategoryList(String userId, String branchCode,
			@Valid String fromDate, @Valid String toDate, String corporateId) {
		LOGGER.info("Getting AllocationReconCategory List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, SUM(A.COUNT_RECON) AS COUNT_RECON FROM(  " ) 
		.append("    SELECT 'Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Failed' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partiallly Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Unallocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO)  AS COUNT_RECON   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append("     ) A GROUP BY A.RECON_CATEGORY ");

		LOGGER.info("getAllocationReconTrendGraphList Query :{}",selectQuery.toString());

		List<Object[]> resultSet = obReconAllocPmtTxnRepository.executeNativeQuery(selectQuery.toString());
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		response.setCategoryPercentage(populateReconCountList(resultSet, "Allocation Recon"));
		return response;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTrendGraphResponseDto getAllocationReconTrendGraphList(String userId, String branchCode,
			@Valid String fromDate, @Valid String toDate, String corporateId) {
		LOGGER.info("Getting AllocationReconTrendGraph List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, COUNT(A.PAYMENT_REF_NO), TO_CHAR(A.PAYMENT_DATE, 'MON') AS MONTH , TO_CHAR(A.PAYMENT_DATE, 'YYYY') AS YEAR, TO_CHAR(A.PAYMENT_DATE, 'MON-YYYY') AS PAYMENT_DATE  FROM(  " ) 
		.append("    SELECT 'Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE     FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Failed' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO, PAYMENT.PAYMENT_DATE   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Failed' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'FAILED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Partiallly Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Partiallly Allocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'PART_ALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO       AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID       AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'   WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
		.append(UNION_ALL).append("     SELECT 'Unallocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO      AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'      WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append(UNION_ALL).append("    SELECT 'Unallocated' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO        AND RPAYMENT.PMT_ALLOCATION_STATUS = 'UNALLOCATED'     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID      AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
		.append("     ) A GROUP BY A.PAYMENT_DATE, A.RECON_CATEGORY ORDER BY A.PAYMENT_DATE ");
		
		List<Object[]> resultSet = obReconAllocPmtTxnRepository.executeNativeQuery(selectQuery.toString());
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		response.setCategoryCount(populateReconTrendGraphList(resultSet, fromDate, toDate, "Allocation"));
		return response;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconCategoryResponseDto getFinanceReconCategoryList(String userId, String branchCode,
			@Valid String fromDate, @Valid String toDate, String corporateId) {
		LOGGER.info("Getting FinanceReconCategory List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, SUM(A.COUNT_RECON) AS COUNT_RECON FROM(  " ) 
        .append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
        .append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, COUNT(RPAYMENT.PAYMENT_REF_NO) AS COUNT_RECON FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
        .append(UNION_ALL).append("       SELECT 'Unreconciled' AS RECON_CATEGORY, COUNT(PAYMENT_REF_NO) AS COUNT_RECON FROM ( ")
        .append("                        SELECT PAYMENT.PAYMENT_REF_NO FROM SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION).append("          SELECT PAYMENT.PAYMENT_REF_NO FROM SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' " ).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION).append("          SELECT PAYMENT.PAYMENT_REF_NO FROM SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'" ).append(reconFormToDateQuery).append(corporateIDClause)
        .append("                      )")
        .append("     ) A GROUP BY A.RECON_CATEGORY ");
		
		List<Object[]> resultSet = obFinPmtReconRepository.executeNativeQuery(selectQuery.toString());
		
		ReconCategoryResponseDto response = new ReconCategoryResponseDto();
		response.setCategoryPercentage(populateReconCountList(resultSet, "Finance Recon"));
		return response;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTrendGraphResponseDto getFinanceReconTrendGraphList(String userId, String branchCode,
			@Valid String fromDate, @Valid String toDate, String corporateId) {
		LOGGER.info("Getting FinanceReconTrendGraph List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT A.RECON_CATEGORY, COUNT(A.PAYMENT_REF_NO), TO_CHAR(A.PAYMENT_DATE, 'MON') AS MONTH , TO_CHAR(A.PAYMENT_DATE, 'YYYY') AS YEAR, TO_CHAR(A.PAYMENT_DATE, 'MON-YYYY') AS PAYMENT_DATE  FROM(  " ) 
        .append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Auto Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'A' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Manual Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M' ) AND PAYMENT.PAYMENT_STATUS = 'MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("    SELECT 'Partially Reconciled' AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT JOIN SCFCM_PAYMENT PAYMENT ON RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO AND RPAYMENT.RECON_TYPE IN ( 'M','A' ) AND PAYMENT.PAYMENT_STATUS = 'PART_MATCHED' JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION_ALL).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause) 
        .append(UNION).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
        .append(UNION).append("     SELECT 'Unreconciled' AS RECON_CATEGORY, PAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE  FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F' AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'  " ).append(reconFormToDateQuery).append(corporateIDClause)  
        .append("     ) A GROUP BY A.PAYMENT_DATE, A.RECON_CATEGORY ORDER BY A.PAYMENT_DATE ");
		
		List<Object[]> resultSet = obFinPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		response.setCategoryCount(populateReconTrendGraphList(resultSet, fromDate, toDate, "Finance Recon"));
		return response;
	}

	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTopCorporateResponseDto getCashflowReconTopCorporateList(String userId, String branchCode, String fromDate, String toDate, String reconTypeRequest, String trendlineFlag, String trendlineDate, String corporateId) {
LOGGER.info("Getting getCashflowReconTopCorporate List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U
		String reconType = "A".equalsIgnoreCase(reconTypeRequest)? "'A'" : "M".equalsIgnoreCase(reconTypeRequest) ? "'M'" :"P".equalsIgnoreCase(reconTypeRequest) ? "'M','A'" : "";
		String paymentStatus = ("A".equalsIgnoreCase(reconTypeRequest) || "M".equalsIgnoreCase(reconTypeRequest)) ? "MATCHED" : "P".equalsIgnoreCase(reconTypeRequest) ? "PART_MATCHED" : "";
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		if("U".equalsIgnoreCase(reconTypeRequest)) {
			selectQuery.append("  SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.CORPORATE_NAME) AS CORPORATE_COUNT FROM( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.ACCOUNT_OWNER   " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)     " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL  " + 
					"     AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL    " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'E' " ).append(reconFormToDateQuery).append(corporateIDClause).append("    ) A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ROWNUM <=5 ");
		}else {
		   selectQuery.append(" SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append(" LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER   " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL" + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).
			append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append(" LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID   " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL" + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).
append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  " + 
		"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   " + 
		"				     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
		"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
		"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE   " + 
				"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'" + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).
append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  " + 
		"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   " + 
		"				     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
		"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
		"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID   " + 
				"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL " + 
				"                     AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'" + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).
append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  " + 
		"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   " + 
		"				     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
		"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
		"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("				     LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID   " + 
				"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL  " + 
				"                     AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'" + 
				"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).
append(" )A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ROWNUM <=5");
		}
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		response.setCorporateCount(populateCorporateCountList(resultSet,userId, branchCode,fromDate,toDate,reconType,paymentStatus,"Cashflow Recon", reconTypeRequest, trendlineFlag, trendlineDate));
		
		return response;
		
	}

	@Transactional("PLATO_TX_MANAGER")
	public ReconCorporateDetailsResponseDto getCashflowReconCorporateDetailsList(String userId, String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String corporateID, String reconTypeRequest, String trendlineFlag, String trendlineDate) {
		LOGGER.info("Getting getCashflowReconCorporateDetails List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U and P
		//String reconType = "A".equalsIgnoreCase(reconCetegory)? "'A'" : "M".equalsIgnoreCase(reconCetegory) ? "'M'" :"P".equalsIgnoreCase(reconCetegory) ? "'M','A'" : "";
		//String paymentStatus = ("A".equalsIgnoreCase(reconCetegory) || "M".equalsIgnoreCase(reconCetegory)) ? "MATCHED" : "P".equalsIgnoreCase(reconCetegory) ? "PART_MATCHED" : "";
		
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateID)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateID).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		
		if("U".equalsIgnoreCase(reconTypeRequest)) {
			selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
					"    PAYMENT.PAYMENT_CURRENCY, 'EPR' AS RECON_CATEGORY, CASE PAYMENT.DEBIT_CREDIT_INDICATOR WHEN 'D' THEN 'O'  WHEN 'C' THEN 'I' END AS CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.ACCOUNT_OWNER   " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
					"    PAYMENT.PAYMENT_CURRENCY, 'EPR' AS RECON_CATEGORY, CASE PAYMENT.DEBIT_CREDIT_INDICATOR WHEN 'D' THEN 'O'  WHEN 'C' THEN 'I' END AS CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL   " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " +  
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
					"    PAYMENT.PAYMENT_CURRENCY, 'EPR' AS RECON_CATEGORY, CASE PAYMENT.DEBIT_CREDIT_INDICATOR WHEN 'D' THEN 'O'  WHEN 'C' THEN 'I' END AS CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR   " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)     " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND  PAYMENT.BENEFICIARY_ID IS NULL  " + 
					"     AND  PAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND RPAYMENT.PAYMENT_REF_NO IS NULL  AND PAYMENT.FLAG = 'E'  " + 
					"     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause) 
			.append(UNION_ALL).append( "SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, NULL AS PMT_STATUS, 'RECONCILE' AS RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
					"    PAYMENT.PAYMENT_CURRENCY, 'EPR' AS RECON_CATEGORY, CASE PAYMENT.DEBIT_CREDIT_INDICATOR WHEN 'D' THEN 'O'  WHEN 'C' THEN 'I' END AS CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR  " + 
					"     FROM  SCFCM_PAYMENT PAYMENT LEFT OUTER JOIN OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT ON   " + 
					"    (PAYMENT.PAYMENT_REF_NO = RPAYMENT.PAYMENT_REF_NO)    AND RPAYMENT.PAYMENT_REF_NO IS NULL    " + 
					"     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID   " + 
					"     AND PAYMENT.ACCOUNT_OWNER IS NULL AND PAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
					"    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
					"    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'E'" ).append(reconFormToDateQuery).append(corporateIDClause);
		}else {
			selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, RPAYMENT.CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT    " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON    " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("	LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER    " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, RPAYMENT.CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT    " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON    " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN " ).append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID    " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, RPAYMENT.CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT    " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON    " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN  ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("	LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE    " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, RPAYMENT.CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT    " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON    " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("	LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID    " + 
							"   AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL  " + 
							"    AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, RPAYMENT.CASHFLOW_TYPE, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR " + 
					"				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT    " + 
					"				     JOIN SCFCM_PAYMENT PAYMENT ON    " + 
					"     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
					"    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("	LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID    " + 
							"                     AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL   " + 
							"                     AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause);
		}
		List<Object[]> resultSet = obExpDrCrPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconCorporateDetailsResponseDto response = new ReconCorporateDetailsResponseDto();
		response.setCorporateCount(populateCashflowCorporateDetails(resultSet, branchCode, userId));
		
		return response;
		
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTopCorporateResponseDto getFinanceReconTopCorporateList(String userId, String branchCode, String fromDate, String toDate, String reconTypeRequest, String trendlineFlag, String trendlineDate, String corporateId) {
		LOGGER.info("Getting getFinanceReconTopCorporateList List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U
		StringBuilder selectQuery = new StringBuilder();
		String reconType = "A".equalsIgnoreCase(reconTypeRequest)? "'A'" : "M".equalsIgnoreCase(reconTypeRequest) ? "'M'" :"P".equalsIgnoreCase(reconTypeRequest) ? "'M','A'" : "";
		String paymentStatus = ("A".equalsIgnoreCase(reconTypeRequest) || "M".equalsIgnoreCase(reconTypeRequest)) ? "MATCHED" : "P".equalsIgnoreCase(reconTypeRequest) ? "PART_MATCHED" : "";
		
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		if("U".equalsIgnoreCase(reconTypeRequest)) {
            selectQuery.append(" SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.CORPORATE_NAME) AS CORPORATE_COUNT FROM( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
                            "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " +
                            "    WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F'  " + 
                            "     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause).append(UNION).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
                                    "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
                                    "    WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F'  " + 
                                    "     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   " ).append(reconFormToDateQuery).append(corporateIDClause).append(UNION).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
                                            "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID   " + 
                                            "    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
                                            "    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'F'" ).append(reconFormToDateQuery).append(corporateIDClause).append("    ) A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ").append(ROWNUM_CLAUSE);
        }else {
            selectQuery.append("SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM ( SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
                                    "     FROM OB_RECON_FIN_PMT_TXN RPAYMENT   " + 
                                    "     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
                                    "     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
                                    "    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID   " + 
                                            "    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO   " + 
                                                    "     FROM OB_RECON_FIN_PMT_TXN RPAYMENT   " + 
                                                    "     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
                                                    "     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
                                                    "    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID   " + 
                                                            "    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append("    )A GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC ) WHERE ").append(ROWNUM_CLAUSE);
        }
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		response.setCorporateCount(populateCorporateCountList(resultSet,userId, branchCode,fromDate,toDate,reconType,paymentStatus,"Finance Recon", reconTypeRequest, trendlineFlag, trendlineDate));
		
		return response;
		
	}

	@Transactional("PLATO_TX_MANAGER")
	public ReconCorporateDetailsResponseDto getFinanceReconCorporateDetailsList(String userId, String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String corporateID, String reconTypeRequest, String trendlineFlag, String trendlineDate) {
		LOGGER.info("Getting getFinanceReconCorporateDetails List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U and P
		//String reconType = "A".equalsIgnoreCase(reconCetegory)? "'A'" : "M".equalsIgnoreCase(reconCetegory) ? "'M'" :"P".equalsIgnoreCase(reconCetegory) ? "'M','A'" : "";
		//String paymentStatus = ("A".equalsIgnoreCase(reconCetegory) || "M".equalsIgnoreCase(reconCetegory)) ? "MATCHED" : "P".equalsIgnoreCase(reconCetegory) ? "PART_MATCHED" : "";
		
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateID)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateID).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		
		if("U".equalsIgnoreCase(reconTypeRequest)) {
            selectQuery.append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, PAYMENT.PAYMENT_STATUS, 'RECONCILE' AS RECON_ACTION, " + 
                    "    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
                    "    PAYMENT.PAYMENT_CURRENCY, 'FPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, " +
                    "    NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME   " + 
                    "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.BENEFICIARY_ID   " +
                    "    WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F'  " + 
                    "     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
            .append(UNION).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, PAYMENT.PAYMENT_STATUS, 'RECONCILE' AS RECON_ACTION, " + 
                    "    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
                    "    PAYMENT.PAYMENT_CURRENCY, 'FPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, " +
                    "    NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME   " + 
                    "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.COUNTERPARTY_ID   " + 
                    "    WHERE REL.CORPORATE_NAME IS NOT NULL AND PAYMENT.FLAG = 'F'  " + 
                    "     AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED'   ").append(reconFormToDateQuery).append(corporateIDClause)
            .append(UNION).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, PAYMENT.PAYMENT_STATUS, 'RECONCILE' AS RECON_ACTION, " + 
                    "    PAYMENT.PAYMENT_AMOUNT, 0 AS RECONCILED_PMT_AMT, PAYMENT.PAYMENT_AMOUNT AS NOT_RECONCILED_AMOUNT,  " + 
                    "    PAYMENT.PAYMENT_CURRENCY, 'FPR' AS RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, " +
                    "    NULL AS INV_BUYER_ID, NULL AS INV_BUYER_NAME, NULL AS INV_SUPPLIER_ID, NULL AS INV_SUPPLIER_NAME   " +  
                    "     FROM  SCFCM_PAYMENT PAYMENT JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = PAYMENT.PAYMENT_PARTY_ID   " +
                    "    WHERE REL.CORPORATE_NAME IS NOT NULL  " + 
                    "    AND PAYMENT.PAYMENT_STATUS = 'UNMATCHED' AND PAYMENT.FLAG = 'F'  ").append(reconFormToDateQuery).append(corporateIDClause);
        }else {
            selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
                                    "    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
                                    "    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
                                    "    PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID, FIN_BUYER_NAME, FIN_SUPPLIER_ID, FIN_SUPPLIER_NAME " + 
                                    "     FROM OB_RECON_FIN_PMT_TXN RPAYMENT   " + 
                                    "     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
                                    "     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
                                    "    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_BUYER_ID   " + 
                                            "    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append( "SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,    " + 
                                                    "    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
                                                    "    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,  " + 
                                                    "    PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID, FIN_BUYER_NAME, FIN_SUPPLIER_ID, FIN_SUPPLIER_NAME  " + 
                                                    "     FROM OB_RECON_FIN_PMT_TXN RPAYMENT   " + 
                                                    "     JOIN SCFCM_PAYMENT PAYMENT ON   " + 
                                                    "     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO " +
                                                    "    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append(" AND PAYMENT.PAYMENT_STATUS = ").append(reconPaymentStatusQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.FIN_SUPPLIER_ID   " + 
                                                            "    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause);
        }
		List<Object[]> resultSet = obFinPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconCorporateDetailsResponseDto response = new ReconCorporateDetailsResponseDto();
		response.setCorporateCount(populateCorporateDetails(resultSet, branchCode, userId));
		
		return response;
		
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTopCorporateResponseDto getAllocationReconTopCorporateList(String userId, String branchCode, String fromDate, String toDate, String reconTypeRequest, String trendlineFlag, String trendlineDate, String corporateId) {
		LOGGER.info("Getting getAllocationReconTopCorporate List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U
		String reconType = "A".equalsIgnoreCase(reconTypeRequest)? "'A'" : "M".equalsIgnoreCase(reconTypeRequest) ? "'M'" :"P".equalsIgnoreCase(reconTypeRequest) ? "'M','A'" : "";
		String paymentStatus = "A".equalsIgnoreCase(reconTypeRequest) ? "ALLOCATED" : "P".equalsIgnoreCase(reconTypeRequest) ? "PART_ALLOCATED" : "U".equalsIgnoreCase(reconTypeRequest) ? "UNALLOCATED" : "F".equalsIgnoreCase(reconTypeRequest) ? "FAILED" : "";
		
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
			selectQuery.append(" SELECT * FROM ( SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM (SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO  " + 
					"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
					"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery).append( "    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ") .append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO  " + 
									"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
									"     AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery).append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID   " + 
											"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
											"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO  " + 
													"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
													"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery).append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID " + 
															"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
															"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO  " + 
																	"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
																	"      AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery).append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  " + 
																			"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
																			"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause).append(" )A  " + 
																					"    GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ").append(ROWNUM_CLAUSE);
		
		
		
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		response.setCorporateCount(populateCorporateCountList(resultSet,userId, branchCode,fromDate,toDate,reconType, paymentStatus, "Allocation", reconTypeRequest, trendlineFlag, trendlineDate)); 
		
		return response;
		
	}

	@Transactional("PLATO_TX_MANAGER")
	public ReconCorporateDetailsResponseDto getAllocationReconCorporateDetailsList(String userId, String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String corporateID, String reconTypeRequest, String trendlineFlag, String trendlineDate) {
		LOGGER.info("Getting getAllocationReconCorporateDetails List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U and P
		//String reconType = "A".equalsIgnoreCase(reconCetegory)? "'A'" : "M".equalsIgnoreCase(reconCetegory) ? "'M'" :"P".equalsIgnoreCase(reconCetegory) ? "'M','A'" : "";
		//String paymentStatus = ("A".equalsIgnoreCase(reconCetegory) || "M".equalsIgnoreCase(reconCetegory)) ? "MATCHED" : "P".equalsIgnoreCase(reconCetegory) ? "PART_MATCHED" : "";
		
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateID)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateID).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		
			
			selectQuery.append(" SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_ALLOCATION_STATUS,  RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.ALLOCATED_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.ALLOCATED_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER , PAYMENT.DEBIT_CREDIT_INDICATOR,  " + 
					"    RPAYMENT.ALLOC_STATUS, PAYMENT.FLAG AS PAYMENT_TOWARDS, PAYMENT.PAYMENT_STATUS " + 
					"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
					"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery)  
			.append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER     " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
			.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_ALLOCATION_STATUS,  RPAYMENT.RECON_ACTION,    " + 
					"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.ALLOCATED_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.ALLOCATED_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
					"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER , PAYMENT.DEBIT_CREDIT_INDICATOR, " + 
					"    RPAYMENT.ALLOC_STATUS, PAYMENT.FLAG AS PAYMENT_TOWARDS, PAYMENT.PAYMENT_STATUS " + 
					"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
					"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery)
							.append( "    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID   " + 
							"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
							"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
							.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_ALLOCATION_STATUS, RPAYMENT.RECON_ACTION,    " + 
									"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.ALLOCATED_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.ALLOCATED_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
									"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER , PAYMENT.DEBIT_CREDIT_INDICATOR, " + 
									"    RPAYMENT.ALLOC_STATUS, PAYMENT.FLAG AS PAYMENT_TOWARDS, PAYMENT.PAYMENT_STATUS " + 
									"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
									"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery) 
							.append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID   " + 
									"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C' " + 
									"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = ").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
							.append(UNION_ALL).append("    SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_ALLOCATION_STATUS,  RPAYMENT.RECON_ACTION,    " + 
									"    PAYMENT.PAYMENT_AMOUNT, RPAYMENT.ALLOCATED_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.ALLOCATED_AMT) AS NOT_RECONCILED_AMOUNT,  " + 
									"    RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER , PAYMENT.DEBIT_CREDIT_INDICATOR, " + 
									"    RPAYMENT.ALLOC_STATUS, PAYMENT.FLAG AS PAYMENT_TOWARDS, PAYMENT.PAYMENT_STATUS " + 
									"    FROM OB_RECON_ALLOC_PMT_TXN RPAYMENT     JOIN SCFCM_PAYMENT PAYMENT ON     RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  " + 
									"    AND RPAYMENT.PMT_ALLOCATION_STATUS = ").append(reconPaymentStatusQuery)
							.append("    LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID  " + 
									"    AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D' " + 
									"    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =").append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause);
		List<Object[]> resultSet = obReconAllocPmtTxnRepository.executeNativeQuery(selectQuery.toString());
		ReconCorporateDetailsResponseDto response = new ReconCorporateDetailsResponseDto();
		response.setCorporateCount(populateAllocationCorporateDetails(resultSet, branchCode, userId));
		
		return response;
		
	}
	
	
	@Transactional("PLATO_TX_MANAGER")
	public ReconTrendGraphResponseDto getReconciliationTrendGraphList(String userId, String branchCode, String fromDate, String toDate, String corporateId) {
		LOGGER.info("Getting getReconciliationTrendGraphList List in service");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append("  SELECT A.RECON_CATEGORY, COUNT(A.PAYMENT_REF_NO) AS COUNT_RECON, TO_CHAR(A.PAYMENT_DATE, 'MON') AS MONTH , TO_CHAR(A.PAYMENT_DATE, 'YYYY') AS YEAR, TO_CHAR(A.PAYMENT_DATE, 'MON-YYYY') AS PAYMENT_DATE FROM (  ") 
			    .append("	                SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE   FROM OB_RECON_INV_PMT_TXN RPAYMENT      JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ('M','A')   JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ('M','A')    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ('M','A')    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A')    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ('M','A')  JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PMT_PROGRAM_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A') JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A') JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A') LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER                        AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A') LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')	AND RPAYMENT.RECON_TYPE IN ('M','A')  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A')  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL                      AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(UNION_ALL).append("	SELECT CASE RECON_TYPE WHEN 'A' THEN 'Auto Reconciled' ELSE 'Manual Reconciled' END AS RECON_CATEGORY, RPAYMENT.PAYMENT_REF_NO , PAYMENT.PAYMENT_DATE FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED') AND RPAYMENT.RECON_TYPE IN ('M','A')  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL                       AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
				.append(" ) A GROUP BY A.PAYMENT_DATE, A.RECON_CATEGORY ORDER BY A.PAYMENT_DATE ");
		
		List<Object[]> resultSet = obExpDrCrPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTrendGraphResponseDto response = new ReconTrendGraphResponseDto();
		response.setCategoryCount(populateReconTrendGraphList(resultSet, fromDate, toDate, "Trend Graph"));
		return response;
		
	}
	
	@Override
	@Transactional("PLATO_TX_MANAGER")
	public ReconTopCorporateResponseDto getReconciliationTopCorporateList(String userId, String branchCode, String fromDate, String toDate, String reconTypeRequest, String trendlineFlag, String trendlineDate, String corporateId) {
		LOGGER.info("Getting getReconciliationTopCorporateList List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U
		String reconType = "A".equalsIgnoreCase(reconTypeRequest)? "'A'" : "M".equalsIgnoreCase(reconTypeRequest) ? "'M'" :"P".equalsIgnoreCase(reconTypeRequest) ? "'M','A'" : "";
		String paymentStatus = ("A".equalsIgnoreCase(reconTypeRequest) || "M".equalsIgnoreCase(reconTypeRequest)) ? "MATCHED" : "P".equalsIgnoreCase(reconTypeRequest) ? "PART_MATCHED" : "";
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = "Y".equalsIgnoreCase(trendlineFlag)?formDateQueryForTrendline(trendlineDate) : formDateQuery(fromDate,toDate);
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateId)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateId).append("' ") : new StringBuilder("");
		
		StringBuilder selectQuery = new StringBuilder();
		selectQuery.append(" SELECT * FROM (SELECT A.CORPORATE_NAME, A.CORPORATE_ID, COUNT(A.PAYMENT_REF_NO) CORPORATE_COUNT FROM (  " + 
						"	SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
						.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
							.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
							.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
							.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID , PAYMENT.PAYMENT_REF_NO      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PMT_PROGRAM_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
								.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO    AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
								.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID, PAYMENT.PAYMENT_REF_NO        FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
								.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER                        AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
										.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
												.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')	  AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
														.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  			     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL                      AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
																.append(UNION_ALL).append("				SELECT REL.CORPORATE_NAME, REL.CORPORATE_ID,  PAYMENT.PAYMENT_REF_NO  				     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL                       AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
																								.append("	)A  GROUP BY A.CORPORATE_NAME, A.CORPORATE_ID ORDER BY CORPORATE_COUNT DESC) WHERE ").append(ROWNUM_CLAUSE);
		
		
		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconTopCorporateResponseDto response = new ReconTopCorporateResponseDto();
		response.setCorporateCount(populateCorporateCountList(resultSet,userId, branchCode,fromDate,toDate,reconType, paymentStatus,"All", reconTypeRequest, trendlineFlag, trendlineDate));
		
		return response;
		
	}
	
	
	@Transactional("PLATO_TX_MANAGER")
	public ReconCorporateDetailsResponseDto getReconciliationCorporateDetailsList(String userId, String branchCode, String fromDate, String toDate, String reconType, String paymentStatus, String corporateID, String reconTypeRequest, String trendlineFlag, String trendlineDate) {
		LOGGER.info("Getting getAllocationReconCorporateDetails List in service");
		
		//reconcategory is mandatory in this API.. need to validate
		// Need to add logic for U and P
		//String reconType = "A".equalsIgnoreCase(reconCetegory)? "'A'" : "M".equalsIgnoreCase(reconCetegory) ? "'M'" :"P".equalsIgnoreCase(reconCetegory) ? "'M','A'" : "";
		//String paymentStatus = ("A".equalsIgnoreCase(reconCetegory) || "M".equalsIgnoreCase(reconCetegory)) ? "MATCHED" : "P".equalsIgnoreCase(reconCetegory) ? "PART_MATCHED" : "";
		
		StringBuilder corporateIDClause = !StringUtils.isEmpty(corporateID)? new StringBuilder(" AND REL.CORPORATE_ID ='").append(corporateID).append("' ") : new StringBuilder("");
		StringBuilder selectQuery = new StringBuilder();
		StringBuilder reconTypeQuery = new StringBuilder("( ").append(reconType).append(" )");
		StringBuilder reconBranchCodeQuery = new StringBuilder("'").append(branchCode).append("' ");
		String reconFormToDateQuery = formDateQuery(fromDate,toDate);
		StringBuilder reconPaymentStatusQuery = new StringBuilder("'").append(paymentStatus).append("' ");
		
			
		selectQuery.append("SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME            FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN  ").append(reconTypeQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.RELATIONSHIP_CODE = RPAYMENT.PMT_RELATIONSHIP_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
				.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME   FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENEFICIARY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE =" ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
					.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("      JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_COUNTERPARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND  RPAYMENT.BENEFICIARY_ID IS NULL       AND  RPAYMENT.PAYMENT_COUNTERPARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
					.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,INV_BUYER_ID AS BUYER_ID, INV_BUYER_NAME AS BUYER_NAME, INV_SUPPLIER_ID AS SUPPLIER_ID, INV_SUPPLIER_NAME AS SUPPLIER_NAME    FROM OB_RECON_INV_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_RELATIONSHIP_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
					.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO  AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')      AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("     JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PMT_PROGRAM_CODE       WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
						.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO    AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
						.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS, RPAYMENT.RECON_ACTION,        PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER,      PAYMENT.DEBIT_CREDIT_INDICATOR,FIN_BUYER_ID AS BUYER_ID, FIN_BUYER_NAME AS BUYER_NAME, FIN_SUPPLIER_ID AS SUPPLIER_ID, FIN_SUPPLIER_NAME AS SUPPLIER_NAME      FROM OB_RECON_FIN_PMT_TXN RPAYMENT        JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("    JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID        AND RPAYMENT.PMT_PROGRAM_CODE IS NULL AND RPAYMENT.PAYMENT_PARTY_ID IS NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'     WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
						.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME 	     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')     AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("  LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.ACCOUNT_OWNER                        AND RPAYMENT.ACCOUNT_OWNER IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause)
								.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME 		     FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.CORPORATE_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NOT NULL    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
										.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')	  AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.BENE_CODE                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND  RPAYMENT.BENE_CODE IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL  AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
												.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')   AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.COUNTERPARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL AND RPAYMENT.BENE_CODE IS NULL                      AND RPAYMENT.COUNTERPARTY_ID IS NOT NULL  AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'C'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause) 
														.append(UNION_ALL).append("	SELECT REL.CORPORATE_NAME,REL.CORPORATE_ID, RPAYMENT.PMT_STATUS,  RPAYMENT.RECON_ACTION,       PAYMENT.PAYMENT_AMOUNT, RPAYMENT.RECONCILED_PMT_AMT, (PAYMENT.PAYMENT_AMOUNT - RPAYMENT.RECONCILED_PMT_AMT) AS NOT_RECONCILED_AMOUNT,      RPAYMENT.PAYMENT_CCY, RPAYMENT.RECON_CATEGORY, PAYMENT.PAYMENT_NUMBER, PAYMENT.DEBIT_CREDIT_INDICATOR, NULL AS BUYER_ID, NULL AS BUYER_NAME, NULL AS SUPPLIER_ID, NULL AS SUPPLIER_NAME    FROM OB_EXP_DRCR_PMT_RECON_TXN RPAYMENT   				     JOIN SCFCM_PAYMENT PAYMENT ON        RPAYMENT.PAYMENT_REF_NO = PAYMENT.PAYMENT_REF_NO   AND PAYMENT.PAYMENT_STATUS IN ('MATCHED','PART_MATCHED')    AND RPAYMENT.RECON_TYPE IN ").append(reconTypeQuery).append("   LEFT OUTER JOIN OBCM_TM_RELATIONSHIP_MASTER REL ON REL.CORPORATE_ID = RPAYMENT.PAYMENT_PARTY_ID                        AND RPAYMENT.ACCOUNT_OWNER IS NULL AND RPAYMENT.CORPORATE_ID IS NULL                       AND RPAYMENT.PAYMENT_PARTY_ID IS NOT NULL AND PAYMENT.DEBIT_CREDIT_INDICATOR = 'D'    WHERE REL.CORPORATE_NAME IS NOT NULL AND RPAYMENT.BRANCH_CODE = " ).append(reconBranchCodeQuery).append(reconFormToDateQuery).append(corporateIDClause);


		List<Object[]> resultSet = obInvPmtReconRepository.executeNativeQuery(selectQuery.toString());
		ReconCorporateDetailsResponseDto response = new ReconCorporateDetailsResponseDto();
		response.setCorporateCount(populateCorporateDetails(resultSet, branchCode, userId));
		
		return response;
		
	}
	
	
	private static List<String>  getMonthAndDateOfGivenRange(String date1, String date2) {
		List<String> dates = new ArrayList<>();
    	DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd", Locale.ROOT);
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MMM-yyyy", Locale.ROOT);
        YearMonth endMonth = YearMonth.parse(date2, dateFormatter);
        for (YearMonth month = YearMonth.parse(date1, dateFormatter);
                ! month.isAfter(endMonth);
                month = month.plusMonths(1)) {
        	dates.add(month.format(monthFormatter).toUpperCase());
        }
    	
    	return dates;
    }
	
	private BigDecimal getAmountInLocalCurrency(String localBranchCCY, BigDecimal paymentAmount, String paymentCCY, String branchCode,
			String userId, String simpleBranchDate) {
		int retryCount = 3;
		BigDecimal paymentAmountInLCY = BigDecimal.ZERO;
		CurrencyUtilityDto currencyUtilityDto = null;
		if (!paymentCCY.equalsIgnoreCase(localBranchCCY) && paymentAmount != null
				&& !paymentAmount.equals(new BigDecimal(0))) {
			
			while(null == currencyUtilityDto && retryCount > 0) {
				currencyUtilityDto = cmcCurrencyServiceClient.getConvertedCcyValue(paymentCCY, localBranchCCY,
						paymentAmount, simpleBranchDate, userId, branchCode);
				retryCount--;
			}
			
			paymentAmountInLCY = (currencyUtilityDto != null ) ? currencyUtilityDto.getAmount() : BigDecimal.ZERO;
		}
		return paymentAmountInLCY;
	}

}